{-# LANGUAGE MagicHash #-}
{-# OPTIONS_GHC -fno-warn-overlapping-patterns #-}


module Curry_UnificationSpec (C_Term (..), C_UnificationError (..), C_VarIdx, C_TermEq, C_TermEqs, C_Subst, d_C_showSubst, nd_C_showSubst, d_C_emptySubst, nd_C_emptySubst, d_C_extendSubst, nd_C_extendSubst, d_C_lookupSubst, nd_C_lookupSubst, d_C_applySubst, nd_C_applySubst, d_C_unify, nd_C_unify) where

import Basics
import qualified Curry_FiniteMap
import qualified Curry_Prelude

type C_VarIdx = Curry_Prelude.C_Int

type C_TermEq = Curry_Prelude.OP_Tuple2 C_Term C_Term

type C_TermEqs = Curry_Prelude.OP_List (Curry_Prelude.OP_Tuple2 C_Term C_Term)

type C_Subst = Curry_FiniteMap.C_FM Curry_Prelude.C_Int C_Term

data C_Term
     = C_TermVar Curry_Prelude.C_Int
     | C_TermCons (Curry_Prelude.OP_List Curry_Prelude.C_Char) (Curry_Prelude.OP_List C_Term)
     | Choice_C_Term Cover ID C_Term C_Term
     | Choices_C_Term Cover ID ([C_Term])
     | Fail_C_Term Cover FailInfo
     | Guard_C_Term Cover Constraints C_Term

instance Show C_Term where
  showsPrec d (Choice_C_Term cd i x y) = showsChoice d cd i x y
  showsPrec d (Choices_C_Term cd i xs) = showsChoices d cd i xs
  showsPrec d (Guard_C_Term cd c e) = showsGuard d cd c e
  showsPrec _ (Fail_C_Term cd info) = showChar '!'
  showsPrec _ (C_TermVar x1) = (showString "(TermVar") . ((showChar ' ') . ((shows x1) . (showChar ')')))
  showsPrec _ (C_TermCons x1 x2) = (showString "(TermCons") . ((showChar ' ') . ((shows x1) . ((showChar ' ') . ((shows x2) . (showChar ')')))))


instance Read C_Term where
  readsPrec d s = (readParen (d > 10) (\r -> [ (C_TermVar x1,r1) | (_,r0) <- readQualified "UnificationSpec" "TermVar" r, (x1,r1) <- readsPrec 11 r0]) s) ++ (readParen (d > 10) (\r -> [ (C_TermCons x1 x2,r2) | (_,r0) <- readQualified "UnificationSpec" "TermCons" r, (x1,r1) <- readsPrec 11 r0, (x2,r2) <- readsPrec 11 r1]) s)


instance NonDet C_Term where
  choiceCons = Choice_C_Term
  choicesCons = Choices_C_Term
  failCons = Fail_C_Term
  guardCons = Guard_C_Term
  try (Choice_C_Term cd i x y) = tryChoice cd i x y
  try (Choices_C_Term cd i xs) = tryChoices cd i xs
  try (Fail_C_Term cd info) = Fail cd info
  try (Guard_C_Term cd c e) = Guard cd c e
  try x = Val x
  match f _ _ _ _ _ (Choice_C_Term cd i x y) = f cd i x y
  match _ f _ _ _ _ (Choices_C_Term cd i@(NarrowedID _ _) xs) = f cd i xs
  match _ _ f _ _ _ (Choices_C_Term cd i@(FreeID _ _) xs) = f cd i xs
  match _ _ _ _ _ _ (Choices_C_Term cd i _) = error ("UnificationSpec.Term.match: Choices with ChoiceID " ++ (show i))
  match _ _ _ f _ _ (Fail_C_Term cd info) = f cd info
  match _ _ _ _ f _ (Guard_C_Term cd c e) = f cd c e
  match _ _ _ _ _ f x = f x


instance Generable C_Term where
  generate s c = Choices_C_Term c (freeID [1,2] s) [(C_TermVar (generate (leftSupply s) c)),(C_TermCons (generate (leftSupply s) c) (generate (rightSupply s) c))]


instance NormalForm C_Term where
  ($!!) cont (C_TermVar x1) d cs = (((\y1 d cs -> cont (C_TermVar y1) d cs) $!! x1) d) cs
  ($!!) cont (C_TermCons x1 x2) d cs = (((\y1 d cs -> (((\y2 d cs -> cont (C_TermCons y1 y2) d cs) $!! x2) d) cs) $!! x1) d) cs
  ($!!) cont (Choice_C_Term cd i x y) d cs = nfChoice cont cd i x y cd cs
  ($!!) cont (Choices_C_Term cd i xs) d cs = nfChoices cont cd i xs d cs
  ($!!) cont (Guard_C_Term cd c e) d cs = guardCons cd c (((cont $!! e) d) (addCs c cs))
  ($!!) _ (Fail_C_Term cd info) _ _ = failCons cd info
  ($##) cont (C_TermVar x1) d cs = (((\y1 d cs -> cont (C_TermVar y1) d cs) $## x1) d) cs
  ($##) cont (C_TermCons x1 x2) d cs = (((\y1 d cs -> (((\y2 d cs -> cont (C_TermCons y1 y2) d cs) $## x2) d) cs) $## x1) d) cs
  ($##) cont (Choice_C_Term cd i x y) d cs = gnfChoice cont cd i x y cd cs
  ($##) cont (Choices_C_Term cd i xs) d cs = gnfChoices cont cd i xs d cs
  ($##) cont (Guard_C_Term cd c e) d cs = guardCons cd c (((cont $## e) d) (addCs c cs))
  ($##) _ (Fail_C_Term cd info) _ _ = failCons cd info
  showCons (C_TermVar _) = "UnificationSpec.TermVar _"
  showCons (C_TermCons _ _) = "UnificationSpec.TermCons _ _"
  showCons x = error ("UnificationSpec.Term.showCons: no constructor: " ++ (show x))
  searchNF search cont (C_TermVar x1) = search (\y1 -> cont (C_TermVar y1)) x1
  searchNF search cont (C_TermCons x1 x2) = search (\y1 -> search (\y2 -> cont (C_TermCons y1 y2)) x2) x1
  searchNF _ _ x = error ("UnificationSpec.Term.searchNF: no constructor: " ++ (show x))


instance Unifiable C_Term where
  (=.=) (C_TermVar x1) (C_TermVar y1) d cs = ((x1 =:= y1) d) cs
  (=.=) (C_TermCons x1 x2) (C_TermCons y1 y2) d cs = (((((x1 =:= y1) d) cs) & (((x2 =:= y2) d) cs)) d) cs
  (=.=) a b cd _ = Fail_C_Success cd (unificationFail (showCons a) (showCons b))
  (=.<=) (C_TermVar x1) (C_TermVar y1) d cs = ((x1 =:<= y1) d) cs
  (=.<=) (C_TermCons x1 x2) (C_TermCons y1 y2) d cs = (((((x1 =:<= y1) d) cs) & (((x2 =:<= y2) d) cs)) d) cs
  (=.<=) a b cd _ = Fail_C_Success cd (unificationFail (showCons a) (showCons b))
  bind cd i (C_TermVar x3) = ((i :=: (ChooseN 0 1)):(concat [(bind cd (leftID i) x3)]))
  bind cd i (C_TermCons x3 x4) = ((i :=: (ChooseN 1 2)):(concat [(bind cd (leftID i) x3),(bind cd (rightID i) x4)]))
  bind d i (Choice_C_Term cd j x y) = [(ConstraintChoice cd j (bind d i x) (bind d i y))]
  bind d i (Choices_C_Term cd j@(FreeID _ _) xs) = bindOrNarrow d i cd j xs
  bind d i (Choices_C_Term cd j@(NarrowedID _ _) xs) = [(ConstraintChoices cd j (map (bind d i) xs))]
  bind _ _ (Choices_C_Term cd i _) = error ("UnificationSpec.Term.bind: Choices with ChoiceID: " ++ (show i))
  bind _ _ (Fail_C_Term cd info) = [(Unsolvable info)]
  bind d i (Guard_C_Term cd c e) = (getConstrList c) ++ (bind d i e)
  lazyBind cd i (C_TermVar x3) = [(i :=: (ChooseN 0 1)),((leftID i) :=: (LazyBind (lazyBind cd (leftID i) x3)))]
  lazyBind cd i (C_TermCons x3 x4) = [(i :=: (ChooseN 1 2)),((leftID i) :=: (LazyBind (lazyBind cd (leftID i) x3))),((rightID i) :=: (LazyBind (lazyBind cd (rightID i) x4)))]
  lazyBind d i (Choice_C_Term cd j x y) = [(ConstraintChoice cd j (lazyBind d i x) (lazyBind d i y))]
  lazyBind d i (Choices_C_Term cd j@(FreeID _ _) xs) = lazyBindOrNarrow d i cd j xs
  lazyBind d i (Choices_C_Term cd j@(NarrowedID _ _) xs) = [(ConstraintChoices cd j (map (lazyBind d i) xs))]
  lazyBind _ _ (Choices_C_Term cd i _) = error ("UnificationSpec.Term.lazyBind: Choices with ChoiceID: " ++ (show i))
  lazyBind _ _ (Fail_C_Term cd info) = [(Unsolvable info)]
  lazyBind d i (Guard_C_Term cd c e) = (getConstrList c) ++ [(i :=: (LazyBind (lazyBind d i e)))]


instance Curry_Prelude.Curry C_Term where
  (=?=) (Choice_C_Term cd i x y) z d cs = narrow cd i (((x Curry_Prelude.=?= z) d) cs) (((y Curry_Prelude.=?= z) d) cs)
  (=?=) (Choices_C_Term cd i xs) y d cs = narrows cs cd i (\x -> ((x Curry_Prelude.=?= y) d) cs) xs
  (=?=) (Guard_C_Term cd c e) y d cs = guardCons cd c (((e Curry_Prelude.=?= y) d) (addCs c cs))
  (=?=) (Fail_C_Term cd info) _ _ _ = failCons cd info
  (=?=) z (Choice_C_Term cd i x y) d cs = narrow cd i (((z Curry_Prelude.=?= x) d) cs) (((z Curry_Prelude.=?= y) d) cs)
  (=?=) y (Choices_C_Term cd i xs) d cs = narrows cs cd i (\x -> ((y Curry_Prelude.=?= x) d) cs) xs
  (=?=) y (Guard_C_Term cd c e) d cs = guardCons cd c (((y Curry_Prelude.=?= e) d) (addCs c cs))
  (=?=) _ (Fail_C_Term cd info) _ _ = failCons cd info
  (=?=) (C_TermVar x1) (C_TermVar y1) d cs = ((x1 Curry_Prelude.=?= y1) d) cs
  (=?=) (C_TermCons x1 x2) (C_TermCons y1 y2) d cs = Curry_Prelude.d_OP_amp_amp (((x1 Curry_Prelude.=?= y1) d) cs) (((x2 Curry_Prelude.=?= y2) d) cs) d cs
  (=?=) _ _ d _ = Curry_Prelude.C_False
  (<?=) (Choice_C_Term cd i x y) z d cs = narrow cd i (((x Curry_Prelude.<?= z) d) cs) (((y Curry_Prelude.<?= z) d) cs)
  (<?=) (Choices_C_Term cd i xs) y d cs = narrows cs cd i (\x -> ((x Curry_Prelude.<?= y) d) cs) xs
  (<?=) (Guard_C_Term cd c e) y d cs = guardCons cd c (((e Curry_Prelude.<?= y) d) (addCs c cs))
  (<?=) (Fail_C_Term cd info) _ _ _ = failCons cd info
  (<?=) z (Choice_C_Term cd i x y) d cs = narrow cd i (((z Curry_Prelude.<?= x) d) cs) (((z Curry_Prelude.<?= y) d) cs)
  (<?=) y (Choices_C_Term cd i xs) d cs = narrows cs cd i (\x -> ((y Curry_Prelude.<?= x) d) cs) xs
  (<?=) y (Guard_C_Term cd c e) d cs = guardCons cd c (((y Curry_Prelude.<?= e) d) (addCs c cs))
  (<?=) _ (Fail_C_Term cd info) _ _ = failCons cd info
  (<?=) (C_TermVar x1) (C_TermVar y1) d cs = ((x1 Curry_Prelude.<?= y1) d) cs
  (<?=) (C_TermVar _) (C_TermCons _ _) _ _ = Curry_Prelude.C_True
  (<?=) (C_TermCons x1 x2) (C_TermCons y1 y2) d cs = Curry_Prelude.d_OP_bar_bar (Curry_Prelude.d_OP_lt x1 y1 d cs) (Curry_Prelude.d_OP_amp_amp (((x1 Curry_Prelude.=?= y1) d) cs) (((x2 Curry_Prelude.<?= y2) d) cs) d cs) d cs
  (<?=) _ _ d _ = Curry_Prelude.C_False


data C_UnificationError
     = C_Clash C_Term C_Term
     | C_OccurCheck Curry_Prelude.C_Int C_Term
     | Choice_C_UnificationError Cover ID C_UnificationError C_UnificationError
     | Choices_C_UnificationError Cover ID ([C_UnificationError])
     | Fail_C_UnificationError Cover FailInfo
     | Guard_C_UnificationError Cover Constraints C_UnificationError

instance Show C_UnificationError where
  showsPrec d (Choice_C_UnificationError cd i x y) = showsChoice d cd i x y
  showsPrec d (Choices_C_UnificationError cd i xs) = showsChoices d cd i xs
  showsPrec d (Guard_C_UnificationError cd c e) = showsGuard d cd c e
  showsPrec _ (Fail_C_UnificationError cd info) = showChar '!'
  showsPrec _ (C_Clash x1 x2) = (showString "(Clash") . ((showChar ' ') . ((shows x1) . ((showChar ' ') . ((shows x2) . (showChar ')')))))
  showsPrec _ (C_OccurCheck x1 x2) = (showString "(OccurCheck") . ((showChar ' ') . ((shows x1) . ((showChar ' ') . ((shows x2) . (showChar ')')))))


instance Read C_UnificationError where
  readsPrec d s = (readParen (d > 10) (\r -> [ (C_Clash x1 x2,r2) | (_,r0) <- readQualified "UnificationSpec" "Clash" r, (x1,r1) <- readsPrec 11 r0, (x2,r2) <- readsPrec 11 r1]) s) ++ (readParen (d > 10) (\r -> [ (C_OccurCheck x1 x2,r2) | (_,r0) <- readQualified "UnificationSpec" "OccurCheck" r, (x1,r1) <- readsPrec 11 r0, (x2,r2) <- readsPrec 11 r1]) s)


instance NonDet C_UnificationError where
  choiceCons = Choice_C_UnificationError
  choicesCons = Choices_C_UnificationError
  failCons = Fail_C_UnificationError
  guardCons = Guard_C_UnificationError
  try (Choice_C_UnificationError cd i x y) = tryChoice cd i x y
  try (Choices_C_UnificationError cd i xs) = tryChoices cd i xs
  try (Fail_C_UnificationError cd info) = Fail cd info
  try (Guard_C_UnificationError cd c e) = Guard cd c e
  try x = Val x
  match f _ _ _ _ _ (Choice_C_UnificationError cd i x y) = f cd i x y
  match _ f _ _ _ _ (Choices_C_UnificationError cd i@(NarrowedID _ _) xs) = f cd i xs
  match _ _ f _ _ _ (Choices_C_UnificationError cd i@(FreeID _ _) xs) = f cd i xs
  match _ _ _ _ _ _ (Choices_C_UnificationError cd i _) = error ("UnificationSpec.UnificationError.match: Choices with ChoiceID " ++ (show i))
  match _ _ _ f _ _ (Fail_C_UnificationError cd info) = f cd info
  match _ _ _ _ f _ (Guard_C_UnificationError cd c e) = f cd c e
  match _ _ _ _ _ f x = f x


instance Generable C_UnificationError where
  generate s c = Choices_C_UnificationError c (freeID [2,2] s) [(C_Clash (generate (leftSupply s) c) (generate (rightSupply s) c)),(C_OccurCheck (generate (leftSupply s) c) (generate (rightSupply s) c))]


instance NormalForm C_UnificationError where
  ($!!) cont (C_Clash x1 x2) d cs = (((\y1 d cs -> (((\y2 d cs -> cont (C_Clash y1 y2) d cs) $!! x2) d) cs) $!! x1) d) cs
  ($!!) cont (C_OccurCheck x1 x2) d cs = (((\y1 d cs -> (((\y2 d cs -> cont (C_OccurCheck y1 y2) d cs) $!! x2) d) cs) $!! x1) d) cs
  ($!!) cont (Choice_C_UnificationError cd i x y) d cs = nfChoice cont cd i x y cd cs
  ($!!) cont (Choices_C_UnificationError cd i xs) d cs = nfChoices cont cd i xs d cs
  ($!!) cont (Guard_C_UnificationError cd c e) d cs = guardCons cd c (((cont $!! e) d) (addCs c cs))
  ($!!) _ (Fail_C_UnificationError cd info) _ _ = failCons cd info
  ($##) cont (C_Clash x1 x2) d cs = (((\y1 d cs -> (((\y2 d cs -> cont (C_Clash y1 y2) d cs) $## x2) d) cs) $## x1) d) cs
  ($##) cont (C_OccurCheck x1 x2) d cs = (((\y1 d cs -> (((\y2 d cs -> cont (C_OccurCheck y1 y2) d cs) $## x2) d) cs) $## x1) d) cs
  ($##) cont (Choice_C_UnificationError cd i x y) d cs = gnfChoice cont cd i x y cd cs
  ($##) cont (Choices_C_UnificationError cd i xs) d cs = gnfChoices cont cd i xs d cs
  ($##) cont (Guard_C_UnificationError cd c e) d cs = guardCons cd c (((cont $## e) d) (addCs c cs))
  ($##) _ (Fail_C_UnificationError cd info) _ _ = failCons cd info
  showCons (C_Clash _ _) = "UnificationSpec.Clash _ _"
  showCons (C_OccurCheck _ _) = "UnificationSpec.OccurCheck _ _"
  showCons x = error ("UnificationSpec.UnificationError.showCons: no constructor: " ++ (show x))
  searchNF search cont (C_Clash x1 x2) = search (\y1 -> search (\y2 -> cont (C_Clash y1 y2)) x2) x1
  searchNF search cont (C_OccurCheck x1 x2) = search (\y1 -> search (\y2 -> cont (C_OccurCheck y1 y2)) x2) x1
  searchNF _ _ x = error ("UnificationSpec.UnificationError.searchNF: no constructor: " ++ (show x))


instance Unifiable C_UnificationError where
  (=.=) (C_Clash x1 x2) (C_Clash y1 y2) d cs = (((((x1 =:= y1) d) cs) & (((x2 =:= y2) d) cs)) d) cs
  (=.=) (C_OccurCheck x1 x2) (C_OccurCheck y1 y2) d cs = (((((x1 =:= y1) d) cs) & (((x2 =:= y2) d) cs)) d) cs
  (=.=) a b cd _ = Fail_C_Success cd (unificationFail (showCons a) (showCons b))
  (=.<=) (C_Clash x1 x2) (C_Clash y1 y2) d cs = (((((x1 =:<= y1) d) cs) & (((x2 =:<= y2) d) cs)) d) cs
  (=.<=) (C_OccurCheck x1 x2) (C_OccurCheck y1 y2) d cs = (((((x1 =:<= y1) d) cs) & (((x2 =:<= y2) d) cs)) d) cs
  (=.<=) a b cd _ = Fail_C_Success cd (unificationFail (showCons a) (showCons b))
  bind cd i (C_Clash x3 x4) = ((i :=: (ChooseN 0 2)):(concat [(bind cd (leftID i) x3),(bind cd (rightID i) x4)]))
  bind cd i (C_OccurCheck x3 x4) = ((i :=: (ChooseN 1 2)):(concat [(bind cd (leftID i) x3),(bind cd (rightID i) x4)]))
  bind d i (Choice_C_UnificationError cd j x y) = [(ConstraintChoice cd j (bind d i x) (bind d i y))]
  bind d i (Choices_C_UnificationError cd j@(FreeID _ _) xs) = bindOrNarrow d i cd j xs
  bind d i (Choices_C_UnificationError cd j@(NarrowedID _ _) xs) = [(ConstraintChoices cd j (map (bind d i) xs))]
  bind _ _ (Choices_C_UnificationError cd i _) = error ("UnificationSpec.UnificationError.bind: Choices with ChoiceID: " ++ (show i))
  bind _ _ (Fail_C_UnificationError cd info) = [(Unsolvable info)]
  bind d i (Guard_C_UnificationError cd c e) = (getConstrList c) ++ (bind d i e)
  lazyBind cd i (C_Clash x3 x4) = [(i :=: (ChooseN 0 2)),((leftID i) :=: (LazyBind (lazyBind cd (leftID i) x3))),((rightID i) :=: (LazyBind (lazyBind cd (rightID i) x4)))]
  lazyBind cd i (C_OccurCheck x3 x4) = [(i :=: (ChooseN 1 2)),((leftID i) :=: (LazyBind (lazyBind cd (leftID i) x3))),((rightID i) :=: (LazyBind (lazyBind cd (rightID i) x4)))]
  lazyBind d i (Choice_C_UnificationError cd j x y) = [(ConstraintChoice cd j (lazyBind d i x) (lazyBind d i y))]
  lazyBind d i (Choices_C_UnificationError cd j@(FreeID _ _) xs) = lazyBindOrNarrow d i cd j xs
  lazyBind d i (Choices_C_UnificationError cd j@(NarrowedID _ _) xs) = [(ConstraintChoices cd j (map (lazyBind d i) xs))]
  lazyBind _ _ (Choices_C_UnificationError cd i _) = error ("UnificationSpec.UnificationError.lazyBind: Choices with ChoiceID: " ++ (show i))
  lazyBind _ _ (Fail_C_UnificationError cd info) = [(Unsolvable info)]
  lazyBind d i (Guard_C_UnificationError cd c e) = (getConstrList c) ++ [(i :=: (LazyBind (lazyBind d i e)))]


instance Curry_Prelude.Curry C_UnificationError where
  (=?=) (Choice_C_UnificationError cd i x y) z d cs = narrow cd i (((x Curry_Prelude.=?= z) d) cs) (((y Curry_Prelude.=?= z) d) cs)
  (=?=) (Choices_C_UnificationError cd i xs) y d cs = narrows cs cd i (\x -> ((x Curry_Prelude.=?= y) d) cs) xs
  (=?=) (Guard_C_UnificationError cd c e) y d cs = guardCons cd c (((e Curry_Prelude.=?= y) d) (addCs c cs))
  (=?=) (Fail_C_UnificationError cd info) _ _ _ = failCons cd info
  (=?=) z (Choice_C_UnificationError cd i x y) d cs = narrow cd i (((z Curry_Prelude.=?= x) d) cs) (((z Curry_Prelude.=?= y) d) cs)
  (=?=) y (Choices_C_UnificationError cd i xs) d cs = narrows cs cd i (\x -> ((y Curry_Prelude.=?= x) d) cs) xs
  (=?=) y (Guard_C_UnificationError cd c e) d cs = guardCons cd c (((y Curry_Prelude.=?= e) d) (addCs c cs))
  (=?=) _ (Fail_C_UnificationError cd info) _ _ = failCons cd info
  (=?=) (C_Clash x1 x2) (C_Clash y1 y2) d cs = Curry_Prelude.d_OP_amp_amp (((x1 Curry_Prelude.=?= y1) d) cs) (((x2 Curry_Prelude.=?= y2) d) cs) d cs
  (=?=) (C_OccurCheck x1 x2) (C_OccurCheck y1 y2) d cs = Curry_Prelude.d_OP_amp_amp (((x1 Curry_Prelude.=?= y1) d) cs) (((x2 Curry_Prelude.=?= y2) d) cs) d cs
  (=?=) _ _ d _ = Curry_Prelude.C_False
  (<?=) (Choice_C_UnificationError cd i x y) z d cs = narrow cd i (((x Curry_Prelude.<?= z) d) cs) (((y Curry_Prelude.<?= z) d) cs)
  (<?=) (Choices_C_UnificationError cd i xs) y d cs = narrows cs cd i (\x -> ((x Curry_Prelude.<?= y) d) cs) xs
  (<?=) (Guard_C_UnificationError cd c e) y d cs = guardCons cd c (((e Curry_Prelude.<?= y) d) (addCs c cs))
  (<?=) (Fail_C_UnificationError cd info) _ _ _ = failCons cd info
  (<?=) z (Choice_C_UnificationError cd i x y) d cs = narrow cd i (((z Curry_Prelude.<?= x) d) cs) (((z Curry_Prelude.<?= y) d) cs)
  (<?=) y (Choices_C_UnificationError cd i xs) d cs = narrows cs cd i (\x -> ((y Curry_Prelude.<?= x) d) cs) xs
  (<?=) y (Guard_C_UnificationError cd c e) d cs = guardCons cd c (((y Curry_Prelude.<?= e) d) (addCs c cs))
  (<?=) _ (Fail_C_UnificationError cd info) _ _ = failCons cd info
  (<?=) (C_Clash x1 x2) (C_Clash y1 y2) d cs = Curry_Prelude.d_OP_bar_bar (Curry_Prelude.d_OP_lt x1 y1 d cs) (Curry_Prelude.d_OP_amp_amp (((x1 Curry_Prelude.=?= y1) d) cs) (((x2 Curry_Prelude.<?= y2) d) cs) d cs) d cs
  (<?=) (C_Clash _ _) (C_OccurCheck _ _) _ _ = Curry_Prelude.C_True
  (<?=) (C_OccurCheck x1 x2) (C_OccurCheck y1 y2) d cs = Curry_Prelude.d_OP_bar_bar (Curry_Prelude.d_OP_lt x1 y1 d cs) (Curry_Prelude.d_OP_amp_amp (((x1 Curry_Prelude.=?= y1) d) cs) (((x2 Curry_Prelude.<?= y2) d) cs) d cs) d cs
  (<?=) _ _ d _ = Curry_Prelude.C_False


d_C_showSubst :: Cover -> ConstStore -> Curry_FiniteMap.C_FM Curry_Prelude.C_Int C_Term -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C_showSubst cd cs = Curry_Prelude.d_OP_dot Curry_Prelude.d_C_unlines (Curry_Prelude.d_OP_dot (Curry_Prelude.d_C_map d_OP_showSubst_dot_showOne_dot_2) Curry_FiniteMap.d_C_fmToList cd cs) cd cs

nd_C_showSubst :: IDSupply -> Cover -> ConstStore -> Func (Curry_FiniteMap.C_FM Curry_Prelude.C_Int C_Term) (Curry_Prelude.OP_List Curry_Prelude.C_Char)
nd_C_showSubst s cd cs = let
     s2 = s
      in (seq s2 (let
          s1 = leftSupply s2
          s0 = rightSupply s2
           in (seq s1 (seq s0 (Curry_Prelude.nd_OP_dot (wrapDX id Curry_Prelude.d_C_unlines) (Curry_Prelude.nd_OP_dot (wrapNX id (Curry_Prelude.nd_C_map (wrapDX id d_OP_showSubst_dot_showOne_dot_2))) (wrapNX id Curry_FiniteMap.nd_C_fmToList) s0 cd cs) s1 cd cs)))))

d_OP_showSubst_dot_showOne_dot_2 :: (Curry_Prelude.Curry t0,Curry_Prelude.Curry t1) => Curry_Prelude.OP_Tuple2 t0 t1 -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_OP_showSubst_dot_showOne_dot_2 x1 cd cs = case x1 of
     (Curry_Prelude.OP_Tuple2 x2 x3) -> Curry_Prelude.d_OP_plus_plus (Curry_Prelude.d_C_show x2 cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString " -> ") (Curry_Prelude.d_C_show x3 cd cs) cd cs) cd cs
     (Curry_Prelude.Choice_OP_Tuple2 d i l r) -> narrow d i (d_OP_showSubst_dot_showOne_dot_2 l cd cs) (d_OP_showSubst_dot_showOne_dot_2 r cd cs)
     (Curry_Prelude.Choices_OP_Tuple2 d i xs) -> narrows cs d i (\z -> d_OP_showSubst_dot_showOne_dot_2 z cd cs) xs
     (Curry_Prelude.Guard_OP_Tuple2 d c e) -> guardCons d c ((d_OP_showSubst_dot_showOne_dot_2 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_Tuple2 d info) -> failCons d (traceFail "UnificationSpec.showSubst.showOne.2" [(show x1)] info)
     _ -> failCons cd (consFail "UnificationSpec.showSubst.showOne.2" (showCons x1))

d_C_emptySubst :: Cover -> ConstStore -> Curry_FiniteMap.C_FM Curry_Prelude.C_Int C_Term
d_C_emptySubst cd cs = Curry_FiniteMap.d_C_emptyFM (acceptCs id Curry_Prelude.d_OP_lt) cd cs

nd_C_emptySubst :: IDSupply -> Cover -> ConstStore -> Curry_FiniteMap.C_FM Curry_Prelude.C_Int C_Term
nd_C_emptySubst s cd cs = let
     s0 = s
      in (seq s0 (Curry_FiniteMap.nd_C_emptyFM (wrapDX (wrapDX id) (acceptCs id Curry_Prelude.d_OP_lt)) s0 cd cs))

d_C_extendSubst :: Cover -> ConstStore -> Curry_FiniteMap.C_FM Curry_Prelude.C_Int C_Term -> Cover -> ConstStore -> Curry_Prelude.C_Int -> Cover -> ConstStore -> C_Term -> Cover -> ConstStore -> Curry_FiniteMap.C_FM Curry_Prelude.C_Int C_Term
d_C_extendSubst cd cs = acceptCs (acceptCs id) Curry_FiniteMap.d_C_addToFM

nd_C_extendSubst :: IDSupply -> Cover -> ConstStore -> Func (Curry_FiniteMap.C_FM Curry_Prelude.C_Int C_Term) (Func Curry_Prelude.C_Int (Func C_Term (Curry_FiniteMap.C_FM Curry_Prelude.C_Int C_Term)))
nd_C_extendSubst s cd cs = wrapDX (wrapDX (wrapNX id)) (acceptCs (acceptCs id) Curry_FiniteMap.nd_C_addToFM)

d_C_lookupSubst :: Cover -> ConstStore -> Curry_FiniteMap.C_FM Curry_Prelude.C_Int C_Term -> Cover -> ConstStore -> Curry_Prelude.C_Int -> Cover -> ConstStore -> Curry_Prelude.C_Maybe C_Term
d_C_lookupSubst cd cs = acceptCs id Curry_FiniteMap.d_C_lookupFM

nd_C_lookupSubst :: IDSupply -> Cover -> ConstStore -> Func (Curry_FiniteMap.C_FM Curry_Prelude.C_Int C_Term) (Func Curry_Prelude.C_Int (Curry_Prelude.C_Maybe C_Term))
nd_C_lookupSubst s cd cs = wrapDX (wrapNX id) (acceptCs id Curry_FiniteMap.nd_C_lookupFM)

d_C_applySubst :: Curry_FiniteMap.C_FM Curry_Prelude.C_Int C_Term -> C_Term -> Cover -> ConstStore -> C_Term
d_C_applySubst x1 x2 cd cs = case x2 of
     (C_TermVar x3) -> Curry_Prelude.d_C_maybe x2 Curry_Prelude.d_C_id (Curry_Prelude.d_C_apply (Curry_Prelude.d_C_apply (d_C_lookupSubst cd cs) x1 cd cs) x3 cd cs) cd cs
     (C_TermCons x4 x5) -> C_TermCons x4 (Curry_Prelude.d_C_map (d_C_applySubst x1) x5 cd cs)
     (Choice_C_Term d i l r) -> narrow d i (d_C_applySubst x1 l cd cs) (d_C_applySubst x1 r cd cs)
     (Choices_C_Term d i xs) -> narrows cs d i (\z -> d_C_applySubst x1 z cd cs) xs
     (Guard_C_Term d c e) -> guardCons d c ((d_C_applySubst x1 e cd) $! (addCs c cs))
     (Fail_C_Term d info) -> failCons d (traceFail "UnificationSpec.applySubst" [(show x1),(show x2)] info)
     _ -> failCons cd (consFail "UnificationSpec.applySubst" (showCons x2))

nd_C_applySubst :: Curry_FiniteMap.C_FM Curry_Prelude.C_Int C_Term -> C_Term -> IDSupply -> Cover -> ConstStore -> C_Term
nd_C_applySubst x1 x2 s cd cs = case x2 of
     (C_TermVar x3) -> let
          s6 = s
           in (seq s6 (let
               s5 = leftSupply s6
               s4 = rightSupply s6
                in (seq s5 (seq s4 (Curry_Prelude.nd_C_maybe x2 (wrapDX id Curry_Prelude.d_C_id) (let
                    s3 = leftSupply s4
                    s2 = rightSupply s4
                     in (seq s3 (seq s2 (Curry_Prelude.nd_C_apply (let
                         s1 = leftSupply s2
                         s0 = rightSupply s2
                          in (seq s1 (seq s0 (Curry_Prelude.nd_C_apply (nd_C_lookupSubst s0 cd cs) x1 s1 cd cs)))) x3 s3 cd cs)))) s5 cd cs)))))
     (C_TermCons x4 x5) -> let
          s0 = s
           in (seq s0 (C_TermCons x4 (Curry_Prelude.nd_C_map (wrapNX id (nd_C_applySubst x1)) x5 s0 cd cs)))
     (Choice_C_Term d i l r) -> narrow d i (nd_C_applySubst x1 l s cd cs) (nd_C_applySubst x1 r s cd cs)
     (Choices_C_Term d i xs) -> narrows cs d i (\z -> nd_C_applySubst x1 z s cd cs) xs
     (Guard_C_Term d c e) -> guardCons d c ((nd_C_applySubst x1 e s cd) $! (addCs c cs))
     (Fail_C_Term d info) -> failCons d (traceFail "UnificationSpec.applySubst" [(show x1),(show x2)] info)
     _ -> failCons cd (consFail "UnificationSpec.applySubst" (showCons x2))

d_C_substituteSingle :: Curry_FiniteMap.C_FM Curry_Prelude.C_Int C_Term -> Curry_Prelude.OP_Tuple2 C_Term C_Term -> Cover -> ConstStore -> Curry_Prelude.OP_Tuple2 C_Term C_Term
d_C_substituteSingle x1 x2 cd cs = case x2 of
     (Curry_Prelude.OP_Tuple2 x3 x4) -> Curry_Prelude.OP_Tuple2 (d_C_applySubst x1 x3 cd cs) (d_C_applySubst x1 x4 cd cs)
     (Curry_Prelude.Choice_OP_Tuple2 d i l r) -> narrow d i (d_C_substituteSingle x1 l cd cs) (d_C_substituteSingle x1 r cd cs)
     (Curry_Prelude.Choices_OP_Tuple2 d i xs) -> narrows cs d i (\z -> d_C_substituteSingle x1 z cd cs) xs
     (Curry_Prelude.Guard_OP_Tuple2 d c e) -> guardCons d c ((d_C_substituteSingle x1 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_Tuple2 d info) -> failCons d (traceFail "UnificationSpec.substituteSingle" [(show x1),(show x2)] info)
     _ -> failCons cd (consFail "UnificationSpec.substituteSingle" (showCons x2))

nd_C_substituteSingle :: Curry_FiniteMap.C_FM Curry_Prelude.C_Int C_Term -> Curry_Prelude.OP_Tuple2 C_Term C_Term -> IDSupply -> Cover -> ConstStore -> Curry_Prelude.OP_Tuple2 C_Term C_Term
nd_C_substituteSingle x1 x2 s cd cs = case x2 of
     (Curry_Prelude.OP_Tuple2 x3 x4) -> let
          s2 = s
           in (seq s2 (let
               s0 = leftSupply s2
               s1 = rightSupply s2
                in (seq s0 (seq s1 (Curry_Prelude.OP_Tuple2 (nd_C_applySubst x1 x3 s0 cd cs) (nd_C_applySubst x1 x4 s1 cd cs))))))
     (Curry_Prelude.Choice_OP_Tuple2 d i l r) -> narrow d i (nd_C_substituteSingle x1 l s cd cs) (nd_C_substituteSingle x1 r s cd cs)
     (Curry_Prelude.Choices_OP_Tuple2 d i xs) -> narrows cs d i (\z -> nd_C_substituteSingle x1 z s cd cs) xs
     (Curry_Prelude.Guard_OP_Tuple2 d c e) -> guardCons d c ((nd_C_substituteSingle x1 e s cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_Tuple2 d info) -> failCons d (traceFail "UnificationSpec.substituteSingle" [(show x1),(show x2)] info)
     _ -> failCons cd (consFail "UnificationSpec.substituteSingle" (showCons x2))

d_C_substitute :: Curry_FiniteMap.C_FM Curry_Prelude.C_Int C_Term -> Curry_Prelude.OP_List (Curry_Prelude.OP_Tuple2 C_Term C_Term) -> Cover -> ConstStore -> Curry_Prelude.OP_List (Curry_Prelude.OP_Tuple2 C_Term C_Term)
d_C_substitute x1 x2 cd cs = Curry_Prelude.d_C_map (d_C_substituteSingle x1) x2 cd cs

nd_C_substitute :: Curry_FiniteMap.C_FM Curry_Prelude.C_Int C_Term -> Curry_Prelude.OP_List (Curry_Prelude.OP_Tuple2 C_Term C_Term) -> IDSupply -> Cover -> ConstStore -> Curry_Prelude.OP_List (Curry_Prelude.OP_Tuple2 C_Term C_Term)
nd_C_substitute x1 x2 s cd cs = let
     s0 = s
      in (seq s0 (Curry_Prelude.nd_C_map (wrapNX id (nd_C_substituteSingle x1)) x2 s0 cd cs))

d_C_unify :: Curry_Prelude.OP_List (Curry_Prelude.OP_Tuple2 C_Term C_Term) -> Cover -> ConstStore -> Curry_Prelude.C_Either C_UnificationError (Curry_FiniteMap.C_FM Curry_Prelude.C_Int C_Term)
d_C_unify x1 cd cs = Curry_Prelude.d_C_either (acceptCs id Curry_Prelude.C_Left) (Curry_Prelude.d_OP_dot (acceptCs id Curry_Prelude.C_Right) d_C_eqsToSubst cd cs) (d_C_unify' x1 Curry_Prelude.OP_List cd cs) cd cs

nd_C_unify :: Curry_Prelude.OP_List (Curry_Prelude.OP_Tuple2 C_Term C_Term) -> IDSupply -> Cover -> ConstStore -> Curry_Prelude.C_Either C_UnificationError (Curry_FiniteMap.C_FM Curry_Prelude.C_Int C_Term)
nd_C_unify x1 s cd cs = let
     s2 = s
      in (seq s2 (let
          s1 = leftSupply s2
          s0 = rightSupply s2
           in (seq s1 (seq s0 (Curry_Prelude.nd_C_either (wrapDX id (acceptCs id Curry_Prelude.C_Left)) (Curry_Prelude.nd_OP_dot (wrapDX id (acceptCs id Curry_Prelude.C_Right)) (wrapNX id nd_C_eqsToSubst) s0 cd cs) (d_C_unify' x1 Curry_Prelude.OP_List cd cs) s1 cd cs)))))

d_C_eqsToSubst :: Curry_Prelude.OP_List (Curry_Prelude.OP_Tuple2 C_Term C_Term) -> Cover -> ConstStore -> Curry_FiniteMap.C_FM Curry_Prelude.C_Int C_Term
d_C_eqsToSubst x1 cd cs = case x1 of
     Curry_Prelude.OP_List -> d_C_emptySubst cd cs
     (Curry_Prelude.OP_Cons x2 x3) -> d_C__case_14 x3 x2 cd cs
     (Curry_Prelude.Choice_OP_List d i l r) -> narrow d i (d_C_eqsToSubst l cd cs) (d_C_eqsToSubst r cd cs)
     (Curry_Prelude.Choices_OP_List d i xs) -> narrows cs d i (\z -> d_C_eqsToSubst z cd cs) xs
     (Curry_Prelude.Guard_OP_List d c e) -> guardCons d c ((d_C_eqsToSubst e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_List d info) -> failCons d (traceFail "UnificationSpec.eqsToSubst" [(show x1)] info)
     _ -> failCons cd (consFail "UnificationSpec.eqsToSubst" (showCons x1))

nd_C_eqsToSubst :: Curry_Prelude.OP_List (Curry_Prelude.OP_Tuple2 C_Term C_Term) -> IDSupply -> Cover -> ConstStore -> Curry_FiniteMap.C_FM Curry_Prelude.C_Int C_Term
nd_C_eqsToSubst x1 s cd cs = case x1 of
     Curry_Prelude.OP_List -> let
          s0 = s
           in (seq s0 (nd_C_emptySubst s0 cd cs))
     (Curry_Prelude.OP_Cons x2 x3) -> let
          s0 = s
           in (seq s0 (nd_C__case_14 x3 x2 s0 cd cs))
     (Curry_Prelude.Choice_OP_List d i l r) -> narrow d i (nd_C_eqsToSubst l s cd cs) (nd_C_eqsToSubst r s cd cs)
     (Curry_Prelude.Choices_OP_List d i xs) -> narrows cs d i (\z -> nd_C_eqsToSubst z s cd cs) xs
     (Curry_Prelude.Guard_OP_List d c e) -> guardCons d c ((nd_C_eqsToSubst e s cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_List d info) -> failCons d (traceFail "UnificationSpec.eqsToSubst" [(show x1)] info)
     _ -> failCons cd (consFail "UnificationSpec.eqsToSubst" (showCons x1))

d_C_unify' :: Curry_Prelude.OP_List (Curry_Prelude.OP_Tuple2 C_Term C_Term) -> Curry_Prelude.OP_List (Curry_Prelude.OP_Tuple2 C_Term C_Term) -> Cover -> ConstStore -> Curry_Prelude.C_Either C_UnificationError (Curry_Prelude.OP_List (Curry_Prelude.OP_Tuple2 C_Term C_Term))
d_C_unify' x1 x2 cd cs = case x2 of
     Curry_Prelude.OP_List -> Curry_Prelude.C_Right x1
     (Curry_Prelude.OP_Cons x3 x4) -> d_C__case_11 x4 x1 x3 cd cs
     (Curry_Prelude.Choice_OP_List d i l r) -> narrow d i (d_C_unify' x1 l cd cs) (d_C_unify' x1 r cd cs)
     (Curry_Prelude.Choices_OP_List d i xs) -> narrows cs d i (\z -> d_C_unify' x1 z cd cs) xs
     (Curry_Prelude.Guard_OP_List d c e) -> guardCons d c ((d_C_unify' x1 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_List d info) -> failCons d (traceFail "UnificationSpec.unify'" [(show x1),(show x2)] info)
     _ -> failCons cd (consFail "UnificationSpec.unify'" (showCons x2))

d_C_elim :: Curry_Prelude.OP_List (Curry_Prelude.OP_Tuple2 C_Term C_Term) -> Curry_Prelude.C_Int -> C_Term -> Curry_Prelude.OP_List (Curry_Prelude.OP_Tuple2 C_Term C_Term) -> Cover -> ConstStore -> Curry_Prelude.C_Either C_UnificationError (Curry_Prelude.OP_List (Curry_Prelude.OP_Tuple2 C_Term C_Term))
d_C_elim x1 x2 x3 x4 cd cs = let
     x5 = Curry_Prelude.OP_Cons (Curry_Prelude.OP_Tuple2 (C_TermVar x2) x3) (Curry_Prelude.d_C_map (d_OP_elim_dot_uscore_hash_lambda3 x2 x3) x1 cd cs)
      in (d_C__case_3 x3 x2 x4 x5 (d_C_dependsOn (C_TermVar x2) x3 cd cs) cd cs)

d_OP_elim_dot_uscore_hash_lambda3 :: Curry_Prelude.C_Int -> C_Term -> Curry_Prelude.OP_Tuple2 C_Term C_Term -> Cover -> ConstStore -> Curry_Prelude.OP_Tuple2 C_Term C_Term
d_OP_elim_dot_uscore_hash_lambda3 x1 x2 x3 cd cs = case x3 of
     (Curry_Prelude.OP_Tuple2 x4 x5) -> Curry_Prelude.OP_Tuple2 x4 (d_C_termSubstitute' x1 x2 x5 cd cs)
     (Curry_Prelude.Choice_OP_Tuple2 d i l r) -> narrow d i (d_OP_elim_dot_uscore_hash_lambda3 x1 x2 l cd cs) (d_OP_elim_dot_uscore_hash_lambda3 x1 x2 r cd cs)
     (Curry_Prelude.Choices_OP_Tuple2 d i xs) -> narrows cs d i (\z -> d_OP_elim_dot_uscore_hash_lambda3 x1 x2 z cd cs) xs
     (Curry_Prelude.Guard_OP_Tuple2 d c e) -> guardCons d c ((d_OP_elim_dot_uscore_hash_lambda3 x1 x2 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_Tuple2 d info) -> failCons d (traceFail "UnificationSpec.elim._#lambda3" [(show x1),(show x2),(show x3)] info)
     _ -> failCons cd (consFail "UnificationSpec.elim._#lambda3" (showCons x3))

d_C_termSubstitute' :: Curry_Prelude.C_Int -> C_Term -> C_Term -> Cover -> ConstStore -> C_Term
d_C_termSubstitute' x1 x2 x3 cd cs = case x3 of
     (C_TermVar x4) -> d_C__case_1 x1 x4 x3 x2 (Curry_Prelude.d_OP_eq_eq x4 x1 cd cs) cd cs
     (C_TermCons x5 x6) -> C_TermCons x5 (d_C_termsSubstitute' x1 x2 x6 cd cs)
     (Choice_C_Term d i l r) -> narrow d i (d_C_termSubstitute' x1 x2 l cd cs) (d_C_termSubstitute' x1 x2 r cd cs)
     (Choices_C_Term d i xs) -> narrows cs d i (\z -> d_C_termSubstitute' x1 x2 z cd cs) xs
     (Guard_C_Term d c e) -> guardCons d c ((d_C_termSubstitute' x1 x2 e cd) $! (addCs c cs))
     (Fail_C_Term d info) -> failCons d (traceFail "UnificationSpec.termSubstitute'" [(show x1),(show x2),(show x3)] info)
     _ -> failCons cd (consFail "UnificationSpec.termSubstitute'" (showCons x3))

d_C_termsSubstitute' :: Curry_Prelude.C_Int -> C_Term -> Curry_Prelude.OP_List C_Term -> Cover -> ConstStore -> Curry_Prelude.OP_List C_Term
d_C_termsSubstitute' x1 x2 x3 cd cs = Curry_Prelude.d_C_map (d_C_termSubstitute' x1 x2) x3 cd cs

d_C_substitute' :: Curry_Prelude.C_Int -> C_Term -> Curry_Prelude.OP_List (Curry_Prelude.OP_Tuple2 C_Term C_Term) -> Cover -> ConstStore -> Curry_Prelude.OP_List (Curry_Prelude.OP_Tuple2 C_Term C_Term)
d_C_substitute' x1 x2 x3 cd cs = Curry_Prelude.d_C_map (d_C_substituteSingle' x1 x2) x3 cd cs

d_C_substituteSingle' :: Curry_Prelude.C_Int -> C_Term -> Curry_Prelude.OP_Tuple2 C_Term C_Term -> Cover -> ConstStore -> Curry_Prelude.OP_Tuple2 C_Term C_Term
d_C_substituteSingle' x1 x2 x3 cd cs = case x3 of
     (Curry_Prelude.OP_Tuple2 x4 x5) -> Curry_Prelude.OP_Tuple2 (d_C_termSubstitute' x1 x2 x4 cd cs) (d_C_termSubstitute' x1 x2 x5 cd cs)
     (Curry_Prelude.Choice_OP_Tuple2 d i l r) -> narrow d i (d_C_substituteSingle' x1 x2 l cd cs) (d_C_substituteSingle' x1 x2 r cd cs)
     (Curry_Prelude.Choices_OP_Tuple2 d i xs) -> narrows cs d i (\z -> d_C_substituteSingle' x1 x2 z cd cs) xs
     (Curry_Prelude.Guard_OP_Tuple2 d c e) -> guardCons d c ((d_C_substituteSingle' x1 x2 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_Tuple2 d info) -> failCons d (traceFail "UnificationSpec.substituteSingle'" [(show x1),(show x2),(show x3)] info)
     _ -> failCons cd (consFail "UnificationSpec.substituteSingle'" (showCons x3))

d_C_dependsOn :: C_Term -> C_Term -> Cover -> ConstStore -> Curry_Prelude.C_Bool
d_C_dependsOn x1 x2 cd cs = Curry_Prelude.d_C_apply (Curry_Prelude.d_C_and cd cs) (Curry_Prelude.OP_Cons (Curry_Prelude.d_C_not (Curry_Prelude.d_OP_eq_eq x1 x2 cd cs) cd cs) (Curry_Prelude.OP_Cons (d_OP_dependsOn_dot_dependsOn_tick_dot_65 x1 x2 cd cs) Curry_Prelude.OP_List)) cd cs

d_OP_dependsOn_dot_dependsOn_tick_dot_65 :: C_Term -> C_Term -> Cover -> ConstStore -> Curry_Prelude.C_Bool
d_OP_dependsOn_dot_dependsOn_tick_dot_65 x1 x2 cd cs = case x2 of
     (C_TermVar x3) -> Curry_Prelude.d_OP_eq_eq x1 x2 cd cs
     (C_TermCons x4 x5) -> Curry_Prelude.d_C_apply (Curry_Prelude.d_C_any Curry_Prelude.d_C_id cd cs) (Curry_Prelude.d_C_map (d_OP_dependsOn_dot_dependsOn_tick_dot_65 x1) x5 cd cs) cd cs
     (Choice_C_Term d i l r) -> narrow d i (d_OP_dependsOn_dot_dependsOn_tick_dot_65 x1 l cd cs) (d_OP_dependsOn_dot_dependsOn_tick_dot_65 x1 r cd cs)
     (Choices_C_Term d i xs) -> narrows cs d i (\z -> d_OP_dependsOn_dot_dependsOn_tick_dot_65 x1 z cd cs) xs
     (Guard_C_Term d c e) -> guardCons d c ((d_OP_dependsOn_dot_dependsOn_tick_dot_65 x1 e cd) $! (addCs c cs))
     (Fail_C_Term d info) -> failCons d (traceFail "UnificationSpec.dependsOn.dependsOn'.65" [(show x1),(show x2)] info)
     _ -> failCons cd (consFail "UnificationSpec.dependsOn.dependsOn'.65" (showCons x2))

d_C__case_1 :: Curry_Prelude.C_Int -> Curry_Prelude.C_Int -> C_Term -> C_Term -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> C_Term
d_C__case_1 x1 x4 x3 x2 x5 cd cs = case x5 of
     Curry_Prelude.C_True -> x2
     Curry_Prelude.C_False -> d_C__case_0 x3 (Curry_Prelude.d_C_otherwise cd cs) cd cs
     (Curry_Prelude.Choice_C_Bool d i l r) -> narrow d i (d_C__case_1 x1 x4 x3 x2 l cd cs) (d_C__case_1 x1 x4 x3 x2 r cd cs)
     (Curry_Prelude.Choices_C_Bool d i xs) -> narrows cs d i (\z -> d_C__case_1 x1 x4 x3 x2 z cd cs) xs
     (Curry_Prelude.Guard_C_Bool d c e) -> guardCons d c ((d_C__case_1 x1 x4 x3 x2 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_C_Bool d info) -> failCons d (traceFail "UnificationSpec._case_1" [(show x1),(show x4),(show x3),(show x2),(show x5)] info)
     _ -> failCons cd (consFail "UnificationSpec._case_1" (showCons x5))

d_C__case_0 :: C_Term -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> C_Term
d_C__case_0 x3 x4 cd cs = case x4 of
     Curry_Prelude.C_True -> x3
     Curry_Prelude.C_False -> Curry_Prelude.d_C_failed cd cs
     (Curry_Prelude.Choice_C_Bool d i l r) -> narrow d i (d_C__case_0 x3 l cd cs) (d_C__case_0 x3 r cd cs)
     (Curry_Prelude.Choices_C_Bool d i xs) -> narrows cs d i (\z -> d_C__case_0 x3 z cd cs) xs
     (Curry_Prelude.Guard_C_Bool d c e) -> guardCons d c ((d_C__case_0 x3 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_C_Bool d info) -> failCons d (traceFail "UnificationSpec._case_0" [(show x3),(show x4)] info)
     _ -> failCons cd (consFail "UnificationSpec._case_0" (showCons x4))

d_C__case_3 :: C_Term -> Curry_Prelude.C_Int -> Curry_Prelude.OP_List (Curry_Prelude.OP_Tuple2 C_Term C_Term) -> Curry_Prelude.OP_List (Curry_Prelude.OP_Tuple2 C_Term C_Term) -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> Curry_Prelude.C_Either C_UnificationError (Curry_Prelude.OP_List (Curry_Prelude.OP_Tuple2 C_Term C_Term))
d_C__case_3 x3 x2 x4 x5 x6 cd cs = case x6 of
     Curry_Prelude.C_True -> Curry_Prelude.C_Left (C_OccurCheck x2 x3)
     Curry_Prelude.C_False -> d_C__case_2 x4 x3 x2 x5 (Curry_Prelude.d_C_otherwise cd cs) cd cs
     (Curry_Prelude.Choice_C_Bool d i l r) -> narrow d i (d_C__case_3 x3 x2 x4 x5 l cd cs) (d_C__case_3 x3 x2 x4 x5 r cd cs)
     (Curry_Prelude.Choices_C_Bool d i xs) -> narrows cs d i (\z -> d_C__case_3 x3 x2 x4 x5 z cd cs) xs
     (Curry_Prelude.Guard_C_Bool d c e) -> guardCons d c ((d_C__case_3 x3 x2 x4 x5 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_C_Bool d info) -> failCons d (traceFail "UnificationSpec._case_3" [(show x3),(show x2),(show x4),(show x5),(show x6)] info)
     _ -> failCons cd (consFail "UnificationSpec._case_3" (showCons x6))

d_C__case_2 :: Curry_Prelude.OP_List (Curry_Prelude.OP_Tuple2 C_Term C_Term) -> C_Term -> Curry_Prelude.C_Int -> Curry_Prelude.OP_List (Curry_Prelude.OP_Tuple2 C_Term C_Term) -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> Curry_Prelude.C_Either C_UnificationError (Curry_Prelude.OP_List (Curry_Prelude.OP_Tuple2 C_Term C_Term))
d_C__case_2 x4 x3 x2 x5 x6 cd cs = case x6 of
     Curry_Prelude.C_True -> d_C_unify' x5 (d_C_substitute' x2 x3 x4 cd cs) cd cs
     Curry_Prelude.C_False -> Curry_Prelude.d_C_failed cd cs
     (Curry_Prelude.Choice_C_Bool d i l r) -> narrow d i (d_C__case_2 x4 x3 x2 x5 l cd cs) (d_C__case_2 x4 x3 x2 x5 r cd cs)
     (Curry_Prelude.Choices_C_Bool d i xs) -> narrows cs d i (\z -> d_C__case_2 x4 x3 x2 x5 z cd cs) xs
     (Curry_Prelude.Guard_C_Bool d c e) -> guardCons d c ((d_C__case_2 x4 x3 x2 x5 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_C_Bool d info) -> failCons d (traceFail "UnificationSpec._case_2" [(show x4),(show x3),(show x2),(show x5),(show x6)] info)
     _ -> failCons cd (consFail "UnificationSpec._case_2" (showCons x6))

d_C__case_11 :: Curry_Prelude.OP_List (Curry_Prelude.OP_Tuple2 C_Term C_Term) -> Curry_Prelude.OP_List (Curry_Prelude.OP_Tuple2 C_Term C_Term) -> Curry_Prelude.OP_Tuple2 C_Term C_Term -> Cover -> ConstStore -> Curry_Prelude.C_Either C_UnificationError (Curry_Prelude.OP_List (Curry_Prelude.OP_Tuple2 C_Term C_Term))
d_C__case_11 x4 x1 x3 cd cs = case x3 of
     (Curry_Prelude.OP_Tuple2 x5 x6) -> d_C__case_10 x6 x4 x1 x5 cd cs
     (Curry_Prelude.Choice_OP_Tuple2 d i l r) -> narrow d i (d_C__case_11 x4 x1 l cd cs) (d_C__case_11 x4 x1 r cd cs)
     (Curry_Prelude.Choices_OP_Tuple2 d i xs) -> narrows cs d i (\z -> d_C__case_11 x4 x1 z cd cs) xs
     (Curry_Prelude.Guard_OP_Tuple2 d c e) -> guardCons d c ((d_C__case_11 x4 x1 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_Tuple2 d info) -> failCons d (traceFail "UnificationSpec._case_11" [(show x4),(show x1),(show x3)] info)
     _ -> failCons cd (consFail "UnificationSpec._case_11" (showCons x3))

d_C__case_10 :: C_Term -> Curry_Prelude.OP_List (Curry_Prelude.OP_Tuple2 C_Term C_Term) -> Curry_Prelude.OP_List (Curry_Prelude.OP_Tuple2 C_Term C_Term) -> C_Term -> Cover -> ConstStore -> Curry_Prelude.C_Either C_UnificationError (Curry_Prelude.OP_List (Curry_Prelude.OP_Tuple2 C_Term C_Term))
d_C__case_10 x6 x4 x1 x5 cd cs = case x5 of
     (C_TermVar x7) -> d_C__case_9 x7 x4 x1 x6 cd cs
     (C_TermCons x11 x12) -> d_C__case_6 x11 x5 x4 x12 x1 x6 cd cs
     (Choice_C_Term d i l r) -> narrow d i (d_C__case_10 x6 x4 x1 l cd cs) (d_C__case_10 x6 x4 x1 r cd cs)
     (Choices_C_Term d i xs) -> narrows cs d i (\z -> d_C__case_10 x6 x4 x1 z cd cs) xs
     (Guard_C_Term d c e) -> guardCons d c ((d_C__case_10 x6 x4 x1 e cd) $! (addCs c cs))
     (Fail_C_Term d info) -> failCons d (traceFail "UnificationSpec._case_10" [(show x6),(show x4),(show x1),(show x5)] info)
     _ -> failCons cd (consFail "UnificationSpec._case_10" (showCons x5))

d_C__case_6 :: Curry_Prelude.OP_List Curry_Prelude.C_Char -> C_Term -> Curry_Prelude.OP_List (Curry_Prelude.OP_Tuple2 C_Term C_Term) -> Curry_Prelude.OP_List C_Term -> Curry_Prelude.OP_List (Curry_Prelude.OP_Tuple2 C_Term C_Term) -> C_Term -> Cover -> ConstStore -> Curry_Prelude.C_Either C_UnificationError (Curry_Prelude.OP_List (Curry_Prelude.OP_Tuple2 C_Term C_Term))
d_C__case_6 x11 x5 x4 x12 x1 x6 cd cs = case x6 of
     (C_TermVar x13) -> d_C_elim x1 x13 x5 x4 cd cs
     (C_TermCons x14 x15) -> d_C__case_5 x14 x11 x6 x5 x4 x15 x12 x1 (Curry_Prelude.d_OP_eq_eq x11 x14 cd cs) cd cs
     (Choice_C_Term d i l r) -> narrow d i (d_C__case_6 x11 x5 x4 x12 x1 l cd cs) (d_C__case_6 x11 x5 x4 x12 x1 r cd cs)
     (Choices_C_Term d i xs) -> narrows cs d i (\z -> d_C__case_6 x11 x5 x4 x12 x1 z cd cs) xs
     (Guard_C_Term d c e) -> guardCons d c ((d_C__case_6 x11 x5 x4 x12 x1 e cd) $! (addCs c cs))
     (Fail_C_Term d info) -> failCons d (traceFail "UnificationSpec._case_6" [(show x11),(show x5),(show x4),(show x12),(show x1),(show x6)] info)
     _ -> failCons cd (consFail "UnificationSpec._case_6" (showCons x6))

d_C__case_5 :: Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> C_Term -> C_Term -> Curry_Prelude.OP_List (Curry_Prelude.OP_Tuple2 C_Term C_Term) -> Curry_Prelude.OP_List C_Term -> Curry_Prelude.OP_List C_Term -> Curry_Prelude.OP_List (Curry_Prelude.OP_Tuple2 C_Term C_Term) -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> Curry_Prelude.C_Either C_UnificationError (Curry_Prelude.OP_List (Curry_Prelude.OP_Tuple2 C_Term C_Term))
d_C__case_5 x14 x11 x6 x5 x4 x15 x12 x1 x16 cd cs = case x16 of
     Curry_Prelude.C_True -> d_C_unify' x1 (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.d_C_zip x12 x15 cd cs) x4 cd cs) cd cs
     Curry_Prelude.C_False -> d_C__case_4 x6 x5 (Curry_Prelude.d_C_otherwise cd cs) cd cs
     (Curry_Prelude.Choice_C_Bool d i l r) -> narrow d i (d_C__case_5 x14 x11 x6 x5 x4 x15 x12 x1 l cd cs) (d_C__case_5 x14 x11 x6 x5 x4 x15 x12 x1 r cd cs)
     (Curry_Prelude.Choices_C_Bool d i xs) -> narrows cs d i (\z -> d_C__case_5 x14 x11 x6 x5 x4 x15 x12 x1 z cd cs) xs
     (Curry_Prelude.Guard_C_Bool d c e) -> guardCons d c ((d_C__case_5 x14 x11 x6 x5 x4 x15 x12 x1 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_C_Bool d info) -> failCons d (traceFail "UnificationSpec._case_5" [(show x14),(show x11),(show x6),(show x5),(show x4),(show x15),(show x12),(show x1),(show x16)] info)
     _ -> failCons cd (consFail "UnificationSpec._case_5" (showCons x16))

d_C__case_4 :: C_Term -> C_Term -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> Curry_Prelude.C_Either C_UnificationError (Curry_Prelude.OP_List (Curry_Prelude.OP_Tuple2 C_Term C_Term))
d_C__case_4 x6 x5 x7 cd cs = case x7 of
     Curry_Prelude.C_True -> Curry_Prelude.C_Left (C_Clash x5 x6)
     Curry_Prelude.C_False -> Curry_Prelude.d_C_failed cd cs
     (Curry_Prelude.Choice_C_Bool d i l r) -> narrow d i (d_C__case_4 x6 x5 l cd cs) (d_C__case_4 x6 x5 r cd cs)
     (Curry_Prelude.Choices_C_Bool d i xs) -> narrows cs d i (\z -> d_C__case_4 x6 x5 z cd cs) xs
     (Curry_Prelude.Guard_C_Bool d c e) -> guardCons d c ((d_C__case_4 x6 x5 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_C_Bool d info) -> failCons d (traceFail "UnificationSpec._case_4" [(show x6),(show x5),(show x7)] info)
     _ -> failCons cd (consFail "UnificationSpec._case_4" (showCons x7))

d_C__case_9 :: Curry_Prelude.C_Int -> Curry_Prelude.OP_List (Curry_Prelude.OP_Tuple2 C_Term C_Term) -> Curry_Prelude.OP_List (Curry_Prelude.OP_Tuple2 C_Term C_Term) -> C_Term -> Cover -> ConstStore -> Curry_Prelude.C_Either C_UnificationError (Curry_Prelude.OP_List (Curry_Prelude.OP_Tuple2 C_Term C_Term))
d_C__case_9 x7 x4 x1 x6 cd cs = case x6 of
     (C_TermCons x8 x9) -> d_C_elim x1 x7 x6 x4 cd cs
     (C_TermVar x10) -> d_C__case_8 x10 x7 x4 x6 x1 (Curry_Prelude.d_OP_eq_eq x7 x10 cd cs) cd cs
     (Choice_C_Term d i l r) -> narrow d i (d_C__case_9 x7 x4 x1 l cd cs) (d_C__case_9 x7 x4 x1 r cd cs)
     (Choices_C_Term d i xs) -> narrows cs d i (\z -> d_C__case_9 x7 x4 x1 z cd cs) xs
     (Guard_C_Term d c e) -> guardCons d c ((d_C__case_9 x7 x4 x1 e cd) $! (addCs c cs))
     (Fail_C_Term d info) -> failCons d (traceFail "UnificationSpec._case_9" [(show x7),(show x4),(show x1),(show x6)] info)
     _ -> failCons cd (consFail "UnificationSpec._case_9" (showCons x6))

d_C__case_8 :: Curry_Prelude.C_Int -> Curry_Prelude.C_Int -> Curry_Prelude.OP_List (Curry_Prelude.OP_Tuple2 C_Term C_Term) -> C_Term -> Curry_Prelude.OP_List (Curry_Prelude.OP_Tuple2 C_Term C_Term) -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> Curry_Prelude.C_Either C_UnificationError (Curry_Prelude.OP_List (Curry_Prelude.OP_Tuple2 C_Term C_Term))
d_C__case_8 x10 x7 x4 x6 x1 x11 cd cs = case x11 of
     Curry_Prelude.C_True -> d_C_unify' x1 x4 cd cs
     Curry_Prelude.C_False -> d_C__case_7 x4 x6 x7 x1 (Curry_Prelude.d_C_otherwise cd cs) cd cs
     (Curry_Prelude.Choice_C_Bool d i l r) -> narrow d i (d_C__case_8 x10 x7 x4 x6 x1 l cd cs) (d_C__case_8 x10 x7 x4 x6 x1 r cd cs)
     (Curry_Prelude.Choices_C_Bool d i xs) -> narrows cs d i (\z -> d_C__case_8 x10 x7 x4 x6 x1 z cd cs) xs
     (Curry_Prelude.Guard_C_Bool d c e) -> guardCons d c ((d_C__case_8 x10 x7 x4 x6 x1 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_C_Bool d info) -> failCons d (traceFail "UnificationSpec._case_8" [(show x10),(show x7),(show x4),(show x6),(show x1),(show x11)] info)
     _ -> failCons cd (consFail "UnificationSpec._case_8" (showCons x11))

d_C__case_7 :: Curry_Prelude.OP_List (Curry_Prelude.OP_Tuple2 C_Term C_Term) -> C_Term -> Curry_Prelude.C_Int -> Curry_Prelude.OP_List (Curry_Prelude.OP_Tuple2 C_Term C_Term) -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> Curry_Prelude.C_Either C_UnificationError (Curry_Prelude.OP_List (Curry_Prelude.OP_Tuple2 C_Term C_Term))
d_C__case_7 x4 x6 x7 x1 x8 cd cs = case x8 of
     Curry_Prelude.C_True -> d_C_elim x1 x7 x6 x4 cd cs
     Curry_Prelude.C_False -> Curry_Prelude.d_C_failed cd cs
     (Curry_Prelude.Choice_C_Bool d i l r) -> narrow d i (d_C__case_7 x4 x6 x7 x1 l cd cs) (d_C__case_7 x4 x6 x7 x1 r cd cs)
     (Curry_Prelude.Choices_C_Bool d i xs) -> narrows cs d i (\z -> d_C__case_7 x4 x6 x7 x1 z cd cs) xs
     (Curry_Prelude.Guard_C_Bool d c e) -> guardCons d c ((d_C__case_7 x4 x6 x7 x1 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_C_Bool d info) -> failCons d (traceFail "UnificationSpec._case_7" [(show x4),(show x6),(show x7),(show x1),(show x8)] info)
     _ -> failCons cd (consFail "UnificationSpec._case_7" (showCons x8))

d_C__case_14 :: Curry_Prelude.OP_List (Curry_Prelude.OP_Tuple2 C_Term C_Term) -> Curry_Prelude.OP_Tuple2 C_Term C_Term -> Cover -> ConstStore -> Curry_FiniteMap.C_FM Curry_Prelude.C_Int C_Term
d_C__case_14 x3 x2 cd cs = case x2 of
     (Curry_Prelude.OP_Tuple2 x4 x5) -> d_C__case_13 x5 x3 x4 cd cs
     (Curry_Prelude.Choice_OP_Tuple2 d i l r) -> narrow d i (d_C__case_14 x3 l cd cs) (d_C__case_14 x3 r cd cs)
     (Curry_Prelude.Choices_OP_Tuple2 d i xs) -> narrows cs d i (\z -> d_C__case_14 x3 z cd cs) xs
     (Curry_Prelude.Guard_OP_Tuple2 d c e) -> guardCons d c ((d_C__case_14 x3 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_Tuple2 d info) -> failCons d (traceFail "UnificationSpec._case_14" [(show x3),(show x2)] info)
     _ -> failCons cd (consFail "UnificationSpec._case_14" (showCons x2))

nd_C__case_14 :: Curry_Prelude.OP_List (Curry_Prelude.OP_Tuple2 C_Term C_Term) -> Curry_Prelude.OP_Tuple2 C_Term C_Term -> IDSupply -> Cover -> ConstStore -> Curry_FiniteMap.C_FM Curry_Prelude.C_Int C_Term
nd_C__case_14 x3 x2 s cd cs = case x2 of
     (Curry_Prelude.OP_Tuple2 x4 x5) -> let
          s0 = s
           in (seq s0 (nd_C__case_13 x5 x3 x4 s0 cd cs))
     (Curry_Prelude.Choice_OP_Tuple2 d i l r) -> narrow d i (nd_C__case_14 x3 l s cd cs) (nd_C__case_14 x3 r s cd cs)
     (Curry_Prelude.Choices_OP_Tuple2 d i xs) -> narrows cs d i (\z -> nd_C__case_14 x3 z s cd cs) xs
     (Curry_Prelude.Guard_OP_Tuple2 d c e) -> guardCons d c ((nd_C__case_14 x3 e s cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_Tuple2 d info) -> failCons d (traceFail "UnificationSpec._case_14" [(show x3),(show x2)] info)
     _ -> failCons cd (consFail "UnificationSpec._case_14" (showCons x2))

d_C__case_13 :: C_Term -> Curry_Prelude.OP_List (Curry_Prelude.OP_Tuple2 C_Term C_Term) -> C_Term -> Cover -> ConstStore -> Curry_FiniteMap.C_FM Curry_Prelude.C_Int C_Term
d_C__case_13 x5 x3 x4 cd cs = case x4 of
     (C_TermVar x6) -> Curry_Prelude.d_C_apply (Curry_Prelude.d_C_apply (Curry_Prelude.d_C_apply (d_C_extendSubst cd cs) (d_C_eqsToSubst x3 cd cs) cd cs) x6 cd cs) x5 cd cs
     (C_TermCons x7 x8) -> d_C__case_12 x4 x3 x5 cd cs
     (Choice_C_Term d i l r) -> narrow d i (d_C__case_13 x5 x3 l cd cs) (d_C__case_13 x5 x3 r cd cs)
     (Choices_C_Term d i xs) -> narrows cs d i (\z -> d_C__case_13 x5 x3 z cd cs) xs
     (Guard_C_Term d c e) -> guardCons d c ((d_C__case_13 x5 x3 e cd) $! (addCs c cs))
     (Fail_C_Term d info) -> failCons d (traceFail "UnificationSpec._case_13" [(show x5),(show x3),(show x4)] info)
     _ -> failCons cd (consFail "UnificationSpec._case_13" (showCons x4))

nd_C__case_13 :: C_Term -> Curry_Prelude.OP_List (Curry_Prelude.OP_Tuple2 C_Term C_Term) -> C_Term -> IDSupply -> Cover -> ConstStore -> Curry_FiniteMap.C_FM Curry_Prelude.C_Int C_Term
nd_C__case_13 x5 x3 x4 s cd cs = case x4 of
     (C_TermVar x6) -> let
          s8 = s
           in (seq s8 (let
               s7 = leftSupply s8
               s6 = rightSupply s8
                in (seq s7 (seq s6 (Curry_Prelude.nd_C_apply (let
                    s5 = leftSupply s6
                    s3 = rightSupply s6
                     in (seq s5 (seq s3 (Curry_Prelude.nd_C_apply (let
                         s2 = leftSupply s3
                         s4 = rightSupply s3
                         s0 = leftSupply s4
                         s1 = rightSupply s4
                          in (seq s2 (seq s4 (seq s0 (seq s1 (Curry_Prelude.nd_C_apply (nd_C_extendSubst s0 cd cs) (nd_C_eqsToSubst x3 s1 cd cs) s2 cd cs)))))) x6 s5 cd cs)))) x5 s7 cd cs)))))
     (C_TermCons x7 x8) -> let
          s0 = s
           in (seq s0 (nd_C__case_12 x4 x3 x5 s0 cd cs))
     (Choice_C_Term d i l r) -> narrow d i (nd_C__case_13 x5 x3 l s cd cs) (nd_C__case_13 x5 x3 r s cd cs)
     (Choices_C_Term d i xs) -> narrows cs d i (\z -> nd_C__case_13 x5 x3 z s cd cs) xs
     (Guard_C_Term d c e) -> guardCons d c ((nd_C__case_13 x5 x3 e s cd) $! (addCs c cs))
     (Fail_C_Term d info) -> failCons d (traceFail "UnificationSpec._case_13" [(show x5),(show x3),(show x4)] info)
     _ -> failCons cd (consFail "UnificationSpec._case_13" (showCons x4))

d_C__case_12 :: C_Term -> Curry_Prelude.OP_List (Curry_Prelude.OP_Tuple2 C_Term C_Term) -> C_Term -> Cover -> ConstStore -> Curry_FiniteMap.C_FM Curry_Prelude.C_Int C_Term
d_C__case_12 x4 x3 x5 cd cs = case x5 of
     (C_TermVar x9) -> Curry_Prelude.d_C_apply (Curry_Prelude.d_C_apply (Curry_Prelude.d_C_apply (d_C_extendSubst cd cs) (d_C_eqsToSubst x3 cd cs) cd cs) x9 cd cs) x4 cd cs
     (C_TermCons x10 x11) -> Curry_Prelude.d_OP_dollar Curry_Prelude.d_C_error (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString "eqsToSubst: ") (Curry_Prelude.d_C_show (Curry_Prelude.OP_Tuple2 x4 x5) cd cs) cd cs) cd cs
     (Choice_C_Term d i l r) -> narrow d i (d_C__case_12 x4 x3 l cd cs) (d_C__case_12 x4 x3 r cd cs)
     (Choices_C_Term d i xs) -> narrows cs d i (\z -> d_C__case_12 x4 x3 z cd cs) xs
     (Guard_C_Term d c e) -> guardCons d c ((d_C__case_12 x4 x3 e cd) $! (addCs c cs))
     (Fail_C_Term d info) -> failCons d (traceFail "UnificationSpec._case_12" [(show x4),(show x3),(show x5)] info)
     _ -> failCons cd (consFail "UnificationSpec._case_12" (showCons x5))

nd_C__case_12 :: C_Term -> Curry_Prelude.OP_List (Curry_Prelude.OP_Tuple2 C_Term C_Term) -> C_Term -> IDSupply -> Cover -> ConstStore -> Curry_FiniteMap.C_FM Curry_Prelude.C_Int C_Term
nd_C__case_12 x4 x3 x5 s cd cs = case x5 of
     (C_TermVar x9) -> let
          s8 = s
           in (seq s8 (let
               s7 = leftSupply s8
               s6 = rightSupply s8
                in (seq s7 (seq s6 (Curry_Prelude.nd_C_apply (let
                    s5 = leftSupply s6
                    s3 = rightSupply s6
                     in (seq s5 (seq s3 (Curry_Prelude.nd_C_apply (let
                         s2 = leftSupply s3
                         s4 = rightSupply s3
                         s0 = leftSupply s4
                         s1 = rightSupply s4
                          in (seq s2 (seq s4 (seq s0 (seq s1 (Curry_Prelude.nd_C_apply (nd_C_extendSubst s0 cd cs) (nd_C_eqsToSubst x3 s1 cd cs) s2 cd cs)))))) x9 s5 cd cs)))) x4 s7 cd cs)))))
     (C_TermCons x10 x11) -> let
          s0 = s
           in (seq s0 (Curry_Prelude.nd_OP_dollar (wrapDX id Curry_Prelude.d_C_error) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString "eqsToSubst: ") (Curry_Prelude.d_C_show (Curry_Prelude.OP_Tuple2 x4 x5) cd cs) cd cs) s0 cd cs))
     (Choice_C_Term d i l r) -> narrow d i (nd_C__case_12 x4 x3 l s cd cs) (nd_C__case_12 x4 x3 r s cd cs)
     (Choices_C_Term d i xs) -> narrows cs d i (\z -> nd_C__case_12 x4 x3 z s cd cs) xs
     (Guard_C_Term d c e) -> guardCons d c ((nd_C__case_12 x4 x3 e s cd) $! (addCs c cs))
     (Fail_C_Term d info) -> failCons d (traceFail "UnificationSpec._case_12" [(show x4),(show x3),(show x5)] info)
     _ -> failCons cd (consFail "UnificationSpec._case_12" (showCons x5))