{-# LANGUAGE MagicHash #-}
{-# OPTIONS_GHC -fno-warn-overlapping-patterns #-}


module Curry_AbstractHaskellPrinter (C_Options, d_C_showProg, d_C_showModuleHeader, d_C_showDecls, d_C_showTypeDecls, d_C_showTypeDecl, d_C_showTypeExpr, d_C_showFuncDecl, d_C_showExpr, d_C_showPattern, d_C_showLiteral) where

import Basics
import qualified Curry_AbstractHaskell
import qualified Curry_Char
import qualified Curry_List
import qualified Curry_Maybe
import qualified Curry_Names
import qualified Curry_Prelude
import qualified Curry_Read

data C_Options
     = C_Options (Curry_Prelude.OP_List Curry_Prelude.C_Char) Curry_Prelude.C_Bool
     | Choice_C_Options Cover ID C_Options C_Options
     | Choices_C_Options Cover ID ([C_Options])
     | Fail_C_Options Cover FailInfo
     | Guard_C_Options Cover Constraints C_Options

instance Show C_Options where
  showsPrec d (Choice_C_Options cd i x y) = showsChoice d cd i x y
  showsPrec d (Choices_C_Options cd i xs) = showsChoices d cd i xs
  showsPrec d (Guard_C_Options cd c e) = showsGuard d cd c e
  showsPrec _ (Fail_C_Options cd info) = showChar '!'
  showsPrec _ (C_Options x1 x2) = (showString "(Options") . ((showChar ' ') . ((shows x1) . ((showChar ' ') . ((shows x2) . (showChar ')')))))


instance Read C_Options where
  readsPrec d s = readParen (d > 10) (\r -> [ (C_Options x1 x2,r2) | (_,r0) <- readQualified "AbstractHaskellPrinter" "Options" r, (x1,r1) <- readsPrec 11 r0, (x2,r2) <- readsPrec 11 r1]) s


instance NonDet C_Options where
  choiceCons = Choice_C_Options
  choicesCons = Choices_C_Options
  failCons = Fail_C_Options
  guardCons = Guard_C_Options
  try (Choice_C_Options cd i x y) = tryChoice cd i x y
  try (Choices_C_Options cd i xs) = tryChoices cd i xs
  try (Fail_C_Options cd info) = Fail cd info
  try (Guard_C_Options cd c e) = Guard cd c e
  try x = Val x
  match f _ _ _ _ _ (Choice_C_Options cd i x y) = f cd i x y
  match _ f _ _ _ _ (Choices_C_Options cd i@(NarrowedID _ _) xs) = f cd i xs
  match _ _ f _ _ _ (Choices_C_Options cd i@(FreeID _ _) xs) = f cd i xs
  match _ _ _ _ _ _ (Choices_C_Options cd i _) = error ("AbstractHaskellPrinter.Options.match: Choices with ChoiceID " ++ (show i))
  match _ _ _ f _ _ (Fail_C_Options cd info) = f cd info
  match _ _ _ _ f _ (Guard_C_Options cd c e) = f cd c e
  match _ _ _ _ _ f x = f x


instance Generable C_Options where
  generate s c = Choices_C_Options c (freeID [2] s) [(C_Options (generate (leftSupply s) c) (generate (rightSupply s) c))]


instance NormalForm C_Options where
  ($!!) cont (C_Options x1 x2) d cs = (((\y1 d cs -> (((\y2 d cs -> cont (C_Options y1 y2) d cs) $!! x2) d) cs) $!! x1) d) cs
  ($!!) cont (Choice_C_Options cd i x y) d cs = nfChoice cont cd i x y cd cs
  ($!!) cont (Choices_C_Options cd i xs) d cs = nfChoices cont cd i xs d cs
  ($!!) cont (Guard_C_Options cd c e) d cs = guardCons cd c (((cont $!! e) d) (addCs c cs))
  ($!!) _ (Fail_C_Options cd info) _ _ = failCons cd info
  ($##) cont (C_Options x1 x2) d cs = (((\y1 d cs -> (((\y2 d cs -> cont (C_Options y1 y2) d cs) $## x2) d) cs) $## x1) d) cs
  ($##) cont (Choice_C_Options cd i x y) d cs = gnfChoice cont cd i x y cd cs
  ($##) cont (Choices_C_Options cd i xs) d cs = gnfChoices cont cd i xs d cs
  ($##) cont (Guard_C_Options cd c e) d cs = guardCons cd c (((cont $## e) d) (addCs c cs))
  ($##) _ (Fail_C_Options cd info) _ _ = failCons cd info
  showCons (C_Options _ _) = "AbstractHaskellPrinter.Options _ _"
  showCons x = error ("AbstractHaskellPrinter.Options.showCons: no constructor: " ++ (show x))
  searchNF search cont (C_Options x1 x2) = search (\y1 -> search (\y2 -> cont (C_Options y1 y2)) x2) x1
  searchNF _ _ x = error ("AbstractHaskellPrinter.Options.searchNF: no constructor: " ++ (show x))


instance Unifiable C_Options where
  (=.=) (C_Options x1 x2) (C_Options y1 y2) d cs = (((((x1 =:= y1) d) cs) & (((x2 =:= y2) d) cs)) d) cs
  (=.=) a b cd _ = Fail_C_Success cd (unificationFail (showCons a) (showCons b))
  (=.<=) (C_Options x1 x2) (C_Options y1 y2) d cs = (((((x1 =:<= y1) d) cs) & (((x2 =:<= y2) d) cs)) d) cs
  (=.<=) a b cd _ = Fail_C_Success cd (unificationFail (showCons a) (showCons b))
  bind cd i (C_Options x3 x4) = ((i :=: (ChooseN 0 2)):(concat [(bind cd (leftID i) x3),(bind cd (rightID i) x4)]))
  bind d i (Choice_C_Options cd j x y) = [(ConstraintChoice cd j (bind d i x) (bind d i y))]
  bind d i (Choices_C_Options cd j@(FreeID _ _) xs) = bindOrNarrow d i cd j xs
  bind d i (Choices_C_Options cd j@(NarrowedID _ _) xs) = [(ConstraintChoices cd j (map (bind d i) xs))]
  bind _ _ (Choices_C_Options cd i _) = error ("AbstractHaskellPrinter.Options.bind: Choices with ChoiceID: " ++ (show i))
  bind _ _ (Fail_C_Options cd info) = [(Unsolvable info)]
  bind d i (Guard_C_Options cd c e) = (getConstrList c) ++ (bind d i e)
  lazyBind cd i (C_Options x3 x4) = [(i :=: (ChooseN 0 2)),((leftID i) :=: (LazyBind (lazyBind cd (leftID i) x3))),((rightID i) :=: (LazyBind (lazyBind cd (rightID i) x4)))]
  lazyBind d i (Choice_C_Options cd j x y) = [(ConstraintChoice cd j (lazyBind d i x) (lazyBind d i y))]
  lazyBind d i (Choices_C_Options cd j@(FreeID _ _) xs) = lazyBindOrNarrow d i cd j xs
  lazyBind d i (Choices_C_Options cd j@(NarrowedID _ _) xs) = [(ConstraintChoices cd j (map (lazyBind d i) xs))]
  lazyBind _ _ (Choices_C_Options cd i _) = error ("AbstractHaskellPrinter.Options.lazyBind: Choices with ChoiceID: " ++ (show i))
  lazyBind _ _ (Fail_C_Options cd info) = [(Unsolvable info)]
  lazyBind d i (Guard_C_Options cd c e) = (getConstrList c) ++ [(i :=: (LazyBind (lazyBind d i e)))]


instance Curry_Prelude.Curry C_Options where
  (=?=) (Choice_C_Options cd i x y) z d cs = narrow cd i (((x Curry_Prelude.=?= z) d) cs) (((y Curry_Prelude.=?= z) d) cs)
  (=?=) (Choices_C_Options cd i xs) y d cs = narrows cs cd i (\x -> ((x Curry_Prelude.=?= y) d) cs) xs
  (=?=) (Guard_C_Options cd c e) y d cs = guardCons cd c (((e Curry_Prelude.=?= y) d) (addCs c cs))
  (=?=) (Fail_C_Options cd info) _ _ _ = failCons cd info
  (=?=) z (Choice_C_Options cd i x y) d cs = narrow cd i (((z Curry_Prelude.=?= x) d) cs) (((z Curry_Prelude.=?= y) d) cs)
  (=?=) y (Choices_C_Options cd i xs) d cs = narrows cs cd i (\x -> ((y Curry_Prelude.=?= x) d) cs) xs
  (=?=) y (Guard_C_Options cd c e) d cs = guardCons cd c (((y Curry_Prelude.=?= e) d) (addCs c cs))
  (=?=) _ (Fail_C_Options cd info) _ _ = failCons cd info
  (=?=) (C_Options x1 x2) (C_Options y1 y2) d cs = Curry_Prelude.d_OP_amp_amp (((x1 Curry_Prelude.=?= y1) d) cs) (((x2 Curry_Prelude.=?= y2) d) cs) d cs
  (<?=) (Choice_C_Options cd i x y) z d cs = narrow cd i (((x Curry_Prelude.<?= z) d) cs) (((y Curry_Prelude.<?= z) d) cs)
  (<?=) (Choices_C_Options cd i xs) y d cs = narrows cs cd i (\x -> ((x Curry_Prelude.<?= y) d) cs) xs
  (<?=) (Guard_C_Options cd c e) y d cs = guardCons cd c (((e Curry_Prelude.<?= y) d) (addCs c cs))
  (<?=) (Fail_C_Options cd info) _ _ _ = failCons cd info
  (<?=) z (Choice_C_Options cd i x y) d cs = narrow cd i (((z Curry_Prelude.<?= x) d) cs) (((z Curry_Prelude.<?= y) d) cs)
  (<?=) y (Choices_C_Options cd i xs) d cs = narrows cs cd i (\x -> ((y Curry_Prelude.<?= x) d) cs) xs
  (<?=) y (Guard_C_Options cd c e) d cs = guardCons cd c (((y Curry_Prelude.<?= e) d) (addCs c cs))
  (<?=) _ (Fail_C_Options cd info) _ _ = failCons cd info
  (<?=) (C_Options x1 x2) (C_Options y1 y2) d cs = Curry_Prelude.d_OP_bar_bar (Curry_Prelude.d_OP_lt x1 y1 d cs) (Curry_Prelude.d_OP_amp_amp (((x1 Curry_Prelude.=?= y1) d) cs) (((x2 Curry_Prelude.<?= y2) d) cs) d cs) d cs


d_OP_uscore_hash_selR_at_Options_dot_currentModule :: C_Options -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_OP_uscore_hash_selR_at_Options_dot_currentModule x1 cd cs = case x1 of
     (C_Options x2 x3) -> x2
     (Choice_C_Options d i l r) -> narrow d i (d_OP_uscore_hash_selR_at_Options_dot_currentModule l cd cs) (d_OP_uscore_hash_selR_at_Options_dot_currentModule r cd cs)
     (Choices_C_Options d i xs) -> narrows cs d i (\z -> d_OP_uscore_hash_selR_at_Options_dot_currentModule z cd cs) xs
     (Guard_C_Options d c e) -> guardCons d c ((d_OP_uscore_hash_selR_at_Options_dot_currentModule e cd) $! (addCs c cs))
     (Fail_C_Options d info) -> failCons d (traceFail "AbstractHaskellPrinter._#selR@Options.currentModule" [(show x1)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._#selR@Options.currentModule" (showCons x1))

d_OP_uscore_hash_updR_at_Options_dot_currentModule :: C_Options -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Cover -> ConstStore -> C_Options
d_OP_uscore_hash_updR_at_Options_dot_currentModule x1 x2 cd cs = case x1 of
     (C_Options x3 x4) -> C_Options x2 x4
     (Choice_C_Options d i l r) -> narrow d i (d_OP_uscore_hash_updR_at_Options_dot_currentModule l x2 cd cs) (d_OP_uscore_hash_updR_at_Options_dot_currentModule r x2 cd cs)
     (Choices_C_Options d i xs) -> narrows cs d i (\z -> d_OP_uscore_hash_updR_at_Options_dot_currentModule z x2 cd cs) xs
     (Guard_C_Options d c e) -> guardCons d c ((d_OP_uscore_hash_updR_at_Options_dot_currentModule e x2 cd) $! (addCs c cs))
     (Fail_C_Options d info) -> failCons d (traceFail "AbstractHaskellPrinter._#updR@Options.currentModule" [(show x1),(show x2)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._#updR@Options.currentModule" (showCons x1))

d_OP_uscore_hash_selR_at_Options_dot_traceFailure :: C_Options -> Cover -> ConstStore -> Curry_Prelude.C_Bool
d_OP_uscore_hash_selR_at_Options_dot_traceFailure x1 cd cs = case x1 of
     (C_Options x2 x3) -> x3
     (Choice_C_Options d i l r) -> narrow d i (d_OP_uscore_hash_selR_at_Options_dot_traceFailure l cd cs) (d_OP_uscore_hash_selR_at_Options_dot_traceFailure r cd cs)
     (Choices_C_Options d i xs) -> narrows cs d i (\z -> d_OP_uscore_hash_selR_at_Options_dot_traceFailure z cd cs) xs
     (Guard_C_Options d c e) -> guardCons d c ((d_OP_uscore_hash_selR_at_Options_dot_traceFailure e cd) $! (addCs c cs))
     (Fail_C_Options d info) -> failCons d (traceFail "AbstractHaskellPrinter._#selR@Options.traceFailure" [(show x1)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._#selR@Options.traceFailure" (showCons x1))

d_OP_uscore_hash_updR_at_Options_dot_traceFailure :: C_Options -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> C_Options
d_OP_uscore_hash_updR_at_Options_dot_traceFailure x1 x2 cd cs = case x1 of
     (C_Options x3 x4) -> C_Options x3 x2
     (Choice_C_Options d i l r) -> narrow d i (d_OP_uscore_hash_updR_at_Options_dot_traceFailure l x2 cd cs) (d_OP_uscore_hash_updR_at_Options_dot_traceFailure r x2 cd cs)
     (Choices_C_Options d i xs) -> narrows cs d i (\z -> d_OP_uscore_hash_updR_at_Options_dot_traceFailure z x2 cd cs) xs
     (Guard_C_Options d c e) -> guardCons d c ((d_OP_uscore_hash_updR_at_Options_dot_traceFailure e x2 cd) $! (addCs c cs))
     (Fail_C_Options d info) -> failCons d (traceFail "AbstractHaskellPrinter._#updR@Options.traceFailure" [(show x1),(show x2)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._#updR@Options.traceFailure" (showCons x1))

d_C_defaultOptions :: Cover -> ConstStore -> C_Options
d_C_defaultOptions cd cs = C_Options Curry_Prelude.OP_List Curry_Prelude.C_False

d_C_showProg :: Curry_Prelude.C_Bool -> Curry_AbstractHaskell.C_Prog -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C_showProg x1 x2 cd cs = case x2 of
     (Curry_AbstractHaskell.C_Prog x3 x4 x5 x6 x7) -> Curry_Prelude.d_OP_dollar (Curry_List.d_C_intercalate (Curry_Prelude.toCurryString "\n\n")) (Curry_Prelude.d_OP_dollar (Curry_Prelude.d_C_filter (Curry_Prelude.d_OP_dot Curry_Prelude.d_C_not Curry_Prelude.d_C_null cd cs)) (Curry_Prelude.OP_Cons (d_C_showModuleHeader x1 x3 x5 x6 x4 cd cs) (Curry_Prelude.OP_Cons (d_C_showDecls x1 x3 x7 x5 x6 cd cs) Curry_Prelude.OP_List)) cd cs) cd cs
     (Curry_AbstractHaskell.Choice_C_Prog d i l r) -> narrow d i (d_C_showProg x1 l cd cs) (d_C_showProg x1 r cd cs)
     (Curry_AbstractHaskell.Choices_C_Prog d i xs) -> narrows cs d i (\z -> d_C_showProg x1 z cd cs) xs
     (Curry_AbstractHaskell.Guard_C_Prog d c e) -> guardCons d c ((d_C_showProg x1 e cd) $! (addCs c cs))
     (Curry_AbstractHaskell.Fail_C_Prog d info) -> failCons d (traceFail "AbstractHaskellPrinter.showProg" [(show x1),(show x2)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter.showProg" (showCons x2))

d_C_showModuleHeader :: Curry_Prelude.C_Bool -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_TypeDecl -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_FuncDecl -> Curry_Prelude.OP_List (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C_showModuleHeader x1 x2 x3 x4 x5 cd cs = let
     x6 = d_C_showExports x3 x4 cd cs
     x7 = d_C_showImports x1 x5 cd cs
      in (Curry_Prelude.d_C_concat (Curry_Prelude.OP_Cons (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString "module ") (d_C__case_164 x2 x1 cd cs) cd cs) (Curry_Prelude.OP_Cons (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString " (") (Curry_Prelude.d_OP_plus_plus x6 (Curry_Prelude.toCurryString ")") cd cs) cd cs) (Curry_Prelude.OP_Cons (Curry_Prelude.toCurryString " where") (Curry_Prelude.OP_Cons (d_C__case_163 x7 (Curry_Prelude.d_C_null x7 cd cs) cd cs) Curry_Prelude.OP_List)))) cd cs)

d_C_showDecls :: Curry_Prelude.C_Bool -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_OpDecl -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_TypeDecl -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_FuncDecl -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C_showDecls x1 x2 x3 x4 x5 cd cs = let
     x6 = C_Options x2 x1
      in (Curry_Prelude.d_OP_dollar (Curry_List.d_C_intercalate (Curry_Prelude.toCurryString "\n\n")) (Curry_Prelude.d_C_filter (Curry_Prelude.d_OP_dot Curry_Prelude.d_C_not Curry_Prelude.d_C_null cd cs) (Curry_Prelude.OP_Cons (d_C_showOpDecls x3 cd cs) (Curry_Prelude.OP_Cons (d_C_showTypeDecls x6 x4 cd cs) (Curry_Prelude.OP_Cons (d_C_showFuncDecls x6 x5 cd cs) Curry_Prelude.OP_List))) cd cs) cd cs)

d_C_showExports :: Curry_Prelude.OP_List Curry_AbstractHaskell.C_TypeDecl -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_FuncDecl -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C_showExports x1 x2 cd cs = let
     x3 = Curry_Prelude.d_C_filter d_OP_showExports_dot_isPublicType_dot_13 x1 cd cs
     x4 = Curry_List.d_C_partition d_OP_showExports_dot_allPublicCons_dot_13 x3 cd cs
     x5 = d_OP_showExports_dot_uscore_hash_selFP2_hash_withCons x4 cd cs
     x6 = d_OP_showExports_dot_uscore_hash_selFP3_hash_withoutCons x4 cd cs
      in (Curry_Prelude.d_OP_dollar (Curry_List.d_C_intercalate (Curry_Prelude.toCurryString ", ")) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.d_C_map (Curry_Prelude.d_OP_dot (Curry_Prelude.d_C_flip (acceptCs id Curry_Prelude.d_OP_plus_plus) (Curry_Prelude.toCurryString " (..)")) d_OP_showExports_dot_getTypeName_dot_13 cd cs) x5 cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.d_C_map d_OP_showExports_dot_getTypeName_dot_13 x6 cd cs) (Curry_Prelude.d_C_map d_OP_showExports_dot_getFuncName_dot_13 (Curry_Prelude.d_C_filter d_OP_showExports_dot_isPublicFunc_dot_13 x2 cd cs) cd cs) cd cs) cd cs) cd cs)

d_OP_showExports_dot_isPublicType_dot_13 :: Curry_AbstractHaskell.C_TypeDecl -> Cover -> ConstStore -> Curry_Prelude.C_Bool
d_OP_showExports_dot_isPublicType_dot_13 x1 cd cs = case x1 of
     (Curry_AbstractHaskell.C_Type x2 x3 x4 x5) -> Curry_Prelude.d_OP_eq_eq x3 Curry_AbstractHaskell.C_Public cd cs
     (Curry_AbstractHaskell.C_TypeSyn x6 x7 x8 x9) -> Curry_Prelude.d_OP_eq_eq x7 Curry_AbstractHaskell.C_Public cd cs
     (Curry_AbstractHaskell.C_Instance x10 x11 x12 x13) -> Curry_Prelude.C_False
     (Curry_AbstractHaskell.Choice_C_TypeDecl d i l r) -> narrow d i (d_OP_showExports_dot_isPublicType_dot_13 l cd cs) (d_OP_showExports_dot_isPublicType_dot_13 r cd cs)
     (Curry_AbstractHaskell.Choices_C_TypeDecl d i xs) -> narrows cs d i (\z -> d_OP_showExports_dot_isPublicType_dot_13 z cd cs) xs
     (Curry_AbstractHaskell.Guard_C_TypeDecl d c e) -> guardCons d c ((d_OP_showExports_dot_isPublicType_dot_13 e cd) $! (addCs c cs))
     (Curry_AbstractHaskell.Fail_C_TypeDecl d info) -> failCons d (traceFail "AbstractHaskellPrinter.showExports.isPublicType.13" [(show x1)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter.showExports.isPublicType.13" (showCons x1))

d_OP_showExports_dot_isPublicFunc_dot_13 :: Curry_AbstractHaskell.C_FuncDecl -> Cover -> ConstStore -> Curry_Prelude.C_Bool
d_OP_showExports_dot_isPublicFunc_dot_13 x1 cd cs = case x1 of
     (Curry_AbstractHaskell.C_Func x2 x3 x4 x5 x6 x7) -> Curry_Prelude.d_OP_eq_eq x5 Curry_AbstractHaskell.C_Public cd cs
     (Curry_AbstractHaskell.Choice_C_FuncDecl d i l r) -> narrow d i (d_OP_showExports_dot_isPublicFunc_dot_13 l cd cs) (d_OP_showExports_dot_isPublicFunc_dot_13 r cd cs)
     (Curry_AbstractHaskell.Choices_C_FuncDecl d i xs) -> narrows cs d i (\z -> d_OP_showExports_dot_isPublicFunc_dot_13 z cd cs) xs
     (Curry_AbstractHaskell.Guard_C_FuncDecl d c e) -> guardCons d c ((d_OP_showExports_dot_isPublicFunc_dot_13 e cd) $! (addCs c cs))
     (Curry_AbstractHaskell.Fail_C_FuncDecl d info) -> failCons d (traceFail "AbstractHaskellPrinter.showExports.isPublicFunc.13" [(show x1)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter.showExports.isPublicFunc.13" (showCons x1))

d_OP_showExports_dot_getTypeName_dot_13 :: Curry_AbstractHaskell.C_TypeDecl -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_OP_showExports_dot_getTypeName_dot_13 x1 cd cs = case x1 of
     (Curry_AbstractHaskell.C_Type x2 x3 x4 x5) -> d_C__case_162 x2 cd cs
     (Curry_AbstractHaskell.C_TypeSyn x8 x9 x10 x11) -> d_C__case_161 x8 cd cs
     (Curry_AbstractHaskell.C_Instance x14 x15 x16 x17) -> d_C__case_160 x14 cd cs
     (Curry_AbstractHaskell.Choice_C_TypeDecl d i l r) -> narrow d i (d_OP_showExports_dot_getTypeName_dot_13 l cd cs) (d_OP_showExports_dot_getTypeName_dot_13 r cd cs)
     (Curry_AbstractHaskell.Choices_C_TypeDecl d i xs) -> narrows cs d i (\z -> d_OP_showExports_dot_getTypeName_dot_13 z cd cs) xs
     (Curry_AbstractHaskell.Guard_C_TypeDecl d c e) -> guardCons d c ((d_OP_showExports_dot_getTypeName_dot_13 e cd) $! (addCs c cs))
     (Curry_AbstractHaskell.Fail_C_TypeDecl d info) -> failCons d (traceFail "AbstractHaskellPrinter.showExports.getTypeName.13" [(show x1)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter.showExports.getTypeName.13" (showCons x1))

d_OP_showExports_dot_allPublicCons_dot_13 :: Curry_AbstractHaskell.C_TypeDecl -> Cover -> ConstStore -> Curry_Prelude.C_Bool
d_OP_showExports_dot_allPublicCons_dot_13 x1 cd cs = case x1 of
     (Curry_AbstractHaskell.C_Type x2 x3 x4 x5) -> Curry_Prelude.d_OP_eq_eq (Curry_Prelude.d_C_length (Curry_Prelude.d_C_filter d_OP_showExports_dot_allPublicCons_dot_13_dot_isPublicCons_dot_59 x5 cd cs) cd cs) (Curry_Prelude.d_C_length x5 cd cs) cd cs
     (Curry_AbstractHaskell.C_TypeSyn x6 x7 x8 x9) -> Curry_Prelude.C_False
     (Curry_AbstractHaskell.C_Instance x10 x11 x12 x13) -> Curry_Prelude.C_False
     (Curry_AbstractHaskell.Choice_C_TypeDecl d i l r) -> narrow d i (d_OP_showExports_dot_allPublicCons_dot_13 l cd cs) (d_OP_showExports_dot_allPublicCons_dot_13 r cd cs)
     (Curry_AbstractHaskell.Choices_C_TypeDecl d i xs) -> narrows cs d i (\z -> d_OP_showExports_dot_allPublicCons_dot_13 z cd cs) xs
     (Curry_AbstractHaskell.Guard_C_TypeDecl d c e) -> guardCons d c ((d_OP_showExports_dot_allPublicCons_dot_13 e cd) $! (addCs c cs))
     (Curry_AbstractHaskell.Fail_C_TypeDecl d info) -> failCons d (traceFail "AbstractHaskellPrinter.showExports.allPublicCons.13" [(show x1)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter.showExports.allPublicCons.13" (showCons x1))

d_OP_showExports_dot_allPublicCons_dot_13_dot_isPublicCons_dot_59 :: Curry_AbstractHaskell.C_ConsDecl -> Cover -> ConstStore -> Curry_Prelude.C_Bool
d_OP_showExports_dot_allPublicCons_dot_13_dot_isPublicCons_dot_59 x1 cd cs = case x1 of
     (Curry_AbstractHaskell.C_Cons x2 x3 x4 x5) -> Curry_Prelude.d_OP_eq_eq x4 Curry_AbstractHaskell.C_Public cd cs
     (Curry_AbstractHaskell.Choice_C_ConsDecl d i l r) -> narrow d i (d_OP_showExports_dot_allPublicCons_dot_13_dot_isPublicCons_dot_59 l cd cs) (d_OP_showExports_dot_allPublicCons_dot_13_dot_isPublicCons_dot_59 r cd cs)
     (Curry_AbstractHaskell.Choices_C_ConsDecl d i xs) -> narrows cs d i (\z -> d_OP_showExports_dot_allPublicCons_dot_13_dot_isPublicCons_dot_59 z cd cs) xs
     (Curry_AbstractHaskell.Guard_C_ConsDecl d c e) -> guardCons d c ((d_OP_showExports_dot_allPublicCons_dot_13_dot_isPublicCons_dot_59 e cd) $! (addCs c cs))
     (Curry_AbstractHaskell.Fail_C_ConsDecl d info) -> failCons d (traceFail "AbstractHaskellPrinter.showExports.allPublicCons.13.isPublicCons.59" [(show x1)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter.showExports.allPublicCons.13.isPublicCons.59" (showCons x1))

d_OP_showExports_dot_getFuncName_dot_13 :: Curry_AbstractHaskell.C_FuncDecl -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_OP_showExports_dot_getFuncName_dot_13 x1 cd cs = case x1 of
     (Curry_AbstractHaskell.C_Func x2 x3 x4 x5 x6 x7) -> d_C__case_159 x3 cd cs
     (Curry_AbstractHaskell.Choice_C_FuncDecl d i l r) -> narrow d i (d_OP_showExports_dot_getFuncName_dot_13 l cd cs) (d_OP_showExports_dot_getFuncName_dot_13 r cd cs)
     (Curry_AbstractHaskell.Choices_C_FuncDecl d i xs) -> narrows cs d i (\z -> d_OP_showExports_dot_getFuncName_dot_13 z cd cs) xs
     (Curry_AbstractHaskell.Guard_C_FuncDecl d c e) -> guardCons d c ((d_OP_showExports_dot_getFuncName_dot_13 e cd) $! (addCs c cs))
     (Curry_AbstractHaskell.Fail_C_FuncDecl d info) -> failCons d (traceFail "AbstractHaskellPrinter.showExports.getFuncName.13" [(show x1)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter.showExports.getFuncName.13" (showCons x1))

d_OP_showExports_dot_uscore_hash_selFP2_hash_withCons :: Curry_Prelude.OP_Tuple2 (Curry_Prelude.OP_List Curry_AbstractHaskell.C_TypeDecl) (Curry_Prelude.OP_List Curry_AbstractHaskell.C_TypeDecl) -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_TypeDecl
d_OP_showExports_dot_uscore_hash_selFP2_hash_withCons x1 cd cs = case x1 of
     (Curry_Prelude.OP_Tuple2 x2 x3) -> x2
     (Curry_Prelude.Choice_OP_Tuple2 d i l r) -> narrow d i (d_OP_showExports_dot_uscore_hash_selFP2_hash_withCons l cd cs) (d_OP_showExports_dot_uscore_hash_selFP2_hash_withCons r cd cs)
     (Curry_Prelude.Choices_OP_Tuple2 d i xs) -> narrows cs d i (\z -> d_OP_showExports_dot_uscore_hash_selFP2_hash_withCons z cd cs) xs
     (Curry_Prelude.Guard_OP_Tuple2 d c e) -> guardCons d c ((d_OP_showExports_dot_uscore_hash_selFP2_hash_withCons e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_Tuple2 d info) -> failCons d (traceFail "AbstractHaskellPrinter.showExports._#selFP2#withCons" [(show x1)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter.showExports._#selFP2#withCons" (showCons x1))

d_OP_showExports_dot_uscore_hash_selFP3_hash_withoutCons :: Curry_Prelude.OP_Tuple2 (Curry_Prelude.OP_List Curry_AbstractHaskell.C_TypeDecl) (Curry_Prelude.OP_List Curry_AbstractHaskell.C_TypeDecl) -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_TypeDecl
d_OP_showExports_dot_uscore_hash_selFP3_hash_withoutCons x1 cd cs = case x1 of
     (Curry_Prelude.OP_Tuple2 x2 x3) -> x3
     (Curry_Prelude.Choice_OP_Tuple2 d i l r) -> narrow d i (d_OP_showExports_dot_uscore_hash_selFP3_hash_withoutCons l cd cs) (d_OP_showExports_dot_uscore_hash_selFP3_hash_withoutCons r cd cs)
     (Curry_Prelude.Choices_OP_Tuple2 d i xs) -> narrows cs d i (\z -> d_OP_showExports_dot_uscore_hash_selFP3_hash_withoutCons z cd cs) xs
     (Curry_Prelude.Guard_OP_Tuple2 d c e) -> guardCons d c ((d_OP_showExports_dot_uscore_hash_selFP3_hash_withoutCons e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_Tuple2 d info) -> failCons d (traceFail "AbstractHaskellPrinter.showExports._#selFP3#withoutCons" [(show x1)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter.showExports._#selFP3#withoutCons" (showCons x1))

d_C_showImports :: Curry_Prelude.C_Bool -> Curry_Prelude.OP_List (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C_showImports x1 x2 cd cs = d_C_prefixInter (d_OP_showImports_dot_showImport_dot_89 x1) x2 (Curry_Prelude.toCurryString "\n") cd cs

d_OP_showImports_dot_showImport_dot_89 :: Curry_Prelude.C_Bool -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_OP_showImports_dot_showImport_dot_89 x1 x2 cd cs = d_C__case_157 x2 x1 (Curry_Prelude.d_C_apply (Curry_Names.d_C_isHaskellModule cd cs) x2 cd cs) cd cs

d_C_showOpDecls :: Curry_Prelude.OP_List Curry_AbstractHaskell.C_OpDecl -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C_showOpDecls x1 cd cs = d_C_prefixInter d_C_showOpDecl x1 (Curry_Prelude.toCurryString "\n") cd cs

d_C_showOpDecl :: Curry_AbstractHaskell.C_OpDecl -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C_showOpDecl x1 cd cs = case x1 of
     (Curry_AbstractHaskell.C_Op x2 x3 x4) -> d_C__case_154 x4 x3 x2 cd cs
     (Curry_AbstractHaskell.Choice_C_OpDecl d i l r) -> narrow d i (d_C_showOpDecl l cd cs) (d_C_showOpDecl r cd cs)
     (Curry_AbstractHaskell.Choices_C_OpDecl d i xs) -> narrows cs d i (\z -> d_C_showOpDecl z cd cs) xs
     (Curry_AbstractHaskell.Guard_C_OpDecl d c e) -> guardCons d c ((d_C_showOpDecl e cd) $! (addCs c cs))
     (Curry_AbstractHaskell.Fail_C_OpDecl d info) -> failCons d (traceFail "AbstractHaskellPrinter.showOpDecl" [(show x1)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter.showOpDecl" (showCons x1))

d_C_showFixity :: Curry_AbstractHaskell.C_Fixity -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C_showFixity x1 cd cs = case x1 of
     Curry_AbstractHaskell.C_InfixOp -> Curry_Prelude.toCurryString "infix"
     Curry_AbstractHaskell.C_InfixlOp -> Curry_Prelude.toCurryString "infixl"
     Curry_AbstractHaskell.C_InfixrOp -> Curry_Prelude.toCurryString "infixr"
     (Curry_AbstractHaskell.Choice_C_Fixity d i l r) -> narrow d i (d_C_showFixity l cd cs) (d_C_showFixity r cd cs)
     (Curry_AbstractHaskell.Choices_C_Fixity d i xs) -> narrows cs d i (\z -> d_C_showFixity z cd cs) xs
     (Curry_AbstractHaskell.Guard_C_Fixity d c e) -> guardCons d c ((d_C_showFixity e cd) $! (addCs c cs))
     (Curry_AbstractHaskell.Fail_C_Fixity d info) -> failCons d (traceFail "AbstractHaskellPrinter.showFixity" [(show x1)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter.showFixity" (showCons x1))

d_C_showTypeDecls :: C_Options -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_TypeDecl -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C_showTypeDecls x1 x2 cd cs = d_C_prefixInter (d_C_showTypeDecl x1) x2 (Curry_Prelude.toCurryString "\n\n") cd cs

d_C_showTypeDecl :: C_Options -> Curry_AbstractHaskell.C_TypeDecl -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C_showTypeDecl x1 x2 cd cs = case x2 of
     (Curry_AbstractHaskell.C_TypeSyn x3 x4 x5 x6) -> Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString "type ") (Curry_Prelude.d_OP_plus_plus (d_C_showSymbol x1 x3 cd cs) (Curry_Prelude.d_OP_plus_plus (d_C_prefixMap (d_C_showTypeExpr x1 Curry_Prelude.C_False) (Curry_Prelude.d_C_map (acceptCs id Curry_AbstractHaskell.C_TVar) x5 cd cs) (Curry_Prelude.toCurryString " ") cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString " = ") (d_C_showTypeExpr x1 Curry_Prelude.C_False x6 cd cs) cd cs) cd cs) cd cs) cd cs
     (Curry_AbstractHaskell.C_Type x7 x8 x9 x10) -> d_C__case_152 x10 x1 x9 x7 (Curry_Prelude.d_C_null x10 cd cs) cd cs
     (Curry_AbstractHaskell.C_Instance x11 x12 x13 x14) -> Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString "instance ") (Curry_Prelude.d_OP_plus_plus (d_C_showContext x1 x13 cd cs) (Curry_Prelude.d_OP_plus_plus (d_C_showSymbol x1 x11 cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString " ") (Curry_Prelude.d_OP_plus_plus (d_C_showTypeExpr x1 Curry_Prelude.C_True x12 cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString " where\n") (Curry_Prelude.d_C_apply (Curry_Prelude.d_C_concatMap (d_OP_showTypeDecl_dot_showInstRule_dot_112 x1) cd cs) x14 cd cs) cd cs) cd cs) cd cs) cd cs) cd cs) cd cs
     (Curry_AbstractHaskell.Choice_C_TypeDecl d i l r) -> narrow d i (d_C_showTypeDecl x1 l cd cs) (d_C_showTypeDecl x1 r cd cs)
     (Curry_AbstractHaskell.Choices_C_TypeDecl d i xs) -> narrows cs d i (\z -> d_C_showTypeDecl x1 z cd cs) xs
     (Curry_AbstractHaskell.Guard_C_TypeDecl d c e) -> guardCons d c ((d_C_showTypeDecl x1 e cd) $! (addCs c cs))
     (Curry_AbstractHaskell.Fail_C_TypeDecl d info) -> failCons d (traceFail "AbstractHaskellPrinter.showTypeDecl" [(show x1),(show x2)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter.showTypeDecl" (showCons x2))

d_OP_showTypeDecl_dot_showInstRule_dot_112 :: Curry_Prelude.Curry t0 => C_Options -> Curry_Prelude.OP_Tuple2 (Curry_Prelude.OP_Tuple2 t0 (Curry_Prelude.OP_List Curry_Prelude.C_Char)) Curry_AbstractHaskell.C_Rule -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_OP_showTypeDecl_dot_showInstRule_dot_112 x1 x2 cd cs = case x2 of
     (Curry_Prelude.OP_Tuple2 x3 x4) -> d_C__case_150 x4 x1 x3 cd cs
     (Curry_Prelude.Choice_OP_Tuple2 d i l r) -> narrow d i (d_OP_showTypeDecl_dot_showInstRule_dot_112 x1 l cd cs) (d_OP_showTypeDecl_dot_showInstRule_dot_112 x1 r cd cs)
     (Curry_Prelude.Choices_OP_Tuple2 d i xs) -> narrows cs d i (\z -> d_OP_showTypeDecl_dot_showInstRule_dot_112 x1 z cd cs) xs
     (Curry_Prelude.Guard_OP_Tuple2 d c e) -> guardCons d c ((d_OP_showTypeDecl_dot_showInstRule_dot_112 x1 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_Tuple2 d info) -> failCons d (traceFail "AbstractHaskellPrinter.showTypeDecl.showInstRule.112" [(show x1),(show x2)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter.showTypeDecl.showInstRule.112" (showCons x2))

d_C_showContext :: C_Options -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_Context -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C_showContext x1 x2 cd cs = case x2 of
     Curry_Prelude.OP_List -> Curry_Prelude.OP_List
     (Curry_Prelude.OP_Cons x3 x4) -> d_C__case_148 x2 x1 x3 x4 cd cs
     (Curry_Prelude.Choice_OP_List d i l r) -> narrow d i (d_C_showContext x1 l cd cs) (d_C_showContext x1 r cd cs)
     (Curry_Prelude.Choices_OP_List d i xs) -> narrows cs d i (\z -> d_C_showContext x1 z cd cs) xs
     (Curry_Prelude.Guard_OP_List d c e) -> guardCons d c ((d_C_showContext x1 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_List d info) -> failCons d (traceFail "AbstractHaskellPrinter.showContext" [(show x1),(show x2)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter.showContext" (showCons x2))

d_C_showClass :: C_Options -> Curry_AbstractHaskell.C_Context -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C_showClass x1 x2 cd cs = case x2 of
     (Curry_AbstractHaskell.C_Context x3 x4) -> d_C_showTypeExpr x1 Curry_Prelude.C_False (Curry_AbstractHaskell.C_TCons x3 (Curry_Prelude.d_C_map (acceptCs id Curry_AbstractHaskell.C_TVar) x4 cd cs)) cd cs
     (Curry_AbstractHaskell.Choice_C_Context d i l r) -> narrow d i (d_C_showClass x1 l cd cs) (d_C_showClass x1 r cd cs)
     (Curry_AbstractHaskell.Choices_C_Context d i xs) -> narrows cs d i (\z -> d_C_showClass x1 z cd cs) xs
     (Curry_AbstractHaskell.Guard_C_Context d c e) -> guardCons d c ((d_C_showClass x1 e cd) $! (addCs c cs))
     (Curry_AbstractHaskell.Fail_C_Context d info) -> failCons d (traceFail "AbstractHaskellPrinter.showClass" [(show x1),(show x2)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter.showClass" (showCons x2))

d_C_showConsDecl :: C_Options -> Curry_AbstractHaskell.C_ConsDecl -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C_showConsDecl x1 x2 cd cs = case x2 of
     (Curry_AbstractHaskell.C_Cons x3 x4 x5 x6) -> Curry_Prelude.d_OP_plus_plus (d_C_showSymbol x1 x3 cd cs) (d_C_prefixMap (d_C_showTypeExpr x1 Curry_Prelude.C_True) x6 (Curry_Prelude.toCurryString " ") cd cs) cd cs
     (Curry_AbstractHaskell.Choice_C_ConsDecl d i l r) -> narrow d i (d_C_showConsDecl x1 l cd cs) (d_C_showConsDecl x1 r cd cs)
     (Curry_AbstractHaskell.Choices_C_ConsDecl d i xs) -> narrows cs d i (\z -> d_C_showConsDecl x1 z cd cs) xs
     (Curry_AbstractHaskell.Guard_C_ConsDecl d c e) -> guardCons d c ((d_C_showConsDecl x1 e cd) $! (addCs c cs))
     (Curry_AbstractHaskell.Fail_C_ConsDecl d info) -> failCons d (traceFail "AbstractHaskellPrinter.showConsDecl" [(show x1),(show x2)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter.showConsDecl" (showCons x2))

d_C_showTypeExpr :: C_Options -> Curry_Prelude.C_Bool -> Curry_AbstractHaskell.C_TypeExpr -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C_showTypeExpr x1 x2 x3 cd cs = case x3 of
     (Curry_AbstractHaskell.C_TVar x4) -> d_C__case_147 x4 cd cs
     (Curry_AbstractHaskell.C_FuncType x7 x8) -> Curry_Prelude.d_OP_dollar (d_C_maybeShowBrackets x2) (Curry_Prelude.d_OP_plus_plus (d_C_showTypeExpr x1 (d_C_isFuncType x7 cd cs) x7 cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString " -> ") (d_C_showTypeExpr x1 Curry_Prelude.C_False x8 cd cs) cd cs) cd cs) cd cs
     (Curry_AbstractHaskell.C_TCons x9 x10) -> d_C__case_146 x10 x1 x2 x9 cd cs
     (Curry_AbstractHaskell.Choice_C_TypeExpr d i l r) -> narrow d i (d_C_showTypeExpr x1 x2 l cd cs) (d_C_showTypeExpr x1 x2 r cd cs)
     (Curry_AbstractHaskell.Choices_C_TypeExpr d i xs) -> narrows cs d i (\z -> d_C_showTypeExpr x1 x2 z cd cs) xs
     (Curry_AbstractHaskell.Guard_C_TypeExpr d c e) -> guardCons d c ((d_C_showTypeExpr x1 x2 e cd) $! (addCs c cs))
     (Curry_AbstractHaskell.Fail_C_TypeExpr d info) -> failCons d (traceFail "AbstractHaskellPrinter.showTypeExpr" [(show x1),(show x2),(show x3)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter.showTypeExpr" (showCons x3))

d_C_showTypeSig :: C_Options -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_AbstractHaskell.C_TypeSig -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C_showTypeSig x1 x2 x3 cd cs = case x3 of
     Curry_AbstractHaskell.C_Untyped -> Curry_Prelude.OP_List
     (Curry_AbstractHaskell.C_FType x4) -> Curry_Prelude.d_OP_plus_plus (d_C__case_143 x2 (Curry_Prelude.d_C_apply (d_C_isInfixOpName cd cs) x2 cd cs) cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString " :: ") (Curry_Prelude.d_OP_plus_plus (d_C_showTypeExpr x1 Curry_Prelude.C_False x4 cd cs) (Curry_Prelude.toCurryString "\n") cd cs) cd cs) cd cs
     (Curry_AbstractHaskell.C_CType x5 x6) -> Curry_Prelude.d_OP_plus_plus (d_C__case_142 x2 (Curry_Prelude.d_C_apply (d_C_isInfixOpName cd cs) x2 cd cs) cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString " :: ") (Curry_Prelude.d_OP_plus_plus (d_C_showContext x1 x5 cd cs) (Curry_Prelude.d_OP_plus_plus (d_C_showTypeExpr x1 Curry_Prelude.C_False x6 cd cs) (Curry_Prelude.toCurryString "\n") cd cs) cd cs) cd cs) cd cs
     (Curry_AbstractHaskell.Choice_C_TypeSig d i l r) -> narrow d i (d_C_showTypeSig x1 x2 l cd cs) (d_C_showTypeSig x1 x2 r cd cs)
     (Curry_AbstractHaskell.Choices_C_TypeSig d i xs) -> narrows cs d i (\z -> d_C_showTypeSig x1 x2 z cd cs) xs
     (Curry_AbstractHaskell.Guard_C_TypeSig d c e) -> guardCons d c ((d_C_showTypeSig x1 x2 e cd) $! (addCs c cs))
     (Curry_AbstractHaskell.Fail_C_TypeSig d info) -> failCons d (traceFail "AbstractHaskellPrinter.showTypeSig" [(show x1),(show x2),(show x3)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter.showTypeSig" (showCons x3))

d_C_showTypeVar :: Curry_Prelude.OP_List Curry_Prelude.C_Char -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C_showTypeVar x1 cd cs = case x1 of
     Curry_Prelude.OP_List -> Curry_Prelude.OP_List
     (Curry_Prelude.OP_Cons x2 x3) -> d_C__case_141 x3 x2 (Curry_Prelude.d_OP_amp_amp (Curry_Prelude.d_OP_eq_eq x2 (Curry_Prelude.C_Char 'a'#) cd cs) (Curry_Prelude.d_OP_amp_amp (Curry_Prelude.d_C_not (Curry_Prelude.d_C_null x3 cd cs) cd cs) (Curry_Prelude.d_C_apply (Curry_Prelude.d_C_all Curry_Char.d_C_isDigit cd cs) x3 cd cs) cd cs) cd cs) cd cs
     (Curry_Prelude.Choice_OP_List d i l r) -> narrow d i (d_C_showTypeVar l cd cs) (d_C_showTypeVar r cd cs)
     (Curry_Prelude.Choices_OP_List d i xs) -> narrows cs d i (\z -> d_C_showTypeVar z cd cs) xs
     (Curry_Prelude.Guard_OP_List d c e) -> guardCons d c ((d_C_showTypeVar e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_List d info) -> failCons d (traceFail "AbstractHaskellPrinter.showTypeVar" [(show x1)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter.showTypeVar" (showCons x1))

d_C_showIdentifier :: Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C_showIdentifier cd cs = Curry_Prelude.d_C_filter (Curry_Prelude.d_OP_dot Curry_Prelude.d_C_not (Curry_Prelude.d_C_flip Curry_Prelude.d_C_elem (Curry_Prelude.toCurryString "<>")) cd cs)

d_C_showFuncDecl :: Cover -> ConstStore -> Curry_AbstractHaskell.C_FuncDecl -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C_showFuncDecl cd cs = d_C_showFuncDeclOpt (d_C_defaultOptions cd cs)

d_C_showFuncDecls :: C_Options -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_FuncDecl -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C_showFuncDecls x1 x2 cd cs = d_C_prefixInter (d_C_showFuncDeclOpt x1) x2 (Curry_Prelude.toCurryString "\n\n") cd cs

d_C_showFuncDeclOpt :: C_Options -> Curry_AbstractHaskell.C_FuncDecl -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C_showFuncDeclOpt x1 x2 cd cs = case x2 of
     (Curry_AbstractHaskell.C_Func x3 x4 x5 x6 x7 x8) -> d_C__case_140 x4 x1 x5 x7 x3 x8 cd cs
     (Curry_AbstractHaskell.Choice_C_FuncDecl d i l r) -> narrow d i (d_C_showFuncDeclOpt x1 l cd cs) (d_C_showFuncDeclOpt x1 r cd cs)
     (Curry_AbstractHaskell.Choices_C_FuncDecl d i xs) -> narrows cs d i (\z -> d_C_showFuncDeclOpt x1 z cd cs) xs
     (Curry_AbstractHaskell.Guard_C_FuncDecl d c e) -> guardCons d c ((d_C_showFuncDeclOpt x1 e cd) $! (addCs c cs))
     (Curry_AbstractHaskell.Fail_C_FuncDecl d info) -> failCons d (traceFail "AbstractHaskellPrinter.showFuncDeclOpt" [(show x1),(show x2)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter.showFuncDeclOpt" (showCons x2))

d_OP_showFuncDeclOpt_dot_insertName_dot_162 :: Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.C_Int -> Curry_Prelude.OP_Tuple2 (Curry_Prelude.OP_List Curry_Prelude.C_Char) (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_OP_showFuncDeclOpt_dot_insertName_dot_162 x1 x2 x3 x4 cd cs = case x4 of
     (Curry_Prelude.OP_Tuple2 x5 x6) -> d_C__case_136 x3 x6 x5 x1 x2 (Curry_Prelude.d_OP_slash_eq x3 (Curry_Prelude.C_Int 0#) cd cs) cd cs
     (Curry_Prelude.Choice_OP_Tuple2 d i l r) -> narrow d i (d_OP_showFuncDeclOpt_dot_insertName_dot_162 x1 x2 x3 l cd cs) (d_OP_showFuncDeclOpt_dot_insertName_dot_162 x1 x2 x3 r cd cs)
     (Curry_Prelude.Choices_OP_Tuple2 d i xs) -> narrows cs d i (\z -> d_OP_showFuncDeclOpt_dot_insertName_dot_162 x1 x2 x3 z cd cs) xs
     (Curry_Prelude.Guard_OP_Tuple2 d c e) -> guardCons d c ((d_OP_showFuncDeclOpt_dot_insertName_dot_162 x1 x2 x3 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_Tuple2 d info) -> failCons d (traceFail "AbstractHaskellPrinter.showFuncDeclOpt.insertName.162" [(show x1),(show x2),(show x3),(show x4)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter.showFuncDeclOpt.insertName.162" (showCons x4))

d_OP_showFuncDeclOpt_dot_rulePrints_dot_162 :: Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> C_Options -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_Rule -> Curry_Prelude.C_Int -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_OP_showFuncDeclOpt_dot_rulePrints_dot_162 x1 x2 x3 x4 x5 cd cs = Curry_Prelude.d_OP_dollar (Curry_List.d_C_intercalate (Curry_Prelude.toCurryString "\n")) (Curry_Prelude.d_C_map (Curry_Prelude.d_OP_dot (d_OP_showFuncDeclOpt_dot_insertName_dot_162 x1 x2 x5) (Curry_Prelude.d_OP_dot (Curry_Prelude.d_C_span (Curry_Prelude.d_C_flip (acceptCs id Curry_Prelude.d_OP_slash_eq) (Curry_Prelude.C_Char ' '#))) (Curry_Prelude.d_OP_dot Curry_Prelude.d_C_tail (d_C_showRule x3) cd cs) cd cs) cd cs) x4 cd cs) cd cs

d_C_funcComment :: Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C_funcComment cd cs = Curry_Prelude.d_OP_dot Curry_Prelude.d_C_unlines (Curry_Prelude.d_OP_dot (Curry_Prelude.d_C_map (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString "--- "))) Curry_Prelude.d_C_lines cd cs) cd cs

d_C_showLocalFuncDecl :: C_Options -> Cover -> ConstStore -> Curry_AbstractHaskell.C_FuncDecl -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C_showLocalFuncDecl x1 cd cs = d_C_showFuncDeclOpt x1

d_C_showRule :: C_Options -> Curry_AbstractHaskell.C_Rule -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C_showRule x1 x2 cd cs = case x2 of
     (Curry_AbstractHaskell.C_Rule x3 x4 x5) -> Curry_Prelude.d_OP_plus_plus (d_C_prefixMap (d_C_showPattern x1) x3 (Curry_Prelude.toCurryString " ") cd cs) (Curry_Prelude.d_OP_plus_plus (d_C_showCrhsList x1 x4 cd cs) (d_C__case_135 x5 x1 (Curry_Prelude.d_C_null x5 cd cs) cd cs) cd cs) cd cs
     (Curry_AbstractHaskell.Choice_C_Rule d i l r) -> narrow d i (d_C_showRule x1 l cd cs) (d_C_showRule x1 r cd cs)
     (Curry_AbstractHaskell.Choices_C_Rule d i xs) -> narrows cs d i (\z -> d_C_showRule x1 z cd cs) xs
     (Curry_AbstractHaskell.Guard_C_Rule d c e) -> guardCons d c ((d_C_showRule x1 e cd) $! (addCs c cs))
     (Curry_AbstractHaskell.Fail_C_Rule d info) -> failCons d (traceFail "AbstractHaskellPrinter.showRule" [(show x1),(show x2)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter.showRule" (showCons x2))

d_C_showCrhsList :: C_Options -> Curry_Prelude.OP_List (Curry_Prelude.OP_Tuple2 Curry_AbstractHaskell.C_Expr Curry_AbstractHaskell.C_Expr) -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C_showCrhsList x1 x2 cd cs = case x2 of
     Curry_Prelude.OP_List -> Curry_Prelude.OP_List
     (Curry_Prelude.OP_Cons x3 x4) -> d_C__case_134 x4 x1 x3 cd cs
     (Curry_Prelude.Choice_OP_List d i l r) -> narrow d i (d_C_showCrhsList x1 l cd cs) (d_C_showCrhsList x1 r cd cs)
     (Curry_Prelude.Choices_OP_List d i xs) -> narrows cs d i (\z -> d_C_showCrhsList x1 z cd cs) xs
     (Curry_Prelude.Guard_OP_List d c e) -> guardCons d c ((d_C_showCrhsList x1 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_List d info) -> failCons d (traceFail "AbstractHaskellPrinter.showCrhsList" [(show x1),(show x2)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter.showCrhsList" (showCons x2))

d_C_showCrhs :: C_Options -> Curry_Prelude.OP_Tuple2 Curry_AbstractHaskell.C_Expr Curry_AbstractHaskell.C_Expr -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C_showCrhs x1 x2 cd cs = case x2 of
     (Curry_Prelude.OP_Tuple2 x3 x4) -> Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString "| ") (Curry_Prelude.d_OP_plus_plus (d_C_showExprOpt x1 x3 cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString "\n= ") (d_C_showExprOpt x1 x4 cd cs) cd cs) cd cs) cd cs
     (Curry_Prelude.Choice_OP_Tuple2 d i l r) -> narrow d i (d_C_showCrhs x1 l cd cs) (d_C_showCrhs x1 r cd cs)
     (Curry_Prelude.Choices_OP_Tuple2 d i xs) -> narrows cs d i (\z -> d_C_showCrhs x1 z cd cs) xs
     (Curry_Prelude.Guard_OP_Tuple2 d c e) -> guardCons d c ((d_C_showCrhs x1 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_Tuple2 d info) -> failCons d (traceFail "AbstractHaskellPrinter.showCrhs" [(show x1),(show x2)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter.showCrhs" (showCons x2))

d_C_showLocalDecl :: C_Options -> Curry_AbstractHaskell.C_LocalDecl -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C_showLocalDecl x1 x2 cd cs = case x2 of
     (Curry_AbstractHaskell.C_LocalFunc x3) -> Curry_Prelude.d_C_apply (d_C_showLocalFuncDecl x1 cd cs) x3 cd cs
     (Curry_AbstractHaskell.C_LocalPat x4 x5 x6) -> Curry_Prelude.d_OP_plus_plus (d_C_showPattern x1 x4 cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString " = ") (Curry_Prelude.d_OP_plus_plus (d_C_showExprOpt x1 x5 cd cs) (d_C__case_131 x6 x1 (Curry_Prelude.d_C_null x6 cd cs) cd cs) cd cs) cd cs) cd cs
     (Curry_AbstractHaskell.Choice_C_LocalDecl d i l r) -> narrow d i (d_C_showLocalDecl x1 l cd cs) (d_C_showLocalDecl x1 r cd cs)
     (Curry_AbstractHaskell.Choices_C_LocalDecl d i xs) -> narrows cs d i (\z -> d_C_showLocalDecl x1 z cd cs) xs
     (Curry_AbstractHaskell.Guard_C_LocalDecl d c e) -> guardCons d c ((d_C_showLocalDecl x1 e cd) $! (addCs c cs))
     (Curry_AbstractHaskell.Fail_C_LocalDecl d info) -> failCons d (traceFail "AbstractHaskellPrinter.showLocalDecl" [(show x1),(show x2)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter.showLocalDecl" (showCons x2))

d_C_showExpr :: Cover -> ConstStore -> Curry_AbstractHaskell.C_Expr -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C_showExpr cd cs = d_C_showExprOpt (d_C_defaultOptions cd cs)

d_C_showExprOpt :: C_Options -> Curry_AbstractHaskell.C_Expr -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C_showExprOpt x1 x2 cd cs = case x2 of
     (Curry_AbstractHaskell.C_Var x3) -> d_C__case_130 x3 cd cs
     (Curry_AbstractHaskell.C_Lit x6) -> d_C_showLiteral x6 cd cs
     (Curry_AbstractHaskell.C_Symbol x7) -> d_C__case_129 x7 x1 (Curry_Prelude.d_C_apply (d_C_isInfixOpName cd cs) (Curry_Prelude.d_C_snd x7 cd cs) cd cs) cd cs
     (Curry_AbstractHaskell.C_Apply x8 x9) -> d_C_showApplication x1 (Curry_AbstractHaskell.C_Apply x8 x9) cd cs
     (Curry_AbstractHaskell.C_Lambda x10 x11) -> d_C_showLambdaOrSection x1 x10 x11 cd cs
     (Curry_AbstractHaskell.C_Let x12 x13) -> Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString "let\n") (d_C_showBlock (Curry_Prelude.d_OP_plus_plus (d_C_combineMap (d_C_showLocalDecl x1) x12 (Curry_Prelude.toCurryString "\n") cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString "\n in ") (d_C_showBoxedExpr x1 x13 cd cs) cd cs) cd cs) cd cs) cd cs
     (Curry_AbstractHaskell.C_DoExpr x14) -> Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString "\n    do\n") (d_C_showBlock (d_C_combineMap (d_C_showStatement x1) x14 (Curry_Prelude.toCurryString "\n") cd cs) cd cs) cd cs
     (Curry_AbstractHaskell.C_ListComp x15 x16) -> Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString "[ ") (Curry_Prelude.d_OP_plus_plus (d_C_showBoxedExpr x1 x15 cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString " | ") (Curry_Prelude.d_OP_plus_plus (d_C_combineMap (d_C_showStatement x1) x16 (Curry_Prelude.toCurryString ", ") cd cs) (Curry_Prelude.toCurryString "]") cd cs) cd cs) cd cs) cd cs
     (Curry_AbstractHaskell.C_Case x17 x18) -> Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString "case ") (Curry_Prelude.d_OP_plus_plus (d_C_showBoxedExpr x1 x17 cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString " of\n") (d_C_showBlock (d_C_combineMap (d_C_showBranchExpr x1) x18 (Curry_Prelude.toCurryString "\n") cd cs) cd cs) cd cs) cd cs) cd cs
     (Curry_AbstractHaskell.C_Typed x19 x20) -> Curry_Prelude.OP_Cons (Curry_Prelude.C_Char '('#) (Curry_Prelude.d_OP_plus_plus (d_C_showExprOpt x1 x19 cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString " :: ") (Curry_Prelude.d_OP_plus_plus (d_C_showTypeExpr x1 Curry_Prelude.C_False x20 cd cs) (Curry_Prelude.toCurryString ")") cd cs) cd cs) cd cs)
     (Curry_AbstractHaskell.C_IfThenElse x21 x22 x23) -> Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString "if ") (Curry_Prelude.d_OP_plus_plus (d_C_showBoxedExpr x1 x21 cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString "\n    then ") (Curry_Prelude.d_OP_plus_plus (d_C_showBoxedExpr x1 x22 cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString "\n    else ") (d_C_showBoxedExpr x1 x23 cd cs) cd cs) cd cs) cd cs) cd cs) cd cs
     (Curry_AbstractHaskell.Choice_C_Expr d i l r) -> narrow d i (d_C_showExprOpt x1 l cd cs) (d_C_showExprOpt x1 r cd cs)
     (Curry_AbstractHaskell.Choices_C_Expr d i xs) -> narrows cs d i (\z -> d_C_showExprOpt x1 z cd cs) xs
     (Curry_AbstractHaskell.Guard_C_Expr d c e) -> guardCons d c ((d_C_showExprOpt x1 e cd) $! (addCs c cs))
     (Curry_AbstractHaskell.Fail_C_Expr d info) -> failCons d (traceFail "AbstractHaskellPrinter.showExprOpt" [(show x1),(show x2)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter.showExprOpt" (showCons x2))

d_C_showSymbol :: C_Options -> Curry_Prelude.OP_Tuple2 (Curry_Prelude.OP_List Curry_Prelude.C_Char) (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C_showSymbol x1 x2 cd cs = case x2 of
     (Curry_Prelude.OP_Tuple2 x3 x4) -> d_C__case_128 x3 x1 x4 (Curry_Prelude.d_C_apply (Curry_Names.d_C_isHaskellModule cd cs) x3 cd cs) cd cs
     (Curry_Prelude.Choice_OP_Tuple2 d i l r) -> narrow d i (d_C_showSymbol x1 l cd cs) (d_C_showSymbol x1 r cd cs)
     (Curry_Prelude.Choices_OP_Tuple2 d i xs) -> narrows cs d i (\z -> d_C_showSymbol x1 z cd cs) xs
     (Curry_Prelude.Guard_OP_Tuple2 d c e) -> guardCons d c ((d_C_showSymbol x1 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_Tuple2 d info) -> failCons d (traceFail "AbstractHaskellPrinter.showSymbol" [(show x1),(show x2)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter.showSymbol" (showCons x2))

d_C_showLambdaOrSection :: C_Options -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_Pattern -> Curry_AbstractHaskell.C_Expr -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C_showLambdaOrSection x1 x2 x3 cd cs = case x2 of
     (Curry_Prelude.OP_Cons x4 x5) -> d_C__case_125 x3 x2 x1 x5 x4 cd cs
     Curry_Prelude.OP_List -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_Prelude.Choice_OP_List d i l r) -> narrow d i (d_C_showLambdaOrSection x1 l x3 cd cs) (d_C_showLambdaOrSection x1 r x3 cd cs)
     (Curry_Prelude.Choices_OP_List d i xs) -> narrows cs d i (\z -> d_C_showLambdaOrSection x1 z x3 cd cs) xs
     (Curry_Prelude.Guard_OP_List d c e) -> guardCons d c ((d_C_showLambdaOrSection x1 e x3 cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_List d info) -> failCons d (traceFail "AbstractHaskellPrinter.showLambdaOrSection" [(show x1),(show x2),(show x3)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter.showLambdaOrSection" (showCons x2))

d_C_showLambda :: C_Options -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_Pattern -> Curry_AbstractHaskell.C_Expr -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C_showLambda x1 x2 x3 cd cs = Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString "\\") (Curry_Prelude.d_OP_plus_plus (d_C_combineMap (d_C_showPattern x1) x2 (Curry_Prelude.toCurryString " ") cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString " -> ") (d_C_showExprOpt x1 x3 cd cs) cd cs) cd cs) cd cs

d_C_showStatement :: C_Options -> Curry_AbstractHaskell.C_Statement -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C_showStatement x1 x2 cd cs = case x2 of
     (Curry_AbstractHaskell.C_SExpr x3) -> d_C_showExprOpt x1 x3 cd cs
     (Curry_AbstractHaskell.C_SPat x4 x5) -> Curry_Prelude.d_OP_plus_plus (d_C_showPattern x1 x4 cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString " <- ") (d_C_showExprOpt x1 x5 cd cs) cd cs) cd cs
     (Curry_AbstractHaskell.C_SLet x6) -> d_C__case_95 x1 x6 cd cs
     (Curry_AbstractHaskell.Choice_C_Statement d i l r) -> narrow d i (d_C_showStatement x1 l cd cs) (d_C_showStatement x1 r cd cs)
     (Curry_AbstractHaskell.Choices_C_Statement d i xs) -> narrows cs d i (\z -> d_C_showStatement x1 z cd cs) xs
     (Curry_AbstractHaskell.Guard_C_Statement d c e) -> guardCons d c ((d_C_showStatement x1 e cd) $! (addCs c cs))
     (Curry_AbstractHaskell.Fail_C_Statement d info) -> failCons d (traceFail "AbstractHaskellPrinter.showStatement" [(show x1),(show x2)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter.showStatement" (showCons x2))

d_C_showPattern :: C_Options -> Curry_AbstractHaskell.C_Pattern -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C_showPattern x1 x2 cd cs = case x2 of
     (Curry_AbstractHaskell.C_PVar x3) -> d_C__case_93 x3 cd cs
     (Curry_AbstractHaskell.C_PLit x6) -> d_C_showLitPattern x1 x6 cd cs
     (Curry_AbstractHaskell.C_PComb x7 x8) -> d_C__case_92 x7 x1 x8 cd cs
     (Curry_AbstractHaskell.C_PAs x13 x14) -> d_C__case_88 x14 x1 x13 cd cs
     (Curry_AbstractHaskell.C_PFuncComb x17 x18) -> d_C_showPattern x1 (Curry_AbstractHaskell.C_PComb x17 x18) cd cs
     (Curry_AbstractHaskell.Choice_C_Pattern d i l r) -> narrow d i (d_C_showPattern x1 l cd cs) (d_C_showPattern x1 r cd cs)
     (Curry_AbstractHaskell.Choices_C_Pattern d i xs) -> narrows cs d i (\z -> d_C_showPattern x1 z cd cs) xs
     (Curry_AbstractHaskell.Guard_C_Pattern d c e) -> guardCons d c ((d_C_showPattern x1 e cd) $! (addCs c cs))
     (Curry_AbstractHaskell.Fail_C_Pattern d info) -> failCons d (traceFail "AbstractHaskellPrinter.showPattern" [(show x1),(show x2)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter.showPattern" (showCons x2))

d_C_showLitPattern :: C_Options -> Curry_AbstractHaskell.C_Literal -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C_showLitPattern x1 x2 cd cs = case x2 of
     (Curry_AbstractHaskell.C_Charc x3) -> d_OP_showLitPattern_dot_wrapUnboxed_dot_270 x2 x1 (Curry_Prelude.OP_Tuple2 (Curry_Names.d_C_curryPrelude cd cs) (Curry_Prelude.toCurryString "C_Char")) cd cs
     (Curry_AbstractHaskell.C_Floatc x4) -> d_OP_showLitPattern_dot_wrapUnboxed_dot_270 x2 x1 (Curry_Prelude.OP_Tuple2 (Curry_Names.d_C_curryPrelude cd cs) (Curry_Prelude.toCurryString "C_Float")) cd cs
     (Curry_AbstractHaskell.C_Intc x5) -> d_OP_showLitPattern_dot_wrapUnboxed_dot_270 x2 x1 (Curry_Prelude.OP_Tuple2 (Curry_Names.d_C_curryPrelude cd cs) (Curry_Prelude.toCurryString "C_Int")) cd cs
     (Curry_AbstractHaskell.C_Stringc x6) -> d_C_showLiteral x2 cd cs
     (Curry_AbstractHaskell.Choice_C_Literal d i l r) -> narrow d i (d_C_showLitPattern x1 l cd cs) (d_C_showLitPattern x1 r cd cs)
     (Curry_AbstractHaskell.Choices_C_Literal d i xs) -> narrows cs d i (\z -> d_C_showLitPattern x1 z cd cs) xs
     (Curry_AbstractHaskell.Guard_C_Literal d c e) -> guardCons d c ((d_C_showLitPattern x1 e cd) $! (addCs c cs))
     (Curry_AbstractHaskell.Fail_C_Literal d info) -> failCons d (traceFail "AbstractHaskellPrinter.showLitPattern" [(show x1),(show x2)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter.showLitPattern" (showCons x2))

d_OP_showLitPattern_dot_wrapUnboxed_dot_270 :: Curry_AbstractHaskell.C_Literal -> C_Options -> Curry_Prelude.OP_Tuple2 (Curry_Prelude.OP_List Curry_Prelude.C_Char) (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_OP_showLitPattern_dot_wrapUnboxed_dot_270 x1 x2 x3 cd cs = Curry_Prelude.OP_Cons (Curry_Prelude.C_Char '('#) (Curry_Prelude.d_OP_plus_plus (d_C_showSymbol x2 x3 cd cs) (Curry_Prelude.OP_Cons (Curry_Prelude.C_Char ' '#) (Curry_Prelude.d_OP_plus_plus (d_C_showLiteral x1 cd cs) (Curry_Prelude.toCurryString "#)") cd cs)) cd cs)

d_C_showPreludeCons :: C_Options -> Curry_AbstractHaskell.C_Pattern -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C_showPreludeCons x1 x2 cd cs = let
     x3 = d_OP_showPreludeCons_dot_uscore_hash_selFP5_hash_qname x2 cd cs
     x4 = d_OP_showPreludeCons_dot_uscore_hash_selFP6_hash_name x2 cd cs
     x5 = d_OP_showPreludeCons_dot_uscore_hash_selFP7_hash_pattlist x2 cd cs
      in (d_C__case_87 x4 x5 x1 x3 x2 (Curry_Prelude.d_OP_eq_eq x4 (Curry_Prelude.toCurryString ":") cd cs) cd cs)

d_OP_showPreludeCons_dot_uscore_hash_selFP5_hash_qname :: Curry_AbstractHaskell.C_Pattern -> Cover -> ConstStore -> Curry_Prelude.OP_Tuple2 (Curry_Prelude.OP_List Curry_Prelude.C_Char) (Curry_Prelude.OP_List Curry_Prelude.C_Char)
d_OP_showPreludeCons_dot_uscore_hash_selFP5_hash_qname x1 cd cs = case x1 of
     (Curry_AbstractHaskell.C_PComb x2 x3) -> d_C__case_84 x2 cd cs
     (Curry_AbstractHaskell.Choice_C_Pattern d i l r) -> narrow d i (d_OP_showPreludeCons_dot_uscore_hash_selFP5_hash_qname l cd cs) (d_OP_showPreludeCons_dot_uscore_hash_selFP5_hash_qname r cd cs)
     (Curry_AbstractHaskell.Choices_C_Pattern d i xs) -> narrows cs d i (\z -> d_OP_showPreludeCons_dot_uscore_hash_selFP5_hash_qname z cd cs) xs
     (Curry_AbstractHaskell.Guard_C_Pattern d c e) -> guardCons d c ((d_OP_showPreludeCons_dot_uscore_hash_selFP5_hash_qname e cd) $! (addCs c cs))
     (Curry_AbstractHaskell.Fail_C_Pattern d info) -> failCons d (traceFail "AbstractHaskellPrinter.showPreludeCons._#selFP5#qname" [(show x1)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter.showPreludeCons._#selFP5#qname" (showCons x1))

d_OP_showPreludeCons_dot_uscore_hash_selFP6_hash_name :: Curry_AbstractHaskell.C_Pattern -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_OP_showPreludeCons_dot_uscore_hash_selFP6_hash_name x1 cd cs = case x1 of
     (Curry_AbstractHaskell.C_PComb x2 x3) -> d_C__case_83 x2 cd cs
     (Curry_AbstractHaskell.Choice_C_Pattern d i l r) -> narrow d i (d_OP_showPreludeCons_dot_uscore_hash_selFP6_hash_name l cd cs) (d_OP_showPreludeCons_dot_uscore_hash_selFP6_hash_name r cd cs)
     (Curry_AbstractHaskell.Choices_C_Pattern d i xs) -> narrows cs d i (\z -> d_OP_showPreludeCons_dot_uscore_hash_selFP6_hash_name z cd cs) xs
     (Curry_AbstractHaskell.Guard_C_Pattern d c e) -> guardCons d c ((d_OP_showPreludeCons_dot_uscore_hash_selFP6_hash_name e cd) $! (addCs c cs))
     (Curry_AbstractHaskell.Fail_C_Pattern d info) -> failCons d (traceFail "AbstractHaskellPrinter.showPreludeCons._#selFP6#name" [(show x1)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter.showPreludeCons._#selFP6#name" (showCons x1))

d_OP_showPreludeCons_dot_uscore_hash_selFP7_hash_pattlist :: Curry_AbstractHaskell.C_Pattern -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_Pattern
d_OP_showPreludeCons_dot_uscore_hash_selFP7_hash_pattlist x1 cd cs = case x1 of
     (Curry_AbstractHaskell.C_PComb x2 x3) -> d_C__case_82 x3 x2 cd cs
     (Curry_AbstractHaskell.Choice_C_Pattern d i l r) -> narrow d i (d_OP_showPreludeCons_dot_uscore_hash_selFP7_hash_pattlist l cd cs) (d_OP_showPreludeCons_dot_uscore_hash_selFP7_hash_pattlist r cd cs)
     (Curry_AbstractHaskell.Choices_C_Pattern d i xs) -> narrows cs d i (\z -> d_OP_showPreludeCons_dot_uscore_hash_selFP7_hash_pattlist z cd cs) xs
     (Curry_AbstractHaskell.Guard_C_Pattern d c e) -> guardCons d c ((d_OP_showPreludeCons_dot_uscore_hash_selFP7_hash_pattlist e cd) $! (addCs c cs))
     (Curry_AbstractHaskell.Fail_C_Pattern d info) -> failCons d (traceFail "AbstractHaskellPrinter.showPreludeCons._#selFP7#pattlist" [(show x1)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter.showPreludeCons._#selFP7#pattlist" (showCons x1))

d_C_showPatternList :: C_Options -> Curry_AbstractHaskell.C_Pattern -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C_showPatternList x1 x2 cd cs = d_C__case_81 x2 x1 (d_C_isClosedStringPattern x2 cd cs) cd cs

d_C_showPatListElems :: C_Options -> Curry_AbstractHaskell.C_Pattern -> Cover -> ConstStore -> Curry_Prelude.OP_List (Curry_Prelude.OP_List Curry_Prelude.C_Char)
d_C_showPatListElems x1 x2 cd cs = case x2 of
     (Curry_AbstractHaskell.C_PComb x3 x4) -> d_C__case_77 x4 x1 x3 cd cs
     (Curry_AbstractHaskell.C_PVar x25) -> Curry_Prelude.OP_Cons (d_C_showPattern x1 (Curry_AbstractHaskell.C_PVar x25) cd cs) Curry_Prelude.OP_List
     (Curry_AbstractHaskell.C_PAs x26 x27) -> Curry_Prelude.OP_Cons (d_C_showPattern x1 (Curry_AbstractHaskell.C_PAs x26 x27) cd cs) Curry_Prelude.OP_List
     (Curry_AbstractHaskell.C_PLit x28) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showPatListElems") cd cs
     (Curry_AbstractHaskell.C_PFuncComb x29 x30) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showPatListElems") cd cs
     (Curry_AbstractHaskell.Choice_C_Pattern d i l r) -> narrow d i (d_C_showPatListElems x1 l cd cs) (d_C_showPatListElems x1 r cd cs)
     (Curry_AbstractHaskell.Choices_C_Pattern d i xs) -> narrows cs d i (\z -> d_C_showPatListElems x1 z cd cs) xs
     (Curry_AbstractHaskell.Guard_C_Pattern d c e) -> guardCons d c ((d_C_showPatListElems x1 e cd) $! (addCs c cs))
     (Curry_AbstractHaskell.Fail_C_Pattern d info) -> failCons d (traceFail "AbstractHaskellPrinter.showPatListElems" [(show x1),(show x2)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter.showPatListElems" (showCons x2))

d_C_showAsPatternList :: C_Options -> Curry_AbstractHaskell.C_Pattern -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C_showAsPatternList x1 x2 cd cs = case x2 of
     (Curry_AbstractHaskell.C_PAs x3 x4) -> d_C__case_65 x4 x1 x3 cd cs
     (Curry_AbstractHaskell.C_PVar x7) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showAsPatternList") cd cs
     (Curry_AbstractHaskell.C_PLit x8) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showAsPatternList") cd cs
     (Curry_AbstractHaskell.C_PComb x9 x10) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showAsPatternList") cd cs
     (Curry_AbstractHaskell.C_PFuncComb x11 x12) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showAsPatternList") cd cs
     (Curry_AbstractHaskell.Choice_C_Pattern d i l r) -> narrow d i (d_C_showAsPatternList x1 l cd cs) (d_C_showAsPatternList x1 r cd cs)
     (Curry_AbstractHaskell.Choices_C_Pattern d i xs) -> narrows cs d i (\z -> d_C_showAsPatternList x1 z cd cs) xs
     (Curry_AbstractHaskell.Guard_C_Pattern d c e) -> guardCons d c ((d_C_showAsPatternList x1 e cd) $! (addCs c cs))
     (Curry_AbstractHaskell.Fail_C_Pattern d info) -> failCons d (traceFail "AbstractHaskellPrinter.showAsPatternList" [(show x1),(show x2)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter.showAsPatternList" (showCons x2))

d_C_showBranchExpr :: C_Options -> Curry_AbstractHaskell.C_BranchExpr -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C_showBranchExpr x1 x2 cd cs = case x2 of
     (Curry_AbstractHaskell.C_Branch x3 x4) -> Curry_Prelude.d_OP_plus_plus (d_C_showPattern x1 x3 cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString " -> ") (d_C_showExprOpt x1 x4 cd cs) cd cs) cd cs
     (Curry_AbstractHaskell.Choice_C_BranchExpr d i l r) -> narrow d i (d_C_showBranchExpr x1 l cd cs) (d_C_showBranchExpr x1 r cd cs)
     (Curry_AbstractHaskell.Choices_C_BranchExpr d i xs) -> narrows cs d i (\z -> d_C_showBranchExpr x1 z cd cs) xs
     (Curry_AbstractHaskell.Guard_C_BranchExpr d c e) -> guardCons d c ((d_C_showBranchExpr x1 e cd) $! (addCs c cs))
     (Curry_AbstractHaskell.Fail_C_BranchExpr d info) -> failCons d (traceFail "AbstractHaskellPrinter.showBranchExpr" [(show x1),(show x2)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter.showBranchExpr" (showCons x2))

d_C_showLiteral :: Curry_AbstractHaskell.C_Literal -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C_showLiteral x1 cd cs = case x1 of
     (Curry_AbstractHaskell.C_Intc x2) -> Curry_Prelude.d_C_show x2 cd cs
     (Curry_AbstractHaskell.C_Floatc x3) -> Curry_Prelude.d_C_show x3 cd cs
     (Curry_AbstractHaskell.C_Charc x4) -> Curry_Prelude.d_C_show x4 cd cs
     (Curry_AbstractHaskell.C_Stringc x5) -> Curry_Prelude.d_C_show x5 cd cs
     (Curry_AbstractHaskell.Choice_C_Literal d i l r) -> narrow d i (d_C_showLiteral l cd cs) (d_C_showLiteral r cd cs)
     (Curry_AbstractHaskell.Choices_C_Literal d i xs) -> narrows cs d i (\z -> d_C_showLiteral z cd cs) xs
     (Curry_AbstractHaskell.Guard_C_Literal d c e) -> guardCons d c ((d_C_showLiteral e cd) $! (addCs c cs))
     (Curry_AbstractHaskell.Fail_C_Literal d info) -> failCons d (traceFail "AbstractHaskellPrinter.showLiteral" [(show x1)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter.showLiteral" (showCons x1))

d_C_showBlock :: Curry_Prelude.OP_List Curry_Prelude.C_Char -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C_showBlock x1 cd cs = d_C_combineMap Curry_Prelude.d_C_id (Curry_Prelude.d_C_map (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString "     ")) (Curry_Prelude.d_C_filter (Curry_Prelude.d_OP_slash_eq Curry_Prelude.OP_List) (Curry_Prelude.d_C_lines x1 cd cs) cd cs) cd cs) (Curry_Prelude.toCurryString "\n") cd cs

d_C_showTypeCons :: C_Options -> Curry_Prelude.OP_Tuple2 (Curry_Prelude.OP_List Curry_Prelude.C_Char) (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_TypeExpr -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C_showTypeCons x1 x2 x3 cd cs = case x3 of
     Curry_Prelude.OP_List -> d_C_showSymbol x1 x2 cd cs
     (Curry_Prelude.OP_Cons x4 x5) -> d_C__case_64 x5 x4 x1 x2 cd cs
     (Curry_Prelude.Choice_OP_List d i l r) -> narrow d i (d_C_showTypeCons x1 x2 l cd cs) (d_C_showTypeCons x1 x2 r cd cs)
     (Curry_Prelude.Choices_OP_List d i xs) -> narrows cs d i (\z -> d_C_showTypeCons x1 x2 z cd cs) xs
     (Curry_Prelude.Guard_OP_List d c e) -> guardCons d c ((d_C_showTypeCons x1 x2 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_List d info) -> failCons d (traceFail "AbstractHaskellPrinter.showTypeCons" [(show x1),(show x2),(show x3)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter.showTypeCons" (showCons x3))

d_C_showPreludeTypeCons :: C_Options -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_TypeExpr -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C_showPreludeTypeCons x1 x2 x3 cd cs = d_C__case_61 x3 x2 x1 (Curry_Prelude.d_OP_amp_amp (Curry_Prelude.d_OP_eq_eq x2 (Curry_Prelude.toCurryString "[]") cd cs) (Curry_Prelude.d_OP_eq_eq (Curry_Prelude.d_C_head x3 cd cs) (Curry_AbstractHaskell.C_TCons (Curry_Prelude.OP_Tuple2 (Curry_Names.d_C_prelude cd cs) (Curry_Prelude.toCurryString "Char")) Curry_Prelude.OP_List) cd cs) cd cs) cd cs

d_C_showApplication :: C_Options -> Curry_AbstractHaskell.C_Expr -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C_showApplication x1 x2 cd cs = d_C__case_57 x2 x1 (d_C_applicationHead x2 cd cs) cd cs

d_C_applicationHead :: Curry_AbstractHaskell.C_Expr -> Cover -> ConstStore -> Curry_AbstractHaskell.C_Expr
d_C_applicationHead x1 cd cs = case x1 of
     (Curry_AbstractHaskell.C_Apply x2 x3) -> d_C_applicationHead x2 cd cs
     (Curry_AbstractHaskell.C_Var x4) -> x1
     (Curry_AbstractHaskell.C_Lit x5) -> x1
     (Curry_AbstractHaskell.C_Symbol x6) -> x1
     (Curry_AbstractHaskell.C_Lambda x7 x8) -> x1
     (Curry_AbstractHaskell.C_Let x9 x10) -> x1
     (Curry_AbstractHaskell.C_DoExpr x11) -> x1
     (Curry_AbstractHaskell.C_ListComp x12 x13) -> x1
     (Curry_AbstractHaskell.C_Case x14 x15) -> x1
     (Curry_AbstractHaskell.C_Typed x16 x17) -> x1
     (Curry_AbstractHaskell.C_IfThenElse x18 x19 x20) -> x1
     (Curry_AbstractHaskell.Choice_C_Expr d i l r) -> narrow d i (d_C_applicationHead l cd cs) (d_C_applicationHead r cd cs)
     (Curry_AbstractHaskell.Choices_C_Expr d i xs) -> narrows cs d i (\z -> d_C_applicationHead z cd cs) xs
     (Curry_AbstractHaskell.Guard_C_Expr d c e) -> guardCons d c ((d_C_applicationHead e cd) $! (addCs c cs))
     (Curry_AbstractHaskell.Fail_C_Expr d info) -> failCons d (traceFail "AbstractHaskellPrinter.applicationHead" [(show x1)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter.applicationHead" (showCons x1))

d_C_showSymbolApplication :: C_Options -> Curry_Prelude.OP_Tuple2 (Curry_Prelude.OP_List Curry_Prelude.C_Char) (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Curry_AbstractHaskell.C_Expr -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C_showSymbolApplication x1 x2 x3 cd cs = case x2 of
     (Curry_Prelude.OP_Tuple2 x4 x5) -> d_C__case_56 x5 x4 x3 x1 (Curry_Prelude.d_OP_amp_amp (Curry_Prelude.d_OP_eq_eq x4 (Curry_Names.d_C_prelude cd cs) cd cs) (Curry_Prelude.d_OP_eq_eq x5 (Curry_Prelude.toCurryString ":") cd cs) cd cs) cd cs
     (Curry_Prelude.Choice_OP_Tuple2 d i l r) -> narrow d i (d_C_showSymbolApplication x1 l x3 cd cs) (d_C_showSymbolApplication x1 r x3 cd cs)
     (Curry_Prelude.Choices_OP_Tuple2 d i xs) -> narrows cs d i (\z -> d_C_showSymbolApplication x1 z x3 cd cs) xs
     (Curry_Prelude.Guard_OP_Tuple2 d c e) -> guardCons d c ((d_C_showSymbolApplication x1 e x3 cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_Tuple2 d info) -> failCons d (traceFail "AbstractHaskellPrinter.showSymbolApplication" [(show x1),(show x2),(show x3)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter.showSymbolApplication" (showCons x2))

d_C_showListApplication :: C_Options -> Curry_AbstractHaskell.C_Expr -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C_showListApplication x1 x2 cd cs = d_C__case_51 x2 x1 (d_C_isClosedList x2 cd cs) cd cs

d_C_showConsListApplication :: C_Options -> Curry_AbstractHaskell.C_Expr -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C_showConsListApplication x1 x2 cd cs = case x2 of
     (Curry_AbstractHaskell.C_Apply x3 x4) -> d_C__case_49 x4 x1 x3 cd cs
     (Curry_AbstractHaskell.C_Var x43) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showConsListApplication") cd cs
     (Curry_AbstractHaskell.C_Lit x44) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showConsListApplication") cd cs
     (Curry_AbstractHaskell.C_Symbol x45) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showConsListApplication") cd cs
     (Curry_AbstractHaskell.C_Lambda x46 x47) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showConsListApplication") cd cs
     (Curry_AbstractHaskell.C_Let x48 x49) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showConsListApplication") cd cs
     (Curry_AbstractHaskell.C_DoExpr x50) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showConsListApplication") cd cs
     (Curry_AbstractHaskell.C_ListComp x51 x52) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showConsListApplication") cd cs
     (Curry_AbstractHaskell.C_Case x53 x54) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showConsListApplication") cd cs
     (Curry_AbstractHaskell.C_Typed x55 x56) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showConsListApplication") cd cs
     (Curry_AbstractHaskell.C_IfThenElse x57 x58 x59) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showConsListApplication") cd cs
     (Curry_AbstractHaskell.Choice_C_Expr d i l r) -> narrow d i (d_C_showConsListApplication x1 l cd cs) (d_C_showConsListApplication x1 r cd cs)
     (Curry_AbstractHaskell.Choices_C_Expr d i xs) -> narrows cs d i (\z -> d_C_showConsListApplication x1 z cd cs) xs
     (Curry_AbstractHaskell.Guard_C_Expr d c e) -> guardCons d c ((d_C_showConsListApplication x1 e cd) $! (addCs c cs))
     (Curry_AbstractHaskell.Fail_C_Expr d info) -> failCons d (traceFail "AbstractHaskellPrinter.showConsListApplication" [(show x1),(show x2)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter.showConsListApplication" (showCons x2))

d_C_showSimpleListApplication :: C_Options -> Curry_AbstractHaskell.C_Expr -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C_showSimpleListApplication x1 x2 cd cs = case x2 of
     (Curry_AbstractHaskell.C_Apply x3 x4) -> d_C__case_47 x4 x1 x3 cd cs
     (Curry_AbstractHaskell.C_Var x45) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showSimpleListApplication") cd cs
     (Curry_AbstractHaskell.C_Lit x46) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showSimpleListApplication") cd cs
     (Curry_AbstractHaskell.C_Symbol x47) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showSimpleListApplication") cd cs
     (Curry_AbstractHaskell.C_Lambda x48 x49) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showSimpleListApplication") cd cs
     (Curry_AbstractHaskell.C_Let x50 x51) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showSimpleListApplication") cd cs
     (Curry_AbstractHaskell.C_DoExpr x52) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showSimpleListApplication") cd cs
     (Curry_AbstractHaskell.C_ListComp x53 x54) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showSimpleListApplication") cd cs
     (Curry_AbstractHaskell.C_Case x55 x56) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showSimpleListApplication") cd cs
     (Curry_AbstractHaskell.C_Typed x57 x58) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showSimpleListApplication") cd cs
     (Curry_AbstractHaskell.C_IfThenElse x59 x60 x61) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showSimpleListApplication") cd cs
     (Curry_AbstractHaskell.Choice_C_Expr d i l r) -> narrow d i (d_C_showSimpleListApplication x1 l cd cs) (d_C_showSimpleListApplication x1 r cd cs)
     (Curry_AbstractHaskell.Choices_C_Expr d i xs) -> narrows cs d i (\z -> d_C_showSimpleListApplication x1 z cd cs) xs
     (Curry_AbstractHaskell.Guard_C_Expr d c e) -> guardCons d c ((d_C_showSimpleListApplication x1 e cd) $! (addCs c cs))
     (Curry_AbstractHaskell.Fail_C_Expr d info) -> failCons d (traceFail "AbstractHaskellPrinter.showSimpleListApplication" [(show x1),(show x2)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter.showSimpleListApplication" (showCons x2))

d_C_showInfixApplication :: C_Options -> Curry_Prelude.OP_Tuple2 (Curry_Prelude.OP_List Curry_Prelude.C_Char) (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Curry_AbstractHaskell.C_Expr -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C_showInfixApplication x1 x2 x3 cd cs = case x3 of
     (Curry_AbstractHaskell.C_Apply x4 x5) -> d_C__case_44 x5 x1 x2 x4 cd cs
     (Curry_AbstractHaskell.C_Var x44) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showInfixApplication") cd cs
     (Curry_AbstractHaskell.C_Lit x45) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showInfixApplication") cd cs
     (Curry_AbstractHaskell.C_Symbol x46) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showInfixApplication") cd cs
     (Curry_AbstractHaskell.C_Lambda x47 x48) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showInfixApplication") cd cs
     (Curry_AbstractHaskell.C_Let x49 x50) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showInfixApplication") cd cs
     (Curry_AbstractHaskell.C_DoExpr x51) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showInfixApplication") cd cs
     (Curry_AbstractHaskell.C_ListComp x52 x53) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showInfixApplication") cd cs
     (Curry_AbstractHaskell.C_Case x54 x55) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showInfixApplication") cd cs
     (Curry_AbstractHaskell.C_Typed x56 x57) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showInfixApplication") cd cs
     (Curry_AbstractHaskell.C_IfThenElse x58 x59 x60) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showInfixApplication") cd cs
     (Curry_AbstractHaskell.Choice_C_Expr d i l r) -> narrow d i (d_C_showInfixApplication x1 x2 l cd cs) (d_C_showInfixApplication x1 x2 r cd cs)
     (Curry_AbstractHaskell.Choices_C_Expr d i xs) -> narrows cs d i (\z -> d_C_showInfixApplication x1 x2 z cd cs) xs
     (Curry_AbstractHaskell.Guard_C_Expr d c e) -> guardCons d c ((d_C_showInfixApplication x1 x2 e cd) $! (addCs c cs))
     (Curry_AbstractHaskell.Fail_C_Expr d info) -> failCons d (traceFail "AbstractHaskellPrinter.showInfixApplication" [(show x1),(show x2),(show x3)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter.showInfixApplication" (showCons x3))

d_C_showITEApplication :: C_Options -> Curry_AbstractHaskell.C_Expr -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C_showITEApplication x1 x2 cd cs = case x2 of
     (Curry_AbstractHaskell.C_Apply x3 x4) -> d_C__case_42 x4 x1 x3 cd cs
     (Curry_AbstractHaskell.C_Var x62) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showITEApplication") cd cs
     (Curry_AbstractHaskell.C_Lit x63) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showITEApplication") cd cs
     (Curry_AbstractHaskell.C_Symbol x64) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showITEApplication") cd cs
     (Curry_AbstractHaskell.C_Lambda x65 x66) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showITEApplication") cd cs
     (Curry_AbstractHaskell.C_Let x67 x68) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showITEApplication") cd cs
     (Curry_AbstractHaskell.C_DoExpr x69) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showITEApplication") cd cs
     (Curry_AbstractHaskell.C_ListComp x70 x71) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showITEApplication") cd cs
     (Curry_AbstractHaskell.C_Case x72 x73) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showITEApplication") cd cs
     (Curry_AbstractHaskell.C_Typed x74 x75) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showITEApplication") cd cs
     (Curry_AbstractHaskell.C_IfThenElse x76 x77 x78) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showITEApplication") cd cs
     (Curry_AbstractHaskell.Choice_C_Expr d i l r) -> narrow d i (d_C_showITEApplication x1 l cd cs) (d_C_showITEApplication x1 r cd cs)
     (Curry_AbstractHaskell.Choices_C_Expr d i xs) -> narrows cs d i (\z -> d_C_showITEApplication x1 z cd cs) xs
     (Curry_AbstractHaskell.Guard_C_Expr d c e) -> guardCons d c ((d_C_showITEApplication x1 e cd) $! (addCs c cs))
     (Curry_AbstractHaskell.Fail_C_Expr d info) -> failCons d (traceFail "AbstractHaskellPrinter.showITEApplication" [(show x1),(show x2)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter.showITEApplication" (showCons x2))

d_C_showTupleApplication :: C_Options -> Curry_AbstractHaskell.C_Expr -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C_showTupleApplication x1 x2 cd cs = Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString "(") (Curry_Prelude.d_OP_plus_plus (d_OP_showTupleApplication_dot_p_uscore_showTuple_dot_416 x1 x2 cd cs) (Curry_Prelude.toCurryString ")") cd cs) cd cs

d_OP_showTupleApplication_dot_p_uscore_showTuple_dot_416 :: C_Options -> Curry_AbstractHaskell.C_Expr -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_OP_showTupleApplication_dot_p_uscore_showTuple_dot_416 x1 x2 cd cs = case x2 of
     (Curry_AbstractHaskell.C_Apply x3 x4) -> d_C__case_39 x4 x1 x3 cd cs
     (Curry_AbstractHaskell.C_Var x24) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showTupleApplication") cd cs
     (Curry_AbstractHaskell.C_Lit x25) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showTupleApplication") cd cs
     (Curry_AbstractHaskell.C_Symbol x26) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showTupleApplication") cd cs
     (Curry_AbstractHaskell.C_Lambda x27 x28) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showTupleApplication") cd cs
     (Curry_AbstractHaskell.C_Let x29 x30) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showTupleApplication") cd cs
     (Curry_AbstractHaskell.C_DoExpr x31) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showTupleApplication") cd cs
     (Curry_AbstractHaskell.C_ListComp x32 x33) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showTupleApplication") cd cs
     (Curry_AbstractHaskell.C_Case x34 x35) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showTupleApplication") cd cs
     (Curry_AbstractHaskell.C_Typed x36 x37) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showTupleApplication") cd cs
     (Curry_AbstractHaskell.C_IfThenElse x38 x39 x40) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showTupleApplication") cd cs
     (Curry_AbstractHaskell.Choice_C_Expr d i l r) -> narrow d i (d_OP_showTupleApplication_dot_p_uscore_showTuple_dot_416 x1 l cd cs) (d_OP_showTupleApplication_dot_p_uscore_showTuple_dot_416 x1 r cd cs)
     (Curry_AbstractHaskell.Choices_C_Expr d i xs) -> narrows cs d i (\z -> d_OP_showTupleApplication_dot_p_uscore_showTuple_dot_416 x1 z cd cs) xs
     (Curry_AbstractHaskell.Guard_C_Expr d c e) -> guardCons d c ((d_OP_showTupleApplication_dot_p_uscore_showTuple_dot_416 x1 e cd) $! (addCs c cs))
     (Curry_AbstractHaskell.Fail_C_Expr d info) -> failCons d (traceFail "AbstractHaskellPrinter.showTupleApplication.p_showTuple.416" [(show x1),(show x2)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter.showTupleApplication.p_showTuple.416" (showCons x2))

d_C_showSimpleApplication :: C_Options -> Curry_AbstractHaskell.C_Expr -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C_showSimpleApplication x1 x2 cd cs = case x2 of
     (Curry_AbstractHaskell.C_Apply x3 x4) -> Curry_Prelude.d_OP_plus_plus (d_C_showSimpleApplication x1 x3 cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString " ") (d_C_showBoxedExpr x1 x4 cd cs) cd cs) cd cs
     (Curry_AbstractHaskell.C_Var x5) -> d_C_showBoxedExpr x1 x2 cd cs
     (Curry_AbstractHaskell.C_Lit x6) -> d_C_showBoxedExpr x1 x2 cd cs
     (Curry_AbstractHaskell.C_Symbol x7) -> d_C_showBoxedExpr x1 x2 cd cs
     (Curry_AbstractHaskell.C_Lambda x8 x9) -> d_C_showBoxedExpr x1 x2 cd cs
     (Curry_AbstractHaskell.C_Let x10 x11) -> d_C_showBoxedExpr x1 x2 cd cs
     (Curry_AbstractHaskell.C_DoExpr x12) -> d_C_showBoxedExpr x1 x2 cd cs
     (Curry_AbstractHaskell.C_ListComp x13 x14) -> d_C_showBoxedExpr x1 x2 cd cs
     (Curry_AbstractHaskell.C_Case x15 x16) -> d_C_showBoxedExpr x1 x2 cd cs
     (Curry_AbstractHaskell.C_Typed x17 x18) -> d_C_showBoxedExpr x1 x2 cd cs
     (Curry_AbstractHaskell.C_IfThenElse x19 x20 x21) -> d_C_showBoxedExpr x1 x2 cd cs
     (Curry_AbstractHaskell.Choice_C_Expr d i l r) -> narrow d i (d_C_showSimpleApplication x1 l cd cs) (d_C_showSimpleApplication x1 r cd cs)
     (Curry_AbstractHaskell.Choices_C_Expr d i xs) -> narrows cs d i (\z -> d_C_showSimpleApplication x1 z cd cs) xs
     (Curry_AbstractHaskell.Guard_C_Expr d c e) -> guardCons d c ((d_C_showSimpleApplication x1 e cd) $! (addCs c cs))
     (Curry_AbstractHaskell.Fail_C_Expr d info) -> failCons d (traceFail "AbstractHaskellPrinter.showSimpleApplication" [(show x1),(show x2)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter.showSimpleApplication" (showCons x2))

d_C_showBoxedExpr :: C_Options -> Curry_AbstractHaskell.C_Expr -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C_showBoxedExpr x1 x2 cd cs = d_C__case_38 x2 x1 (d_C_isSimpleExpr x2 cd cs) cd cs

d_C_prefixMap :: Curry_Prelude.Curry t0 => (t0 -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Curry_Prelude.OP_List t0 -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C_prefixMap x1 x2 x3 cd cs = Curry_Prelude.d_C_apply (Curry_Prelude.d_C_concatMap (Curry_Prelude.d_OP_plus_plus x3) cd cs) (Curry_Prelude.d_C_map x1 x2 cd cs) cd cs

nd_C_prefixMap :: Curry_Prelude.Curry t0 => Func t0 (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Curry_Prelude.OP_List t0 -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> IDSupply -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
nd_C_prefixMap x1 x2 x3 s cd cs = let
     s3 = s
      in (seq s3 (let
          s2 = leftSupply s3
          s4 = rightSupply s3
          s0 = leftSupply s4
          s1 = rightSupply s4
           in (seq s2 (seq s4 (seq s0 (seq s1 (Curry_Prelude.nd_C_apply (Curry_Prelude.nd_C_concatMap (wrapDX id (Curry_Prelude.d_OP_plus_plus x3)) s0 cd cs) (Curry_Prelude.nd_C_map x1 x2 s1 cd cs) s2 cd cs)))))))

d_C_prefixInter :: Curry_Prelude.Curry t0 => (t0 -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Curry_Prelude.OP_List t0 -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C_prefixInter x1 x2 x3 cd cs = Curry_List.d_C_intercalate x3 (Curry_Prelude.d_C_map x1 x2 cd cs) cd cs

nd_C_prefixInter :: Curry_Prelude.Curry t0 => Func t0 (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Curry_Prelude.OP_List t0 -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> IDSupply -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
nd_C_prefixInter x1 x2 x3 s cd cs = let
     s0 = s
      in (seq s0 (Curry_List.d_C_intercalate x3 (Curry_Prelude.nd_C_map x1 x2 s0 cd cs) cd cs))

d_C_combineMap :: Curry_Prelude.Curry t0 => (t0 -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Curry_Prelude.OP_List t0 -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C_combineMap x1 x2 x3 cd cs = case x2 of
     Curry_Prelude.OP_List -> Curry_Prelude.OP_List
     (Curry_Prelude.OP_Cons x4 x5) -> Curry_Prelude.d_OP_plus_plus (Curry_Prelude.d_C_apply x1 x4 cd cs) (d_C_prefixMap x1 x5 x3 cd cs) cd cs
     (Curry_Prelude.Choice_OP_List d i l r) -> narrow d i (d_C_combineMap x1 l x3 cd cs) (d_C_combineMap x1 r x3 cd cs)
     (Curry_Prelude.Choices_OP_List d i xs) -> narrows cs d i (\z -> d_C_combineMap x1 z x3 cd cs) xs
     (Curry_Prelude.Guard_OP_List d c e) -> guardCons d c ((d_C_combineMap x1 e x3 cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_List d info) -> failCons d (traceFail "AbstractHaskellPrinter.combineMap" [(show x1),(show x2),(show x3)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter.combineMap" (showCons x2))

nd_C_combineMap :: Curry_Prelude.Curry t0 => Func t0 (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Curry_Prelude.OP_List t0 -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> IDSupply -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
nd_C_combineMap x1 x2 x3 s cd cs = case x2 of
     Curry_Prelude.OP_List -> Curry_Prelude.OP_List
     (Curry_Prelude.OP_Cons x4 x5) -> let
          s2 = s
           in (seq s2 (let
               s0 = leftSupply s2
               s1 = rightSupply s2
                in (seq s0 (seq s1 (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.nd_C_apply x1 x4 s0 cd cs) (nd_C_prefixMap x1 x5 x3 s1 cd cs) cd cs)))))
     (Curry_Prelude.Choice_OP_List d i l r) -> narrow d i (nd_C_combineMap x1 l x3 s cd cs) (nd_C_combineMap x1 r x3 s cd cs)
     (Curry_Prelude.Choices_OP_List d i xs) -> narrows cs d i (\z -> nd_C_combineMap x1 z x3 s cd cs) xs
     (Curry_Prelude.Guard_OP_List d c e) -> guardCons d c ((nd_C_combineMap x1 e x3 s cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_List d info) -> failCons d (traceFail "AbstractHaskellPrinter.combineMap" [(show x1),(show x2),(show x3)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter.combineMap" (showCons x2))

d_C_dropTags :: Curry_Prelude.OP_List Curry_Prelude.C_Char -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C_dropTags x1 cd cs = case x1 of
     Curry_Prelude.OP_List -> Curry_Prelude.OP_List
     (Curry_Prelude.OP_Cons x2 x3) -> let
          x4 = x2
           in (d_C__case_36 x4 x3 (Curry_Prelude.d_OP_eq_eq x4 (Curry_Prelude.C_Char '"'#) cd cs) cd cs)
     (Curry_Prelude.Choice_OP_List d i l r) -> narrow d i (d_C_dropTags l cd cs) (d_C_dropTags r cd cs)
     (Curry_Prelude.Choices_OP_List d i xs) -> narrows cs d i (\z -> d_C_dropTags z cd cs) xs
     (Curry_Prelude.Guard_OP_List d c e) -> guardCons d c ((d_C_dropTags e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_List d info) -> failCons d (traceFail "AbstractHaskellPrinter.dropTags" [(show x1)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter.dropTags" (showCons x1))

d_C_isClosedPatternList :: Curry_AbstractHaskell.C_Pattern -> Cover -> ConstStore -> Curry_Prelude.C_Bool
d_C_isClosedPatternList x1 cd cs = case x1 of
     (Curry_AbstractHaskell.C_PComb x2 x3) -> d_C__case_34 x3 x2 cd cs
     (Curry_AbstractHaskell.C_PVar x24) -> Curry_Prelude.C_False
     (Curry_AbstractHaskell.C_PAs x25 x26) -> d_C_isClosedPatternList x26 cd cs
     (Curry_AbstractHaskell.C_PLit x27) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.isClosedPatternList") cd cs
     (Curry_AbstractHaskell.C_PFuncComb x28 x29) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.isClosedPatternList") cd cs
     (Curry_AbstractHaskell.Choice_C_Pattern d i l r) -> narrow d i (d_C_isClosedPatternList l cd cs) (d_C_isClosedPatternList r cd cs)
     (Curry_AbstractHaskell.Choices_C_Pattern d i xs) -> narrows cs d i (\z -> d_C_isClosedPatternList z cd cs) xs
     (Curry_AbstractHaskell.Guard_C_Pattern d c e) -> guardCons d c ((d_C_isClosedPatternList e cd) $! (addCs c cs))
     (Curry_AbstractHaskell.Fail_C_Pattern d info) -> failCons d (traceFail "AbstractHaskellPrinter.isClosedPatternList" [(show x1)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter.isClosedPatternList" (showCons x1))

d_C_isClosedStringPattern :: Curry_AbstractHaskell.C_Pattern -> Cover -> ConstStore -> Curry_Prelude.C_Bool
d_C_isClosedStringPattern x1 cd cs = case x1 of
     (Curry_AbstractHaskell.C_PComb x2 x3) -> d_C__case_22 x3 x2 cd cs
     (Curry_AbstractHaskell.C_PVar x24) -> Curry_Prelude.C_False
     (Curry_AbstractHaskell.C_PAs x25 x26) -> Curry_Prelude.C_False
     (Curry_AbstractHaskell.C_PLit x27) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.isClosedStringPattern") cd cs
     (Curry_AbstractHaskell.C_PFuncComb x28 x29) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.isClosedStringPattern") cd cs
     (Curry_AbstractHaskell.Choice_C_Pattern d i l r) -> narrow d i (d_C_isClosedStringPattern l cd cs) (d_C_isClosedStringPattern r cd cs)
     (Curry_AbstractHaskell.Choices_C_Pattern d i xs) -> narrows cs d i (\z -> d_C_isClosedStringPattern z cd cs) xs
     (Curry_AbstractHaskell.Guard_C_Pattern d c e) -> guardCons d c ((d_C_isClosedStringPattern e cd) $! (addCs c cs))
     (Curry_AbstractHaskell.Fail_C_Pattern d info) -> failCons d (traceFail "AbstractHaskellPrinter.isClosedStringPattern" [(show x1)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter.isClosedStringPattern" (showCons x1))

d_C_isCharPattern :: Curry_AbstractHaskell.C_Pattern -> Cover -> ConstStore -> Curry_Prelude.C_Bool
d_C_isCharPattern x1 cd cs = case x1 of
     (Curry_AbstractHaskell.C_PLit x2) -> d_C__case_10 x2 cd cs
     (Curry_AbstractHaskell.C_PVar x7) -> Curry_Prelude.C_False
     (Curry_AbstractHaskell.C_PComb x8 x9) -> Curry_Prelude.C_False
     (Curry_AbstractHaskell.C_PAs x10 x11) -> Curry_Prelude.C_False
     (Curry_AbstractHaskell.C_PFuncComb x12 x13) -> Curry_Prelude.C_False
     (Curry_AbstractHaskell.Choice_C_Pattern d i l r) -> narrow d i (d_C_isCharPattern l cd cs) (d_C_isCharPattern r cd cs)
     (Curry_AbstractHaskell.Choices_C_Pattern d i xs) -> narrows cs d i (\z -> d_C_isCharPattern z cd cs) xs
     (Curry_AbstractHaskell.Guard_C_Pattern d c e) -> guardCons d c ((d_C_isCharPattern e cd) $! (addCs c cs))
     (Curry_AbstractHaskell.Fail_C_Pattern d info) -> failCons d (traceFail "AbstractHaskellPrinter.isCharPattern" [(show x1)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter.isCharPattern" (showCons x1))

d_C_isAsPattern :: Curry_AbstractHaskell.C_Pattern -> Cover -> ConstStore -> Curry_Prelude.C_Bool
d_C_isAsPattern x1 cd cs = case x1 of
     (Curry_AbstractHaskell.C_PAs x2 x3) -> Curry_Prelude.C_True
     (Curry_AbstractHaskell.C_PVar x4) -> Curry_Prelude.C_False
     (Curry_AbstractHaskell.C_PLit x5) -> Curry_Prelude.C_False
     (Curry_AbstractHaskell.C_PComb x6 x7) -> Curry_Prelude.C_False
     (Curry_AbstractHaskell.C_PFuncComb x8 x9) -> Curry_Prelude.C_False
     (Curry_AbstractHaskell.Choice_C_Pattern d i l r) -> narrow d i (d_C_isAsPattern l cd cs) (d_C_isAsPattern r cd cs)
     (Curry_AbstractHaskell.Choices_C_Pattern d i xs) -> narrows cs d i (\z -> d_C_isAsPattern z cd cs) xs
     (Curry_AbstractHaskell.Guard_C_Pattern d c e) -> guardCons d c ((d_C_isAsPattern e cd) $! (addCs c cs))
     (Curry_AbstractHaskell.Fail_C_Pattern d info) -> failCons d (traceFail "AbstractHaskellPrinter.isAsPattern" [(show x1)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter.isAsPattern" (showCons x1))

d_C_isInfixOpName :: Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Cover -> ConstStore -> Curry_Prelude.C_Bool
d_C_isInfixOpName cd cs = Curry_Prelude.d_C_all (Curry_Prelude.d_C_flip Curry_Prelude.d_C_elem (d_C_infixIDs cd cs)) cd cs

d_C_isClosedList :: Curry_AbstractHaskell.C_Expr -> Cover -> ConstStore -> Curry_Prelude.C_Bool
d_C_isClosedList x1 cd cs = case x1 of
     (Curry_AbstractHaskell.C_Apply x2 x3) -> d_C__case_9 x3 x2 cd cs
     (Curry_AbstractHaskell.C_Symbol x44) -> d_C__case_6 x44 cd cs
     (Curry_AbstractHaskell.C_Var x47) -> Curry_Prelude.C_False
     (Curry_AbstractHaskell.C_Lit x48) -> Curry_Prelude.C_False
     (Curry_AbstractHaskell.C_Lambda x49 x50) -> Curry_Prelude.C_False
     (Curry_AbstractHaskell.C_Let x51 x52) -> Curry_Prelude.C_False
     (Curry_AbstractHaskell.C_DoExpr x53) -> Curry_Prelude.C_False
     (Curry_AbstractHaskell.C_ListComp x54 x55) -> Curry_Prelude.C_False
     (Curry_AbstractHaskell.C_Case x56 x57) -> Curry_Prelude.C_False
     (Curry_AbstractHaskell.C_Typed x58 x59) -> Curry_Prelude.C_False
     (Curry_AbstractHaskell.C_IfThenElse x60 x61 x62) -> Curry_Prelude.C_False
     (Curry_AbstractHaskell.Choice_C_Expr d i l r) -> narrow d i (d_C_isClosedList l cd cs) (d_C_isClosedList r cd cs)
     (Curry_AbstractHaskell.Choices_C_Expr d i xs) -> narrows cs d i (\z -> d_C_isClosedList z cd cs) xs
     (Curry_AbstractHaskell.Guard_C_Expr d c e) -> guardCons d c ((d_C_isClosedList e cd) $! (addCs c cs))
     (Curry_AbstractHaskell.Fail_C_Expr d info) -> failCons d (traceFail "AbstractHaskellPrinter.isClosedList" [(show x1)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter.isClosedList" (showCons x1))

d_C_isSimpleExpr :: Curry_AbstractHaskell.C_Expr -> Cover -> ConstStore -> Curry_Prelude.C_Bool
d_C_isSimpleExpr x1 cd cs = case x1 of
     (Curry_AbstractHaskell.C_Var x2) -> Curry_Prelude.C_True
     (Curry_AbstractHaskell.C_Lit x3) -> Curry_Prelude.C_True
     (Curry_AbstractHaskell.C_Symbol x4) -> d_C__case_5 x4 cd cs
     (Curry_AbstractHaskell.C_Apply x7 x8) -> d_C__case_4 x7 (d_C_applicationHead x7 cd cs) cd cs
     (Curry_AbstractHaskell.C_Lambda x30 x31) -> Curry_Prelude.C_False
     (Curry_AbstractHaskell.C_Let x32 x33) -> Curry_Prelude.C_False
     (Curry_AbstractHaskell.C_DoExpr x34) -> Curry_Prelude.C_False
     (Curry_AbstractHaskell.C_ListComp x35 x36) -> Curry_Prelude.C_False
     (Curry_AbstractHaskell.C_Case x37 x38) -> Curry_Prelude.C_False
     (Curry_AbstractHaskell.C_Typed x39 x40) -> Curry_Prelude.C_False
     (Curry_AbstractHaskell.C_IfThenElse x41 x42 x43) -> Curry_Prelude.C_False
     (Curry_AbstractHaskell.Choice_C_Expr d i l r) -> narrow d i (d_C_isSimpleExpr l cd cs) (d_C_isSimpleExpr r cd cs)
     (Curry_AbstractHaskell.Choices_C_Expr d i xs) -> narrows cs d i (\z -> d_C_isSimpleExpr z cd cs) xs
     (Curry_AbstractHaskell.Guard_C_Expr d c e) -> guardCons d c ((d_C_isSimpleExpr e cd) $! (addCs c cs))
     (Curry_AbstractHaskell.Fail_C_Expr d info) -> failCons d (traceFail "AbstractHaskellPrinter.isSimpleExpr" [(show x1)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter.isSimpleExpr" (showCons x1))

d_C_isFuncType :: Curry_AbstractHaskell.C_TypeExpr -> Cover -> ConstStore -> Curry_Prelude.C_Bool
d_C_isFuncType x1 cd cs = case x1 of
     (Curry_AbstractHaskell.C_FuncType x2 x3) -> Curry_Prelude.C_True
     (Curry_AbstractHaskell.C_TVar x4) -> Curry_Prelude.C_False
     (Curry_AbstractHaskell.C_TCons x5 x6) -> Curry_Prelude.C_False
     (Curry_AbstractHaskell.Choice_C_TypeExpr d i l r) -> narrow d i (d_C_isFuncType l cd cs) (d_C_isFuncType r cd cs)
     (Curry_AbstractHaskell.Choices_C_TypeExpr d i xs) -> narrows cs d i (\z -> d_C_isFuncType z cd cs) xs
     (Curry_AbstractHaskell.Guard_C_TypeExpr d c e) -> guardCons d c ((d_C_isFuncType e cd) $! (addCs c cs))
     (Curry_AbstractHaskell.Fail_C_TypeExpr d info) -> failCons d (traceFail "AbstractHaskellPrinter.isFuncType" [(show x1)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter.isFuncType" (showCons x1))

d_C_isAtom :: Curry_AbstractHaskell.C_Expr -> Cover -> ConstStore -> Curry_Prelude.C_Bool
d_C_isAtom x1 cd cs = case x1 of
     (Curry_AbstractHaskell.C_Var x2) -> Curry_Prelude.C_True
     (Curry_AbstractHaskell.C_Lit x3) -> Curry_Prelude.C_True
     (Curry_AbstractHaskell.C_Symbol x4) -> d_C__case_2 x4 cd cs
     (Curry_AbstractHaskell.C_Apply x7 x8) -> Curry_Prelude.C_False
     (Curry_AbstractHaskell.C_Lambda x9 x10) -> Curry_Prelude.C_False
     (Curry_AbstractHaskell.C_Let x11 x12) -> Curry_Prelude.C_False
     (Curry_AbstractHaskell.C_DoExpr x13) -> Curry_Prelude.C_False
     (Curry_AbstractHaskell.C_ListComp x14 x15) -> Curry_Prelude.C_False
     (Curry_AbstractHaskell.C_Case x16 x17) -> Curry_Prelude.C_False
     (Curry_AbstractHaskell.C_Typed x18 x19) -> Curry_Prelude.C_False
     (Curry_AbstractHaskell.C_IfThenElse x20 x21 x22) -> Curry_Prelude.C_False
     (Curry_AbstractHaskell.Choice_C_Expr d i l r) -> narrow d i (d_C_isAtom l cd cs) (d_C_isAtom r cd cs)
     (Curry_AbstractHaskell.Choices_C_Expr d i xs) -> narrows cs d i (\z -> d_C_isAtom z cd cs) xs
     (Curry_AbstractHaskell.Guard_C_Expr d c e) -> guardCons d c ((d_C_isAtom e cd) $! (addCs c cs))
     (Curry_AbstractHaskell.Fail_C_Expr d info) -> failCons d (traceFail "AbstractHaskellPrinter.isAtom" [(show x1)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter.isAtom" (showCons x1))

d_C_isTuple :: Curry_Prelude.OP_List Curry_Prelude.C_Char -> Cover -> ConstStore -> Curry_Prelude.C_Bool
d_C_isTuple x1 cd cs = case x1 of
     Curry_Prelude.OP_List -> Curry_Prelude.C_False
     (Curry_Prelude.OP_Cons x2 x3) -> Curry_Prelude.d_OP_amp_amp (Curry_Prelude.d_OP_eq_eq x2 (Curry_Prelude.C_Char '('#) cd cs) (d_OP_isTuple_dot_p1_uscore_isTuple_dot_566 x3 cd cs) cd cs
     (Curry_Prelude.Choice_OP_List d i l r) -> narrow d i (d_C_isTuple l cd cs) (d_C_isTuple r cd cs)
     (Curry_Prelude.Choices_OP_List d i xs) -> narrows cs d i (\z -> d_C_isTuple z cd cs) xs
     (Curry_Prelude.Guard_OP_List d c e) -> guardCons d c ((d_C_isTuple e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_List d info) -> failCons d (traceFail "AbstractHaskellPrinter.isTuple" [(show x1)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter.isTuple" (showCons x1))

d_OP_isTuple_dot_p1_uscore_isTuple_dot_566 :: Curry_Prelude.OP_List Curry_Prelude.C_Char -> Cover -> ConstStore -> Curry_Prelude.C_Bool
d_OP_isTuple_dot_p1_uscore_isTuple_dot_566 x1 cd cs = case x1 of
     Curry_Prelude.OP_List -> Curry_Prelude.C_False
     (Curry_Prelude.OP_Cons x2 x3) -> d_C__case_1 x2 x3 cd cs
     (Curry_Prelude.Choice_OP_List d i l r) -> narrow d i (d_OP_isTuple_dot_p1_uscore_isTuple_dot_566 l cd cs) (d_OP_isTuple_dot_p1_uscore_isTuple_dot_566 r cd cs)
     (Curry_Prelude.Choices_OP_List d i xs) -> narrows cs d i (\z -> d_OP_isTuple_dot_p1_uscore_isTuple_dot_566 z cd cs) xs
     (Curry_Prelude.Guard_OP_List d c e) -> guardCons d c ((d_OP_isTuple_dot_p1_uscore_isTuple_dot_566 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_List d info) -> failCons d (traceFail "AbstractHaskellPrinter.isTuple.p1_isTuple.566" [(show x1)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter.isTuple.p1_isTuple.566" (showCons x1))

d_C_infixIDs :: Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C_infixIDs cd cs = Curry_Prelude.toCurryString "~!@#$%^&*+-=<>?./|\\:"

d_C_maybeShowBrackets :: Curry_Prelude.C_Bool -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C_maybeShowBrackets x1 x2 cd cs = case x1 of
     Curry_Prelude.C_True -> Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString "(") (Curry_Prelude.d_OP_plus_plus x2 (Curry_Prelude.toCurryString ")") cd cs) cd cs
     Curry_Prelude.C_False -> d_C__case_0 x2 (Curry_Prelude.d_C_otherwise cd cs) cd cs
     (Curry_Prelude.Choice_C_Bool d i l r) -> narrow d i (d_C_maybeShowBrackets l x2 cd cs) (d_C_maybeShowBrackets r x2 cd cs)
     (Curry_Prelude.Choices_C_Bool d i xs) -> narrows cs d i (\z -> d_C_maybeShowBrackets z x2 cd cs) xs
     (Curry_Prelude.Guard_C_Bool d c e) -> guardCons d c ((d_C_maybeShowBrackets e x2 cd) $! (addCs c cs))
     (Curry_Prelude.Fail_C_Bool d info) -> failCons d (traceFail "AbstractHaskellPrinter.maybeShowBrackets" [(show x1),(show x2)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter.maybeShowBrackets" (showCons x1))

d_C__case_0 :: Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_0 x2 x3 cd cs = case x3 of
     Curry_Prelude.C_True -> x2
     Curry_Prelude.C_False -> Curry_Prelude.d_C_failed cd cs
     (Curry_Prelude.Choice_C_Bool d i l r) -> narrow d i (d_C__case_0 x2 l cd cs) (d_C__case_0 x2 r cd cs)
     (Curry_Prelude.Choices_C_Bool d i xs) -> narrows cs d i (\z -> d_C__case_0 x2 z cd cs) xs
     (Curry_Prelude.Guard_C_Bool d c e) -> guardCons d c ((d_C__case_0 x2 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_C_Bool d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_0" [(show x2),(show x3)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_0" (showCons x3))

d_C__case_1 :: Curry_Prelude.C_Char -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Cover -> ConstStore -> Curry_Prelude.C_Bool
d_C__case_1 x2 x3 cd cs = case x3 of
     Curry_Prelude.OP_List -> Curry_Prelude.d_OP_eq_eq x2 (Curry_Prelude.C_Char ')'#) cd cs
     (Curry_Prelude.OP_Cons x4 x5) -> Curry_Prelude.d_OP_amp_amp (Curry_Prelude.d_OP_eq_eq x2 (Curry_Prelude.C_Char ','#) cd cs) (d_OP_isTuple_dot_p1_uscore_isTuple_dot_566 (Curry_Prelude.OP_Cons x4 x5) cd cs) cd cs
     (Curry_Prelude.Choice_OP_List d i l r) -> narrow d i (d_C__case_1 x2 l cd cs) (d_C__case_1 x2 r cd cs)
     (Curry_Prelude.Choices_OP_List d i xs) -> narrows cs d i (\z -> d_C__case_1 x2 z cd cs) xs
     (Curry_Prelude.Guard_OP_List d c e) -> guardCons d c ((d_C__case_1 x2 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_List d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_1" [(show x2),(show x3)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_1" (showCons x3))

d_C__case_2 :: Curry_Prelude.OP_Tuple2 (Curry_Prelude.OP_List Curry_Prelude.C_Char) (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Cover -> ConstStore -> Curry_Prelude.C_Bool
d_C__case_2 x4 cd cs = case x4 of
     (Curry_Prelude.OP_Tuple2 x5 x6) -> Curry_Prelude.d_OP_dollar Curry_Prelude.d_C_not (Curry_Prelude.d_C_apply (d_C_isInfixOpName cd cs) x6 cd cs) cd cs
     (Curry_Prelude.Choice_OP_Tuple2 d i l r) -> narrow d i (d_C__case_2 l cd cs) (d_C__case_2 r cd cs)
     (Curry_Prelude.Choices_OP_Tuple2 d i xs) -> narrows cs d i (\z -> d_C__case_2 z cd cs) xs
     (Curry_Prelude.Guard_OP_Tuple2 d c e) -> guardCons d c ((d_C__case_2 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_Tuple2 d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_2" [(show x4)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_2" (showCons x4))

d_C__case_4 :: Curry_AbstractHaskell.C_Expr -> Curry_AbstractHaskell.C_Expr -> Cover -> ConstStore -> Curry_Prelude.C_Bool
d_C__case_4 x7 x30 cd cs = case x30 of
     (Curry_AbstractHaskell.C_Symbol x9) -> d_C__case_3 x9 cd cs
     (Curry_AbstractHaskell.C_Var x12) -> Curry_Prelude.C_False
     (Curry_AbstractHaskell.C_Lit x13) -> Curry_Prelude.C_False
     (Curry_AbstractHaskell.C_Apply x14 x15) -> Curry_Prelude.C_False
     (Curry_AbstractHaskell.C_Lambda x16 x17) -> Curry_Prelude.C_False
     (Curry_AbstractHaskell.C_Let x18 x19) -> Curry_Prelude.C_False
     (Curry_AbstractHaskell.C_DoExpr x20) -> Curry_Prelude.C_False
     (Curry_AbstractHaskell.C_ListComp x21 x22) -> Curry_Prelude.C_False
     (Curry_AbstractHaskell.C_Case x23 x24) -> Curry_Prelude.C_False
     (Curry_AbstractHaskell.C_Typed x25 x26) -> Curry_Prelude.C_False
     (Curry_AbstractHaskell.C_IfThenElse x27 x28 x29) -> Curry_Prelude.C_False
     (Curry_AbstractHaskell.Choice_C_Expr d i l r) -> narrow d i (d_C__case_4 x7 l cd cs) (d_C__case_4 x7 r cd cs)
     (Curry_AbstractHaskell.Choices_C_Expr d i xs) -> narrows cs d i (\z -> d_C__case_4 x7 z cd cs) xs
     (Curry_AbstractHaskell.Guard_C_Expr d c e) -> guardCons d c ((d_C__case_4 x7 e cd) $! (addCs c cs))
     (Curry_AbstractHaskell.Fail_C_Expr d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_4" [(show x7),(show x30)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_4" (showCons x30))

d_C__case_3 :: Curry_Prelude.OP_Tuple2 (Curry_Prelude.OP_List Curry_Prelude.C_Char) (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Cover -> ConstStore -> Curry_Prelude.C_Bool
d_C__case_3 x9 cd cs = case x9 of
     (Curry_Prelude.OP_Tuple2 x10 x11) -> Curry_Prelude.d_OP_amp_amp (Curry_Prelude.d_OP_eq_eq x10 (Curry_Names.d_C_prelude cd cs) cd cs) (Curry_Prelude.d_OP_bar_bar (Curry_Prelude.d_OP_eq_eq x11 (Curry_Prelude.toCurryString ":") cd cs) (Curry_Prelude.d_OP_bar_bar (Curry_Prelude.d_OP_eq_eq x11 (Curry_Prelude.toCurryString "[]") cd cs) (Curry_Prelude.d_OP_bar_bar (Curry_Prelude.d_OP_eq_eq x11 (Curry_Prelude.toCurryString "()") cd cs) (d_C_isTuple x11 cd cs) cd cs) cd cs) cd cs) cd cs
     (Curry_Prelude.Choice_OP_Tuple2 d i l r) -> narrow d i (d_C__case_3 l cd cs) (d_C__case_3 r cd cs)
     (Curry_Prelude.Choices_OP_Tuple2 d i xs) -> narrows cs d i (\z -> d_C__case_3 z cd cs) xs
     (Curry_Prelude.Guard_OP_Tuple2 d c e) -> guardCons d c ((d_C__case_3 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_Tuple2 d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_3" [(show x9)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_3" (showCons x9))

d_C__case_5 :: Curry_Prelude.OP_Tuple2 (Curry_Prelude.OP_List Curry_Prelude.C_Char) (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Cover -> ConstStore -> Curry_Prelude.C_Bool
d_C__case_5 x4 cd cs = case x4 of
     (Curry_Prelude.OP_Tuple2 x5 x6) -> Curry_Prelude.d_OP_dollar Curry_Prelude.d_C_not (Curry_Prelude.d_C_apply (d_C_isInfixOpName cd cs) x6 cd cs) cd cs
     (Curry_Prelude.Choice_OP_Tuple2 d i l r) -> narrow d i (d_C__case_5 l cd cs) (d_C__case_5 r cd cs)
     (Curry_Prelude.Choices_OP_Tuple2 d i xs) -> narrows cs d i (\z -> d_C__case_5 z cd cs) xs
     (Curry_Prelude.Guard_OP_Tuple2 d c e) -> guardCons d c ((d_C__case_5 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_Tuple2 d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_5" [(show x4)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_5" (showCons x4))

d_C__case_6 :: Curry_Prelude.OP_Tuple2 (Curry_Prelude.OP_List Curry_Prelude.C_Char) (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Cover -> ConstStore -> Curry_Prelude.C_Bool
d_C__case_6 x44 cd cs = case x44 of
     (Curry_Prelude.OP_Tuple2 x45 x46) -> Curry_Prelude.d_OP_amp_amp (Curry_Prelude.d_OP_eq_eq x45 (Curry_Names.d_C_prelude cd cs) cd cs) (Curry_Prelude.d_OP_eq_eq x46 (Curry_Prelude.toCurryString "[]") cd cs) cd cs
     (Curry_Prelude.Choice_OP_Tuple2 d i l r) -> narrow d i (d_C__case_6 l cd cs) (d_C__case_6 r cd cs)
     (Curry_Prelude.Choices_OP_Tuple2 d i xs) -> narrows cs d i (\z -> d_C__case_6 z cd cs) xs
     (Curry_Prelude.Guard_OP_Tuple2 d c e) -> guardCons d c ((d_C__case_6 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_Tuple2 d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_6" [(show x44)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_6" (showCons x44))

d_C__case_9 :: Curry_AbstractHaskell.C_Expr -> Curry_AbstractHaskell.C_Expr -> Cover -> ConstStore -> Curry_Prelude.C_Bool
d_C__case_9 x3 x2 cd cs = case x2 of
     (Curry_AbstractHaskell.C_Apply x4 x5) -> d_C__case_8 x3 x4 cd cs
     (Curry_AbstractHaskell.C_Var x27) -> Curry_Prelude.C_False
     (Curry_AbstractHaskell.C_Lit x28) -> Curry_Prelude.C_False
     (Curry_AbstractHaskell.C_Symbol x29) -> Curry_Prelude.C_False
     (Curry_AbstractHaskell.C_Lambda x30 x31) -> Curry_Prelude.C_False
     (Curry_AbstractHaskell.C_Let x32 x33) -> Curry_Prelude.C_False
     (Curry_AbstractHaskell.C_DoExpr x34) -> Curry_Prelude.C_False
     (Curry_AbstractHaskell.C_ListComp x35 x36) -> Curry_Prelude.C_False
     (Curry_AbstractHaskell.C_Case x37 x38) -> Curry_Prelude.C_False
     (Curry_AbstractHaskell.C_Typed x39 x40) -> Curry_Prelude.C_False
     (Curry_AbstractHaskell.C_IfThenElse x41 x42 x43) -> Curry_Prelude.C_False
     (Curry_AbstractHaskell.Choice_C_Expr d i l r) -> narrow d i (d_C__case_9 x3 l cd cs) (d_C__case_9 x3 r cd cs)
     (Curry_AbstractHaskell.Choices_C_Expr d i xs) -> narrows cs d i (\z -> d_C__case_9 x3 z cd cs) xs
     (Curry_AbstractHaskell.Guard_C_Expr d c e) -> guardCons d c ((d_C__case_9 x3 e cd) $! (addCs c cs))
     (Curry_AbstractHaskell.Fail_C_Expr d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_9" [(show x3),(show x2)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_9" (showCons x2))

d_C__case_8 :: Curry_AbstractHaskell.C_Expr -> Curry_AbstractHaskell.C_Expr -> Cover -> ConstStore -> Curry_Prelude.C_Bool
d_C__case_8 x3 x4 cd cs = case x4 of
     (Curry_AbstractHaskell.C_Symbol x6) -> d_C__case_7 x3 x6 cd cs
     (Curry_AbstractHaskell.C_Var x9) -> Curry_Prelude.C_False
     (Curry_AbstractHaskell.C_Lit x10) -> Curry_Prelude.C_False
     (Curry_AbstractHaskell.C_Apply x11 x12) -> Curry_Prelude.C_False
     (Curry_AbstractHaskell.C_Lambda x13 x14) -> Curry_Prelude.C_False
     (Curry_AbstractHaskell.C_Let x15 x16) -> Curry_Prelude.C_False
     (Curry_AbstractHaskell.C_DoExpr x17) -> Curry_Prelude.C_False
     (Curry_AbstractHaskell.C_ListComp x18 x19) -> Curry_Prelude.C_False
     (Curry_AbstractHaskell.C_Case x20 x21) -> Curry_Prelude.C_False
     (Curry_AbstractHaskell.C_Typed x22 x23) -> Curry_Prelude.C_False
     (Curry_AbstractHaskell.C_IfThenElse x24 x25 x26) -> Curry_Prelude.C_False
     (Curry_AbstractHaskell.Choice_C_Expr d i l r) -> narrow d i (d_C__case_8 x3 l cd cs) (d_C__case_8 x3 r cd cs)
     (Curry_AbstractHaskell.Choices_C_Expr d i xs) -> narrows cs d i (\z -> d_C__case_8 x3 z cd cs) xs
     (Curry_AbstractHaskell.Guard_C_Expr d c e) -> guardCons d c ((d_C__case_8 x3 e cd) $! (addCs c cs))
     (Curry_AbstractHaskell.Fail_C_Expr d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_8" [(show x3),(show x4)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_8" (showCons x4))

d_C__case_7 :: Curry_AbstractHaskell.C_Expr -> Curry_Prelude.OP_Tuple2 (Curry_Prelude.OP_List Curry_Prelude.C_Char) (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Cover -> ConstStore -> Curry_Prelude.C_Bool
d_C__case_7 x3 x6 cd cs = case x6 of
     (Curry_Prelude.OP_Tuple2 x7 x8) -> Curry_Prelude.d_OP_amp_amp (Curry_Prelude.d_OP_eq_eq x7 (Curry_Names.d_C_prelude cd cs) cd cs) (Curry_Prelude.d_OP_amp_amp (Curry_Prelude.d_OP_eq_eq x8 (Curry_Prelude.toCurryString ":") cd cs) (d_C_isClosedList x3 cd cs) cd cs) cd cs
     (Curry_Prelude.Choice_OP_Tuple2 d i l r) -> narrow d i (d_C__case_7 x3 l cd cs) (d_C__case_7 x3 r cd cs)
     (Curry_Prelude.Choices_OP_Tuple2 d i xs) -> narrows cs d i (\z -> d_C__case_7 x3 z cd cs) xs
     (Curry_Prelude.Guard_OP_Tuple2 d c e) -> guardCons d c ((d_C__case_7 x3 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_Tuple2 d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_7" [(show x3),(show x6)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_7" (showCons x6))

d_C__case_10 :: Curry_AbstractHaskell.C_Literal -> Cover -> ConstStore -> Curry_Prelude.C_Bool
d_C__case_10 x2 cd cs = case x2 of
     (Curry_AbstractHaskell.C_Charc x3) -> Curry_Prelude.C_True
     (Curry_AbstractHaskell.C_Intc x4) -> Curry_Prelude.C_False
     (Curry_AbstractHaskell.C_Floatc x5) -> Curry_Prelude.C_False
     (Curry_AbstractHaskell.C_Stringc x6) -> Curry_Prelude.C_False
     (Curry_AbstractHaskell.Choice_C_Literal d i l r) -> narrow d i (d_C__case_10 l cd cs) (d_C__case_10 r cd cs)
     (Curry_AbstractHaskell.Choices_C_Literal d i xs) -> narrows cs d i (\z -> d_C__case_10 z cd cs) xs
     (Curry_AbstractHaskell.Guard_C_Literal d c e) -> guardCons d c ((d_C__case_10 e cd) $! (addCs c cs))
     (Curry_AbstractHaskell.Fail_C_Literal d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_10" [(show x2)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_10" (showCons x2))

d_C__case_22 :: Curry_Prelude.OP_List Curry_AbstractHaskell.C_Pattern -> Curry_Prelude.OP_Tuple2 (Curry_Prelude.OP_List Curry_Prelude.C_Char) (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Cover -> ConstStore -> Curry_Prelude.C_Bool
d_C__case_22 x3 x2 cd cs = case x2 of
     (Curry_Prelude.OP_Tuple2 x4 x5) -> d_C__case_21 x3 x4 x5 cd cs
     (Curry_Prelude.Choice_OP_Tuple2 d i l r) -> narrow d i (d_C__case_22 x3 l cd cs) (d_C__case_22 x3 r cd cs)
     (Curry_Prelude.Choices_OP_Tuple2 d i xs) -> narrows cs d i (\z -> d_C__case_22 x3 z cd cs) xs
     (Curry_Prelude.Guard_OP_Tuple2 d c e) -> guardCons d c ((d_C__case_22 x3 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_Tuple2 d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_22" [(show x3),(show x2)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_22" (showCons x2))

d_C__case_21 :: Curry_Prelude.OP_List Curry_AbstractHaskell.C_Pattern -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Cover -> ConstStore -> Curry_Prelude.C_Bool
d_C__case_21 x3 x4 x5 cd cs = case x5 of
     (Curry_Prelude.OP_Cons x6 x7) -> let
          x8 = x6
           in (d_C__case_20 x8 x7 x3 x4 (Curry_Prelude.d_OP_eq_eq x8 (Curry_Prelude.C_Char ':'#) cd cs) cd cs)
     Curry_Prelude.OP_List -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.isClosedStringPattern") cd cs
     (Curry_Prelude.Choice_OP_List d i l r) -> narrow d i (d_C__case_21 x3 x4 l cd cs) (d_C__case_21 x3 x4 r cd cs)
     (Curry_Prelude.Choices_OP_List d i xs) -> narrows cs d i (\z -> d_C__case_21 x3 x4 z cd cs) xs
     (Curry_Prelude.Guard_OP_List d c e) -> guardCons d c ((d_C__case_21 x3 x4 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_List d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_21" [(show x3),(show x4),(show x5)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_21" (showCons x5))

d_C__case_20 :: Curry_Prelude.C_Char -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_Pattern -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> Curry_Prelude.C_Bool
d_C__case_20 x8 x7 x3 x4 x9 cd cs = case x9 of
     Curry_Prelude.C_True -> d_C__case_19 x3 x4 x7 cd cs
     Curry_Prelude.C_False -> d_C__case_15 x8 x7 x3 x4 (Curry_Prelude.d_OP_eq_eq x8 (Curry_Prelude.C_Char '['#) cd cs) cd cs
     (Curry_Prelude.Choice_C_Bool d i l r) -> narrow d i (d_C__case_20 x8 x7 x3 x4 l cd cs) (d_C__case_20 x8 x7 x3 x4 r cd cs)
     (Curry_Prelude.Choices_C_Bool d i xs) -> narrows cs d i (\z -> d_C__case_20 x8 x7 x3 x4 z cd cs) xs
     (Curry_Prelude.Guard_C_Bool d c e) -> guardCons d c ((d_C__case_20 x8 x7 x3 x4 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_C_Bool d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_20" [(show x8),(show x7),(show x3),(show x4),(show x9)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_20" (showCons x9))

d_C__case_15 :: Curry_Prelude.C_Char -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_Pattern -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> Curry_Prelude.C_Bool
d_C__case_15 x8 x7 x3 x4 x9 cd cs = case x9 of
     Curry_Prelude.C_True -> d_C__case_14 x3 x4 x7 cd cs
     Curry_Prelude.C_False -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.isClosedStringPattern") cd cs
     (Curry_Prelude.Choice_C_Bool d i l r) -> narrow d i (d_C__case_15 x8 x7 x3 x4 l cd cs) (d_C__case_15 x8 x7 x3 x4 r cd cs)
     (Curry_Prelude.Choices_C_Bool d i xs) -> narrows cs d i (\z -> d_C__case_15 x8 x7 x3 x4 z cd cs) xs
     (Curry_Prelude.Guard_C_Bool d c e) -> guardCons d c ((d_C__case_15 x8 x7 x3 x4 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_C_Bool d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_15" [(show x8),(show x7),(show x3),(show x4),(show x9)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_15" (showCons x9))

d_C__case_14 :: Curry_Prelude.OP_List Curry_AbstractHaskell.C_Pattern -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Cover -> ConstStore -> Curry_Prelude.C_Bool
d_C__case_14 x3 x4 x7 cd cs = case x7 of
     (Curry_Prelude.OP_Cons x17 x18) -> let
          x19 = x17
           in (d_C__case_13 x19 x18 x3 x4 (Curry_Prelude.d_OP_eq_eq x19 (Curry_Prelude.C_Char ']'#) cd cs) cd cs)
     Curry_Prelude.OP_List -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.isClosedStringPattern") cd cs
     (Curry_Prelude.Choice_OP_List d i l r) -> narrow d i (d_C__case_14 x3 x4 l cd cs) (d_C__case_14 x3 x4 r cd cs)
     (Curry_Prelude.Choices_OP_List d i xs) -> narrows cs d i (\z -> d_C__case_14 x3 x4 z cd cs) xs
     (Curry_Prelude.Guard_OP_List d c e) -> guardCons d c ((d_C__case_14 x3 x4 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_List d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_14" [(show x3),(show x4),(show x7)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_14" (showCons x7))

d_C__case_13 :: Curry_Prelude.C_Char -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_Pattern -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> Curry_Prelude.C_Bool
d_C__case_13 x19 x18 x3 x4 x20 cd cs = case x20 of
     Curry_Prelude.C_True -> d_C__case_12 x3 x4 x18 cd cs
     Curry_Prelude.C_False -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.isClosedStringPattern") cd cs
     (Curry_Prelude.Choice_C_Bool d i l r) -> narrow d i (d_C__case_13 x19 x18 x3 x4 l cd cs) (d_C__case_13 x19 x18 x3 x4 r cd cs)
     (Curry_Prelude.Choices_C_Bool d i xs) -> narrows cs d i (\z -> d_C__case_13 x19 x18 x3 x4 z cd cs) xs
     (Curry_Prelude.Guard_C_Bool d c e) -> guardCons d c ((d_C__case_13 x19 x18 x3 x4 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_C_Bool d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_13" [(show x19),(show x18),(show x3),(show x4),(show x20)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_13" (showCons x20))

d_C__case_12 :: Curry_Prelude.OP_List Curry_AbstractHaskell.C_Pattern -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Cover -> ConstStore -> Curry_Prelude.C_Bool
d_C__case_12 x3 x4 x18 cd cs = case x18 of
     Curry_Prelude.OP_List -> d_C__case_11 x4 x3 cd cs
     (Curry_Prelude.OP_Cons x22 x23) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.isClosedStringPattern") cd cs
     (Curry_Prelude.Choice_OP_List d i l r) -> narrow d i (d_C__case_12 x3 x4 l cd cs) (d_C__case_12 x3 x4 r cd cs)
     (Curry_Prelude.Choices_OP_List d i xs) -> narrows cs d i (\z -> d_C__case_12 x3 x4 z cd cs) xs
     (Curry_Prelude.Guard_OP_List d c e) -> guardCons d c ((d_C__case_12 x3 x4 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_List d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_12" [(show x3),(show x4),(show x18)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_12" (showCons x18))

d_C__case_11 :: Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_Pattern -> Cover -> ConstStore -> Curry_Prelude.C_Bool
d_C__case_11 x4 x3 cd cs = case x3 of
     Curry_Prelude.OP_List -> Curry_Prelude.d_OP_eq_eq x4 (Curry_Names.d_C_prelude cd cs) cd cs
     (Curry_Prelude.OP_Cons x20 x21) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.isClosedStringPattern") cd cs
     (Curry_Prelude.Choice_OP_List d i l r) -> narrow d i (d_C__case_11 x4 l cd cs) (d_C__case_11 x4 r cd cs)
     (Curry_Prelude.Choices_OP_List d i xs) -> narrows cs d i (\z -> d_C__case_11 x4 z cd cs) xs
     (Curry_Prelude.Guard_OP_List d c e) -> guardCons d c ((d_C__case_11 x4 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_List d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_11" [(show x4),(show x3)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_11" (showCons x3))

d_C__case_19 :: Curry_Prelude.OP_List Curry_AbstractHaskell.C_Pattern -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Cover -> ConstStore -> Curry_Prelude.C_Bool
d_C__case_19 x3 x4 x7 cd cs = case x7 of
     Curry_Prelude.OP_List -> d_C__case_18 x4 x3 cd cs
     (Curry_Prelude.OP_Cons x15 x16) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.isClosedStringPattern") cd cs
     (Curry_Prelude.Choice_OP_List d i l r) -> narrow d i (d_C__case_19 x3 x4 l cd cs) (d_C__case_19 x3 x4 r cd cs)
     (Curry_Prelude.Choices_OP_List d i xs) -> narrows cs d i (\z -> d_C__case_19 x3 x4 z cd cs) xs
     (Curry_Prelude.Guard_OP_List d c e) -> guardCons d c ((d_C__case_19 x3 x4 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_List d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_19" [(show x3),(show x4),(show x7)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_19" (showCons x7))

d_C__case_18 :: Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_Pattern -> Cover -> ConstStore -> Curry_Prelude.C_Bool
d_C__case_18 x4 x3 cd cs = case x3 of
     (Curry_Prelude.OP_Cons x9 x10) -> d_C__case_17 x9 x4 x10 cd cs
     Curry_Prelude.OP_List -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.isClosedStringPattern") cd cs
     (Curry_Prelude.Choice_OP_List d i l r) -> narrow d i (d_C__case_18 x4 l cd cs) (d_C__case_18 x4 r cd cs)
     (Curry_Prelude.Choices_OP_List d i xs) -> narrows cs d i (\z -> d_C__case_18 x4 z cd cs) xs
     (Curry_Prelude.Guard_OP_List d c e) -> guardCons d c ((d_C__case_18 x4 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_List d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_18" [(show x4),(show x3)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_18" (showCons x3))

d_C__case_17 :: Curry_AbstractHaskell.C_Pattern -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_Pattern -> Cover -> ConstStore -> Curry_Prelude.C_Bool
d_C__case_17 x9 x4 x10 cd cs = case x10 of
     (Curry_Prelude.OP_Cons x11 x12) -> d_C__case_16 x11 x9 x4 x12 cd cs
     Curry_Prelude.OP_List -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.isClosedStringPattern") cd cs
     (Curry_Prelude.Choice_OP_List d i l r) -> narrow d i (d_C__case_17 x9 x4 l cd cs) (d_C__case_17 x9 x4 r cd cs)
     (Curry_Prelude.Choices_OP_List d i xs) -> narrows cs d i (\z -> d_C__case_17 x9 x4 z cd cs) xs
     (Curry_Prelude.Guard_OP_List d c e) -> guardCons d c ((d_C__case_17 x9 x4 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_List d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_17" [(show x9),(show x4),(show x10)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_17" (showCons x10))

d_C__case_16 :: Curry_AbstractHaskell.C_Pattern -> Curry_AbstractHaskell.C_Pattern -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_Pattern -> Cover -> ConstStore -> Curry_Prelude.C_Bool
d_C__case_16 x11 x9 x4 x12 cd cs = case x12 of
     Curry_Prelude.OP_List -> Curry_Prelude.d_OP_amp_amp (Curry_Prelude.d_OP_eq_eq x4 (Curry_Names.d_C_prelude cd cs) cd cs) (Curry_Prelude.d_OP_amp_amp (d_C_isCharPattern x9 cd cs) (d_C_isClosedStringPattern x11 cd cs) cd cs) cd cs
     (Curry_Prelude.OP_Cons x13 x14) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.isClosedStringPattern") cd cs
     (Curry_Prelude.Choice_OP_List d i l r) -> narrow d i (d_C__case_16 x11 x9 x4 l cd cs) (d_C__case_16 x11 x9 x4 r cd cs)
     (Curry_Prelude.Choices_OP_List d i xs) -> narrows cs d i (\z -> d_C__case_16 x11 x9 x4 z cd cs) xs
     (Curry_Prelude.Guard_OP_List d c e) -> guardCons d c ((d_C__case_16 x11 x9 x4 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_List d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_16" [(show x11),(show x9),(show x4),(show x12)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_16" (showCons x12))

d_C__case_34 :: Curry_Prelude.OP_List Curry_AbstractHaskell.C_Pattern -> Curry_Prelude.OP_Tuple2 (Curry_Prelude.OP_List Curry_Prelude.C_Char) (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Cover -> ConstStore -> Curry_Prelude.C_Bool
d_C__case_34 x3 x2 cd cs = case x2 of
     (Curry_Prelude.OP_Tuple2 x4 x5) -> d_C__case_33 x3 x4 x5 cd cs
     (Curry_Prelude.Choice_OP_Tuple2 d i l r) -> narrow d i (d_C__case_34 x3 l cd cs) (d_C__case_34 x3 r cd cs)
     (Curry_Prelude.Choices_OP_Tuple2 d i xs) -> narrows cs d i (\z -> d_C__case_34 x3 z cd cs) xs
     (Curry_Prelude.Guard_OP_Tuple2 d c e) -> guardCons d c ((d_C__case_34 x3 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_Tuple2 d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_34" [(show x3),(show x2)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_34" (showCons x2))

d_C__case_33 :: Curry_Prelude.OP_List Curry_AbstractHaskell.C_Pattern -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Cover -> ConstStore -> Curry_Prelude.C_Bool
d_C__case_33 x3 x4 x5 cd cs = case x5 of
     (Curry_Prelude.OP_Cons x6 x7) -> let
          x8 = x6
           in (d_C__case_32 x8 x7 x3 x4 (Curry_Prelude.d_OP_eq_eq x8 (Curry_Prelude.C_Char ':'#) cd cs) cd cs)
     Curry_Prelude.OP_List -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.isClosedPatternList") cd cs
     (Curry_Prelude.Choice_OP_List d i l r) -> narrow d i (d_C__case_33 x3 x4 l cd cs) (d_C__case_33 x3 x4 r cd cs)
     (Curry_Prelude.Choices_OP_List d i xs) -> narrows cs d i (\z -> d_C__case_33 x3 x4 z cd cs) xs
     (Curry_Prelude.Guard_OP_List d c e) -> guardCons d c ((d_C__case_33 x3 x4 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_List d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_33" [(show x3),(show x4),(show x5)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_33" (showCons x5))

d_C__case_32 :: Curry_Prelude.C_Char -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_Pattern -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> Curry_Prelude.C_Bool
d_C__case_32 x8 x7 x3 x4 x9 cd cs = case x9 of
     Curry_Prelude.C_True -> d_C__case_31 x3 x4 x7 cd cs
     Curry_Prelude.C_False -> d_C__case_27 x8 x7 x3 x4 (Curry_Prelude.d_OP_eq_eq x8 (Curry_Prelude.C_Char '['#) cd cs) cd cs
     (Curry_Prelude.Choice_C_Bool d i l r) -> narrow d i (d_C__case_32 x8 x7 x3 x4 l cd cs) (d_C__case_32 x8 x7 x3 x4 r cd cs)
     (Curry_Prelude.Choices_C_Bool d i xs) -> narrows cs d i (\z -> d_C__case_32 x8 x7 x3 x4 z cd cs) xs
     (Curry_Prelude.Guard_C_Bool d c e) -> guardCons d c ((d_C__case_32 x8 x7 x3 x4 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_C_Bool d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_32" [(show x8),(show x7),(show x3),(show x4),(show x9)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_32" (showCons x9))

d_C__case_27 :: Curry_Prelude.C_Char -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_Pattern -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> Curry_Prelude.C_Bool
d_C__case_27 x8 x7 x3 x4 x9 cd cs = case x9 of
     Curry_Prelude.C_True -> d_C__case_26 x3 x4 x7 cd cs
     Curry_Prelude.C_False -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.isClosedPatternList") cd cs
     (Curry_Prelude.Choice_C_Bool d i l r) -> narrow d i (d_C__case_27 x8 x7 x3 x4 l cd cs) (d_C__case_27 x8 x7 x3 x4 r cd cs)
     (Curry_Prelude.Choices_C_Bool d i xs) -> narrows cs d i (\z -> d_C__case_27 x8 x7 x3 x4 z cd cs) xs
     (Curry_Prelude.Guard_C_Bool d c e) -> guardCons d c ((d_C__case_27 x8 x7 x3 x4 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_C_Bool d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_27" [(show x8),(show x7),(show x3),(show x4),(show x9)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_27" (showCons x9))

d_C__case_26 :: Curry_Prelude.OP_List Curry_AbstractHaskell.C_Pattern -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Cover -> ConstStore -> Curry_Prelude.C_Bool
d_C__case_26 x3 x4 x7 cd cs = case x7 of
     (Curry_Prelude.OP_Cons x17 x18) -> let
          x19 = x17
           in (d_C__case_25 x19 x18 x3 x4 (Curry_Prelude.d_OP_eq_eq x19 (Curry_Prelude.C_Char ']'#) cd cs) cd cs)
     Curry_Prelude.OP_List -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.isClosedPatternList") cd cs
     (Curry_Prelude.Choice_OP_List d i l r) -> narrow d i (d_C__case_26 x3 x4 l cd cs) (d_C__case_26 x3 x4 r cd cs)
     (Curry_Prelude.Choices_OP_List d i xs) -> narrows cs d i (\z -> d_C__case_26 x3 x4 z cd cs) xs
     (Curry_Prelude.Guard_OP_List d c e) -> guardCons d c ((d_C__case_26 x3 x4 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_List d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_26" [(show x3),(show x4),(show x7)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_26" (showCons x7))

d_C__case_25 :: Curry_Prelude.C_Char -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_Pattern -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> Curry_Prelude.C_Bool
d_C__case_25 x19 x18 x3 x4 x20 cd cs = case x20 of
     Curry_Prelude.C_True -> d_C__case_24 x3 x4 x18 cd cs
     Curry_Prelude.C_False -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.isClosedPatternList") cd cs
     (Curry_Prelude.Choice_C_Bool d i l r) -> narrow d i (d_C__case_25 x19 x18 x3 x4 l cd cs) (d_C__case_25 x19 x18 x3 x4 r cd cs)
     (Curry_Prelude.Choices_C_Bool d i xs) -> narrows cs d i (\z -> d_C__case_25 x19 x18 x3 x4 z cd cs) xs
     (Curry_Prelude.Guard_C_Bool d c e) -> guardCons d c ((d_C__case_25 x19 x18 x3 x4 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_C_Bool d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_25" [(show x19),(show x18),(show x3),(show x4),(show x20)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_25" (showCons x20))

d_C__case_24 :: Curry_Prelude.OP_List Curry_AbstractHaskell.C_Pattern -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Cover -> ConstStore -> Curry_Prelude.C_Bool
d_C__case_24 x3 x4 x18 cd cs = case x18 of
     Curry_Prelude.OP_List -> d_C__case_23 x4 x3 cd cs
     (Curry_Prelude.OP_Cons x22 x23) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.isClosedPatternList") cd cs
     (Curry_Prelude.Choice_OP_List d i l r) -> narrow d i (d_C__case_24 x3 x4 l cd cs) (d_C__case_24 x3 x4 r cd cs)
     (Curry_Prelude.Choices_OP_List d i xs) -> narrows cs d i (\z -> d_C__case_24 x3 x4 z cd cs) xs
     (Curry_Prelude.Guard_OP_List d c e) -> guardCons d c ((d_C__case_24 x3 x4 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_List d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_24" [(show x3),(show x4),(show x18)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_24" (showCons x18))

d_C__case_23 :: Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_Pattern -> Cover -> ConstStore -> Curry_Prelude.C_Bool
d_C__case_23 x4 x3 cd cs = case x3 of
     Curry_Prelude.OP_List -> Curry_Prelude.d_OP_eq_eq x4 (Curry_Names.d_C_prelude cd cs) cd cs
     (Curry_Prelude.OP_Cons x20 x21) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.isClosedPatternList") cd cs
     (Curry_Prelude.Choice_OP_List d i l r) -> narrow d i (d_C__case_23 x4 l cd cs) (d_C__case_23 x4 r cd cs)
     (Curry_Prelude.Choices_OP_List d i xs) -> narrows cs d i (\z -> d_C__case_23 x4 z cd cs) xs
     (Curry_Prelude.Guard_OP_List d c e) -> guardCons d c ((d_C__case_23 x4 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_List d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_23" [(show x4),(show x3)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_23" (showCons x3))

d_C__case_31 :: Curry_Prelude.OP_List Curry_AbstractHaskell.C_Pattern -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Cover -> ConstStore -> Curry_Prelude.C_Bool
d_C__case_31 x3 x4 x7 cd cs = case x7 of
     Curry_Prelude.OP_List -> d_C__case_30 x4 x3 cd cs
     (Curry_Prelude.OP_Cons x15 x16) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.isClosedPatternList") cd cs
     (Curry_Prelude.Choice_OP_List d i l r) -> narrow d i (d_C__case_31 x3 x4 l cd cs) (d_C__case_31 x3 x4 r cd cs)
     (Curry_Prelude.Choices_OP_List d i xs) -> narrows cs d i (\z -> d_C__case_31 x3 x4 z cd cs) xs
     (Curry_Prelude.Guard_OP_List d c e) -> guardCons d c ((d_C__case_31 x3 x4 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_List d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_31" [(show x3),(show x4),(show x7)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_31" (showCons x7))

d_C__case_30 :: Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_Pattern -> Cover -> ConstStore -> Curry_Prelude.C_Bool
d_C__case_30 x4 x3 cd cs = case x3 of
     (Curry_Prelude.OP_Cons x9 x10) -> d_C__case_29 x4 x10 cd cs
     Curry_Prelude.OP_List -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.isClosedPatternList") cd cs
     (Curry_Prelude.Choice_OP_List d i l r) -> narrow d i (d_C__case_30 x4 l cd cs) (d_C__case_30 x4 r cd cs)
     (Curry_Prelude.Choices_OP_List d i xs) -> narrows cs d i (\z -> d_C__case_30 x4 z cd cs) xs
     (Curry_Prelude.Guard_OP_List d c e) -> guardCons d c ((d_C__case_30 x4 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_List d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_30" [(show x4),(show x3)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_30" (showCons x3))

d_C__case_29 :: Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_Pattern -> Cover -> ConstStore -> Curry_Prelude.C_Bool
d_C__case_29 x4 x10 cd cs = case x10 of
     (Curry_Prelude.OP_Cons x11 x12) -> d_C__case_28 x11 x4 x12 cd cs
     Curry_Prelude.OP_List -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.isClosedPatternList") cd cs
     (Curry_Prelude.Choice_OP_List d i l r) -> narrow d i (d_C__case_29 x4 l cd cs) (d_C__case_29 x4 r cd cs)
     (Curry_Prelude.Choices_OP_List d i xs) -> narrows cs d i (\z -> d_C__case_29 x4 z cd cs) xs
     (Curry_Prelude.Guard_OP_List d c e) -> guardCons d c ((d_C__case_29 x4 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_List d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_29" [(show x4),(show x10)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_29" (showCons x10))

d_C__case_28 :: Curry_AbstractHaskell.C_Pattern -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_Pattern -> Cover -> ConstStore -> Curry_Prelude.C_Bool
d_C__case_28 x11 x4 x12 cd cs = case x12 of
     Curry_Prelude.OP_List -> Curry_Prelude.d_OP_amp_amp (Curry_Prelude.d_OP_eq_eq x4 (Curry_Names.d_C_prelude cd cs) cd cs) (d_C_isClosedPatternList x11 cd cs) cd cs
     (Curry_Prelude.OP_Cons x13 x14) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.isClosedPatternList") cd cs
     (Curry_Prelude.Choice_OP_List d i l r) -> narrow d i (d_C__case_28 x11 x4 l cd cs) (d_C__case_28 x11 x4 r cd cs)
     (Curry_Prelude.Choices_OP_List d i xs) -> narrows cs d i (\z -> d_C__case_28 x11 x4 z cd cs) xs
     (Curry_Prelude.Guard_OP_List d c e) -> guardCons d c ((d_C__case_28 x11 x4 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_List d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_28" [(show x11),(show x4),(show x12)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_28" (showCons x12))

d_C__case_36 :: Curry_Prelude.C_Char -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_36 x4 x3 x5 cd cs = case x5 of
     Curry_Prelude.C_True -> Curry_Prelude.d_OP_dollar d_C_dropTags (Curry_Prelude.d_OP_dollar Curry_Prelude.d_C_tail (Curry_Prelude.d_C_dropWhile (Curry_Prelude.d_C_flip (acceptCs id Curry_Prelude.d_OP_slash_eq) (Curry_Prelude.C_Char '"'#)) x3 cd cs) cd cs) cd cs
     Curry_Prelude.C_False -> d_C__case_35 x4 x3 (Curry_Prelude.d_OP_eq_eq x4 (Curry_Prelude.C_Char '>'#) cd cs) cd cs
     (Curry_Prelude.Choice_C_Bool d i l r) -> narrow d i (d_C__case_36 x4 x3 l cd cs) (d_C__case_36 x4 x3 r cd cs)
     (Curry_Prelude.Choices_C_Bool d i xs) -> narrows cs d i (\z -> d_C__case_36 x4 x3 z cd cs) xs
     (Curry_Prelude.Guard_C_Bool d c e) -> guardCons d c ((d_C__case_36 x4 x3 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_C_Bool d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_36" [(show x4),(show x3),(show x5)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_36" (showCons x5))

d_C__case_35 :: Curry_Prelude.C_Char -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_35 x4 x3 x5 cd cs = case x5 of
     Curry_Prelude.C_True -> x3
     Curry_Prelude.C_False -> d_C_dropTags x3 cd cs
     (Curry_Prelude.Choice_C_Bool d i l r) -> narrow d i (d_C__case_35 x4 x3 l cd cs) (d_C__case_35 x4 x3 r cd cs)
     (Curry_Prelude.Choices_C_Bool d i xs) -> narrows cs d i (\z -> d_C__case_35 x4 x3 z cd cs) xs
     (Curry_Prelude.Guard_C_Bool d c e) -> guardCons d c ((d_C__case_35 x4 x3 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_C_Bool d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_35" [(show x4),(show x3),(show x5)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_35" (showCons x5))

d_C__case_38 :: Curry_AbstractHaskell.C_Expr -> C_Options -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_38 x2 x1 x3 cd cs = case x3 of
     Curry_Prelude.C_True -> d_C_showExprOpt x1 x2 cd cs
     Curry_Prelude.C_False -> d_C__case_37 x2 x1 (Curry_Prelude.d_C_otherwise cd cs) cd cs
     (Curry_Prelude.Choice_C_Bool d i l r) -> narrow d i (d_C__case_38 x2 x1 l cd cs) (d_C__case_38 x2 x1 r cd cs)
     (Curry_Prelude.Choices_C_Bool d i xs) -> narrows cs d i (\z -> d_C__case_38 x2 x1 z cd cs) xs
     (Curry_Prelude.Guard_C_Bool d c e) -> guardCons d c ((d_C__case_38 x2 x1 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_C_Bool d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_38" [(show x2),(show x1),(show x3)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_38" (showCons x3))

d_C__case_37 :: Curry_AbstractHaskell.C_Expr -> C_Options -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_37 x2 x1 x3 cd cs = case x3 of
     Curry_Prelude.C_True -> Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString "(") (Curry_Prelude.d_OP_plus_plus (d_C_showExprOpt x1 x2 cd cs) (Curry_Prelude.toCurryString ")") cd cs) cd cs
     Curry_Prelude.C_False -> Curry_Prelude.d_C_failed cd cs
     (Curry_Prelude.Choice_C_Bool d i l r) -> narrow d i (d_C__case_37 x2 x1 l cd cs) (d_C__case_37 x2 x1 r cd cs)
     (Curry_Prelude.Choices_C_Bool d i xs) -> narrows cs d i (\z -> d_C__case_37 x2 x1 z cd cs) xs
     (Curry_Prelude.Guard_C_Bool d c e) -> guardCons d c ((d_C__case_37 x2 x1 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_C_Bool d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_37" [(show x2),(show x1),(show x3)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_37" (showCons x3))

d_C__case_39 :: Curry_AbstractHaskell.C_Expr -> C_Options -> Curry_AbstractHaskell.C_Expr -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_39 x4 x1 x3 cd cs = case x3 of
     (Curry_AbstractHaskell.C_Symbol x5) -> d_C_showExprOpt x1 x4 cd cs
     (Curry_AbstractHaskell.C_Apply x6 x7) -> Curry_Prelude.d_OP_plus_plus (d_OP_showTupleApplication_dot_p_uscore_showTuple_dot_416 x1 (Curry_AbstractHaskell.C_Apply x6 x7) cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString ",") (d_C_showExprOpt x1 x4 cd cs) cd cs) cd cs
     (Curry_AbstractHaskell.C_Var x8) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showTupleApplication") cd cs
     (Curry_AbstractHaskell.C_Lit x9) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showTupleApplication") cd cs
     (Curry_AbstractHaskell.C_Lambda x10 x11) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showTupleApplication") cd cs
     (Curry_AbstractHaskell.C_Let x12 x13) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showTupleApplication") cd cs
     (Curry_AbstractHaskell.C_DoExpr x14) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showTupleApplication") cd cs
     (Curry_AbstractHaskell.C_ListComp x15 x16) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showTupleApplication") cd cs
     (Curry_AbstractHaskell.C_Case x17 x18) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showTupleApplication") cd cs
     (Curry_AbstractHaskell.C_Typed x19 x20) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showTupleApplication") cd cs
     (Curry_AbstractHaskell.C_IfThenElse x21 x22 x23) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showTupleApplication") cd cs
     (Curry_AbstractHaskell.Choice_C_Expr d i l r) -> narrow d i (d_C__case_39 x4 x1 l cd cs) (d_C__case_39 x4 x1 r cd cs)
     (Curry_AbstractHaskell.Choices_C_Expr d i xs) -> narrows cs d i (\z -> d_C__case_39 x4 x1 z cd cs) xs
     (Curry_AbstractHaskell.Guard_C_Expr d c e) -> guardCons d c ((d_C__case_39 x4 x1 e cd) $! (addCs c cs))
     (Curry_AbstractHaskell.Fail_C_Expr d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_39" [(show x4),(show x1),(show x3)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_39" (showCons x3))

d_C__case_42 :: Curry_AbstractHaskell.C_Expr -> C_Options -> Curry_AbstractHaskell.C_Expr -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_42 x4 x1 x3 cd cs = case x3 of
     (Curry_AbstractHaskell.C_Apply x5 x6) -> d_C__case_41 x4 x1 x3 x6 x5 cd cs
     (Curry_AbstractHaskell.C_Var x45) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showITEApplication") cd cs
     (Curry_AbstractHaskell.C_Lit x46) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showITEApplication") cd cs
     (Curry_AbstractHaskell.C_Symbol x47) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showITEApplication") cd cs
     (Curry_AbstractHaskell.C_Lambda x48 x49) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showITEApplication") cd cs
     (Curry_AbstractHaskell.C_Let x50 x51) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showITEApplication") cd cs
     (Curry_AbstractHaskell.C_DoExpr x52) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showITEApplication") cd cs
     (Curry_AbstractHaskell.C_ListComp x53 x54) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showITEApplication") cd cs
     (Curry_AbstractHaskell.C_Case x55 x56) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showITEApplication") cd cs
     (Curry_AbstractHaskell.C_Typed x57 x58) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showITEApplication") cd cs
     (Curry_AbstractHaskell.C_IfThenElse x59 x60 x61) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showITEApplication") cd cs
     (Curry_AbstractHaskell.Choice_C_Expr d i l r) -> narrow d i (d_C__case_42 x4 x1 l cd cs) (d_C__case_42 x4 x1 r cd cs)
     (Curry_AbstractHaskell.Choices_C_Expr d i xs) -> narrows cs d i (\z -> d_C__case_42 x4 x1 z cd cs) xs
     (Curry_AbstractHaskell.Guard_C_Expr d c e) -> guardCons d c ((d_C__case_42 x4 x1 e cd) $! (addCs c cs))
     (Curry_AbstractHaskell.Fail_C_Expr d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_42" [(show x4),(show x1),(show x3)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_42" (showCons x3))

d_C__case_41 :: Curry_AbstractHaskell.C_Expr -> C_Options -> Curry_AbstractHaskell.C_Expr -> Curry_AbstractHaskell.C_Expr -> Curry_AbstractHaskell.C_Expr -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_41 x4 x1 x3 x6 x5 cd cs = case x5 of
     (Curry_AbstractHaskell.C_Apply x7 x8) -> d_C__case_40 x4 x1 x3 x6 x8 x7 cd cs
     (Curry_AbstractHaskell.C_Var x28) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showITEApplication") cd cs
     (Curry_AbstractHaskell.C_Lit x29) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showITEApplication") cd cs
     (Curry_AbstractHaskell.C_Symbol x30) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showITEApplication") cd cs
     (Curry_AbstractHaskell.C_Lambda x31 x32) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showITEApplication") cd cs
     (Curry_AbstractHaskell.C_Let x33 x34) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showITEApplication") cd cs
     (Curry_AbstractHaskell.C_DoExpr x35) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showITEApplication") cd cs
     (Curry_AbstractHaskell.C_ListComp x36 x37) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showITEApplication") cd cs
     (Curry_AbstractHaskell.C_Case x38 x39) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showITEApplication") cd cs
     (Curry_AbstractHaskell.C_Typed x40 x41) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showITEApplication") cd cs
     (Curry_AbstractHaskell.C_IfThenElse x42 x43 x44) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showITEApplication") cd cs
     (Curry_AbstractHaskell.Choice_C_Expr d i l r) -> narrow d i (d_C__case_41 x4 x1 x3 x6 l cd cs) (d_C__case_41 x4 x1 x3 x6 r cd cs)
     (Curry_AbstractHaskell.Choices_C_Expr d i xs) -> narrows cs d i (\z -> d_C__case_41 x4 x1 x3 x6 z cd cs) xs
     (Curry_AbstractHaskell.Guard_C_Expr d c e) -> guardCons d c ((d_C__case_41 x4 x1 x3 x6 e cd) $! (addCs c cs))
     (Curry_AbstractHaskell.Fail_C_Expr d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_41" [(show x4),(show x1),(show x3),(show x6),(show x5)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_41" (showCons x5))

d_C__case_40 :: Curry_AbstractHaskell.C_Expr -> C_Options -> Curry_AbstractHaskell.C_Expr -> Curry_AbstractHaskell.C_Expr -> Curry_AbstractHaskell.C_Expr -> Curry_AbstractHaskell.C_Expr -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_40 x4 x1 x3 x6 x8 x7 cd cs = case x7 of
     (Curry_AbstractHaskell.C_Symbol x9) -> Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString "if ") (Curry_Prelude.d_OP_plus_plus (d_C_showExprOpt x1 x8 cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString " then ") (Curry_Prelude.d_OP_plus_plus (d_C_showExprOpt x1 x6 cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString " else ") (d_C_showExprOpt x1 x4 cd cs) cd cs) cd cs) cd cs) cd cs) cd cs
     (Curry_AbstractHaskell.C_Apply x10 x11) -> Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString "(") (Curry_Prelude.d_OP_plus_plus (d_C_showITEApplication x1 x3 cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString ") ") (d_C_showBoxedExpr x1 x4 cd cs) cd cs) cd cs) cd cs
     (Curry_AbstractHaskell.C_Var x12) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showITEApplication") cd cs
     (Curry_AbstractHaskell.C_Lit x13) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showITEApplication") cd cs
     (Curry_AbstractHaskell.C_Lambda x14 x15) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showITEApplication") cd cs
     (Curry_AbstractHaskell.C_Let x16 x17) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showITEApplication") cd cs
     (Curry_AbstractHaskell.C_DoExpr x18) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showITEApplication") cd cs
     (Curry_AbstractHaskell.C_ListComp x19 x20) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showITEApplication") cd cs
     (Curry_AbstractHaskell.C_Case x21 x22) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showITEApplication") cd cs
     (Curry_AbstractHaskell.C_Typed x23 x24) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showITEApplication") cd cs
     (Curry_AbstractHaskell.C_IfThenElse x25 x26 x27) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showITEApplication") cd cs
     (Curry_AbstractHaskell.Choice_C_Expr d i l r) -> narrow d i (d_C__case_40 x4 x1 x3 x6 x8 l cd cs) (d_C__case_40 x4 x1 x3 x6 x8 r cd cs)
     (Curry_AbstractHaskell.Choices_C_Expr d i xs) -> narrows cs d i (\z -> d_C__case_40 x4 x1 x3 x6 x8 z cd cs) xs
     (Curry_AbstractHaskell.Guard_C_Expr d c e) -> guardCons d c ((d_C__case_40 x4 x1 x3 x6 x8 e cd) $! (addCs c cs))
     (Curry_AbstractHaskell.Fail_C_Expr d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_40" [(show x4),(show x1),(show x3),(show x6),(show x8),(show x7)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_40" (showCons x7))

d_C__case_44 :: Curry_AbstractHaskell.C_Expr -> C_Options -> Curry_Prelude.OP_Tuple2 (Curry_Prelude.OP_List Curry_Prelude.C_Char) (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Curry_AbstractHaskell.C_Expr -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_44 x5 x1 x2 x4 cd cs = case x4 of
     (Curry_AbstractHaskell.C_Apply x6 x7) -> d_C__case_43 x5 x1 x2 x7 x4 x6 cd cs
     (Curry_AbstractHaskell.C_Var x27) -> Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString "(") (Curry_Prelude.d_OP_plus_plus (d_C_showSymbol x1 x2 cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString ") ") (d_C_showBoxedExpr x1 x5 cd cs) cd cs) cd cs) cd cs
     (Curry_AbstractHaskell.C_Lit x28) -> Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString "(") (Curry_Prelude.d_OP_plus_plus (d_C_showSymbol x1 x2 cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString ") ") (d_C_showBoxedExpr x1 x5 cd cs) cd cs) cd cs) cd cs
     (Curry_AbstractHaskell.C_Symbol x29) -> Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString "(") (Curry_Prelude.d_OP_plus_plus (d_C_showSymbol x1 x2 cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString ") ") (d_C_showBoxedExpr x1 x5 cd cs) cd cs) cd cs) cd cs
     (Curry_AbstractHaskell.C_Lambda x30 x31) -> Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString "(") (Curry_Prelude.d_OP_plus_plus (d_C_showSymbol x1 x2 cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString ") ") (d_C_showBoxedExpr x1 x5 cd cs) cd cs) cd cs) cd cs
     (Curry_AbstractHaskell.C_Let x32 x33) -> Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString "(") (Curry_Prelude.d_OP_plus_plus (d_C_showSymbol x1 x2 cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString ") ") (d_C_showBoxedExpr x1 x5 cd cs) cd cs) cd cs) cd cs
     (Curry_AbstractHaskell.C_DoExpr x34) -> Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString "(") (Curry_Prelude.d_OP_plus_plus (d_C_showSymbol x1 x2 cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString ") ") (d_C_showBoxedExpr x1 x5 cd cs) cd cs) cd cs) cd cs
     (Curry_AbstractHaskell.C_ListComp x35 x36) -> Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString "(") (Curry_Prelude.d_OP_plus_plus (d_C_showSymbol x1 x2 cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString ") ") (d_C_showBoxedExpr x1 x5 cd cs) cd cs) cd cs) cd cs
     (Curry_AbstractHaskell.C_Case x37 x38) -> Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString "(") (Curry_Prelude.d_OP_plus_plus (d_C_showSymbol x1 x2 cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString ") ") (d_C_showBoxedExpr x1 x5 cd cs) cd cs) cd cs) cd cs
     (Curry_AbstractHaskell.C_Typed x39 x40) -> Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString "(") (Curry_Prelude.d_OP_plus_plus (d_C_showSymbol x1 x2 cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString ") ") (d_C_showBoxedExpr x1 x5 cd cs) cd cs) cd cs) cd cs
     (Curry_AbstractHaskell.C_IfThenElse x41 x42 x43) -> Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString "(") (Curry_Prelude.d_OP_plus_plus (d_C_showSymbol x1 x2 cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString ") ") (d_C_showBoxedExpr x1 x5 cd cs) cd cs) cd cs) cd cs
     (Curry_AbstractHaskell.Choice_C_Expr d i l r) -> narrow d i (d_C__case_44 x5 x1 x2 l cd cs) (d_C__case_44 x5 x1 x2 r cd cs)
     (Curry_AbstractHaskell.Choices_C_Expr d i xs) -> narrows cs d i (\z -> d_C__case_44 x5 x1 x2 z cd cs) xs
     (Curry_AbstractHaskell.Guard_C_Expr d c e) -> guardCons d c ((d_C__case_44 x5 x1 x2 e cd) $! (addCs c cs))
     (Curry_AbstractHaskell.Fail_C_Expr d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_44" [(show x5),(show x1),(show x2),(show x4)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_44" (showCons x4))

d_C__case_43 :: Curry_AbstractHaskell.C_Expr -> C_Options -> Curry_Prelude.OP_Tuple2 (Curry_Prelude.OP_List Curry_Prelude.C_Char) (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Curry_AbstractHaskell.C_Expr -> Curry_AbstractHaskell.C_Expr -> Curry_AbstractHaskell.C_Expr -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_43 x5 x1 x2 x7 x4 x6 cd cs = case x6 of
     (Curry_AbstractHaskell.C_Apply x8 x9) -> Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString "(") (Curry_Prelude.d_OP_plus_plus (d_C_showInfixApplication x1 x2 x4 cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString ") ") (d_C_showBoxedExpr x1 x5 cd cs) cd cs) cd cs) cd cs
     (Curry_AbstractHaskell.C_Var x10) -> Curry_Prelude.d_OP_plus_plus (d_C_showBoxedExpr x1 x7 cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString " ") (Curry_Prelude.d_OP_plus_plus (d_C_showSymbol x1 x2 cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString " ") (d_C_showBoxedExpr x1 x5 cd cs) cd cs) cd cs) cd cs) cd cs
     (Curry_AbstractHaskell.C_Lit x11) -> Curry_Prelude.d_OP_plus_plus (d_C_showBoxedExpr x1 x7 cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString " ") (Curry_Prelude.d_OP_plus_plus (d_C_showSymbol x1 x2 cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString " ") (d_C_showBoxedExpr x1 x5 cd cs) cd cs) cd cs) cd cs) cd cs
     (Curry_AbstractHaskell.C_Symbol x12) -> Curry_Prelude.d_OP_plus_plus (d_C_showBoxedExpr x1 x7 cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString " ") (Curry_Prelude.d_OP_plus_plus (d_C_showSymbol x1 x2 cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString " ") (d_C_showBoxedExpr x1 x5 cd cs) cd cs) cd cs) cd cs) cd cs
     (Curry_AbstractHaskell.C_Lambda x13 x14) -> Curry_Prelude.d_OP_plus_plus (d_C_showBoxedExpr x1 x7 cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString " ") (Curry_Prelude.d_OP_plus_plus (d_C_showSymbol x1 x2 cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString " ") (d_C_showBoxedExpr x1 x5 cd cs) cd cs) cd cs) cd cs) cd cs
     (Curry_AbstractHaskell.C_Let x15 x16) -> Curry_Prelude.d_OP_plus_plus (d_C_showBoxedExpr x1 x7 cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString " ") (Curry_Prelude.d_OP_plus_plus (d_C_showSymbol x1 x2 cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString " ") (d_C_showBoxedExpr x1 x5 cd cs) cd cs) cd cs) cd cs) cd cs
     (Curry_AbstractHaskell.C_DoExpr x17) -> Curry_Prelude.d_OP_plus_plus (d_C_showBoxedExpr x1 x7 cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString " ") (Curry_Prelude.d_OP_plus_plus (d_C_showSymbol x1 x2 cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString " ") (d_C_showBoxedExpr x1 x5 cd cs) cd cs) cd cs) cd cs) cd cs
     (Curry_AbstractHaskell.C_ListComp x18 x19) -> Curry_Prelude.d_OP_plus_plus (d_C_showBoxedExpr x1 x7 cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString " ") (Curry_Prelude.d_OP_plus_plus (d_C_showSymbol x1 x2 cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString " ") (d_C_showBoxedExpr x1 x5 cd cs) cd cs) cd cs) cd cs) cd cs
     (Curry_AbstractHaskell.C_Case x20 x21) -> Curry_Prelude.d_OP_plus_plus (d_C_showBoxedExpr x1 x7 cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString " ") (Curry_Prelude.d_OP_plus_plus (d_C_showSymbol x1 x2 cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString " ") (d_C_showBoxedExpr x1 x5 cd cs) cd cs) cd cs) cd cs) cd cs
     (Curry_AbstractHaskell.C_Typed x22 x23) -> Curry_Prelude.d_OP_plus_plus (d_C_showBoxedExpr x1 x7 cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString " ") (Curry_Prelude.d_OP_plus_plus (d_C_showSymbol x1 x2 cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString " ") (d_C_showBoxedExpr x1 x5 cd cs) cd cs) cd cs) cd cs) cd cs
     (Curry_AbstractHaskell.C_IfThenElse x24 x25 x26) -> Curry_Prelude.d_OP_plus_plus (d_C_showBoxedExpr x1 x7 cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString " ") (Curry_Prelude.d_OP_plus_plus (d_C_showSymbol x1 x2 cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString " ") (d_C_showBoxedExpr x1 x5 cd cs) cd cs) cd cs) cd cs) cd cs
     (Curry_AbstractHaskell.Choice_C_Expr d i l r) -> narrow d i (d_C__case_43 x5 x1 x2 x7 x4 l cd cs) (d_C__case_43 x5 x1 x2 x7 x4 r cd cs)
     (Curry_AbstractHaskell.Choices_C_Expr d i xs) -> narrows cs d i (\z -> d_C__case_43 x5 x1 x2 x7 x4 z cd cs) xs
     (Curry_AbstractHaskell.Guard_C_Expr d c e) -> guardCons d c ((d_C__case_43 x5 x1 x2 x7 x4 e cd) $! (addCs c cs))
     (Curry_AbstractHaskell.Fail_C_Expr d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_43" [(show x5),(show x1),(show x2),(show x7),(show x4),(show x6)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_43" (showCons x6))

d_C__case_47 :: Curry_AbstractHaskell.C_Expr -> C_Options -> Curry_AbstractHaskell.C_Expr -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_47 x4 x1 x3 cd cs = case x3 of
     (Curry_AbstractHaskell.C_Apply x5 x6) -> d_C__case_46 x1 x6 x4 cd cs
     (Curry_AbstractHaskell.C_Symbol x26) -> d_C__case_45 x4 x1 x26 cd cs
     (Curry_AbstractHaskell.C_Var x29) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showSimpleListApplication") cd cs
     (Curry_AbstractHaskell.C_Lit x30) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showSimpleListApplication") cd cs
     (Curry_AbstractHaskell.C_Lambda x31 x32) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showSimpleListApplication") cd cs
     (Curry_AbstractHaskell.C_Let x33 x34) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showSimpleListApplication") cd cs
     (Curry_AbstractHaskell.C_DoExpr x35) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showSimpleListApplication") cd cs
     (Curry_AbstractHaskell.C_ListComp x36 x37) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showSimpleListApplication") cd cs
     (Curry_AbstractHaskell.C_Case x38 x39) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showSimpleListApplication") cd cs
     (Curry_AbstractHaskell.C_Typed x40 x41) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showSimpleListApplication") cd cs
     (Curry_AbstractHaskell.C_IfThenElse x42 x43 x44) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showSimpleListApplication") cd cs
     (Curry_AbstractHaskell.Choice_C_Expr d i l r) -> narrow d i (d_C__case_47 x4 x1 l cd cs) (d_C__case_47 x4 x1 r cd cs)
     (Curry_AbstractHaskell.Choices_C_Expr d i xs) -> narrows cs d i (\z -> d_C__case_47 x4 x1 z cd cs) xs
     (Curry_AbstractHaskell.Guard_C_Expr d c e) -> guardCons d c ((d_C__case_47 x4 x1 e cd) $! (addCs c cs))
     (Curry_AbstractHaskell.Fail_C_Expr d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_47" [(show x4),(show x1),(show x3)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_47" (showCons x3))

d_C__case_45 :: Curry_AbstractHaskell.C_Expr -> C_Options -> Curry_Prelude.OP_Tuple2 (Curry_Prelude.OP_List Curry_Prelude.C_Char) (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_45 x4 x1 x26 cd cs = case x26 of
     (Curry_Prelude.OP_Tuple2 x27 x28) -> Curry_Prelude.d_OP_plus_plus (d_C_showBoxedExpr x1 x4 cd cs) x28 cd cs
     (Curry_Prelude.Choice_OP_Tuple2 d i l r) -> narrow d i (d_C__case_45 x4 x1 l cd cs) (d_C__case_45 x4 x1 r cd cs)
     (Curry_Prelude.Choices_OP_Tuple2 d i xs) -> narrows cs d i (\z -> d_C__case_45 x4 x1 z cd cs) xs
     (Curry_Prelude.Guard_OP_Tuple2 d c e) -> guardCons d c ((d_C__case_45 x4 x1 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_Tuple2 d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_45" [(show x4),(show x1),(show x26)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_45" (showCons x26))

d_C__case_46 :: C_Options -> Curry_AbstractHaskell.C_Expr -> Curry_AbstractHaskell.C_Expr -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_46 x1 x6 x4 cd cs = case x4 of
     (Curry_AbstractHaskell.C_Symbol x7) -> Curry_Prelude.d_OP_plus_plus (d_C_showBoxedExpr x1 x6 cd cs) (Curry_Prelude.toCurryString ":[]") cd cs
     (Curry_AbstractHaskell.C_Var x8) -> Curry_Prelude.d_OP_plus_plus (d_C_showBoxedExpr x1 x6 cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString ":") (d_C_showBoxedExpr x1 x4 cd cs) cd cs) cd cs
     (Curry_AbstractHaskell.C_Lit x9) -> Curry_Prelude.d_OP_plus_plus (d_C_showBoxedExpr x1 x6 cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString ":") (d_C_showBoxedExpr x1 x4 cd cs) cd cs) cd cs
     (Curry_AbstractHaskell.C_Apply x10 x11) -> Curry_Prelude.d_OP_plus_plus (d_C_showBoxedExpr x1 x6 cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString ":") (d_C_showBoxedExpr x1 x4 cd cs) cd cs) cd cs
     (Curry_AbstractHaskell.C_Lambda x12 x13) -> Curry_Prelude.d_OP_plus_plus (d_C_showBoxedExpr x1 x6 cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString ":") (d_C_showBoxedExpr x1 x4 cd cs) cd cs) cd cs
     (Curry_AbstractHaskell.C_Let x14 x15) -> Curry_Prelude.d_OP_plus_plus (d_C_showBoxedExpr x1 x6 cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString ":") (d_C_showBoxedExpr x1 x4 cd cs) cd cs) cd cs
     (Curry_AbstractHaskell.C_DoExpr x16) -> Curry_Prelude.d_OP_plus_plus (d_C_showBoxedExpr x1 x6 cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString ":") (d_C_showBoxedExpr x1 x4 cd cs) cd cs) cd cs
     (Curry_AbstractHaskell.C_ListComp x17 x18) -> Curry_Prelude.d_OP_plus_plus (d_C_showBoxedExpr x1 x6 cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString ":") (d_C_showBoxedExpr x1 x4 cd cs) cd cs) cd cs
     (Curry_AbstractHaskell.C_Case x19 x20) -> Curry_Prelude.d_OP_plus_plus (d_C_showBoxedExpr x1 x6 cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString ":") (d_C_showBoxedExpr x1 x4 cd cs) cd cs) cd cs
     (Curry_AbstractHaskell.C_Typed x21 x22) -> Curry_Prelude.d_OP_plus_plus (d_C_showBoxedExpr x1 x6 cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString ":") (d_C_showBoxedExpr x1 x4 cd cs) cd cs) cd cs
     (Curry_AbstractHaskell.C_IfThenElse x23 x24 x25) -> Curry_Prelude.d_OP_plus_plus (d_C_showBoxedExpr x1 x6 cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString ":") (d_C_showBoxedExpr x1 x4 cd cs) cd cs) cd cs
     (Curry_AbstractHaskell.Choice_C_Expr d i l r) -> narrow d i (d_C__case_46 x1 x6 l cd cs) (d_C__case_46 x1 x6 r cd cs)
     (Curry_AbstractHaskell.Choices_C_Expr d i xs) -> narrows cs d i (\z -> d_C__case_46 x1 x6 z cd cs) xs
     (Curry_AbstractHaskell.Guard_C_Expr d c e) -> guardCons d c ((d_C__case_46 x1 x6 e cd) $! (addCs c cs))
     (Curry_AbstractHaskell.Fail_C_Expr d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_46" [(show x1),(show x6),(show x4)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_46" (showCons x4))

d_C__case_49 :: Curry_AbstractHaskell.C_Expr -> C_Options -> Curry_AbstractHaskell.C_Expr -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_49 x4 x1 x3 cd cs = case x3 of
     (Curry_AbstractHaskell.C_Apply x5 x6) -> d_C__case_48 x1 x6 x4 cd cs
     (Curry_AbstractHaskell.C_Var x26) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showConsListApplication") cd cs
     (Curry_AbstractHaskell.C_Lit x27) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showConsListApplication") cd cs
     (Curry_AbstractHaskell.C_Symbol x28) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showConsListApplication") cd cs
     (Curry_AbstractHaskell.C_Lambda x29 x30) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showConsListApplication") cd cs
     (Curry_AbstractHaskell.C_Let x31 x32) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showConsListApplication") cd cs
     (Curry_AbstractHaskell.C_DoExpr x33) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showConsListApplication") cd cs
     (Curry_AbstractHaskell.C_ListComp x34 x35) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showConsListApplication") cd cs
     (Curry_AbstractHaskell.C_Case x36 x37) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showConsListApplication") cd cs
     (Curry_AbstractHaskell.C_Typed x38 x39) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showConsListApplication") cd cs
     (Curry_AbstractHaskell.C_IfThenElse x40 x41 x42) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showConsListApplication") cd cs
     (Curry_AbstractHaskell.Choice_C_Expr d i l r) -> narrow d i (d_C__case_49 x4 x1 l cd cs) (d_C__case_49 x4 x1 r cd cs)
     (Curry_AbstractHaskell.Choices_C_Expr d i xs) -> narrows cs d i (\z -> d_C__case_49 x4 x1 z cd cs) xs
     (Curry_AbstractHaskell.Guard_C_Expr d c e) -> guardCons d c ((d_C__case_49 x4 x1 e cd) $! (addCs c cs))
     (Curry_AbstractHaskell.Fail_C_Expr d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_49" [(show x4),(show x1),(show x3)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_49" (showCons x3))

d_C__case_48 :: C_Options -> Curry_AbstractHaskell.C_Expr -> Curry_AbstractHaskell.C_Expr -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_48 x1 x6 x4 cd cs = case x4 of
     (Curry_AbstractHaskell.C_Symbol x7) -> d_C_showBoxedExpr x1 x6 cd cs
     (Curry_AbstractHaskell.C_Var x8) -> Curry_Prelude.d_OP_plus_plus (d_C_showBoxedExpr x1 x6 cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString ",") (d_C_showConsListApplication x1 x4 cd cs) cd cs) cd cs
     (Curry_AbstractHaskell.C_Lit x9) -> Curry_Prelude.d_OP_plus_plus (d_C_showBoxedExpr x1 x6 cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString ",") (d_C_showConsListApplication x1 x4 cd cs) cd cs) cd cs
     (Curry_AbstractHaskell.C_Apply x10 x11) -> Curry_Prelude.d_OP_plus_plus (d_C_showBoxedExpr x1 x6 cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString ",") (d_C_showConsListApplication x1 x4 cd cs) cd cs) cd cs
     (Curry_AbstractHaskell.C_Lambda x12 x13) -> Curry_Prelude.d_OP_plus_plus (d_C_showBoxedExpr x1 x6 cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString ",") (d_C_showConsListApplication x1 x4 cd cs) cd cs) cd cs
     (Curry_AbstractHaskell.C_Let x14 x15) -> Curry_Prelude.d_OP_plus_plus (d_C_showBoxedExpr x1 x6 cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString ",") (d_C_showConsListApplication x1 x4 cd cs) cd cs) cd cs
     (Curry_AbstractHaskell.C_DoExpr x16) -> Curry_Prelude.d_OP_plus_plus (d_C_showBoxedExpr x1 x6 cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString ",") (d_C_showConsListApplication x1 x4 cd cs) cd cs) cd cs
     (Curry_AbstractHaskell.C_ListComp x17 x18) -> Curry_Prelude.d_OP_plus_plus (d_C_showBoxedExpr x1 x6 cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString ",") (d_C_showConsListApplication x1 x4 cd cs) cd cs) cd cs
     (Curry_AbstractHaskell.C_Case x19 x20) -> Curry_Prelude.d_OP_plus_plus (d_C_showBoxedExpr x1 x6 cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString ",") (d_C_showConsListApplication x1 x4 cd cs) cd cs) cd cs
     (Curry_AbstractHaskell.C_Typed x21 x22) -> Curry_Prelude.d_OP_plus_plus (d_C_showBoxedExpr x1 x6 cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString ",") (d_C_showConsListApplication x1 x4 cd cs) cd cs) cd cs
     (Curry_AbstractHaskell.C_IfThenElse x23 x24 x25) -> Curry_Prelude.d_OP_plus_plus (d_C_showBoxedExpr x1 x6 cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString ",") (d_C_showConsListApplication x1 x4 cd cs) cd cs) cd cs
     (Curry_AbstractHaskell.Choice_C_Expr d i l r) -> narrow d i (d_C__case_48 x1 x6 l cd cs) (d_C__case_48 x1 x6 r cd cs)
     (Curry_AbstractHaskell.Choices_C_Expr d i xs) -> narrows cs d i (\z -> d_C__case_48 x1 x6 z cd cs) xs
     (Curry_AbstractHaskell.Guard_C_Expr d c e) -> guardCons d c ((d_C__case_48 x1 x6 e cd) $! (addCs c cs))
     (Curry_AbstractHaskell.Fail_C_Expr d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_48" [(show x1),(show x6),(show x4)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_48" (showCons x4))

d_C__case_51 :: Curry_AbstractHaskell.C_Expr -> C_Options -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_51 x2 x1 x3 cd cs = case x3 of
     Curry_Prelude.C_True -> Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString "[") (Curry_Prelude.d_OP_plus_plus (d_C_showConsListApplication x1 x2 cd cs) (Curry_Prelude.toCurryString "]") cd cs) cd cs
     Curry_Prelude.C_False -> d_C__case_50 x2 x1 (Curry_Prelude.d_C_otherwise cd cs) cd cs
     (Curry_Prelude.Choice_C_Bool d i l r) -> narrow d i (d_C__case_51 x2 x1 l cd cs) (d_C__case_51 x2 x1 r cd cs)
     (Curry_Prelude.Choices_C_Bool d i xs) -> narrows cs d i (\z -> d_C__case_51 x2 x1 z cd cs) xs
     (Curry_Prelude.Guard_C_Bool d c e) -> guardCons d c ((d_C__case_51 x2 x1 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_C_Bool d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_51" [(show x2),(show x1),(show x3)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_51" (showCons x3))

d_C__case_50 :: Curry_AbstractHaskell.C_Expr -> C_Options -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_50 x2 x1 x3 cd cs = case x3 of
     Curry_Prelude.C_True -> Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString "(") (Curry_Prelude.d_OP_plus_plus (d_C_showSimpleListApplication x1 x2 cd cs) (Curry_Prelude.toCurryString ")") cd cs) cd cs
     Curry_Prelude.C_False -> Curry_Prelude.d_C_failed cd cs
     (Curry_Prelude.Choice_C_Bool d i l r) -> narrow d i (d_C__case_50 x2 x1 l cd cs) (d_C__case_50 x2 x1 r cd cs)
     (Curry_Prelude.Choices_C_Bool d i xs) -> narrows cs d i (\z -> d_C__case_50 x2 x1 z cd cs) xs
     (Curry_Prelude.Guard_C_Bool d c e) -> guardCons d c ((d_C__case_50 x2 x1 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_C_Bool d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_50" [(show x2),(show x1),(show x3)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_50" (showCons x3))

d_C__case_56 :: Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_AbstractHaskell.C_Expr -> C_Options -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_56 x5 x4 x3 x1 x6 cd cs = case x6 of
     Curry_Prelude.C_True -> d_C_showListApplication x1 x3 cd cs
     Curry_Prelude.C_False -> d_C__case_55 x5 x4 x3 x1 (Curry_Prelude.d_C_apply (d_C_isInfixOpName cd cs) x5 cd cs) cd cs
     (Curry_Prelude.Choice_C_Bool d i l r) -> narrow d i (d_C__case_56 x5 x4 x3 x1 l cd cs) (d_C__case_56 x5 x4 x3 x1 r cd cs)
     (Curry_Prelude.Choices_C_Bool d i xs) -> narrows cs d i (\z -> d_C__case_56 x5 x4 x3 x1 z cd cs) xs
     (Curry_Prelude.Guard_C_Bool d c e) -> guardCons d c ((d_C__case_56 x5 x4 x3 x1 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_C_Bool d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_56" [(show x5),(show x4),(show x3),(show x1),(show x6)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_56" (showCons x6))

d_C__case_55 :: Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_AbstractHaskell.C_Expr -> C_Options -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_55 x5 x4 x3 x1 x6 cd cs = case x6 of
     Curry_Prelude.C_True -> d_C_showInfixApplication x1 (Curry_Prelude.OP_Tuple2 x4 x5) x3 cd cs
     Curry_Prelude.C_False -> d_C__case_54 x5 x4 x3 x1 (Curry_Prelude.d_OP_amp_amp (Curry_Prelude.d_OP_eq_eq x4 (Curry_Names.d_C_prelude cd cs) cd cs) (Curry_Prelude.d_OP_eq_eq x5 (Curry_Prelude.toCurryString "if_then_else") cd cs) cd cs) cd cs
     (Curry_Prelude.Choice_C_Bool d i l r) -> narrow d i (d_C__case_55 x5 x4 x3 x1 l cd cs) (d_C__case_55 x5 x4 x3 x1 r cd cs)
     (Curry_Prelude.Choices_C_Bool d i xs) -> narrows cs d i (\z -> d_C__case_55 x5 x4 x3 x1 z cd cs) xs
     (Curry_Prelude.Guard_C_Bool d c e) -> guardCons d c ((d_C__case_55 x5 x4 x3 x1 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_C_Bool d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_55" [(show x5),(show x4),(show x3),(show x1),(show x6)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_55" (showCons x6))

d_C__case_54 :: Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_AbstractHaskell.C_Expr -> C_Options -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_54 x5 x4 x3 x1 x6 cd cs = case x6 of
     Curry_Prelude.C_True -> d_C_showITEApplication x1 x3 cd cs
     Curry_Prelude.C_False -> d_C__case_53 x5 x3 x1 (d_C_isTuple x5 cd cs) cd cs
     (Curry_Prelude.Choice_C_Bool d i l r) -> narrow d i (d_C__case_54 x5 x4 x3 x1 l cd cs) (d_C__case_54 x5 x4 x3 x1 r cd cs)
     (Curry_Prelude.Choices_C_Bool d i xs) -> narrows cs d i (\z -> d_C__case_54 x5 x4 x3 x1 z cd cs) xs
     (Curry_Prelude.Guard_C_Bool d c e) -> guardCons d c ((d_C__case_54 x5 x4 x3 x1 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_C_Bool d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_54" [(show x5),(show x4),(show x3),(show x1),(show x6)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_54" (showCons x6))

d_C__case_53 :: Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_AbstractHaskell.C_Expr -> C_Options -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_53 x5 x3 x1 x6 cd cs = case x6 of
     Curry_Prelude.C_True -> d_C_showTupleApplication x1 x3 cd cs
     Curry_Prelude.C_False -> d_C__case_52 x3 x1 (Curry_Prelude.d_C_otherwise cd cs) cd cs
     (Curry_Prelude.Choice_C_Bool d i l r) -> narrow d i (d_C__case_53 x5 x3 x1 l cd cs) (d_C__case_53 x5 x3 x1 r cd cs)
     (Curry_Prelude.Choices_C_Bool d i xs) -> narrows cs d i (\z -> d_C__case_53 x5 x3 x1 z cd cs) xs
     (Curry_Prelude.Guard_C_Bool d c e) -> guardCons d c ((d_C__case_53 x5 x3 x1 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_C_Bool d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_53" [(show x5),(show x3),(show x1),(show x6)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_53" (showCons x6))

d_C__case_52 :: Curry_AbstractHaskell.C_Expr -> C_Options -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_52 x3 x1 x4 cd cs = case x4 of
     Curry_Prelude.C_True -> d_C_showSimpleApplication x1 x3 cd cs
     Curry_Prelude.C_False -> Curry_Prelude.d_C_failed cd cs
     (Curry_Prelude.Choice_C_Bool d i l r) -> narrow d i (d_C__case_52 x3 x1 l cd cs) (d_C__case_52 x3 x1 r cd cs)
     (Curry_Prelude.Choices_C_Bool d i xs) -> narrows cs d i (\z -> d_C__case_52 x3 x1 z cd cs) xs
     (Curry_Prelude.Guard_C_Bool d c e) -> guardCons d c ((d_C__case_52 x3 x1 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_C_Bool d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_52" [(show x3),(show x1),(show x4)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_52" (showCons x4))

d_C__case_57 :: Curry_AbstractHaskell.C_Expr -> C_Options -> Curry_AbstractHaskell.C_Expr -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_57 x2 x1 x22 cd cs = case x22 of
     (Curry_AbstractHaskell.C_Symbol x3) -> d_C_showSymbolApplication x1 x3 x2 cd cs
     (Curry_AbstractHaskell.C_Var x4) -> d_C_showSimpleApplication x1 x2 cd cs
     (Curry_AbstractHaskell.C_Lit x5) -> d_C_showSimpleApplication x1 x2 cd cs
     (Curry_AbstractHaskell.C_Apply x6 x7) -> d_C_showSimpleApplication x1 x2 cd cs
     (Curry_AbstractHaskell.C_Lambda x8 x9) -> d_C_showSimpleApplication x1 x2 cd cs
     (Curry_AbstractHaskell.C_Let x10 x11) -> d_C_showSimpleApplication x1 x2 cd cs
     (Curry_AbstractHaskell.C_DoExpr x12) -> d_C_showSimpleApplication x1 x2 cd cs
     (Curry_AbstractHaskell.C_ListComp x13 x14) -> d_C_showSimpleApplication x1 x2 cd cs
     (Curry_AbstractHaskell.C_Case x15 x16) -> d_C_showSimpleApplication x1 x2 cd cs
     (Curry_AbstractHaskell.C_Typed x17 x18) -> d_C_showSimpleApplication x1 x2 cd cs
     (Curry_AbstractHaskell.C_IfThenElse x19 x20 x21) -> d_C_showSimpleApplication x1 x2 cd cs
     (Curry_AbstractHaskell.Choice_C_Expr d i l r) -> narrow d i (d_C__case_57 x2 x1 l cd cs) (d_C__case_57 x2 x1 r cd cs)
     (Curry_AbstractHaskell.Choices_C_Expr d i xs) -> narrows cs d i (\z -> d_C__case_57 x2 x1 z cd cs) xs
     (Curry_AbstractHaskell.Guard_C_Expr d c e) -> guardCons d c ((d_C__case_57 x2 x1 e cd) $! (addCs c cs))
     (Curry_AbstractHaskell.Fail_C_Expr d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_57" [(show x2),(show x1),(show x22)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_57" (showCons x22))

d_C__case_61 :: Curry_Prelude.OP_List Curry_AbstractHaskell.C_TypeExpr -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> C_Options -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_61 x3 x2 x1 x4 cd cs = case x4 of
     Curry_Prelude.C_True -> Curry_Prelude.toCurryString "String"
     Curry_Prelude.C_False -> d_C__case_60 x2 x3 x1 (Curry_Prelude.d_OP_eq_eq x2 (Curry_Prelude.toCurryString "[]") cd cs) cd cs
     (Curry_Prelude.Choice_C_Bool d i l r) -> narrow d i (d_C__case_61 x3 x2 x1 l cd cs) (d_C__case_61 x3 x2 x1 r cd cs)
     (Curry_Prelude.Choices_C_Bool d i xs) -> narrows cs d i (\z -> d_C__case_61 x3 x2 x1 z cd cs) xs
     (Curry_Prelude.Guard_C_Bool d c e) -> guardCons d c ((d_C__case_61 x3 x2 x1 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_C_Bool d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_61" [(show x3),(show x2),(show x1),(show x4)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_61" (showCons x4))

d_C__case_60 :: Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_TypeExpr -> C_Options -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_60 x2 x3 x1 x4 cd cs = case x4 of
     Curry_Prelude.C_True -> Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString "[") (Curry_Prelude.d_OP_plus_plus (d_C_showTypeExpr x1 Curry_Prelude.C_False (Curry_Prelude.d_C_head x3 cd cs) cd cs) (Curry_Prelude.toCurryString "]") cd cs) cd cs
     Curry_Prelude.C_False -> d_C__case_59 x2 x3 x1 (d_C_isTuple x2 cd cs) cd cs
     (Curry_Prelude.Choice_C_Bool d i l r) -> narrow d i (d_C__case_60 x2 x3 x1 l cd cs) (d_C__case_60 x2 x3 x1 r cd cs)
     (Curry_Prelude.Choices_C_Bool d i xs) -> narrows cs d i (\z -> d_C__case_60 x2 x3 x1 z cd cs) xs
     (Curry_Prelude.Guard_C_Bool d c e) -> guardCons d c ((d_C__case_60 x2 x3 x1 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_C_Bool d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_60" [(show x2),(show x3),(show x1),(show x4)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_60" (showCons x4))

d_C__case_59 :: Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_TypeExpr -> C_Options -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_59 x2 x3 x1 x4 cd cs = case x4 of
     Curry_Prelude.C_True -> Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString "(") (Curry_Prelude.d_OP_plus_plus (d_C_combineMap (d_C_showTypeExpr x1 Curry_Prelude.C_False) x3 (Curry_Prelude.toCurryString ",") cd cs) (Curry_Prelude.toCurryString ")") cd cs) cd cs
     Curry_Prelude.C_False -> d_C__case_58 x3 x1 x2 (Curry_Prelude.d_C_otherwise cd cs) cd cs
     (Curry_Prelude.Choice_C_Bool d i l r) -> narrow d i (d_C__case_59 x2 x3 x1 l cd cs) (d_C__case_59 x2 x3 x1 r cd cs)
     (Curry_Prelude.Choices_C_Bool d i xs) -> narrows cs d i (\z -> d_C__case_59 x2 x3 x1 z cd cs) xs
     (Curry_Prelude.Guard_C_Bool d c e) -> guardCons d c ((d_C__case_59 x2 x3 x1 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_C_Bool d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_59" [(show x2),(show x3),(show x1),(show x4)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_59" (showCons x4))

d_C__case_58 :: Curry_Prelude.OP_List Curry_AbstractHaskell.C_TypeExpr -> C_Options -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_58 x3 x1 x2 x4 cd cs = case x4 of
     Curry_Prelude.C_True -> Curry_Prelude.d_OP_plus_plus x2 (d_C_prefixMap (d_C_showTypeExpr x1 Curry_Prelude.C_True) x3 (Curry_Prelude.toCurryString " ") cd cs) cd cs
     Curry_Prelude.C_False -> Curry_Prelude.d_C_failed cd cs
     (Curry_Prelude.Choice_C_Bool d i l r) -> narrow d i (d_C__case_58 x3 x1 x2 l cd cs) (d_C__case_58 x3 x1 x2 r cd cs)
     (Curry_Prelude.Choices_C_Bool d i xs) -> narrows cs d i (\z -> d_C__case_58 x3 x1 x2 z cd cs) xs
     (Curry_Prelude.Guard_C_Bool d c e) -> guardCons d c ((d_C__case_58 x3 x1 x2 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_C_Bool d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_58" [(show x3),(show x1),(show x2),(show x4)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_58" (showCons x4))

d_C__case_64 :: Curry_Prelude.OP_List Curry_AbstractHaskell.C_TypeExpr -> Curry_AbstractHaskell.C_TypeExpr -> C_Options -> Curry_Prelude.OP_Tuple2 (Curry_Prelude.OP_List Curry_Prelude.C_Char) (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_64 x5 x4 x1 x2 cd cs = case x2 of
     (Curry_Prelude.OP_Tuple2 x6 x7) -> d_C__case_63 x6 x5 x4 x1 x2 x7 (Curry_Prelude.d_OP_eq_eq x6 (Curry_Names.d_C_prelude cd cs) cd cs) cd cs
     (Curry_Prelude.Choice_OP_Tuple2 d i l r) -> narrow d i (d_C__case_64 x5 x4 x1 l cd cs) (d_C__case_64 x5 x4 x1 r cd cs)
     (Curry_Prelude.Choices_OP_Tuple2 d i xs) -> narrows cs d i (\z -> d_C__case_64 x5 x4 x1 z cd cs) xs
     (Curry_Prelude.Guard_OP_Tuple2 d c e) -> guardCons d c ((d_C__case_64 x5 x4 x1 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_Tuple2 d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_64" [(show x5),(show x4),(show x1),(show x2)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_64" (showCons x2))

d_C__case_63 :: Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_TypeExpr -> Curry_AbstractHaskell.C_TypeExpr -> C_Options -> Curry_Prelude.OP_Tuple2 (Curry_Prelude.OP_List Curry_Prelude.C_Char) (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_63 x6 x5 x4 x1 x2 x7 x8 cd cs = case x8 of
     Curry_Prelude.C_True -> d_C_showPreludeTypeCons x1 x7 (Curry_Prelude.OP_Cons x4 x5) cd cs
     Curry_Prelude.C_False -> d_C__case_62 x5 x4 x1 x2 (Curry_Prelude.d_C_otherwise cd cs) cd cs
     (Curry_Prelude.Choice_C_Bool d i l r) -> narrow d i (d_C__case_63 x6 x5 x4 x1 x2 x7 l cd cs) (d_C__case_63 x6 x5 x4 x1 x2 x7 r cd cs)
     (Curry_Prelude.Choices_C_Bool d i xs) -> narrows cs d i (\z -> d_C__case_63 x6 x5 x4 x1 x2 x7 z cd cs) xs
     (Curry_Prelude.Guard_C_Bool d c e) -> guardCons d c ((d_C__case_63 x6 x5 x4 x1 x2 x7 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_C_Bool d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_63" [(show x6),(show x5),(show x4),(show x1),(show x2),(show x7),(show x8)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_63" (showCons x8))

d_C__case_62 :: Curry_Prelude.OP_List Curry_AbstractHaskell.C_TypeExpr -> Curry_AbstractHaskell.C_TypeExpr -> C_Options -> Curry_Prelude.OP_Tuple2 (Curry_Prelude.OP_List Curry_Prelude.C_Char) (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_62 x5 x4 x1 x2 x6 cd cs = case x6 of
     Curry_Prelude.C_True -> Curry_Prelude.d_OP_plus_plus (d_C_showSymbol x1 x2 cd cs) (d_C_prefixMap (d_C_showTypeExpr x1 Curry_Prelude.C_True) (Curry_Prelude.OP_Cons x4 x5) (Curry_Prelude.toCurryString " ") cd cs) cd cs
     Curry_Prelude.C_False -> Curry_Prelude.d_C_failed cd cs
     (Curry_Prelude.Choice_C_Bool d i l r) -> narrow d i (d_C__case_62 x5 x4 x1 x2 l cd cs) (d_C__case_62 x5 x4 x1 x2 r cd cs)
     (Curry_Prelude.Choices_C_Bool d i xs) -> narrows cs d i (\z -> d_C__case_62 x5 x4 x1 x2 z cd cs) xs
     (Curry_Prelude.Guard_C_Bool d c e) -> guardCons d c ((d_C__case_62 x5 x4 x1 x2 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_C_Bool d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_62" [(show x5),(show x4),(show x1),(show x2),(show x6)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_62" (showCons x6))

d_C__case_65 :: Curry_AbstractHaskell.C_Pattern -> C_Options -> Curry_Prelude.OP_Tuple2 Curry_Prelude.C_Int (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_65 x4 x1 x3 cd cs = case x3 of
     (Curry_Prelude.OP_Tuple2 x5 x6) -> Curry_Prelude.d_OP_plus_plus x6 (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString "@") (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString "(") (Curry_Prelude.d_OP_plus_plus (Curry_List.d_C_intercalate (Curry_Prelude.toCurryString ":") (d_C_showPatListElems x1 x4 cd cs) cd cs) (Curry_Prelude.toCurryString ")") cd cs) cd cs) cd cs) cd cs
     (Curry_Prelude.Choice_OP_Tuple2 d i l r) -> narrow d i (d_C__case_65 x4 x1 l cd cs) (d_C__case_65 x4 x1 r cd cs)
     (Curry_Prelude.Choices_OP_Tuple2 d i xs) -> narrows cs d i (\z -> d_C__case_65 x4 x1 z cd cs) xs
     (Curry_Prelude.Guard_OP_Tuple2 d c e) -> guardCons d c ((d_C__case_65 x4 x1 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_Tuple2 d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_65" [(show x4),(show x1),(show x3)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_65" (showCons x3))

d_C__case_77 :: Curry_Prelude.OP_List Curry_AbstractHaskell.C_Pattern -> C_Options -> Curry_Prelude.OP_Tuple2 (Curry_Prelude.OP_List Curry_Prelude.C_Char) (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Cover -> ConstStore -> Curry_Prelude.OP_List (Curry_Prelude.OP_List Curry_Prelude.C_Char)
d_C__case_77 x4 x1 x3 cd cs = case x3 of
     (Curry_Prelude.OP_Tuple2 x5 x6) -> d_C__case_76 x4 x1 x6 cd cs
     (Curry_Prelude.Choice_OP_Tuple2 d i l r) -> narrow d i (d_C__case_77 x4 x1 l cd cs) (d_C__case_77 x4 x1 r cd cs)
     (Curry_Prelude.Choices_OP_Tuple2 d i xs) -> narrows cs d i (\z -> d_C__case_77 x4 x1 z cd cs) xs
     (Curry_Prelude.Guard_OP_Tuple2 d c e) -> guardCons d c ((d_C__case_77 x4 x1 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_Tuple2 d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_77" [(show x4),(show x1),(show x3)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_77" (showCons x3))

d_C__case_76 :: Curry_Prelude.OP_List Curry_AbstractHaskell.C_Pattern -> C_Options -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Cover -> ConstStore -> Curry_Prelude.OP_List (Curry_Prelude.OP_List Curry_Prelude.C_Char)
d_C__case_76 x4 x1 x6 cd cs = case x6 of
     (Curry_Prelude.OP_Cons x7 x8) -> let
          x9 = x7
           in (d_C__case_75 x9 x8 x4 x1 (Curry_Prelude.d_OP_eq_eq x9 (Curry_Prelude.C_Char ':'#) cd cs) cd cs)
     Curry_Prelude.OP_List -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showPatListElems") cd cs
     (Curry_Prelude.Choice_OP_List d i l r) -> narrow d i (d_C__case_76 x4 x1 l cd cs) (d_C__case_76 x4 x1 r cd cs)
     (Curry_Prelude.Choices_OP_List d i xs) -> narrows cs d i (\z -> d_C__case_76 x4 x1 z cd cs) xs
     (Curry_Prelude.Guard_OP_List d c e) -> guardCons d c ((d_C__case_76 x4 x1 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_List d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_76" [(show x4),(show x1),(show x6)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_76" (showCons x6))

d_C__case_75 :: Curry_Prelude.C_Char -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_Pattern -> C_Options -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> Curry_Prelude.OP_List (Curry_Prelude.OP_List Curry_Prelude.C_Char)
d_C__case_75 x9 x8 x4 x1 x10 cd cs = case x10 of
     Curry_Prelude.C_True -> d_C__case_74 x4 x1 x8 cd cs
     Curry_Prelude.C_False -> d_C__case_70 x9 x8 x4 (Curry_Prelude.d_OP_eq_eq x9 (Curry_Prelude.C_Char '['#) cd cs) cd cs
     (Curry_Prelude.Choice_C_Bool d i l r) -> narrow d i (d_C__case_75 x9 x8 x4 x1 l cd cs) (d_C__case_75 x9 x8 x4 x1 r cd cs)
     (Curry_Prelude.Choices_C_Bool d i xs) -> narrows cs d i (\z -> d_C__case_75 x9 x8 x4 x1 z cd cs) xs
     (Curry_Prelude.Guard_C_Bool d c e) -> guardCons d c ((d_C__case_75 x9 x8 x4 x1 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_C_Bool d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_75" [(show x9),(show x8),(show x4),(show x1),(show x10)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_75" (showCons x10))

d_C__case_70 :: Curry_Prelude.C_Char -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_Pattern -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> Curry_Prelude.OP_List (Curry_Prelude.OP_List Curry_Prelude.C_Char)
d_C__case_70 x9 x8 x4 x10 cd cs = case x10 of
     Curry_Prelude.C_True -> d_C__case_69 x4 x8 cd cs
     Curry_Prelude.C_False -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showPatListElems") cd cs
     (Curry_Prelude.Choice_C_Bool d i l r) -> narrow d i (d_C__case_70 x9 x8 x4 l cd cs) (d_C__case_70 x9 x8 x4 r cd cs)
     (Curry_Prelude.Choices_C_Bool d i xs) -> narrows cs d i (\z -> d_C__case_70 x9 x8 x4 z cd cs) xs
     (Curry_Prelude.Guard_C_Bool d c e) -> guardCons d c ((d_C__case_70 x9 x8 x4 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_C_Bool d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_70" [(show x9),(show x8),(show x4),(show x10)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_70" (showCons x10))

d_C__case_69 :: Curry_Prelude.OP_List Curry_AbstractHaskell.C_Pattern -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Cover -> ConstStore -> Curry_Prelude.OP_List (Curry_Prelude.OP_List Curry_Prelude.C_Char)
d_C__case_69 x4 x8 cd cs = case x8 of
     (Curry_Prelude.OP_Cons x18 x19) -> let
          x20 = x18
           in (d_C__case_68 x20 x19 x4 (Curry_Prelude.d_OP_eq_eq x20 (Curry_Prelude.C_Char ']'#) cd cs) cd cs)
     Curry_Prelude.OP_List -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showPatListElems") cd cs
     (Curry_Prelude.Choice_OP_List d i l r) -> narrow d i (d_C__case_69 x4 l cd cs) (d_C__case_69 x4 r cd cs)
     (Curry_Prelude.Choices_OP_List d i xs) -> narrows cs d i (\z -> d_C__case_69 x4 z cd cs) xs
     (Curry_Prelude.Guard_OP_List d c e) -> guardCons d c ((d_C__case_69 x4 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_List d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_69" [(show x4),(show x8)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_69" (showCons x8))

d_C__case_68 :: Curry_Prelude.C_Char -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_Pattern -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> Curry_Prelude.OP_List (Curry_Prelude.OP_List Curry_Prelude.C_Char)
d_C__case_68 x20 x19 x4 x21 cd cs = case x21 of
     Curry_Prelude.C_True -> d_C__case_67 x4 x19 cd cs
     Curry_Prelude.C_False -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showPatListElems") cd cs
     (Curry_Prelude.Choice_C_Bool d i l r) -> narrow d i (d_C__case_68 x20 x19 x4 l cd cs) (d_C__case_68 x20 x19 x4 r cd cs)
     (Curry_Prelude.Choices_C_Bool d i xs) -> narrows cs d i (\z -> d_C__case_68 x20 x19 x4 z cd cs) xs
     (Curry_Prelude.Guard_C_Bool d c e) -> guardCons d c ((d_C__case_68 x20 x19 x4 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_C_Bool d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_68" [(show x20),(show x19),(show x4),(show x21)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_68" (showCons x21))

d_C__case_67 :: Curry_Prelude.OP_List Curry_AbstractHaskell.C_Pattern -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Cover -> ConstStore -> Curry_Prelude.OP_List (Curry_Prelude.OP_List Curry_Prelude.C_Char)
d_C__case_67 x4 x19 cd cs = case x19 of
     Curry_Prelude.OP_List -> d_C__case_66 x4 cd cs
     (Curry_Prelude.OP_Cons x23 x24) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showPatListElems") cd cs
     (Curry_Prelude.Choice_OP_List d i l r) -> narrow d i (d_C__case_67 x4 l cd cs) (d_C__case_67 x4 r cd cs)
     (Curry_Prelude.Choices_OP_List d i xs) -> narrows cs d i (\z -> d_C__case_67 x4 z cd cs) xs
     (Curry_Prelude.Guard_OP_List d c e) -> guardCons d c ((d_C__case_67 x4 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_List d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_67" [(show x4),(show x19)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_67" (showCons x19))

d_C__case_66 :: Curry_Prelude.OP_List Curry_AbstractHaskell.C_Pattern -> Cover -> ConstStore -> Curry_Prelude.OP_List (Curry_Prelude.OP_List Curry_Prelude.C_Char)
d_C__case_66 x4 cd cs = case x4 of
     Curry_Prelude.OP_List -> Curry_Prelude.OP_List
     (Curry_Prelude.OP_Cons x21 x22) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showPatListElems") cd cs
     (Curry_Prelude.Choice_OP_List d i l r) -> narrow d i (d_C__case_66 l cd cs) (d_C__case_66 r cd cs)
     (Curry_Prelude.Choices_OP_List d i xs) -> narrows cs d i (\z -> d_C__case_66 z cd cs) xs
     (Curry_Prelude.Guard_OP_List d c e) -> guardCons d c ((d_C__case_66 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_List d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_66" [(show x4)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_66" (showCons x4))

d_C__case_74 :: Curry_Prelude.OP_List Curry_AbstractHaskell.C_Pattern -> C_Options -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Cover -> ConstStore -> Curry_Prelude.OP_List (Curry_Prelude.OP_List Curry_Prelude.C_Char)
d_C__case_74 x4 x1 x8 cd cs = case x8 of
     Curry_Prelude.OP_List -> d_C__case_73 x1 x4 cd cs
     (Curry_Prelude.OP_Cons x16 x17) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showPatListElems") cd cs
     (Curry_Prelude.Choice_OP_List d i l r) -> narrow d i (d_C__case_74 x4 x1 l cd cs) (d_C__case_74 x4 x1 r cd cs)
     (Curry_Prelude.Choices_OP_List d i xs) -> narrows cs d i (\z -> d_C__case_74 x4 x1 z cd cs) xs
     (Curry_Prelude.Guard_OP_List d c e) -> guardCons d c ((d_C__case_74 x4 x1 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_List d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_74" [(show x4),(show x1),(show x8)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_74" (showCons x8))

d_C__case_73 :: C_Options -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_Pattern -> Cover -> ConstStore -> Curry_Prelude.OP_List (Curry_Prelude.OP_List Curry_Prelude.C_Char)
d_C__case_73 x1 x4 cd cs = case x4 of
     (Curry_Prelude.OP_Cons x10 x11) -> d_C__case_72 x1 x10 x11 cd cs
     Curry_Prelude.OP_List -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showPatListElems") cd cs
     (Curry_Prelude.Choice_OP_List d i l r) -> narrow d i (d_C__case_73 x1 l cd cs) (d_C__case_73 x1 r cd cs)
     (Curry_Prelude.Choices_OP_List d i xs) -> narrows cs d i (\z -> d_C__case_73 x1 z cd cs) xs
     (Curry_Prelude.Guard_OP_List d c e) -> guardCons d c ((d_C__case_73 x1 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_List d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_73" [(show x1),(show x4)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_73" (showCons x4))

d_C__case_72 :: C_Options -> Curry_AbstractHaskell.C_Pattern -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_Pattern -> Cover -> ConstStore -> Curry_Prelude.OP_List (Curry_Prelude.OP_List Curry_Prelude.C_Char)
d_C__case_72 x1 x10 x11 cd cs = case x11 of
     (Curry_Prelude.OP_Cons x12 x13) -> d_C__case_71 x12 x1 x10 x13 cd cs
     Curry_Prelude.OP_List -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showPatListElems") cd cs
     (Curry_Prelude.Choice_OP_List d i l r) -> narrow d i (d_C__case_72 x1 x10 l cd cs) (d_C__case_72 x1 x10 r cd cs)
     (Curry_Prelude.Choices_OP_List d i xs) -> narrows cs d i (\z -> d_C__case_72 x1 x10 z cd cs) xs
     (Curry_Prelude.Guard_OP_List d c e) -> guardCons d c ((d_C__case_72 x1 x10 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_List d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_72" [(show x1),(show x10),(show x11)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_72" (showCons x11))

d_C__case_71 :: Curry_AbstractHaskell.C_Pattern -> C_Options -> Curry_AbstractHaskell.C_Pattern -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_Pattern -> Cover -> ConstStore -> Curry_Prelude.OP_List (Curry_Prelude.OP_List Curry_Prelude.C_Char)
d_C__case_71 x12 x1 x10 x13 cd cs = case x13 of
     Curry_Prelude.OP_List -> Curry_Prelude.OP_Cons (d_C_showPattern x1 x10 cd cs) (d_C_showPatListElems x1 x12 cd cs)
     (Curry_Prelude.OP_Cons x14 x15) -> Curry_Prelude.d_C_error (Curry_Prelude.toCurryString "AbstractHaskellPrinter.showPatListElems") cd cs
     (Curry_Prelude.Choice_OP_List d i l r) -> narrow d i (d_C__case_71 x12 x1 x10 l cd cs) (d_C__case_71 x12 x1 x10 r cd cs)
     (Curry_Prelude.Choices_OP_List d i xs) -> narrows cs d i (\z -> d_C__case_71 x12 x1 x10 z cd cs) xs
     (Curry_Prelude.Guard_OP_List d c e) -> guardCons d c ((d_C__case_71 x12 x1 x10 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_List d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_71" [(show x12),(show x1),(show x10),(show x13)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_71" (showCons x13))

d_C__case_81 :: Curry_AbstractHaskell.C_Pattern -> C_Options -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_81 x2 x1 x3 cd cs = case x3 of
     Curry_Prelude.C_True -> Curry_Prelude.OP_Cons (Curry_Prelude.C_Char '"'#) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.d_C_filter (Curry_Prelude.d_C_flip (acceptCs id Curry_Prelude.d_OP_slash_eq) (Curry_Prelude.C_Char '\''#)) (Curry_Prelude.d_C_concat (d_C_showPatListElems x1 x2 cd cs) cd cs) cd cs) (Curry_Prelude.toCurryString "\"") cd cs)
     Curry_Prelude.C_False -> d_C__case_80 x2 x1 (d_C_isClosedPatternList x2 cd cs) cd cs
     (Curry_Prelude.Choice_C_Bool d i l r) -> narrow d i (d_C__case_81 x2 x1 l cd cs) (d_C__case_81 x2 x1 r cd cs)
     (Curry_Prelude.Choices_C_Bool d i xs) -> narrows cs d i (\z -> d_C__case_81 x2 x1 z cd cs) xs
     (Curry_Prelude.Guard_C_Bool d c e) -> guardCons d c ((d_C__case_81 x2 x1 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_C_Bool d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_81" [(show x2),(show x1),(show x3)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_81" (showCons x3))

d_C__case_80 :: Curry_AbstractHaskell.C_Pattern -> C_Options -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_80 x2 x1 x3 cd cs = case x3 of
     Curry_Prelude.C_True -> Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString "[") (Curry_Prelude.d_OP_plus_plus (Curry_List.d_C_intercalate (Curry_Prelude.toCurryString ",") (d_C_showPatListElems x1 x2 cd cs) cd cs) (Curry_Prelude.toCurryString "]") cd cs) cd cs
     Curry_Prelude.C_False -> d_C__case_79 x2 x1 (d_C_isAsPattern x2 cd cs) cd cs
     (Curry_Prelude.Choice_C_Bool d i l r) -> narrow d i (d_C__case_80 x2 x1 l cd cs) (d_C__case_80 x2 x1 r cd cs)
     (Curry_Prelude.Choices_C_Bool d i xs) -> narrows cs d i (\z -> d_C__case_80 x2 x1 z cd cs) xs
     (Curry_Prelude.Guard_C_Bool d c e) -> guardCons d c ((d_C__case_80 x2 x1 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_C_Bool d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_80" [(show x2),(show x1),(show x3)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_80" (showCons x3))

d_C__case_79 :: Curry_AbstractHaskell.C_Pattern -> C_Options -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_79 x2 x1 x3 cd cs = case x3 of
     Curry_Prelude.C_True -> d_C_showAsPatternList x1 x2 cd cs
     Curry_Prelude.C_False -> d_C__case_78 x2 x1 (Curry_Prelude.d_C_otherwise cd cs) cd cs
     (Curry_Prelude.Choice_C_Bool d i l r) -> narrow d i (d_C__case_79 x2 x1 l cd cs) (d_C__case_79 x2 x1 r cd cs)
     (Curry_Prelude.Choices_C_Bool d i xs) -> narrows cs d i (\z -> d_C__case_79 x2 x1 z cd cs) xs
     (Curry_Prelude.Guard_C_Bool d c e) -> guardCons d c ((d_C__case_79 x2 x1 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_C_Bool d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_79" [(show x2),(show x1),(show x3)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_79" (showCons x3))

d_C__case_78 :: Curry_AbstractHaskell.C_Pattern -> C_Options -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_78 x2 x1 x3 cd cs = case x3 of
     Curry_Prelude.C_True -> Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString "(") (Curry_Prelude.d_OP_plus_plus (Curry_List.d_C_intercalate (Curry_Prelude.toCurryString ":") (d_C_showPatListElems x1 x2 cd cs) cd cs) (Curry_Prelude.toCurryString ")") cd cs) cd cs
     Curry_Prelude.C_False -> Curry_Prelude.d_C_failed cd cs
     (Curry_Prelude.Choice_C_Bool d i l r) -> narrow d i (d_C__case_78 x2 x1 l cd cs) (d_C__case_78 x2 x1 r cd cs)
     (Curry_Prelude.Choices_C_Bool d i xs) -> narrows cs d i (\z -> d_C__case_78 x2 x1 z cd cs) xs
     (Curry_Prelude.Guard_C_Bool d c e) -> guardCons d c ((d_C__case_78 x2 x1 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_C_Bool d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_78" [(show x2),(show x1),(show x3)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_78" (showCons x3))

d_C__case_82 :: Curry_Prelude.OP_List Curry_AbstractHaskell.C_Pattern -> Curry_Prelude.OP_Tuple2 (Curry_Prelude.OP_List Curry_Prelude.C_Char) (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_Pattern
d_C__case_82 x3 x2 cd cs = case x2 of
     (Curry_Prelude.OP_Tuple2 x4 x5) -> x3
     (Curry_Prelude.Choice_OP_Tuple2 d i l r) -> narrow d i (d_C__case_82 x3 l cd cs) (d_C__case_82 x3 r cd cs)
     (Curry_Prelude.Choices_OP_Tuple2 d i xs) -> narrows cs d i (\z -> d_C__case_82 x3 z cd cs) xs
     (Curry_Prelude.Guard_OP_Tuple2 d c e) -> guardCons d c ((d_C__case_82 x3 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_Tuple2 d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_82" [(show x3),(show x2)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_82" (showCons x2))

d_C__case_83 :: Curry_Prelude.OP_Tuple2 (Curry_Prelude.OP_List Curry_Prelude.C_Char) (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_83 x2 cd cs = case x2 of
     (Curry_Prelude.OP_Tuple2 x4 x5) -> x5
     (Curry_Prelude.Choice_OP_Tuple2 d i l r) -> narrow d i (d_C__case_83 l cd cs) (d_C__case_83 r cd cs)
     (Curry_Prelude.Choices_OP_Tuple2 d i xs) -> narrows cs d i (\z -> d_C__case_83 z cd cs) xs
     (Curry_Prelude.Guard_OP_Tuple2 d c e) -> guardCons d c ((d_C__case_83 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_Tuple2 d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_83" [(show x2)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_83" (showCons x2))

d_C__case_84 :: Curry_Prelude.OP_Tuple2 (Curry_Prelude.OP_List Curry_Prelude.C_Char) (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Cover -> ConstStore -> Curry_Prelude.OP_Tuple2 (Curry_Prelude.OP_List Curry_Prelude.C_Char) (Curry_Prelude.OP_List Curry_Prelude.C_Char)
d_C__case_84 x2 cd cs = case x2 of
     (Curry_Prelude.OP_Tuple2 x4 x5) -> x2
     (Curry_Prelude.Choice_OP_Tuple2 d i l r) -> narrow d i (d_C__case_84 l cd cs) (d_C__case_84 r cd cs)
     (Curry_Prelude.Choices_OP_Tuple2 d i xs) -> narrows cs d i (\z -> d_C__case_84 z cd cs) xs
     (Curry_Prelude.Guard_OP_Tuple2 d c e) -> guardCons d c ((d_C__case_84 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_Tuple2 d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_84" [(show x2)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_84" (showCons x2))

d_C__case_87 :: Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_Pattern -> C_Options -> Curry_Prelude.OP_Tuple2 (Curry_Prelude.OP_List Curry_Prelude.C_Char) (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Curry_AbstractHaskell.C_Pattern -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_87 x4 x5 x1 x3 x2 x6 cd cs = case x6 of
     Curry_Prelude.C_True -> d_C_showPatternList x1 x2 cd cs
     Curry_Prelude.C_False -> d_C__case_86 x4 x5 x1 x3 (d_C_isTuple x4 cd cs) cd cs
     (Curry_Prelude.Choice_C_Bool d i l r) -> narrow d i (d_C__case_87 x4 x5 x1 x3 x2 l cd cs) (d_C__case_87 x4 x5 x1 x3 x2 r cd cs)
     (Curry_Prelude.Choices_C_Bool d i xs) -> narrows cs d i (\z -> d_C__case_87 x4 x5 x1 x3 x2 z cd cs) xs
     (Curry_Prelude.Guard_C_Bool d c e) -> guardCons d c ((d_C__case_87 x4 x5 x1 x3 x2 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_C_Bool d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_87" [(show x4),(show x5),(show x1),(show x3),(show x2),(show x6)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_87" (showCons x6))

d_C__case_86 :: Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_Pattern -> C_Options -> Curry_Prelude.OP_Tuple2 (Curry_Prelude.OP_List Curry_Prelude.C_Char) (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_86 x4 x5 x1 x3 x6 cd cs = case x6 of
     Curry_Prelude.C_True -> Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString "(") (Curry_Prelude.d_OP_plus_plus (d_C_combineMap (d_C_showPattern x1) x5 (Curry_Prelude.toCurryString ",") cd cs) (Curry_Prelude.toCurryString ")") cd cs) cd cs
     Curry_Prelude.C_False -> d_C__case_85 x5 x1 x3 (Curry_Prelude.d_C_otherwise cd cs) cd cs
     (Curry_Prelude.Choice_C_Bool d i l r) -> narrow d i (d_C__case_86 x4 x5 x1 x3 l cd cs) (d_C__case_86 x4 x5 x1 x3 r cd cs)
     (Curry_Prelude.Choices_C_Bool d i xs) -> narrows cs d i (\z -> d_C__case_86 x4 x5 x1 x3 z cd cs) xs
     (Curry_Prelude.Guard_C_Bool d c e) -> guardCons d c ((d_C__case_86 x4 x5 x1 x3 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_C_Bool d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_86" [(show x4),(show x5),(show x1),(show x3),(show x6)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_86" (showCons x6))

d_C__case_85 :: Curry_Prelude.OP_List Curry_AbstractHaskell.C_Pattern -> C_Options -> Curry_Prelude.OP_Tuple2 (Curry_Prelude.OP_List Curry_Prelude.C_Char) (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_85 x5 x1 x3 x6 cd cs = case x6 of
     Curry_Prelude.C_True -> Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString "(") (Curry_Prelude.d_OP_plus_plus (d_C_showSymbol x1 x3 cd cs) (Curry_Prelude.d_OP_plus_plus (d_C_prefixMap (d_C_showPattern x1) x5 (Curry_Prelude.toCurryString " ") cd cs) (Curry_Prelude.toCurryString ")") cd cs) cd cs) cd cs
     Curry_Prelude.C_False -> Curry_Prelude.d_C_failed cd cs
     (Curry_Prelude.Choice_C_Bool d i l r) -> narrow d i (d_C__case_85 x5 x1 x3 l cd cs) (d_C__case_85 x5 x1 x3 r cd cs)
     (Curry_Prelude.Choices_C_Bool d i xs) -> narrows cs d i (\z -> d_C__case_85 x5 x1 x3 z cd cs) xs
     (Curry_Prelude.Guard_C_Bool d c e) -> guardCons d c ((d_C__case_85 x5 x1 x3 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_C_Bool d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_85" [(show x5),(show x1),(show x3),(show x6)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_85" (showCons x6))

d_C__case_88 :: Curry_AbstractHaskell.C_Pattern -> C_Options -> Curry_Prelude.OP_Tuple2 Curry_Prelude.C_Int (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_88 x14 x1 x13 cd cs = case x13 of
     (Curry_Prelude.OP_Tuple2 x15 x16) -> Curry_Prelude.d_OP_plus_plus (Curry_Prelude.d_C_apply (d_C_showIdentifier cd cs) x16 cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString "@") (d_C_showPattern x1 x14 cd cs) cd cs) cd cs
     (Curry_Prelude.Choice_OP_Tuple2 d i l r) -> narrow d i (d_C__case_88 x14 x1 l cd cs) (d_C__case_88 x14 x1 r cd cs)
     (Curry_Prelude.Choices_OP_Tuple2 d i xs) -> narrows cs d i (\z -> d_C__case_88 x14 x1 z cd cs) xs
     (Curry_Prelude.Guard_OP_Tuple2 d c e) -> guardCons d c ((d_C__case_88 x14 x1 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_Tuple2 d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_88" [(show x14),(show x1),(show x13)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_88" (showCons x13))

d_C__case_92 :: Curry_Prelude.OP_Tuple2 (Curry_Prelude.OP_List Curry_Prelude.C_Char) (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> C_Options -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_Pattern -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_92 x7 x1 x8 cd cs = case x8 of
     Curry_Prelude.OP_List -> d_C_showSymbol x1 x7 cd cs
     (Curry_Prelude.OP_Cons x9 x10) -> d_C__case_91 x10 x9 x1 x7 cd cs
     (Curry_Prelude.Choice_OP_List d i l r) -> narrow d i (d_C__case_92 x7 x1 l cd cs) (d_C__case_92 x7 x1 r cd cs)
     (Curry_Prelude.Choices_OP_List d i xs) -> narrows cs d i (\z -> d_C__case_92 x7 x1 z cd cs) xs
     (Curry_Prelude.Guard_OP_List d c e) -> guardCons d c ((d_C__case_92 x7 x1 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_List d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_92" [(show x7),(show x1),(show x8)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_92" (showCons x8))

d_C__case_91 :: Curry_Prelude.OP_List Curry_AbstractHaskell.C_Pattern -> Curry_AbstractHaskell.C_Pattern -> C_Options -> Curry_Prelude.OP_Tuple2 (Curry_Prelude.OP_List Curry_Prelude.C_Char) (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_91 x10 x9 x1 x7 cd cs = case x7 of
     (Curry_Prelude.OP_Tuple2 x11 x12) -> d_C__case_90 x11 x10 x9 x1 x7 (Curry_Prelude.d_OP_eq_eq x11 (Curry_Names.d_C_prelude cd cs) cd cs) cd cs
     (Curry_Prelude.Choice_OP_Tuple2 d i l r) -> narrow d i (d_C__case_91 x10 x9 x1 l cd cs) (d_C__case_91 x10 x9 x1 r cd cs)
     (Curry_Prelude.Choices_OP_Tuple2 d i xs) -> narrows cs d i (\z -> d_C__case_91 x10 x9 x1 z cd cs) xs
     (Curry_Prelude.Guard_OP_Tuple2 d c e) -> guardCons d c ((d_C__case_91 x10 x9 x1 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_Tuple2 d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_91" [(show x10),(show x9),(show x1),(show x7)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_91" (showCons x7))

d_C__case_90 :: Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_Pattern -> Curry_AbstractHaskell.C_Pattern -> C_Options -> Curry_Prelude.OP_Tuple2 (Curry_Prelude.OP_List Curry_Prelude.C_Char) (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_90 x11 x10 x9 x1 x7 x12 cd cs = case x12 of
     Curry_Prelude.C_True -> d_C_showPreludeCons x1 (Curry_AbstractHaskell.C_PComb x7 (Curry_Prelude.OP_Cons x9 x10)) cd cs
     Curry_Prelude.C_False -> d_C__case_89 x10 x9 x1 x7 (Curry_Prelude.d_C_otherwise cd cs) cd cs
     (Curry_Prelude.Choice_C_Bool d i l r) -> narrow d i (d_C__case_90 x11 x10 x9 x1 x7 l cd cs) (d_C__case_90 x11 x10 x9 x1 x7 r cd cs)
     (Curry_Prelude.Choices_C_Bool d i xs) -> narrows cs d i (\z -> d_C__case_90 x11 x10 x9 x1 x7 z cd cs) xs
     (Curry_Prelude.Guard_C_Bool d c e) -> guardCons d c ((d_C__case_90 x11 x10 x9 x1 x7 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_C_Bool d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_90" [(show x11),(show x10),(show x9),(show x1),(show x7),(show x12)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_90" (showCons x12))

d_C__case_89 :: Curry_Prelude.OP_List Curry_AbstractHaskell.C_Pattern -> Curry_AbstractHaskell.C_Pattern -> C_Options -> Curry_Prelude.OP_Tuple2 (Curry_Prelude.OP_List Curry_Prelude.C_Char) (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_89 x10 x9 x1 x7 x11 cd cs = case x11 of
     Curry_Prelude.C_True -> Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString "(") (Curry_Prelude.d_OP_plus_plus (d_C_showSymbol x1 x7 cd cs) (Curry_Prelude.d_OP_plus_plus (d_C_prefixMap (d_C_showPattern x1) (Curry_Prelude.OP_Cons x9 x10) (Curry_Prelude.toCurryString " ") cd cs) (Curry_Prelude.toCurryString ")") cd cs) cd cs) cd cs
     Curry_Prelude.C_False -> Curry_Prelude.d_C_failed cd cs
     (Curry_Prelude.Choice_C_Bool d i l r) -> narrow d i (d_C__case_89 x10 x9 x1 x7 l cd cs) (d_C__case_89 x10 x9 x1 x7 r cd cs)
     (Curry_Prelude.Choices_C_Bool d i xs) -> narrows cs d i (\z -> d_C__case_89 x10 x9 x1 x7 z cd cs) xs
     (Curry_Prelude.Guard_C_Bool d c e) -> guardCons d c ((d_C__case_89 x10 x9 x1 x7 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_C_Bool d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_89" [(show x10),(show x9),(show x1),(show x7),(show x11)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_89" (showCons x11))

d_C__case_93 :: Curry_Prelude.OP_Tuple2 Curry_Prelude.C_Int (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_93 x3 cd cs = case x3 of
     (Curry_Prelude.OP_Tuple2 x4 x5) -> Curry_Prelude.d_C_apply (d_C_showIdentifier cd cs) x5 cd cs
     (Curry_Prelude.Choice_OP_Tuple2 d i l r) -> narrow d i (d_C__case_93 l cd cs) (d_C__case_93 r cd cs)
     (Curry_Prelude.Choices_OP_Tuple2 d i xs) -> narrows cs d i (\z -> d_C__case_93 z cd cs) xs
     (Curry_Prelude.Guard_OP_Tuple2 d c e) -> guardCons d c ((d_C__case_93 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_Tuple2 d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_93" [(show x3)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_93" (showCons x3))

d_C__case_95 :: C_Options -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_LocalDecl -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_95 x1 x6 cd cs = case x6 of
     (Curry_Prelude.OP_Cons x7 x8) -> d_C__case_94 x6 x1 x7 x8 cd cs
     Curry_Prelude.OP_List -> Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString "let\n") (d_C_showBlock (d_C_combineMap (d_C_showLocalDecl x1) x6 (Curry_Prelude.toCurryString "\n") cd cs) cd cs) cd cs
     (Curry_Prelude.Choice_OP_List d i l r) -> narrow d i (d_C__case_95 x1 l cd cs) (d_C__case_95 x1 r cd cs)
     (Curry_Prelude.Choices_OP_List d i xs) -> narrows cs d i (\z -> d_C__case_95 x1 z cd cs) xs
     (Curry_Prelude.Guard_OP_List d c e) -> guardCons d c ((d_C__case_95 x1 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_List d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_95" [(show x1),(show x6)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_95" (showCons x6))

d_C__case_94 :: Curry_Prelude.OP_List Curry_AbstractHaskell.C_LocalDecl -> C_Options -> Curry_AbstractHaskell.C_LocalDecl -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_LocalDecl -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_94 x6 x1 x7 x8 cd cs = case x8 of
     Curry_Prelude.OP_List -> Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString "let ") (d_C_showLocalDecl x1 x7 cd cs) cd cs
     (Curry_Prelude.OP_Cons x9 x10) -> Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString "let\n") (d_C_showBlock (d_C_combineMap (d_C_showLocalDecl x1) x6 (Curry_Prelude.toCurryString "\n") cd cs) cd cs) cd cs
     (Curry_Prelude.Choice_OP_List d i l r) -> narrow d i (d_C__case_94 x6 x1 x7 l cd cs) (d_C__case_94 x6 x1 x7 r cd cs)
     (Curry_Prelude.Choices_OP_List d i xs) -> narrows cs d i (\z -> d_C__case_94 x6 x1 x7 z cd cs) xs
     (Curry_Prelude.Guard_OP_List d c e) -> guardCons d c ((d_C__case_94 x6 x1 x7 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_List d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_94" [(show x6),(show x1),(show x7),(show x8)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_94" (showCons x8))

d_C__case_125 :: Curry_AbstractHaskell.C_Expr -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_Pattern -> C_Options -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_Pattern -> Curry_AbstractHaskell.C_Pattern -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_125 x3 x2 x1 x5 x4 cd cs = case x4 of
     (Curry_AbstractHaskell.C_PVar x6) -> d_C__case_124 x3 x2 x1 x6 x5 cd cs
     (Curry_AbstractHaskell.C_PLit x277) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_PComb x278 x279) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_PAs x280 x281) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_PFuncComb x282 x283) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.Choice_C_Pattern d i l r) -> narrow d i (d_C__case_125 x3 x2 x1 x5 l cd cs) (d_C__case_125 x3 x2 x1 x5 r cd cs)
     (Curry_AbstractHaskell.Choices_C_Pattern d i xs) -> narrows cs d i (\z -> d_C__case_125 x3 x2 x1 x5 z cd cs) xs
     (Curry_AbstractHaskell.Guard_C_Pattern d c e) -> guardCons d c ((d_C__case_125 x3 x2 x1 x5 e cd) $! (addCs c cs))
     (Curry_AbstractHaskell.Fail_C_Pattern d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_125" [(show x3),(show x2),(show x1),(show x5),(show x4)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_125" (showCons x4))

d_C__case_124 :: Curry_AbstractHaskell.C_Expr -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_Pattern -> C_Options -> Curry_Prelude.OP_Tuple2 Curry_Prelude.C_Int (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_Pattern -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_124 x3 x2 x1 x6 x5 cd cs = case x5 of
     Curry_Prelude.OP_List -> d_C__case_123 x2 x1 x6 x3 cd cs
     (Curry_Prelude.OP_Cons x275 x276) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_Prelude.Choice_OP_List d i l r) -> narrow d i (d_C__case_124 x3 x2 x1 x6 l cd cs) (d_C__case_124 x3 x2 x1 x6 r cd cs)
     (Curry_Prelude.Choices_OP_List d i xs) -> narrows cs d i (\z -> d_C__case_124 x3 x2 x1 x6 z cd cs) xs
     (Curry_Prelude.Guard_OP_List d c e) -> guardCons d c ((d_C__case_124 x3 x2 x1 x6 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_List d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_124" [(show x3),(show x2),(show x1),(show x6),(show x5)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_124" (showCons x5))

d_C__case_123 :: Curry_Prelude.OP_List Curry_AbstractHaskell.C_Pattern -> C_Options -> Curry_Prelude.OP_Tuple2 Curry_Prelude.C_Int (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Curry_AbstractHaskell.C_Expr -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_123 x2 x1 x6 x3 cd cs = case x3 of
     (Curry_AbstractHaskell.C_Apply x7 x8) -> d_C__case_122 x3 x2 x1 x8 x6 x7 cd cs
     (Curry_AbstractHaskell.C_Var x258) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_Lit x259) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_Symbol x260) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_Lambda x261 x262) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_Let x263 x264) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_DoExpr x265) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_ListComp x266 x267) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_Case x268 x269) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_Typed x270 x271) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_IfThenElse x272 x273 x274) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.Choice_C_Expr d i l r) -> narrow d i (d_C__case_123 x2 x1 x6 l cd cs) (d_C__case_123 x2 x1 x6 r cd cs)
     (Curry_AbstractHaskell.Choices_C_Expr d i xs) -> narrows cs d i (\z -> d_C__case_123 x2 x1 x6 z cd cs) xs
     (Curry_AbstractHaskell.Guard_C_Expr d c e) -> guardCons d c ((d_C__case_123 x2 x1 x6 e cd) $! (addCs c cs))
     (Curry_AbstractHaskell.Fail_C_Expr d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_123" [(show x2),(show x1),(show x6),(show x3)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_123" (showCons x3))

d_C__case_122 :: Curry_AbstractHaskell.C_Expr -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_Pattern -> C_Options -> Curry_AbstractHaskell.C_Expr -> Curry_Prelude.OP_Tuple2 Curry_Prelude.C_Int (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Curry_AbstractHaskell.C_Expr -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_122 x3 x2 x1 x8 x6 x7 cd cs = case x7 of
     (Curry_AbstractHaskell.C_Apply x9 x10) -> d_C__case_121 x3 x2 x1 x8 x10 x6 x9 cd cs
     (Curry_AbstractHaskell.C_Var x241) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_Lit x242) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_Symbol x243) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_Lambda x244 x245) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_Let x246 x247) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_DoExpr x248) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_ListComp x249 x250) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_Case x251 x252) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_Typed x253 x254) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_IfThenElse x255 x256 x257) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.Choice_C_Expr d i l r) -> narrow d i (d_C__case_122 x3 x2 x1 x8 x6 l cd cs) (d_C__case_122 x3 x2 x1 x8 x6 r cd cs)
     (Curry_AbstractHaskell.Choices_C_Expr d i xs) -> narrows cs d i (\z -> d_C__case_122 x3 x2 x1 x8 x6 z cd cs) xs
     (Curry_AbstractHaskell.Guard_C_Expr d c e) -> guardCons d c ((d_C__case_122 x3 x2 x1 x8 x6 e cd) $! (addCs c cs))
     (Curry_AbstractHaskell.Fail_C_Expr d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_122" [(show x3),(show x2),(show x1),(show x8),(show x6),(show x7)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_122" (showCons x7))

d_C__case_121 :: Curry_AbstractHaskell.C_Expr -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_Pattern -> C_Options -> Curry_AbstractHaskell.C_Expr -> Curry_AbstractHaskell.C_Expr -> Curry_Prelude.OP_Tuple2 Curry_Prelude.C_Int (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Curry_AbstractHaskell.C_Expr -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_121 x3 x2 x1 x8 x10 x6 x9 cd cs = case x9 of
     (Curry_AbstractHaskell.C_Symbol x11) -> d_C__case_120 x8 x10 x3 x2 x1 x6 x11 cd cs
     (Curry_AbstractHaskell.C_Var x223) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_Lit x224) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_Apply x225 x226) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_Lambda x227 x228) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_Let x229 x230) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_DoExpr x231) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_ListComp x232 x233) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_Case x234 x235) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_Typed x236 x237) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_IfThenElse x238 x239 x240) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.Choice_C_Expr d i l r) -> narrow d i (d_C__case_121 x3 x2 x1 x8 x10 x6 l cd cs) (d_C__case_121 x3 x2 x1 x8 x10 x6 r cd cs)
     (Curry_AbstractHaskell.Choices_C_Expr d i xs) -> narrows cs d i (\z -> d_C__case_121 x3 x2 x1 x8 x10 x6 z cd cs) xs
     (Curry_AbstractHaskell.Guard_C_Expr d c e) -> guardCons d c ((d_C__case_121 x3 x2 x1 x8 x10 x6 e cd) $! (addCs c cs))
     (Curry_AbstractHaskell.Fail_C_Expr d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_121" [(show x3),(show x2),(show x1),(show x8),(show x10),(show x6),(show x9)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_121" (showCons x9))

d_C__case_120 :: Curry_AbstractHaskell.C_Expr -> Curry_AbstractHaskell.C_Expr -> Curry_AbstractHaskell.C_Expr -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_Pattern -> C_Options -> Curry_Prelude.OP_Tuple2 Curry_Prelude.C_Int (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Curry_Prelude.OP_Tuple2 (Curry_Prelude.OP_List Curry_Prelude.C_Char) (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_120 x8 x10 x3 x2 x1 x6 x11 cd cs = case x11 of
     (Curry_Prelude.OP_Tuple2 x12 x13) -> d_C__case_119 x10 x3 x2 x1 x6 x13 x11 x8 cd cs
     (Curry_Prelude.Choice_OP_Tuple2 d i l r) -> narrow d i (d_C__case_120 x8 x10 x3 x2 x1 x6 l cd cs) (d_C__case_120 x8 x10 x3 x2 x1 x6 r cd cs)
     (Curry_Prelude.Choices_OP_Tuple2 d i xs) -> narrows cs d i (\z -> d_C__case_120 x8 x10 x3 x2 x1 x6 z cd cs) xs
     (Curry_Prelude.Guard_OP_Tuple2 d c e) -> guardCons d c ((d_C__case_120 x8 x10 x3 x2 x1 x6 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_Tuple2 d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_120" [(show x8),(show x10),(show x3),(show x2),(show x1),(show x6),(show x11)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_120" (showCons x11))

d_C__case_119 :: Curry_AbstractHaskell.C_Expr -> Curry_AbstractHaskell.C_Expr -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_Pattern -> C_Options -> Curry_Prelude.OP_Tuple2 Curry_Prelude.C_Int (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.OP_Tuple2 (Curry_Prelude.OP_List Curry_Prelude.C_Char) (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Curry_AbstractHaskell.C_Expr -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_119 x10 x3 x2 x1 x6 x13 x11 x8 cd cs = case x8 of
     (Curry_AbstractHaskell.C_Var x14) -> d_C__case_118 x10 x14 x13 x3 x2 x1 x6 x11 (Curry_Prelude.d_OP_amp_amp (Curry_Prelude.d_C_apply (d_C_isInfixOpName cd cs) x13 cd cs) (Curry_Prelude.d_OP_amp_amp (d_C_isAtom x10 cd cs) (Curry_Prelude.d_OP_slash_eq (Curry_AbstractHaskell.C_Var x14) x10 cd cs) cd cs) cd cs) cd cs
     (Curry_AbstractHaskell.C_Lit x15) -> d_C__case_115 x3 x2 x1 x15 x6 x13 x10 cd cs
     (Curry_AbstractHaskell.C_Symbol x35) -> d_C__case_113 x3 x2 x1 x35 x6 x13 x10 cd cs
     (Curry_AbstractHaskell.C_Apply x55 x56) -> d_C__case_111 x3 x2 x1 x56 x55 x6 x13 x10 cd cs
     (Curry_AbstractHaskell.C_Lambda x76 x77) -> d_C__case_109 x3 x2 x1 x77 x76 x6 x13 x10 cd cs
     (Curry_AbstractHaskell.C_Let x97 x98) -> d_C__case_107 x3 x2 x1 x98 x97 x6 x13 x10 cd cs
     (Curry_AbstractHaskell.C_DoExpr x118) -> d_C__case_105 x3 x2 x1 x118 x6 x13 x10 cd cs
     (Curry_AbstractHaskell.C_ListComp x138 x139) -> d_C__case_103 x3 x2 x1 x139 x138 x6 x13 x10 cd cs
     (Curry_AbstractHaskell.C_Case x159 x160) -> d_C__case_101 x3 x2 x1 x160 x159 x6 x13 x10 cd cs
     (Curry_AbstractHaskell.C_Typed x180 x181) -> d_C__case_99 x3 x2 x1 x181 x180 x6 x13 x10 cd cs
     (Curry_AbstractHaskell.C_IfThenElse x201 x202 x203) -> d_C__case_97 x3 x2 x1 x203 x202 x201 x6 x13 x10 cd cs
     (Curry_AbstractHaskell.Choice_C_Expr d i l r) -> narrow d i (d_C__case_119 x10 x3 x2 x1 x6 x13 x11 l cd cs) (d_C__case_119 x10 x3 x2 x1 x6 x13 x11 r cd cs)
     (Curry_AbstractHaskell.Choices_C_Expr d i xs) -> narrows cs d i (\z -> d_C__case_119 x10 x3 x2 x1 x6 x13 x11 z cd cs) xs
     (Curry_AbstractHaskell.Guard_C_Expr d c e) -> guardCons d c ((d_C__case_119 x10 x3 x2 x1 x6 x13 x11 e cd) $! (addCs c cs))
     (Curry_AbstractHaskell.Fail_C_Expr d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_119" [(show x10),(show x3),(show x2),(show x1),(show x6),(show x13),(show x11),(show x8)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_119" (showCons x8))

d_C__case_97 :: Curry_AbstractHaskell.C_Expr -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_Pattern -> C_Options -> Curry_AbstractHaskell.C_Expr -> Curry_AbstractHaskell.C_Expr -> Curry_AbstractHaskell.C_Expr -> Curry_Prelude.OP_Tuple2 Curry_Prelude.C_Int (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_AbstractHaskell.C_Expr -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_97 x3 x2 x1 x203 x202 x201 x6 x13 x10 cd cs = case x10 of
     (Curry_AbstractHaskell.C_Var x204) -> d_C__case_96 x203 x202 x201 x204 x6 x13 x3 x2 x1 (Curry_Prelude.d_OP_amp_amp (Curry_Prelude.d_C_apply (d_C_isInfixOpName cd cs) x13 cd cs) (Curry_Prelude.d_OP_amp_amp (Curry_Prelude.d_OP_eq_eq x6 x204 cd cs) (Curry_Prelude.d_OP_amp_amp (d_C_isAtom (Curry_AbstractHaskell.C_IfThenElse x201 x202 x203) cd cs) (Curry_Prelude.d_OP_slash_eq (Curry_AbstractHaskell.C_Var x204) (Curry_AbstractHaskell.C_IfThenElse x201 x202 x203) cd cs) cd cs) cd cs) cd cs) cd cs
     (Curry_AbstractHaskell.C_Lit x205) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_Symbol x206) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_Apply x207 x208) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_Lambda x209 x210) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_Let x211 x212) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_DoExpr x213) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_ListComp x214 x215) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_Case x216 x217) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_Typed x218 x219) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_IfThenElse x220 x221 x222) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.Choice_C_Expr d i l r) -> narrow d i (d_C__case_97 x3 x2 x1 x203 x202 x201 x6 x13 l cd cs) (d_C__case_97 x3 x2 x1 x203 x202 x201 x6 x13 r cd cs)
     (Curry_AbstractHaskell.Choices_C_Expr d i xs) -> narrows cs d i (\z -> d_C__case_97 x3 x2 x1 x203 x202 x201 x6 x13 z cd cs) xs
     (Curry_AbstractHaskell.Guard_C_Expr d c e) -> guardCons d c ((d_C__case_97 x3 x2 x1 x203 x202 x201 x6 x13 e cd) $! (addCs c cs))
     (Curry_AbstractHaskell.Fail_C_Expr d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_97" [(show x3),(show x2),(show x1),(show x203),(show x202),(show x201),(show x6),(show x13),(show x10)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_97" (showCons x10))

d_C__case_96 :: Curry_AbstractHaskell.C_Expr -> Curry_AbstractHaskell.C_Expr -> Curry_AbstractHaskell.C_Expr -> Curry_Prelude.OP_Tuple2 Curry_Prelude.C_Int (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Curry_Prelude.OP_Tuple2 Curry_Prelude.C_Int (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_AbstractHaskell.C_Expr -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_Pattern -> C_Options -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_96 x203 x202 x201 x204 x6 x13 x3 x2 x1 x205 cd cs = case x205 of
     Curry_Prelude.C_True -> Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString "(") (Curry_Prelude.d_OP_plus_plus x13 (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString " ") (Curry_Prelude.d_OP_plus_plus (d_C_showBoxedExpr x1 (Curry_AbstractHaskell.C_IfThenElse x201 x202 x203) cd cs) (Curry_Prelude.toCurryString ")") cd cs) cd cs) cd cs) cd cs
     Curry_Prelude.C_False -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_Prelude.Choice_C_Bool d i l r) -> narrow d i (d_C__case_96 x203 x202 x201 x204 x6 x13 x3 x2 x1 l cd cs) (d_C__case_96 x203 x202 x201 x204 x6 x13 x3 x2 x1 r cd cs)
     (Curry_Prelude.Choices_C_Bool d i xs) -> narrows cs d i (\z -> d_C__case_96 x203 x202 x201 x204 x6 x13 x3 x2 x1 z cd cs) xs
     (Curry_Prelude.Guard_C_Bool d c e) -> guardCons d c ((d_C__case_96 x203 x202 x201 x204 x6 x13 x3 x2 x1 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_C_Bool d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_96" [(show x203),(show x202),(show x201),(show x204),(show x6),(show x13),(show x3),(show x2),(show x1),(show x205)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_96" (showCons x205))

d_C__case_99 :: Curry_AbstractHaskell.C_Expr -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_Pattern -> C_Options -> Curry_AbstractHaskell.C_TypeExpr -> Curry_AbstractHaskell.C_Expr -> Curry_Prelude.OP_Tuple2 Curry_Prelude.C_Int (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_AbstractHaskell.C_Expr -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_99 x3 x2 x1 x181 x180 x6 x13 x10 cd cs = case x10 of
     (Curry_AbstractHaskell.C_Var x182) -> d_C__case_98 x181 x180 x182 x6 x13 x3 x2 x1 (Curry_Prelude.d_OP_amp_amp (Curry_Prelude.d_C_apply (d_C_isInfixOpName cd cs) x13 cd cs) (Curry_Prelude.d_OP_amp_amp (Curry_Prelude.d_OP_eq_eq x6 x182 cd cs) (Curry_Prelude.d_OP_amp_amp (d_C_isAtom (Curry_AbstractHaskell.C_Typed x180 x181) cd cs) (Curry_Prelude.d_OP_slash_eq (Curry_AbstractHaskell.C_Var x182) (Curry_AbstractHaskell.C_Typed x180 x181) cd cs) cd cs) cd cs) cd cs) cd cs
     (Curry_AbstractHaskell.C_Lit x183) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_Symbol x184) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_Apply x185 x186) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_Lambda x187 x188) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_Let x189 x190) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_DoExpr x191) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_ListComp x192 x193) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_Case x194 x195) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_Typed x196 x197) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_IfThenElse x198 x199 x200) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.Choice_C_Expr d i l r) -> narrow d i (d_C__case_99 x3 x2 x1 x181 x180 x6 x13 l cd cs) (d_C__case_99 x3 x2 x1 x181 x180 x6 x13 r cd cs)
     (Curry_AbstractHaskell.Choices_C_Expr d i xs) -> narrows cs d i (\z -> d_C__case_99 x3 x2 x1 x181 x180 x6 x13 z cd cs) xs
     (Curry_AbstractHaskell.Guard_C_Expr d c e) -> guardCons d c ((d_C__case_99 x3 x2 x1 x181 x180 x6 x13 e cd) $! (addCs c cs))
     (Curry_AbstractHaskell.Fail_C_Expr d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_99" [(show x3),(show x2),(show x1),(show x181),(show x180),(show x6),(show x13),(show x10)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_99" (showCons x10))

d_C__case_98 :: Curry_AbstractHaskell.C_TypeExpr -> Curry_AbstractHaskell.C_Expr -> Curry_Prelude.OP_Tuple2 Curry_Prelude.C_Int (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Curry_Prelude.OP_Tuple2 Curry_Prelude.C_Int (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_AbstractHaskell.C_Expr -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_Pattern -> C_Options -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_98 x181 x180 x182 x6 x13 x3 x2 x1 x183 cd cs = case x183 of
     Curry_Prelude.C_True -> Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString "(") (Curry_Prelude.d_OP_plus_plus x13 (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString " ") (Curry_Prelude.d_OP_plus_plus (d_C_showBoxedExpr x1 (Curry_AbstractHaskell.C_Typed x180 x181) cd cs) (Curry_Prelude.toCurryString ")") cd cs) cd cs) cd cs) cd cs
     Curry_Prelude.C_False -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_Prelude.Choice_C_Bool d i l r) -> narrow d i (d_C__case_98 x181 x180 x182 x6 x13 x3 x2 x1 l cd cs) (d_C__case_98 x181 x180 x182 x6 x13 x3 x2 x1 r cd cs)
     (Curry_Prelude.Choices_C_Bool d i xs) -> narrows cs d i (\z -> d_C__case_98 x181 x180 x182 x6 x13 x3 x2 x1 z cd cs) xs
     (Curry_Prelude.Guard_C_Bool d c e) -> guardCons d c ((d_C__case_98 x181 x180 x182 x6 x13 x3 x2 x1 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_C_Bool d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_98" [(show x181),(show x180),(show x182),(show x6),(show x13),(show x3),(show x2),(show x1),(show x183)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_98" (showCons x183))

d_C__case_101 :: Curry_AbstractHaskell.C_Expr -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_Pattern -> C_Options -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_BranchExpr -> Curry_AbstractHaskell.C_Expr -> Curry_Prelude.OP_Tuple2 Curry_Prelude.C_Int (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_AbstractHaskell.C_Expr -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_101 x3 x2 x1 x160 x159 x6 x13 x10 cd cs = case x10 of
     (Curry_AbstractHaskell.C_Var x161) -> d_C__case_100 x160 x159 x161 x6 x13 x3 x2 x1 (Curry_Prelude.d_OP_amp_amp (Curry_Prelude.d_C_apply (d_C_isInfixOpName cd cs) x13 cd cs) (Curry_Prelude.d_OP_amp_amp (Curry_Prelude.d_OP_eq_eq x6 x161 cd cs) (Curry_Prelude.d_OP_amp_amp (d_C_isAtom (Curry_AbstractHaskell.C_Case x159 x160) cd cs) (Curry_Prelude.d_OP_slash_eq (Curry_AbstractHaskell.C_Var x161) (Curry_AbstractHaskell.C_Case x159 x160) cd cs) cd cs) cd cs) cd cs) cd cs
     (Curry_AbstractHaskell.C_Lit x162) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_Symbol x163) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_Apply x164 x165) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_Lambda x166 x167) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_Let x168 x169) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_DoExpr x170) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_ListComp x171 x172) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_Case x173 x174) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_Typed x175 x176) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_IfThenElse x177 x178 x179) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.Choice_C_Expr d i l r) -> narrow d i (d_C__case_101 x3 x2 x1 x160 x159 x6 x13 l cd cs) (d_C__case_101 x3 x2 x1 x160 x159 x6 x13 r cd cs)
     (Curry_AbstractHaskell.Choices_C_Expr d i xs) -> narrows cs d i (\z -> d_C__case_101 x3 x2 x1 x160 x159 x6 x13 z cd cs) xs
     (Curry_AbstractHaskell.Guard_C_Expr d c e) -> guardCons d c ((d_C__case_101 x3 x2 x1 x160 x159 x6 x13 e cd) $! (addCs c cs))
     (Curry_AbstractHaskell.Fail_C_Expr d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_101" [(show x3),(show x2),(show x1),(show x160),(show x159),(show x6),(show x13),(show x10)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_101" (showCons x10))

d_C__case_100 :: Curry_Prelude.OP_List Curry_AbstractHaskell.C_BranchExpr -> Curry_AbstractHaskell.C_Expr -> Curry_Prelude.OP_Tuple2 Curry_Prelude.C_Int (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Curry_Prelude.OP_Tuple2 Curry_Prelude.C_Int (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_AbstractHaskell.C_Expr -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_Pattern -> C_Options -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_100 x160 x159 x161 x6 x13 x3 x2 x1 x162 cd cs = case x162 of
     Curry_Prelude.C_True -> Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString "(") (Curry_Prelude.d_OP_plus_plus x13 (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString " ") (Curry_Prelude.d_OP_plus_plus (d_C_showBoxedExpr x1 (Curry_AbstractHaskell.C_Case x159 x160) cd cs) (Curry_Prelude.toCurryString ")") cd cs) cd cs) cd cs) cd cs
     Curry_Prelude.C_False -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_Prelude.Choice_C_Bool d i l r) -> narrow d i (d_C__case_100 x160 x159 x161 x6 x13 x3 x2 x1 l cd cs) (d_C__case_100 x160 x159 x161 x6 x13 x3 x2 x1 r cd cs)
     (Curry_Prelude.Choices_C_Bool d i xs) -> narrows cs d i (\z -> d_C__case_100 x160 x159 x161 x6 x13 x3 x2 x1 z cd cs) xs
     (Curry_Prelude.Guard_C_Bool d c e) -> guardCons d c ((d_C__case_100 x160 x159 x161 x6 x13 x3 x2 x1 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_C_Bool d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_100" [(show x160),(show x159),(show x161),(show x6),(show x13),(show x3),(show x2),(show x1),(show x162)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_100" (showCons x162))

d_C__case_103 :: Curry_AbstractHaskell.C_Expr -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_Pattern -> C_Options -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_Statement -> Curry_AbstractHaskell.C_Expr -> Curry_Prelude.OP_Tuple2 Curry_Prelude.C_Int (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_AbstractHaskell.C_Expr -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_103 x3 x2 x1 x139 x138 x6 x13 x10 cd cs = case x10 of
     (Curry_AbstractHaskell.C_Var x140) -> d_C__case_102 x139 x138 x140 x6 x13 x3 x2 x1 (Curry_Prelude.d_OP_amp_amp (Curry_Prelude.d_C_apply (d_C_isInfixOpName cd cs) x13 cd cs) (Curry_Prelude.d_OP_amp_amp (Curry_Prelude.d_OP_eq_eq x6 x140 cd cs) (Curry_Prelude.d_OP_amp_amp (d_C_isAtom (Curry_AbstractHaskell.C_ListComp x138 x139) cd cs) (Curry_Prelude.d_OP_slash_eq (Curry_AbstractHaskell.C_Var x140) (Curry_AbstractHaskell.C_ListComp x138 x139) cd cs) cd cs) cd cs) cd cs) cd cs
     (Curry_AbstractHaskell.C_Lit x141) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_Symbol x142) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_Apply x143 x144) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_Lambda x145 x146) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_Let x147 x148) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_DoExpr x149) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_ListComp x150 x151) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_Case x152 x153) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_Typed x154 x155) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_IfThenElse x156 x157 x158) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.Choice_C_Expr d i l r) -> narrow d i (d_C__case_103 x3 x2 x1 x139 x138 x6 x13 l cd cs) (d_C__case_103 x3 x2 x1 x139 x138 x6 x13 r cd cs)
     (Curry_AbstractHaskell.Choices_C_Expr d i xs) -> narrows cs d i (\z -> d_C__case_103 x3 x2 x1 x139 x138 x6 x13 z cd cs) xs
     (Curry_AbstractHaskell.Guard_C_Expr d c e) -> guardCons d c ((d_C__case_103 x3 x2 x1 x139 x138 x6 x13 e cd) $! (addCs c cs))
     (Curry_AbstractHaskell.Fail_C_Expr d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_103" [(show x3),(show x2),(show x1),(show x139),(show x138),(show x6),(show x13),(show x10)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_103" (showCons x10))

d_C__case_102 :: Curry_Prelude.OP_List Curry_AbstractHaskell.C_Statement -> Curry_AbstractHaskell.C_Expr -> Curry_Prelude.OP_Tuple2 Curry_Prelude.C_Int (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Curry_Prelude.OP_Tuple2 Curry_Prelude.C_Int (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_AbstractHaskell.C_Expr -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_Pattern -> C_Options -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_102 x139 x138 x140 x6 x13 x3 x2 x1 x141 cd cs = case x141 of
     Curry_Prelude.C_True -> Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString "(") (Curry_Prelude.d_OP_plus_plus x13 (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString " ") (Curry_Prelude.d_OP_plus_plus (d_C_showBoxedExpr x1 (Curry_AbstractHaskell.C_ListComp x138 x139) cd cs) (Curry_Prelude.toCurryString ")") cd cs) cd cs) cd cs) cd cs
     Curry_Prelude.C_False -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_Prelude.Choice_C_Bool d i l r) -> narrow d i (d_C__case_102 x139 x138 x140 x6 x13 x3 x2 x1 l cd cs) (d_C__case_102 x139 x138 x140 x6 x13 x3 x2 x1 r cd cs)
     (Curry_Prelude.Choices_C_Bool d i xs) -> narrows cs d i (\z -> d_C__case_102 x139 x138 x140 x6 x13 x3 x2 x1 z cd cs) xs
     (Curry_Prelude.Guard_C_Bool d c e) -> guardCons d c ((d_C__case_102 x139 x138 x140 x6 x13 x3 x2 x1 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_C_Bool d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_102" [(show x139),(show x138),(show x140),(show x6),(show x13),(show x3),(show x2),(show x1),(show x141)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_102" (showCons x141))

d_C__case_105 :: Curry_AbstractHaskell.C_Expr -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_Pattern -> C_Options -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_Statement -> Curry_Prelude.OP_Tuple2 Curry_Prelude.C_Int (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_AbstractHaskell.C_Expr -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_105 x3 x2 x1 x118 x6 x13 x10 cd cs = case x10 of
     (Curry_AbstractHaskell.C_Var x119) -> d_C__case_104 x118 x119 x6 x13 x3 x2 x1 (Curry_Prelude.d_OP_amp_amp (Curry_Prelude.d_C_apply (d_C_isInfixOpName cd cs) x13 cd cs) (Curry_Prelude.d_OP_amp_amp (Curry_Prelude.d_OP_eq_eq x6 x119 cd cs) (Curry_Prelude.d_OP_amp_amp (d_C_isAtom (Curry_AbstractHaskell.C_DoExpr x118) cd cs) (Curry_Prelude.d_OP_slash_eq (Curry_AbstractHaskell.C_Var x119) (Curry_AbstractHaskell.C_DoExpr x118) cd cs) cd cs) cd cs) cd cs) cd cs
     (Curry_AbstractHaskell.C_Lit x120) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_Symbol x121) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_Apply x122 x123) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_Lambda x124 x125) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_Let x126 x127) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_DoExpr x128) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_ListComp x129 x130) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_Case x131 x132) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_Typed x133 x134) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_IfThenElse x135 x136 x137) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.Choice_C_Expr d i l r) -> narrow d i (d_C__case_105 x3 x2 x1 x118 x6 x13 l cd cs) (d_C__case_105 x3 x2 x1 x118 x6 x13 r cd cs)
     (Curry_AbstractHaskell.Choices_C_Expr d i xs) -> narrows cs d i (\z -> d_C__case_105 x3 x2 x1 x118 x6 x13 z cd cs) xs
     (Curry_AbstractHaskell.Guard_C_Expr d c e) -> guardCons d c ((d_C__case_105 x3 x2 x1 x118 x6 x13 e cd) $! (addCs c cs))
     (Curry_AbstractHaskell.Fail_C_Expr d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_105" [(show x3),(show x2),(show x1),(show x118),(show x6),(show x13),(show x10)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_105" (showCons x10))

d_C__case_104 :: Curry_Prelude.OP_List Curry_AbstractHaskell.C_Statement -> Curry_Prelude.OP_Tuple2 Curry_Prelude.C_Int (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Curry_Prelude.OP_Tuple2 Curry_Prelude.C_Int (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_AbstractHaskell.C_Expr -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_Pattern -> C_Options -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_104 x118 x119 x6 x13 x3 x2 x1 x120 cd cs = case x120 of
     Curry_Prelude.C_True -> Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString "(") (Curry_Prelude.d_OP_plus_plus x13 (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString " ") (Curry_Prelude.d_OP_plus_plus (d_C_showBoxedExpr x1 (Curry_AbstractHaskell.C_DoExpr x118) cd cs) (Curry_Prelude.toCurryString ")") cd cs) cd cs) cd cs) cd cs
     Curry_Prelude.C_False -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_Prelude.Choice_C_Bool d i l r) -> narrow d i (d_C__case_104 x118 x119 x6 x13 x3 x2 x1 l cd cs) (d_C__case_104 x118 x119 x6 x13 x3 x2 x1 r cd cs)
     (Curry_Prelude.Choices_C_Bool d i xs) -> narrows cs d i (\z -> d_C__case_104 x118 x119 x6 x13 x3 x2 x1 z cd cs) xs
     (Curry_Prelude.Guard_C_Bool d c e) -> guardCons d c ((d_C__case_104 x118 x119 x6 x13 x3 x2 x1 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_C_Bool d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_104" [(show x118),(show x119),(show x6),(show x13),(show x3),(show x2),(show x1),(show x120)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_104" (showCons x120))

d_C__case_107 :: Curry_AbstractHaskell.C_Expr -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_Pattern -> C_Options -> Curry_AbstractHaskell.C_Expr -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_LocalDecl -> Curry_Prelude.OP_Tuple2 Curry_Prelude.C_Int (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_AbstractHaskell.C_Expr -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_107 x3 x2 x1 x98 x97 x6 x13 x10 cd cs = case x10 of
     (Curry_AbstractHaskell.C_Var x99) -> d_C__case_106 x98 x97 x99 x6 x13 x3 x2 x1 (Curry_Prelude.d_OP_amp_amp (Curry_Prelude.d_C_apply (d_C_isInfixOpName cd cs) x13 cd cs) (Curry_Prelude.d_OP_amp_amp (Curry_Prelude.d_OP_eq_eq x6 x99 cd cs) (Curry_Prelude.d_OP_amp_amp (d_C_isAtom (Curry_AbstractHaskell.C_Let x97 x98) cd cs) (Curry_Prelude.d_OP_slash_eq (Curry_AbstractHaskell.C_Var x99) (Curry_AbstractHaskell.C_Let x97 x98) cd cs) cd cs) cd cs) cd cs) cd cs
     (Curry_AbstractHaskell.C_Lit x100) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_Symbol x101) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_Apply x102 x103) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_Lambda x104 x105) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_Let x106 x107) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_DoExpr x108) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_ListComp x109 x110) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_Case x111 x112) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_Typed x113 x114) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_IfThenElse x115 x116 x117) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.Choice_C_Expr d i l r) -> narrow d i (d_C__case_107 x3 x2 x1 x98 x97 x6 x13 l cd cs) (d_C__case_107 x3 x2 x1 x98 x97 x6 x13 r cd cs)
     (Curry_AbstractHaskell.Choices_C_Expr d i xs) -> narrows cs d i (\z -> d_C__case_107 x3 x2 x1 x98 x97 x6 x13 z cd cs) xs
     (Curry_AbstractHaskell.Guard_C_Expr d c e) -> guardCons d c ((d_C__case_107 x3 x2 x1 x98 x97 x6 x13 e cd) $! (addCs c cs))
     (Curry_AbstractHaskell.Fail_C_Expr d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_107" [(show x3),(show x2),(show x1),(show x98),(show x97),(show x6),(show x13),(show x10)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_107" (showCons x10))

d_C__case_106 :: Curry_AbstractHaskell.C_Expr -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_LocalDecl -> Curry_Prelude.OP_Tuple2 Curry_Prelude.C_Int (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Curry_Prelude.OP_Tuple2 Curry_Prelude.C_Int (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_AbstractHaskell.C_Expr -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_Pattern -> C_Options -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_106 x98 x97 x99 x6 x13 x3 x2 x1 x100 cd cs = case x100 of
     Curry_Prelude.C_True -> Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString "(") (Curry_Prelude.d_OP_plus_plus x13 (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString " ") (Curry_Prelude.d_OP_plus_plus (d_C_showBoxedExpr x1 (Curry_AbstractHaskell.C_Let x97 x98) cd cs) (Curry_Prelude.toCurryString ")") cd cs) cd cs) cd cs) cd cs
     Curry_Prelude.C_False -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_Prelude.Choice_C_Bool d i l r) -> narrow d i (d_C__case_106 x98 x97 x99 x6 x13 x3 x2 x1 l cd cs) (d_C__case_106 x98 x97 x99 x6 x13 x3 x2 x1 r cd cs)
     (Curry_Prelude.Choices_C_Bool d i xs) -> narrows cs d i (\z -> d_C__case_106 x98 x97 x99 x6 x13 x3 x2 x1 z cd cs) xs
     (Curry_Prelude.Guard_C_Bool d c e) -> guardCons d c ((d_C__case_106 x98 x97 x99 x6 x13 x3 x2 x1 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_C_Bool d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_106" [(show x98),(show x97),(show x99),(show x6),(show x13),(show x3),(show x2),(show x1),(show x100)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_106" (showCons x100))

d_C__case_109 :: Curry_AbstractHaskell.C_Expr -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_Pattern -> C_Options -> Curry_AbstractHaskell.C_Expr -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_Pattern -> Curry_Prelude.OP_Tuple2 Curry_Prelude.C_Int (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_AbstractHaskell.C_Expr -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_109 x3 x2 x1 x77 x76 x6 x13 x10 cd cs = case x10 of
     (Curry_AbstractHaskell.C_Var x78) -> d_C__case_108 x77 x76 x78 x6 x13 x3 x2 x1 (Curry_Prelude.d_OP_amp_amp (Curry_Prelude.d_C_apply (d_C_isInfixOpName cd cs) x13 cd cs) (Curry_Prelude.d_OP_amp_amp (Curry_Prelude.d_OP_eq_eq x6 x78 cd cs) (Curry_Prelude.d_OP_amp_amp (d_C_isAtom (Curry_AbstractHaskell.C_Lambda x76 x77) cd cs) (Curry_Prelude.d_OP_slash_eq (Curry_AbstractHaskell.C_Var x78) (Curry_AbstractHaskell.C_Lambda x76 x77) cd cs) cd cs) cd cs) cd cs) cd cs
     (Curry_AbstractHaskell.C_Lit x79) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_Symbol x80) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_Apply x81 x82) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_Lambda x83 x84) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_Let x85 x86) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_DoExpr x87) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_ListComp x88 x89) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_Case x90 x91) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_Typed x92 x93) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_IfThenElse x94 x95 x96) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.Choice_C_Expr d i l r) -> narrow d i (d_C__case_109 x3 x2 x1 x77 x76 x6 x13 l cd cs) (d_C__case_109 x3 x2 x1 x77 x76 x6 x13 r cd cs)
     (Curry_AbstractHaskell.Choices_C_Expr d i xs) -> narrows cs d i (\z -> d_C__case_109 x3 x2 x1 x77 x76 x6 x13 z cd cs) xs
     (Curry_AbstractHaskell.Guard_C_Expr d c e) -> guardCons d c ((d_C__case_109 x3 x2 x1 x77 x76 x6 x13 e cd) $! (addCs c cs))
     (Curry_AbstractHaskell.Fail_C_Expr d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_109" [(show x3),(show x2),(show x1),(show x77),(show x76),(show x6),(show x13),(show x10)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_109" (showCons x10))

d_C__case_108 :: Curry_AbstractHaskell.C_Expr -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_Pattern -> Curry_Prelude.OP_Tuple2 Curry_Prelude.C_Int (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Curry_Prelude.OP_Tuple2 Curry_Prelude.C_Int (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_AbstractHaskell.C_Expr -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_Pattern -> C_Options -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_108 x77 x76 x78 x6 x13 x3 x2 x1 x79 cd cs = case x79 of
     Curry_Prelude.C_True -> Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString "(") (Curry_Prelude.d_OP_plus_plus x13 (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString " ") (Curry_Prelude.d_OP_plus_plus (d_C_showBoxedExpr x1 (Curry_AbstractHaskell.C_Lambda x76 x77) cd cs) (Curry_Prelude.toCurryString ")") cd cs) cd cs) cd cs) cd cs
     Curry_Prelude.C_False -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_Prelude.Choice_C_Bool d i l r) -> narrow d i (d_C__case_108 x77 x76 x78 x6 x13 x3 x2 x1 l cd cs) (d_C__case_108 x77 x76 x78 x6 x13 x3 x2 x1 r cd cs)
     (Curry_Prelude.Choices_C_Bool d i xs) -> narrows cs d i (\z -> d_C__case_108 x77 x76 x78 x6 x13 x3 x2 x1 z cd cs) xs
     (Curry_Prelude.Guard_C_Bool d c e) -> guardCons d c ((d_C__case_108 x77 x76 x78 x6 x13 x3 x2 x1 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_C_Bool d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_108" [(show x77),(show x76),(show x78),(show x6),(show x13),(show x3),(show x2),(show x1),(show x79)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_108" (showCons x79))

d_C__case_111 :: Curry_AbstractHaskell.C_Expr -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_Pattern -> C_Options -> Curry_AbstractHaskell.C_Expr -> Curry_AbstractHaskell.C_Expr -> Curry_Prelude.OP_Tuple2 Curry_Prelude.C_Int (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_AbstractHaskell.C_Expr -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_111 x3 x2 x1 x56 x55 x6 x13 x10 cd cs = case x10 of
     (Curry_AbstractHaskell.C_Var x57) -> d_C__case_110 x56 x55 x57 x6 x13 x3 x2 x1 (Curry_Prelude.d_OP_amp_amp (Curry_Prelude.d_C_apply (d_C_isInfixOpName cd cs) x13 cd cs) (Curry_Prelude.d_OP_amp_amp (Curry_Prelude.d_OP_eq_eq x6 x57 cd cs) (Curry_Prelude.d_OP_amp_amp (d_C_isAtom (Curry_AbstractHaskell.C_Apply x55 x56) cd cs) (Curry_Prelude.d_OP_slash_eq (Curry_AbstractHaskell.C_Var x57) (Curry_AbstractHaskell.C_Apply x55 x56) cd cs) cd cs) cd cs) cd cs) cd cs
     (Curry_AbstractHaskell.C_Lit x58) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_Symbol x59) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_Apply x60 x61) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_Lambda x62 x63) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_Let x64 x65) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_DoExpr x66) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_ListComp x67 x68) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_Case x69 x70) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_Typed x71 x72) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_IfThenElse x73 x74 x75) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.Choice_C_Expr d i l r) -> narrow d i (d_C__case_111 x3 x2 x1 x56 x55 x6 x13 l cd cs) (d_C__case_111 x3 x2 x1 x56 x55 x6 x13 r cd cs)
     (Curry_AbstractHaskell.Choices_C_Expr d i xs) -> narrows cs d i (\z -> d_C__case_111 x3 x2 x1 x56 x55 x6 x13 z cd cs) xs
     (Curry_AbstractHaskell.Guard_C_Expr d c e) -> guardCons d c ((d_C__case_111 x3 x2 x1 x56 x55 x6 x13 e cd) $! (addCs c cs))
     (Curry_AbstractHaskell.Fail_C_Expr d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_111" [(show x3),(show x2),(show x1),(show x56),(show x55),(show x6),(show x13),(show x10)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_111" (showCons x10))

d_C__case_110 :: Curry_AbstractHaskell.C_Expr -> Curry_AbstractHaskell.C_Expr -> Curry_Prelude.OP_Tuple2 Curry_Prelude.C_Int (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Curry_Prelude.OP_Tuple2 Curry_Prelude.C_Int (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_AbstractHaskell.C_Expr -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_Pattern -> C_Options -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_110 x56 x55 x57 x6 x13 x3 x2 x1 x58 cd cs = case x58 of
     Curry_Prelude.C_True -> Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString "(") (Curry_Prelude.d_OP_plus_plus x13 (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString " ") (Curry_Prelude.d_OP_plus_plus (d_C_showBoxedExpr x1 (Curry_AbstractHaskell.C_Apply x55 x56) cd cs) (Curry_Prelude.toCurryString ")") cd cs) cd cs) cd cs) cd cs
     Curry_Prelude.C_False -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_Prelude.Choice_C_Bool d i l r) -> narrow d i (d_C__case_110 x56 x55 x57 x6 x13 x3 x2 x1 l cd cs) (d_C__case_110 x56 x55 x57 x6 x13 x3 x2 x1 r cd cs)
     (Curry_Prelude.Choices_C_Bool d i xs) -> narrows cs d i (\z -> d_C__case_110 x56 x55 x57 x6 x13 x3 x2 x1 z cd cs) xs
     (Curry_Prelude.Guard_C_Bool d c e) -> guardCons d c ((d_C__case_110 x56 x55 x57 x6 x13 x3 x2 x1 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_C_Bool d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_110" [(show x56),(show x55),(show x57),(show x6),(show x13),(show x3),(show x2),(show x1),(show x58)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_110" (showCons x58))

d_C__case_113 :: Curry_AbstractHaskell.C_Expr -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_Pattern -> C_Options -> Curry_Prelude.OP_Tuple2 (Curry_Prelude.OP_List Curry_Prelude.C_Char) (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Curry_Prelude.OP_Tuple2 Curry_Prelude.C_Int (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_AbstractHaskell.C_Expr -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_113 x3 x2 x1 x35 x6 x13 x10 cd cs = case x10 of
     (Curry_AbstractHaskell.C_Var x36) -> d_C__case_112 x35 x36 x6 x13 x3 x2 x1 (Curry_Prelude.d_OP_amp_amp (Curry_Prelude.d_C_apply (d_C_isInfixOpName cd cs) x13 cd cs) (Curry_Prelude.d_OP_amp_amp (Curry_Prelude.d_OP_eq_eq x6 x36 cd cs) (Curry_Prelude.d_OP_amp_amp (d_C_isAtom (Curry_AbstractHaskell.C_Symbol x35) cd cs) (Curry_Prelude.d_OP_slash_eq (Curry_AbstractHaskell.C_Var x36) (Curry_AbstractHaskell.C_Symbol x35) cd cs) cd cs) cd cs) cd cs) cd cs
     (Curry_AbstractHaskell.C_Lit x37) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_Symbol x38) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_Apply x39 x40) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_Lambda x41 x42) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_Let x43 x44) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_DoExpr x45) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_ListComp x46 x47) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_Case x48 x49) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_Typed x50 x51) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_IfThenElse x52 x53 x54) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.Choice_C_Expr d i l r) -> narrow d i (d_C__case_113 x3 x2 x1 x35 x6 x13 l cd cs) (d_C__case_113 x3 x2 x1 x35 x6 x13 r cd cs)
     (Curry_AbstractHaskell.Choices_C_Expr d i xs) -> narrows cs d i (\z -> d_C__case_113 x3 x2 x1 x35 x6 x13 z cd cs) xs
     (Curry_AbstractHaskell.Guard_C_Expr d c e) -> guardCons d c ((d_C__case_113 x3 x2 x1 x35 x6 x13 e cd) $! (addCs c cs))
     (Curry_AbstractHaskell.Fail_C_Expr d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_113" [(show x3),(show x2),(show x1),(show x35),(show x6),(show x13),(show x10)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_113" (showCons x10))

d_C__case_112 :: Curry_Prelude.OP_Tuple2 (Curry_Prelude.OP_List Curry_Prelude.C_Char) (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Curry_Prelude.OP_Tuple2 Curry_Prelude.C_Int (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Curry_Prelude.OP_Tuple2 Curry_Prelude.C_Int (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_AbstractHaskell.C_Expr -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_Pattern -> C_Options -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_112 x35 x36 x6 x13 x3 x2 x1 x37 cd cs = case x37 of
     Curry_Prelude.C_True -> Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString "(") (Curry_Prelude.d_OP_plus_plus x13 (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString " ") (Curry_Prelude.d_OP_plus_plus (d_C_showBoxedExpr x1 (Curry_AbstractHaskell.C_Symbol x35) cd cs) (Curry_Prelude.toCurryString ")") cd cs) cd cs) cd cs) cd cs
     Curry_Prelude.C_False -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_Prelude.Choice_C_Bool d i l r) -> narrow d i (d_C__case_112 x35 x36 x6 x13 x3 x2 x1 l cd cs) (d_C__case_112 x35 x36 x6 x13 x3 x2 x1 r cd cs)
     (Curry_Prelude.Choices_C_Bool d i xs) -> narrows cs d i (\z -> d_C__case_112 x35 x36 x6 x13 x3 x2 x1 z cd cs) xs
     (Curry_Prelude.Guard_C_Bool d c e) -> guardCons d c ((d_C__case_112 x35 x36 x6 x13 x3 x2 x1 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_C_Bool d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_112" [(show x35),(show x36),(show x6),(show x13),(show x3),(show x2),(show x1),(show x37)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_112" (showCons x37))

d_C__case_115 :: Curry_AbstractHaskell.C_Expr -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_Pattern -> C_Options -> Curry_AbstractHaskell.C_Literal -> Curry_Prelude.OP_Tuple2 Curry_Prelude.C_Int (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_AbstractHaskell.C_Expr -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_115 x3 x2 x1 x15 x6 x13 x10 cd cs = case x10 of
     (Curry_AbstractHaskell.C_Var x16) -> d_C__case_114 x15 x16 x6 x13 x3 x2 x1 (Curry_Prelude.d_OP_amp_amp (Curry_Prelude.d_C_apply (d_C_isInfixOpName cd cs) x13 cd cs) (Curry_Prelude.d_OP_amp_amp (Curry_Prelude.d_OP_eq_eq x6 x16 cd cs) (Curry_Prelude.d_OP_amp_amp (d_C_isAtom (Curry_AbstractHaskell.C_Lit x15) cd cs) (Curry_Prelude.d_OP_slash_eq (Curry_AbstractHaskell.C_Var x16) (Curry_AbstractHaskell.C_Lit x15) cd cs) cd cs) cd cs) cd cs) cd cs
     (Curry_AbstractHaskell.C_Lit x17) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_Symbol x18) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_Apply x19 x20) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_Lambda x21 x22) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_Let x23 x24) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_DoExpr x25) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_ListComp x26 x27) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_Case x28 x29) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_Typed x30 x31) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.C_IfThenElse x32 x33 x34) -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_AbstractHaskell.Choice_C_Expr d i l r) -> narrow d i (d_C__case_115 x3 x2 x1 x15 x6 x13 l cd cs) (d_C__case_115 x3 x2 x1 x15 x6 x13 r cd cs)
     (Curry_AbstractHaskell.Choices_C_Expr d i xs) -> narrows cs d i (\z -> d_C__case_115 x3 x2 x1 x15 x6 x13 z cd cs) xs
     (Curry_AbstractHaskell.Guard_C_Expr d c e) -> guardCons d c ((d_C__case_115 x3 x2 x1 x15 x6 x13 e cd) $! (addCs c cs))
     (Curry_AbstractHaskell.Fail_C_Expr d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_115" [(show x3),(show x2),(show x1),(show x15),(show x6),(show x13),(show x10)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_115" (showCons x10))

d_C__case_114 :: Curry_AbstractHaskell.C_Literal -> Curry_Prelude.OP_Tuple2 Curry_Prelude.C_Int (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Curry_Prelude.OP_Tuple2 Curry_Prelude.C_Int (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_AbstractHaskell.C_Expr -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_Pattern -> C_Options -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_114 x15 x16 x6 x13 x3 x2 x1 x17 cd cs = case x17 of
     Curry_Prelude.C_True -> Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString "(") (Curry_Prelude.d_OP_plus_plus x13 (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString " ") (Curry_Prelude.d_OP_plus_plus (d_C_showBoxedExpr x1 (Curry_AbstractHaskell.C_Lit x15) cd cs) (Curry_Prelude.toCurryString ")") cd cs) cd cs) cd cs) cd cs
     Curry_Prelude.C_False -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_Prelude.Choice_C_Bool d i l r) -> narrow d i (d_C__case_114 x15 x16 x6 x13 x3 x2 x1 l cd cs) (d_C__case_114 x15 x16 x6 x13 x3 x2 x1 r cd cs)
     (Curry_Prelude.Choices_C_Bool d i xs) -> narrows cs d i (\z -> d_C__case_114 x15 x16 x6 x13 x3 x2 x1 z cd cs) xs
     (Curry_Prelude.Guard_C_Bool d c e) -> guardCons d c ((d_C__case_114 x15 x16 x6 x13 x3 x2 x1 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_C_Bool d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_114" [(show x15),(show x16),(show x6),(show x13),(show x3),(show x2),(show x1),(show x17)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_114" (showCons x17))

d_C__case_118 :: Curry_AbstractHaskell.C_Expr -> Curry_Prelude.OP_Tuple2 Curry_Prelude.C_Int (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_AbstractHaskell.C_Expr -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_Pattern -> C_Options -> Curry_Prelude.OP_Tuple2 Curry_Prelude.C_Int (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Curry_Prelude.OP_Tuple2 (Curry_Prelude.OP_List Curry_Prelude.C_Char) (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_118 x10 x14 x13 x3 x2 x1 x6 x11 x15 cd cs = case x15 of
     Curry_Prelude.C_True -> d_C__case_117 x14 x6 x10 x3 x2 x1 x11 (Curry_Prelude.d_OP_eq_eq x6 x14 cd cs) cd cs
     Curry_Prelude.C_False -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_Prelude.Choice_C_Bool d i l r) -> narrow d i (d_C__case_118 x10 x14 x13 x3 x2 x1 x6 x11 l cd cs) (d_C__case_118 x10 x14 x13 x3 x2 x1 x6 x11 r cd cs)
     (Curry_Prelude.Choices_C_Bool d i xs) -> narrows cs d i (\z -> d_C__case_118 x10 x14 x13 x3 x2 x1 x6 x11 z cd cs) xs
     (Curry_Prelude.Guard_C_Bool d c e) -> guardCons d c ((d_C__case_118 x10 x14 x13 x3 x2 x1 x6 x11 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_C_Bool d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_118" [(show x10),(show x14),(show x13),(show x3),(show x2),(show x1),(show x6),(show x11),(show x15)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_118" (showCons x15))

d_C__case_117 :: Curry_Prelude.OP_Tuple2 Curry_Prelude.C_Int (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Curry_Prelude.OP_Tuple2 Curry_Prelude.C_Int (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Curry_AbstractHaskell.C_Expr -> Curry_AbstractHaskell.C_Expr -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_Pattern -> C_Options -> Curry_Prelude.OP_Tuple2 (Curry_Prelude.OP_List Curry_Prelude.C_Char) (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_117 x14 x6 x10 x3 x2 x1 x11 x15 cd cs = case x15 of
     Curry_Prelude.C_True -> Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString "(") (Curry_Prelude.d_OP_plus_plus (d_C_showBoxedExpr x1 x10 cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString " ") (Curry_Prelude.d_OP_plus_plus (d_C_showSymbol x1 x11 cd cs) (Curry_Prelude.toCurryString ")") cd cs) cd cs) cd cs) cd cs
     Curry_Prelude.C_False -> d_C__case_116 x6 x10 x3 x2 x1 x14 x11 (Curry_Prelude.d_OP_eq_eq x10 (Curry_AbstractHaskell.C_Var x6) cd cs) cd cs
     (Curry_Prelude.Choice_C_Bool d i l r) -> narrow d i (d_C__case_117 x14 x6 x10 x3 x2 x1 x11 l cd cs) (d_C__case_117 x14 x6 x10 x3 x2 x1 x11 r cd cs)
     (Curry_Prelude.Choices_C_Bool d i xs) -> narrows cs d i (\z -> d_C__case_117 x14 x6 x10 x3 x2 x1 x11 z cd cs) xs
     (Curry_Prelude.Guard_C_Bool d c e) -> guardCons d c ((d_C__case_117 x14 x6 x10 x3 x2 x1 x11 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_C_Bool d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_117" [(show x14),(show x6),(show x10),(show x3),(show x2),(show x1),(show x11),(show x15)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_117" (showCons x15))

d_C__case_116 :: Curry_Prelude.OP_Tuple2 Curry_Prelude.C_Int (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Curry_AbstractHaskell.C_Expr -> Curry_AbstractHaskell.C_Expr -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_Pattern -> C_Options -> Curry_Prelude.OP_Tuple2 Curry_Prelude.C_Int (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Curry_Prelude.OP_Tuple2 (Curry_Prelude.OP_List Curry_Prelude.C_Char) (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_116 x6 x10 x3 x2 x1 x14 x11 x15 cd cs = case x15 of
     Curry_Prelude.C_True -> Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString "(") (Curry_Prelude.d_OP_plus_plus (d_C_showSymbol x1 x11 cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString " ") (Curry_Prelude.d_OP_plus_plus (d_C_showExprOpt x1 (Curry_AbstractHaskell.C_Var x14) cd cs) (Curry_Prelude.toCurryString ")") cd cs) cd cs) cd cs) cd cs
     Curry_Prelude.C_False -> d_C_showLambda x1 x2 x3 cd cs
     (Curry_Prelude.Choice_C_Bool d i l r) -> narrow d i (d_C__case_116 x6 x10 x3 x2 x1 x14 x11 l cd cs) (d_C__case_116 x6 x10 x3 x2 x1 x14 x11 r cd cs)
     (Curry_Prelude.Choices_C_Bool d i xs) -> narrows cs d i (\z -> d_C__case_116 x6 x10 x3 x2 x1 x14 x11 z cd cs) xs
     (Curry_Prelude.Guard_C_Bool d c e) -> guardCons d c ((d_C__case_116 x6 x10 x3 x2 x1 x14 x11 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_C_Bool d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_116" [(show x6),(show x10),(show x3),(show x2),(show x1),(show x14),(show x11),(show x15)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_116" (showCons x15))

d_C__case_128 :: Curry_Prelude.OP_List Curry_Prelude.C_Char -> C_Options -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_128 x3 x1 x4 x5 cd cs = case x5 of
     Curry_Prelude.C_True -> x4
     Curry_Prelude.C_False -> d_C__case_127 x1 x3 x4 (Curry_Prelude.d_OP_eq_eq x3 (d_OP_uscore_hash_selR_at_Options_dot_currentModule x1 cd cs) cd cs) cd cs
     (Curry_Prelude.Choice_C_Bool d i l r) -> narrow d i (d_C__case_128 x3 x1 x4 l cd cs) (d_C__case_128 x3 x1 x4 r cd cs)
     (Curry_Prelude.Choices_C_Bool d i xs) -> narrows cs d i (\z -> d_C__case_128 x3 x1 x4 z cd cs) xs
     (Curry_Prelude.Guard_C_Bool d c e) -> guardCons d c ((d_C__case_128 x3 x1 x4 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_C_Bool d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_128" [(show x3),(show x1),(show x4),(show x5)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_128" (showCons x5))

d_C__case_127 :: C_Options -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_127 x1 x3 x4 x5 cd cs = case x5 of
     Curry_Prelude.C_True -> x4
     Curry_Prelude.C_False -> d_C__case_126 x4 x3 (Curry_Prelude.d_C_otherwise cd cs) cd cs
     (Curry_Prelude.Choice_C_Bool d i l r) -> narrow d i (d_C__case_127 x1 x3 x4 l cd cs) (d_C__case_127 x1 x3 x4 r cd cs)
     (Curry_Prelude.Choices_C_Bool d i xs) -> narrows cs d i (\z -> d_C__case_127 x1 x3 x4 z cd cs) xs
     (Curry_Prelude.Guard_C_Bool d c e) -> guardCons d c ((d_C__case_127 x1 x3 x4 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_C_Bool d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_127" [(show x1),(show x3),(show x4),(show x5)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_127" (showCons x5))

d_C__case_126 :: Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_126 x4 x3 x5 cd cs = case x5 of
     Curry_Prelude.C_True -> Curry_Prelude.d_OP_plus_plus x3 (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString ".") x4 cd cs) cd cs
     Curry_Prelude.C_False -> Curry_Prelude.d_C_failed cd cs
     (Curry_Prelude.Choice_C_Bool d i l r) -> narrow d i (d_C__case_126 x4 x3 l cd cs) (d_C__case_126 x4 x3 r cd cs)
     (Curry_Prelude.Choices_C_Bool d i xs) -> narrows cs d i (\z -> d_C__case_126 x4 x3 z cd cs) xs
     (Curry_Prelude.Guard_C_Bool d c e) -> guardCons d c ((d_C__case_126 x4 x3 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_C_Bool d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_126" [(show x4),(show x3),(show x5)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_126" (showCons x5))

d_C__case_129 :: Curry_Prelude.OP_Tuple2 (Curry_Prelude.OP_List Curry_Prelude.C_Char) (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> C_Options -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_129 x7 x1 x8 cd cs = case x8 of
     Curry_Prelude.C_True -> Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString "(") (Curry_Prelude.d_OP_plus_plus (d_C_showSymbol x1 x7 cd cs) (Curry_Prelude.toCurryString ")") cd cs) cd cs
     Curry_Prelude.C_False -> d_C_showSymbol x1 x7 cd cs
     (Curry_Prelude.Choice_C_Bool d i l r) -> narrow d i (d_C__case_129 x7 x1 l cd cs) (d_C__case_129 x7 x1 r cd cs)
     (Curry_Prelude.Choices_C_Bool d i xs) -> narrows cs d i (\z -> d_C__case_129 x7 x1 z cd cs) xs
     (Curry_Prelude.Guard_C_Bool d c e) -> guardCons d c ((d_C__case_129 x7 x1 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_C_Bool d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_129" [(show x7),(show x1),(show x8)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_129" (showCons x8))

d_C__case_130 :: Curry_Prelude.OP_Tuple2 Curry_Prelude.C_Int (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_130 x3 cd cs = case x3 of
     (Curry_Prelude.OP_Tuple2 x4 x5) -> Curry_Prelude.d_C_apply (d_C_showIdentifier cd cs) x5 cd cs
     (Curry_Prelude.Choice_OP_Tuple2 d i l r) -> narrow d i (d_C__case_130 l cd cs) (d_C__case_130 r cd cs)
     (Curry_Prelude.Choices_OP_Tuple2 d i xs) -> narrows cs d i (\z -> d_C__case_130 z cd cs) xs
     (Curry_Prelude.Guard_OP_Tuple2 d c e) -> guardCons d c ((d_C__case_130 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_Tuple2 d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_130" [(show x3)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_130" (showCons x3))

d_C__case_131 :: Curry_Prelude.OP_List Curry_AbstractHaskell.C_LocalDecl -> C_Options -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_131 x6 x1 x7 cd cs = case x7 of
     Curry_Prelude.C_True -> Curry_Prelude.OP_List
     Curry_Prelude.C_False -> Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString "\n   where\n") (d_C_showBlock (d_C_prefixMap (d_C_showLocalDecl x1) x6 (Curry_Prelude.toCurryString "\n") cd cs) cd cs) cd cs
     (Curry_Prelude.Choice_C_Bool d i l r) -> narrow d i (d_C__case_131 x6 x1 l cd cs) (d_C__case_131 x6 x1 r cd cs)
     (Curry_Prelude.Choices_C_Bool d i xs) -> narrows cs d i (\z -> d_C__case_131 x6 x1 z cd cs) xs
     (Curry_Prelude.Guard_C_Bool d c e) -> guardCons d c ((d_C__case_131 x6 x1 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_C_Bool d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_131" [(show x6),(show x1),(show x7)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_131" (showCons x7))

d_C__case_134 :: Curry_Prelude.OP_List (Curry_Prelude.OP_Tuple2 Curry_AbstractHaskell.C_Expr Curry_AbstractHaskell.C_Expr) -> C_Options -> Curry_Prelude.OP_Tuple2 Curry_AbstractHaskell.C_Expr Curry_AbstractHaskell.C_Expr -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_134 x4 x1 x3 cd cs = case x3 of
     (Curry_Prelude.OP_Tuple2 x5 x6) -> d_C__case_133 x5 x4 x6 x1 (Curry_Prelude.d_OP_amp_amp (Curry_Prelude.d_OP_eq_eq x4 Curry_Prelude.OP_List cd cs) (Curry_Prelude.d_OP_eq_eq x5 (Curry_AbstractHaskell.C_Symbol (Curry_Prelude.OP_Tuple2 (Curry_Names.d_C_prelude cd cs) (Curry_Prelude.toCurryString "success"))) cd cs) cd cs) cd cs
     (Curry_Prelude.Choice_OP_Tuple2 d i l r) -> narrow d i (d_C__case_134 x4 x1 l cd cs) (d_C__case_134 x4 x1 r cd cs)
     (Curry_Prelude.Choices_OP_Tuple2 d i xs) -> narrows cs d i (\z -> d_C__case_134 x4 x1 z cd cs) xs
     (Curry_Prelude.Guard_OP_Tuple2 d c e) -> guardCons d c ((d_C__case_134 x4 x1 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_Tuple2 d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_134" [(show x4),(show x1),(show x3)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_134" (showCons x3))

d_C__case_133 :: Curry_AbstractHaskell.C_Expr -> Curry_Prelude.OP_List (Curry_Prelude.OP_Tuple2 Curry_AbstractHaskell.C_Expr Curry_AbstractHaskell.C_Expr) -> Curry_AbstractHaskell.C_Expr -> C_Options -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_133 x5 x4 x6 x1 x7 cd cs = case x7 of
     Curry_Prelude.C_True -> Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString " = ") (d_C_showExprOpt x1 x6 cd cs) cd cs
     Curry_Prelude.C_False -> d_C__case_132 x4 x6 x5 x1 (Curry_Prelude.d_C_otherwise cd cs) cd cs
     (Curry_Prelude.Choice_C_Bool d i l r) -> narrow d i (d_C__case_133 x5 x4 x6 x1 l cd cs) (d_C__case_133 x5 x4 x6 x1 r cd cs)
     (Curry_Prelude.Choices_C_Bool d i xs) -> narrows cs d i (\z -> d_C__case_133 x5 x4 x6 x1 z cd cs) xs
     (Curry_Prelude.Guard_C_Bool d c e) -> guardCons d c ((d_C__case_133 x5 x4 x6 x1 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_C_Bool d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_133" [(show x5),(show x4),(show x6),(show x1),(show x7)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_133" (showCons x7))

d_C__case_132 :: Curry_Prelude.OP_List (Curry_Prelude.OP_Tuple2 Curry_AbstractHaskell.C_Expr Curry_AbstractHaskell.C_Expr) -> Curry_AbstractHaskell.C_Expr -> Curry_AbstractHaskell.C_Expr -> C_Options -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_132 x4 x6 x5 x1 x7 cd cs = case x7 of
     Curry_Prelude.C_True -> Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString "\n") (d_C_showBlock (d_C_combineMap (d_C_showCrhs x1) (Curry_Prelude.OP_Cons (Curry_Prelude.OP_Tuple2 x5 x6) x4) (Curry_Prelude.toCurryString "\n") cd cs) cd cs) cd cs
     Curry_Prelude.C_False -> Curry_Prelude.d_C_failed cd cs
     (Curry_Prelude.Choice_C_Bool d i l r) -> narrow d i (d_C__case_132 x4 x6 x5 x1 l cd cs) (d_C__case_132 x4 x6 x5 x1 r cd cs)
     (Curry_Prelude.Choices_C_Bool d i xs) -> narrows cs d i (\z -> d_C__case_132 x4 x6 x5 x1 z cd cs) xs
     (Curry_Prelude.Guard_C_Bool d c e) -> guardCons d c ((d_C__case_132 x4 x6 x5 x1 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_C_Bool d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_132" [(show x4),(show x6),(show x5),(show x1),(show x7)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_132" (showCons x7))

d_C__case_135 :: Curry_Prelude.OP_List Curry_AbstractHaskell.C_LocalDecl -> C_Options -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_135 x5 x1 x6 cd cs = case x6 of
     Curry_Prelude.C_True -> Curry_Prelude.OP_List
     Curry_Prelude.C_False -> Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString "\n   where\n") (d_C_showBlock (d_C_prefixMap (d_C_showLocalDecl x1) x5 (Curry_Prelude.toCurryString "\n") cd cs) cd cs) cd cs
     (Curry_Prelude.Choice_C_Bool d i l r) -> narrow d i (d_C__case_135 x5 x1 l cd cs) (d_C__case_135 x5 x1 r cd cs)
     (Curry_Prelude.Choices_C_Bool d i xs) -> narrows cs d i (\z -> d_C__case_135 x5 x1 z cd cs) xs
     (Curry_Prelude.Guard_C_Bool d c e) -> guardCons d c ((d_C__case_135 x5 x1 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_C_Bool d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_135" [(show x5),(show x1),(show x6)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_135" (showCons x6))

d_C__case_136 :: Curry_Prelude.C_Int -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_136 x3 x6 x5 x1 x2 x7 cd cs = case x7 of
     Curry_Prelude.C_True -> Curry_Prelude.d_OP_plus_plus x5 (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString " ") (Curry_Prelude.d_OP_plus_plus x2 x6 cd cs) cd cs) cd cs
     Curry_Prelude.C_False -> Curry_Prelude.d_OP_plus_plus x1 (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString " ") (Curry_Prelude.d_OP_plus_plus x5 x6 cd cs) cd cs) cd cs
     (Curry_Prelude.Choice_C_Bool d i l r) -> narrow d i (d_C__case_136 x3 x6 x5 x1 x2 l cd cs) (d_C__case_136 x3 x6 x5 x1 x2 r cd cs)
     (Curry_Prelude.Choices_C_Bool d i xs) -> narrows cs d i (\z -> d_C__case_136 x3 x6 x5 x1 x2 z cd cs) xs
     (Curry_Prelude.Guard_C_Bool d c e) -> guardCons d c ((d_C__case_136 x3 x6 x5 x1 x2 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_C_Bool d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_136" [(show x3),(show x6),(show x5),(show x1),(show x2),(show x7)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_136" (showCons x7))

d_C__case_140 :: Curry_Prelude.OP_Tuple2 (Curry_Prelude.OP_List Curry_Prelude.C_Char) (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> C_Options -> Curry_Prelude.C_Int -> Curry_AbstractHaskell.C_TypeSig -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_AbstractHaskell.C_Rules -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_140 x4 x1 x5 x7 x3 x8 cd cs = case x8 of
     (Curry_AbstractHaskell.C_Rules x9) -> d_C__case_139 x9 x1 x5 x7 x3 x4 cd cs
     Curry_AbstractHaskell.C_External -> Curry_Prelude.OP_List
     (Curry_AbstractHaskell.Choice_C_Rules d i l r) -> narrow d i (d_C__case_140 x4 x1 x5 x7 x3 l cd cs) (d_C__case_140 x4 x1 x5 x7 x3 r cd cs)
     (Curry_AbstractHaskell.Choices_C_Rules d i xs) -> narrows cs d i (\z -> d_C__case_140 x4 x1 x5 x7 x3 z cd cs) xs
     (Curry_AbstractHaskell.Guard_C_Rules d c e) -> guardCons d c ((d_C__case_140 x4 x1 x5 x7 x3 e cd) $! (addCs c cs))
     (Curry_AbstractHaskell.Fail_C_Rules d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_140" [(show x4),(show x1),(show x5),(show x7),(show x3),(show x8)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_140" (showCons x8))

d_C__case_139 :: Curry_Prelude.OP_List Curry_AbstractHaskell.C_Rule -> C_Options -> Curry_Prelude.C_Int -> Curry_AbstractHaskell.C_TypeSig -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.OP_Tuple2 (Curry_Prelude.OP_List Curry_Prelude.C_Char) (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_139 x9 x1 x5 x7 x3 x4 cd cs = case x4 of
     (Curry_Prelude.OP_Tuple2 x10 x11) -> let
          x12 = Curry_Prelude.d_C_apply (d_C_isInfixOpName cd cs) x11 cd cs
          x13 = d_C__case_137 x11 x12 cd cs
           in (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.d_C_apply (d_C_funcComment cd cs) x3 cd cs) (Curry_Prelude.d_OP_plus_plus (d_C_showTypeSig x1 x11 x7 cd cs) (d_C__case_138 x11 x9 x1 x5 x13 x12 cd cs) cd cs) cd cs)
     (Curry_Prelude.Choice_OP_Tuple2 d i l r) -> narrow d i (d_C__case_139 x9 x1 x5 x7 x3 l cd cs) (d_C__case_139 x9 x1 x5 x7 x3 r cd cs)
     (Curry_Prelude.Choices_OP_Tuple2 d i xs) -> narrows cs d i (\z -> d_C__case_139 x9 x1 x5 x7 x3 z cd cs) xs
     (Curry_Prelude.Guard_OP_Tuple2 d c e) -> guardCons d c ((d_C__case_139 x9 x1 x5 x7 x3 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_Tuple2 d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_139" [(show x9),(show x1),(show x5),(show x7),(show x3),(show x4)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_139" (showCons x4))

d_C__case_137 :: Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_137 x11 x12 cd cs = case x12 of
     Curry_Prelude.C_True -> Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString "(") (Curry_Prelude.d_OP_plus_plus x11 (Curry_Prelude.toCurryString ")") cd cs) cd cs
     Curry_Prelude.C_False -> x11
     (Curry_Prelude.Choice_C_Bool d i l r) -> narrow d i (d_C__case_137 x11 l cd cs) (d_C__case_137 x11 r cd cs)
     (Curry_Prelude.Choices_C_Bool d i xs) -> narrows cs d i (\z -> d_C__case_137 x11 z cd cs) xs
     (Curry_Prelude.Guard_C_Bool d c e) -> guardCons d c ((d_C__case_137 x11 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_C_Bool d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_137" [(show x11),(show x12)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_137" (showCons x12))

d_C__case_138 :: Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_Rule -> C_Options -> Curry_Prelude.C_Int -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_138 x11 x9 x1 x5 x13 x12 cd cs = case x12 of
     Curry_Prelude.C_True -> d_OP_showFuncDeclOpt_dot_rulePrints_dot_162 x13 x11 x1 x9 x5 cd cs
     Curry_Prelude.C_False -> Curry_Prelude.d_OP_plus_plus x11 (d_C_prefixInter (d_C_showRule x1) x9 (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString "\n") x11 cd cs) cd cs) cd cs
     (Curry_Prelude.Choice_C_Bool d i l r) -> narrow d i (d_C__case_138 x11 x9 x1 x5 x13 l cd cs) (d_C__case_138 x11 x9 x1 x5 x13 r cd cs)
     (Curry_Prelude.Choices_C_Bool d i xs) -> narrows cs d i (\z -> d_C__case_138 x11 x9 x1 x5 x13 z cd cs) xs
     (Curry_Prelude.Guard_C_Bool d c e) -> guardCons d c ((d_C__case_138 x11 x9 x1 x5 x13 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_C_Bool d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_138" [(show x11),(show x9),(show x1),(show x5),(show x13),(show x12)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_138" (showCons x12))

d_C__case_141 :: Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.C_Char -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_141 x3 x2 x4 cd cs = case x4 of
     Curry_Prelude.C_True -> Curry_Prelude.OP_Cons x2 (Curry_Prelude.OP_Cons (Curry_Prelude.C_Char '_'#) x3)
     Curry_Prelude.C_False -> Curry_Prelude.OP_Cons x2 x3
     (Curry_Prelude.Choice_C_Bool d i l r) -> narrow d i (d_C__case_141 x3 x2 l cd cs) (d_C__case_141 x3 x2 r cd cs)
     (Curry_Prelude.Choices_C_Bool d i xs) -> narrows cs d i (\z -> d_C__case_141 x3 x2 z cd cs) xs
     (Curry_Prelude.Guard_C_Bool d c e) -> guardCons d c ((d_C__case_141 x3 x2 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_C_Bool d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_141" [(show x3),(show x2),(show x4)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_141" (showCons x4))

d_C__case_142 :: Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_142 x2 x3 cd cs = case x3 of
     Curry_Prelude.C_True -> Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString "(") (Curry_Prelude.d_OP_plus_plus x2 (Curry_Prelude.toCurryString ")") cd cs) cd cs
     Curry_Prelude.C_False -> x2
     (Curry_Prelude.Choice_C_Bool d i l r) -> narrow d i (d_C__case_142 x2 l cd cs) (d_C__case_142 x2 r cd cs)
     (Curry_Prelude.Choices_C_Bool d i xs) -> narrows cs d i (\z -> d_C__case_142 x2 z cd cs) xs
     (Curry_Prelude.Guard_C_Bool d c e) -> guardCons d c ((d_C__case_142 x2 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_C_Bool d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_142" [(show x2),(show x3)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_142" (showCons x3))

d_C__case_143 :: Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_143 x2 x3 cd cs = case x3 of
     Curry_Prelude.C_True -> Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString "(") (Curry_Prelude.d_OP_plus_plus x2 (Curry_Prelude.toCurryString ")") cd cs) cd cs
     Curry_Prelude.C_False -> x2
     (Curry_Prelude.Choice_C_Bool d i l r) -> narrow d i (d_C__case_143 x2 l cd cs) (d_C__case_143 x2 r cd cs)
     (Curry_Prelude.Choices_C_Bool d i xs) -> narrows cs d i (\z -> d_C__case_143 x2 z cd cs) xs
     (Curry_Prelude.Guard_C_Bool d c e) -> guardCons d c ((d_C__case_143 x2 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_C_Bool d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_143" [(show x2),(show x3)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_143" (showCons x3))

d_C__case_146 :: Curry_Prelude.OP_List Curry_AbstractHaskell.C_TypeExpr -> C_Options -> Curry_Prelude.C_Bool -> Curry_Prelude.OP_Tuple2 (Curry_Prelude.OP_List Curry_Prelude.C_Char) (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_146 x10 x1 x2 x9 cd cs = case x9 of
     (Curry_Prelude.OP_Tuple2 x11 x12) -> d_C__case_145 x12 x11 x10 x9 x1 x2 (Curry_Prelude.d_OP_amp_amp (Curry_Prelude.d_OP_eq_eq x11 (Curry_Names.d_C_prelude cd cs) cd cs) (Curry_Prelude.d_OP_eq_eq x12 (Curry_Prelude.toCurryString "untyped") cd cs) cd cs) cd cs
     (Curry_Prelude.Choice_OP_Tuple2 d i l r) -> narrow d i (d_C__case_146 x10 x1 x2 l cd cs) (d_C__case_146 x10 x1 x2 r cd cs)
     (Curry_Prelude.Choices_OP_Tuple2 d i xs) -> narrows cs d i (\z -> d_C__case_146 x10 x1 x2 z cd cs) xs
     (Curry_Prelude.Guard_OP_Tuple2 d c e) -> guardCons d c ((d_C__case_146 x10 x1 x2 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_Tuple2 d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_146" [(show x10),(show x1),(show x2),(show x9)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_146" (showCons x9))

d_C__case_145 :: Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_TypeExpr -> Curry_Prelude.OP_Tuple2 (Curry_Prelude.OP_List Curry_Prelude.C_Char) (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> C_Options -> Curry_Prelude.C_Bool -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_145 x12 x11 x10 x9 x1 x2 x13 cd cs = case x13 of
     Curry_Prelude.C_True -> Curry_Prelude.toCurryString "-"
     Curry_Prelude.C_False -> d_C__case_144 x10 x9 x1 x2 (Curry_Prelude.d_C_otherwise cd cs) cd cs
     (Curry_Prelude.Choice_C_Bool d i l r) -> narrow d i (d_C__case_145 x12 x11 x10 x9 x1 x2 l cd cs) (d_C__case_145 x12 x11 x10 x9 x1 x2 r cd cs)
     (Curry_Prelude.Choices_C_Bool d i xs) -> narrows cs d i (\z -> d_C__case_145 x12 x11 x10 x9 x1 x2 z cd cs) xs
     (Curry_Prelude.Guard_C_Bool d c e) -> guardCons d c ((d_C__case_145 x12 x11 x10 x9 x1 x2 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_C_Bool d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_145" [(show x12),(show x11),(show x10),(show x9),(show x1),(show x2),(show x13)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_145" (showCons x13))

d_C__case_144 :: Curry_Prelude.OP_List Curry_AbstractHaskell.C_TypeExpr -> Curry_Prelude.OP_Tuple2 (Curry_Prelude.OP_List Curry_Prelude.C_Char) (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> C_Options -> Curry_Prelude.C_Bool -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_144 x10 x9 x1 x2 x11 cd cs = case x11 of
     Curry_Prelude.C_True -> d_C_maybeShowBrackets (Curry_Prelude.d_OP_amp_amp x2 (Curry_Prelude.d_C_not (Curry_Prelude.d_C_null x10 cd cs) cd cs) cd cs) (d_C_showTypeCons x1 x9 x10 cd cs) cd cs
     Curry_Prelude.C_False -> Curry_Prelude.d_C_failed cd cs
     (Curry_Prelude.Choice_C_Bool d i l r) -> narrow d i (d_C__case_144 x10 x9 x1 x2 l cd cs) (d_C__case_144 x10 x9 x1 x2 r cd cs)
     (Curry_Prelude.Choices_C_Bool d i xs) -> narrows cs d i (\z -> d_C__case_144 x10 x9 x1 x2 z cd cs) xs
     (Curry_Prelude.Guard_C_Bool d c e) -> guardCons d c ((d_C__case_144 x10 x9 x1 x2 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_C_Bool d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_144" [(show x10),(show x9),(show x1),(show x2),(show x11)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_144" (showCons x11))

d_C__case_147 :: Curry_Prelude.OP_Tuple2 Curry_Prelude.C_Int (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_147 x4 cd cs = case x4 of
     (Curry_Prelude.OP_Tuple2 x5 x6) -> d_C_showTypeVar (Curry_Prelude.d_C_apply (d_C_showIdentifier cd cs) x6 cd cs) cd cs
     (Curry_Prelude.Choice_OP_Tuple2 d i l r) -> narrow d i (d_C__case_147 l cd cs) (d_C__case_147 r cd cs)
     (Curry_Prelude.Choices_OP_Tuple2 d i xs) -> narrows cs d i (\z -> d_C__case_147 z cd cs) xs
     (Curry_Prelude.Guard_OP_Tuple2 d c e) -> guardCons d c ((d_C__case_147 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_Tuple2 d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_147" [(show x4)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_147" (showCons x4))

d_C__case_148 :: Curry_Prelude.OP_List Curry_AbstractHaskell.C_Context -> C_Options -> Curry_AbstractHaskell.C_Context -> Curry_Prelude.OP_List Curry_AbstractHaskell.C_Context -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_148 x2 x1 x3 x4 cd cs = case x4 of
     Curry_Prelude.OP_List -> Curry_Prelude.d_OP_plus_plus (d_C_showClass x1 x3 cd cs) (Curry_Prelude.toCurryString " => ") cd cs
     (Curry_Prelude.OP_Cons x5 x6) -> Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString "(") (Curry_Prelude.d_OP_plus_plus (Curry_List.d_C_intercalate (Curry_Prelude.toCurryString ",") (Curry_Prelude.d_C_map (d_C_showClass x1) x2 cd cs) cd cs) (Curry_Prelude.toCurryString ") => ") cd cs) cd cs
     (Curry_Prelude.Choice_OP_List d i l r) -> narrow d i (d_C__case_148 x2 x1 x3 l cd cs) (d_C__case_148 x2 x1 x3 r cd cs)
     (Curry_Prelude.Choices_OP_List d i xs) -> narrows cs d i (\z -> d_C__case_148 x2 x1 x3 z cd cs) xs
     (Curry_Prelude.Guard_OP_List d c e) -> guardCons d c ((d_C__case_148 x2 x1 x3 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_List d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_148" [(show x2),(show x1),(show x3),(show x4)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_148" (showCons x4))

d_C__case_150 :: Curry_Prelude.Curry t0 => Curry_AbstractHaskell.C_Rule -> C_Options -> Curry_Prelude.OP_Tuple2 t0 (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_150 x4 x1 x3 cd cs = case x3 of
     (Curry_Prelude.OP_Tuple2 x5 x6) -> d_C__case_149 x6 x4 x1 (Curry_Prelude.d_C_apply (d_C_isInfixOpName cd cs) x6 cd cs) cd cs
     (Curry_Prelude.Choice_OP_Tuple2 d i l r) -> narrow d i (d_C__case_150 x4 x1 l cd cs) (d_C__case_150 x4 x1 r cd cs)
     (Curry_Prelude.Choices_OP_Tuple2 d i xs) -> narrows cs d i (\z -> d_C__case_150 x4 x1 z cd cs) xs
     (Curry_Prelude.Guard_OP_Tuple2 d c e) -> guardCons d c ((d_C__case_150 x4 x1 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_Tuple2 d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_150" [(show x4),(show x1),(show x3)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_150" (showCons x3))

d_C__case_149 :: Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_AbstractHaskell.C_Rule -> C_Options -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_149 x6 x4 x1 x7 cd cs = case x7 of
     Curry_Prelude.C_True -> Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString "  (") (Curry_Prelude.d_OP_plus_plus x6 (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString ")") (Curry_Prelude.d_OP_plus_plus (d_C_showRule x1 x4 cd cs) (Curry_Prelude.toCurryString "\n") cd cs) cd cs) cd cs) cd cs
     Curry_Prelude.C_False -> Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString "  ") (Curry_Prelude.d_OP_plus_plus x6 (Curry_Prelude.d_OP_plus_plus (d_C_showRule x1 x4 cd cs) (Curry_Prelude.toCurryString "\n") cd cs) cd cs) cd cs
     (Curry_Prelude.Choice_C_Bool d i l r) -> narrow d i (d_C__case_149 x6 x4 x1 l cd cs) (d_C__case_149 x6 x4 x1 r cd cs)
     (Curry_Prelude.Choices_C_Bool d i xs) -> narrows cs d i (\z -> d_C__case_149 x6 x4 x1 z cd cs) xs
     (Curry_Prelude.Guard_C_Bool d c e) -> guardCons d c ((d_C__case_149 x6 x4 x1 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_C_Bool d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_149" [(show x6),(show x4),(show x1),(show x7)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_149" (showCons x7))

d_C__case_152 :: Curry_Prelude.OP_List Curry_AbstractHaskell.C_ConsDecl -> C_Options -> Curry_Prelude.OP_List (Curry_Prelude.OP_Tuple2 Curry_Prelude.C_Int (Curry_Prelude.OP_List Curry_Prelude.C_Char)) -> Curry_Prelude.OP_Tuple2 (Curry_Prelude.OP_List Curry_Prelude.C_Char) (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_152 x10 x1 x9 x7 x11 cd cs = case x11 of
     Curry_Prelude.C_True -> Curry_Prelude.OP_List
     Curry_Prelude.C_False -> d_C__case_151 x10 x1 x9 x7 (Curry_Prelude.d_C_otherwise cd cs) cd cs
     (Curry_Prelude.Choice_C_Bool d i l r) -> narrow d i (d_C__case_152 x10 x1 x9 x7 l cd cs) (d_C__case_152 x10 x1 x9 x7 r cd cs)
     (Curry_Prelude.Choices_C_Bool d i xs) -> narrows cs d i (\z -> d_C__case_152 x10 x1 x9 x7 z cd cs) xs
     (Curry_Prelude.Guard_C_Bool d c e) -> guardCons d c ((d_C__case_152 x10 x1 x9 x7 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_C_Bool d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_152" [(show x10),(show x1),(show x9),(show x7),(show x11)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_152" (showCons x11))

d_C__case_151 :: Curry_Prelude.OP_List Curry_AbstractHaskell.C_ConsDecl -> C_Options -> Curry_Prelude.OP_List (Curry_Prelude.OP_Tuple2 Curry_Prelude.C_Int (Curry_Prelude.OP_List Curry_Prelude.C_Char)) -> Curry_Prelude.OP_Tuple2 (Curry_Prelude.OP_List Curry_Prelude.C_Char) (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_151 x10 x1 x9 x7 x11 cd cs = case x11 of
     Curry_Prelude.C_True -> Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString "data ") (Curry_Prelude.d_OP_plus_plus (d_C_showSymbol x1 x7 cd cs) (Curry_Prelude.d_OP_plus_plus (d_C_prefixMap (d_C_showTypeExpr x1 Curry_Prelude.C_False) (Curry_Prelude.d_C_map (acceptCs id Curry_AbstractHaskell.C_TVar) x9 cd cs) (Curry_Prelude.toCurryString " ") cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString "\n") (d_C_showBlock (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString "= ") (d_C_combineMap (d_C_showConsDecl x1) x10 (Curry_Prelude.toCurryString "\n| ") cd cs) cd cs) cd cs) cd cs) cd cs) cd cs) cd cs
     Curry_Prelude.C_False -> Curry_Prelude.d_C_failed cd cs
     (Curry_Prelude.Choice_C_Bool d i l r) -> narrow d i (d_C__case_151 x10 x1 x9 x7 l cd cs) (d_C__case_151 x10 x1 x9 x7 r cd cs)
     (Curry_Prelude.Choices_C_Bool d i xs) -> narrows cs d i (\z -> d_C__case_151 x10 x1 x9 x7 z cd cs) xs
     (Curry_Prelude.Guard_C_Bool d c e) -> guardCons d c ((d_C__case_151 x10 x1 x9 x7 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_C_Bool d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_151" [(show x10),(show x1),(show x9),(show x7),(show x11)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_151" (showCons x11))

d_C__case_154 :: Curry_Prelude.C_Int -> Curry_AbstractHaskell.C_Fixity -> Curry_Prelude.OP_Tuple2 (Curry_Prelude.OP_List Curry_Prelude.C_Char) (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_154 x4 x3 x2 cd cs = case x2 of
     (Curry_Prelude.OP_Tuple2 x5 x6) -> Curry_Prelude.d_OP_plus_plus (d_C_showFixity x3 cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString " ") (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.d_C_show x4 cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString " ") (d_C__case_153 x6 (Curry_Prelude.d_C_apply (d_C_isInfixOpName cd cs) x6 cd cs) cd cs) cd cs) cd cs) cd cs) cd cs
     (Curry_Prelude.Choice_OP_Tuple2 d i l r) -> narrow d i (d_C__case_154 x4 x3 l cd cs) (d_C__case_154 x4 x3 r cd cs)
     (Curry_Prelude.Choices_OP_Tuple2 d i xs) -> narrows cs d i (\z -> d_C__case_154 x4 x3 z cd cs) xs
     (Curry_Prelude.Guard_OP_Tuple2 d c e) -> guardCons d c ((d_C__case_154 x4 x3 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_Tuple2 d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_154" [(show x4),(show x3),(show x2)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_154" (showCons x2))

d_C__case_153 :: Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_153 x6 x7 cd cs = case x7 of
     Curry_Prelude.C_True -> x6
     Curry_Prelude.C_False -> Curry_Prelude.OP_Cons (Curry_Prelude.C_Char '`'#) (Curry_Prelude.d_OP_plus_plus x6 (Curry_Prelude.toCurryString "`") cd cs)
     (Curry_Prelude.Choice_C_Bool d i l r) -> narrow d i (d_C__case_153 x6 l cd cs) (d_C__case_153 x6 r cd cs)
     (Curry_Prelude.Choices_C_Bool d i xs) -> narrows cs d i (\z -> d_C__case_153 x6 z cd cs) xs
     (Curry_Prelude.Guard_C_Bool d c e) -> guardCons d c ((d_C__case_153 x6 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_C_Bool d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_153" [(show x6),(show x7)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_153" (showCons x7))

d_C__case_157 :: Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.C_Bool -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_157 x2 x1 x3 cd cs = case x3 of
     Curry_Prelude.C_True -> Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString "import ") x2 cd cs
     Curry_Prelude.C_False -> d_C__case_156 x2 x1 cd cs
     (Curry_Prelude.Choice_C_Bool d i l r) -> narrow d i (d_C__case_157 x2 x1 l cd cs) (d_C__case_157 x2 x1 r cd cs)
     (Curry_Prelude.Choices_C_Bool d i xs) -> narrows cs d i (\z -> d_C__case_157 x2 x1 z cd cs) xs
     (Curry_Prelude.Guard_C_Bool d c e) -> guardCons d c ((d_C__case_157 x2 x1 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_C_Bool d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_157" [(show x2),(show x1),(show x3)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_157" (showCons x3))

d_C__case_156 :: Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_156 x2 x1 cd cs = case x1 of
     Curry_Prelude.C_True -> Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString "import qualified ") (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.d_C_apply (Curry_Names.d_C_addTrace cd cs) x2 cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString " as ") x2 cd cs) cd cs) cd cs
     Curry_Prelude.C_False -> d_C__case_155 x2 (Curry_Prelude.d_C_otherwise cd cs) cd cs
     (Curry_Prelude.Choice_C_Bool d i l r) -> narrow d i (d_C__case_156 x2 l cd cs) (d_C__case_156 x2 r cd cs)
     (Curry_Prelude.Choices_C_Bool d i xs) -> narrows cs d i (\z -> d_C__case_156 x2 z cd cs) xs
     (Curry_Prelude.Guard_C_Bool d c e) -> guardCons d c ((d_C__case_156 x2 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_C_Bool d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_156" [(show x2),(show x1)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_156" (showCons x1))

d_C__case_155 :: Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_155 x2 x3 cd cs = case x3 of
     Curry_Prelude.C_True -> Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString "import qualified ") x2 cd cs
     Curry_Prelude.C_False -> Curry_Prelude.d_C_failed cd cs
     (Curry_Prelude.Choice_C_Bool d i l r) -> narrow d i (d_C__case_155 x2 l cd cs) (d_C__case_155 x2 r cd cs)
     (Curry_Prelude.Choices_C_Bool d i xs) -> narrows cs d i (\z -> d_C__case_155 x2 z cd cs) xs
     (Curry_Prelude.Guard_C_Bool d c e) -> guardCons d c ((d_C__case_155 x2 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_C_Bool d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_155" [(show x2),(show x3)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_155" (showCons x3))

d_C__case_159 :: Curry_Prelude.OP_Tuple2 (Curry_Prelude.OP_List Curry_Prelude.C_Char) (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_159 x3 cd cs = case x3 of
     (Curry_Prelude.OP_Tuple2 x8 x9) -> d_C__case_158 x9 (Curry_Prelude.d_C_apply (d_C_isInfixOpName cd cs) x9 cd cs) cd cs
     (Curry_Prelude.Choice_OP_Tuple2 d i l r) -> narrow d i (d_C__case_159 l cd cs) (d_C__case_159 r cd cs)
     (Curry_Prelude.Choices_OP_Tuple2 d i xs) -> narrows cs d i (\z -> d_C__case_159 z cd cs) xs
     (Curry_Prelude.Guard_OP_Tuple2 d c e) -> guardCons d c ((d_C__case_159 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_Tuple2 d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_159" [(show x3)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_159" (showCons x3))

d_C__case_158 :: Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_158 x9 x10 cd cs = case x10 of
     Curry_Prelude.C_True -> Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString "(") (Curry_Prelude.d_OP_plus_plus x9 (Curry_Prelude.toCurryString ")") cd cs) cd cs
     Curry_Prelude.C_False -> x9
     (Curry_Prelude.Choice_C_Bool d i l r) -> narrow d i (d_C__case_158 x9 l cd cs) (d_C__case_158 x9 r cd cs)
     (Curry_Prelude.Choices_C_Bool d i xs) -> narrows cs d i (\z -> d_C__case_158 x9 z cd cs) xs
     (Curry_Prelude.Guard_C_Bool d c e) -> guardCons d c ((d_C__case_158 x9 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_C_Bool d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_158" [(show x9),(show x10)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_158" (showCons x10))

d_C__case_160 :: Curry_Prelude.OP_Tuple2 (Curry_Prelude.OP_List Curry_Prelude.C_Char) (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_160 x14 cd cs = case x14 of
     (Curry_Prelude.OP_Tuple2 x18 x19) -> x19
     (Curry_Prelude.Choice_OP_Tuple2 d i l r) -> narrow d i (d_C__case_160 l cd cs) (d_C__case_160 r cd cs)
     (Curry_Prelude.Choices_OP_Tuple2 d i xs) -> narrows cs d i (\z -> d_C__case_160 z cd cs) xs
     (Curry_Prelude.Guard_OP_Tuple2 d c e) -> guardCons d c ((d_C__case_160 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_Tuple2 d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_160" [(show x14)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_160" (showCons x14))

d_C__case_161 :: Curry_Prelude.OP_Tuple2 (Curry_Prelude.OP_List Curry_Prelude.C_Char) (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_161 x8 cd cs = case x8 of
     (Curry_Prelude.OP_Tuple2 x12 x13) -> x13
     (Curry_Prelude.Choice_OP_Tuple2 d i l r) -> narrow d i (d_C__case_161 l cd cs) (d_C__case_161 r cd cs)
     (Curry_Prelude.Choices_OP_Tuple2 d i xs) -> narrows cs d i (\z -> d_C__case_161 z cd cs) xs
     (Curry_Prelude.Guard_OP_Tuple2 d c e) -> guardCons d c ((d_C__case_161 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_Tuple2 d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_161" [(show x8)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_161" (showCons x8))

d_C__case_162 :: Curry_Prelude.OP_Tuple2 (Curry_Prelude.OP_List Curry_Prelude.C_Char) (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_162 x2 cd cs = case x2 of
     (Curry_Prelude.OP_Tuple2 x6 x7) -> x7
     (Curry_Prelude.Choice_OP_Tuple2 d i l r) -> narrow d i (d_C__case_162 l cd cs) (d_C__case_162 r cd cs)
     (Curry_Prelude.Choices_OP_Tuple2 d i xs) -> narrows cs d i (\z -> d_C__case_162 z cd cs) xs
     (Curry_Prelude.Guard_OP_Tuple2 d c e) -> guardCons d c ((d_C__case_162 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_OP_Tuple2 d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_162" [(show x2)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_162" (showCons x2))

d_C__case_163 :: Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_163 x7 x8 cd cs = case x8 of
     Curry_Prelude.C_True -> Curry_Prelude.OP_List
     Curry_Prelude.C_False -> Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString "\n\n") x7 cd cs
     (Curry_Prelude.Choice_C_Bool d i l r) -> narrow d i (d_C__case_163 x7 l cd cs) (d_C__case_163 x7 r cd cs)
     (Curry_Prelude.Choices_C_Bool d i xs) -> narrows cs d i (\z -> d_C__case_163 x7 z cd cs) xs
     (Curry_Prelude.Guard_C_Bool d c e) -> guardCons d c ((d_C__case_163 x7 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_C_Bool d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_163" [(show x7),(show x8)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_163" (showCons x8))

d_C__case_164 :: Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_164 x2 x1 cd cs = case x1 of
     Curry_Prelude.C_True -> Curry_Prelude.d_C_apply (Curry_Names.d_C_addTrace cd cs) x2 cd cs
     Curry_Prelude.C_False -> x2
     (Curry_Prelude.Choice_C_Bool d i l r) -> narrow d i (d_C__case_164 x2 l cd cs) (d_C__case_164 x2 r cd cs)
     (Curry_Prelude.Choices_C_Bool d i xs) -> narrows cs d i (\z -> d_C__case_164 x2 z cd cs) xs
     (Curry_Prelude.Guard_C_Bool d c e) -> guardCons d c ((d_C__case_164 x2 e cd) $! (addCs c cs))
     (Curry_Prelude.Fail_C_Bool d info) -> failCons d (traceFail "AbstractHaskellPrinter._case_164" [(show x2),(show x1)] info)
     _ -> failCons cd (consFail "AbstractHaskellPrinter._case_164" (showCons x1))